@@ -1,6 +1,6 @@
-/* $Id: e2p_config.c 1265 2008-09-22 02:13:38Z tpgww $
+/* $Id: e2p_config.c 2344 2011-04-11 22:46:24Z tpgww $
 
-Copyright (C) 2006-2008 tooar <tooar@gmx.net>
+Copyright (C) 2006-2011 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -14,8 +14,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 /**
@@ -126,9 +125,9 @@ static gchar *set_private_names [] =
 
 //cache for toggle values, static for the session
 static gboolean flags[MAX_FLAGS];
-GPtrArray *treeset_names = NULL;	//names to use when checking importable tree options
-gboolean rebuild_needed;
-E2P_ConfigData *srt;	//copy of runtime ptr for some funcs that don't get rt directly
+static GPtrArray *treeset_names = NULL;	//names to use when checking importable tree options
+static gboolean rebuild_needed;
+static E2P_ConfigData *srt;	//copy of runtime ptr for func(s) that don't get rt directly
 
   /*********************/
  /***** utilities *****/
@@ -221,20 +220,20 @@ static void _e2pc_toggle_cb (GtkWidget *widget, gpointer flagnum)
 @param f enumerated value of flag to be associated with the button
 @param state T/F initial state of the toggle
 @param label  translated string for the button label
-@param callback callback function to be connected
+@param callback "toggled" signal callback function to be connected
 
 @return the button widget
 */
 /*static GtkWidget *_e2pc_create_option_button (GtkWidget *container,
-	flag_t f, gboolean state, gchar *label, gpointer callback_fn)
+	flag_t f, gboolean state, gchar *label,
+	void (*callback)(GtkToggleButton*,gpointer))
 {
 	GtkWidget *btn = gtk_check_button_new_with_mnemonic (label);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (btn), state);
-	g_signal_connect (G_OBJECT (btn), "toggled", G_CALLBACK (callback_fn),
+	g_signal_connect (G_OBJECT (btn), "toggled", G_CALLBACK (callback),
 		(gpointer) f);  //CHECKME = correct value?
 	gtk_container_add (GTK_CONTAINER (container), btn);
-	gtk_widget_show (btn);
-	return (btn);
+	return btn;
 } */
 /**
 @brief create and show a check button in a specified container
@@ -249,15 +248,12 @@ static void _e2pc_toggle_cb (GtkWidget *widget, gpointer flagnum)
 static GtkWidget *_e2pc_create_check_button (GtkWidget *container,
 	flag_t f, gboolean state, gchar *label)
 {
-//	GtkWidget *btn = _e2pc_create_option_button
-//		(container, f, state, label, _e2pc_toggle_cb);
 	GtkWidget *btn = gtk_check_button_new_with_mnemonic (label);
 	g_signal_connect (G_OBJECT (btn), "toggled", G_CALLBACK (_e2pc_toggle_cb),
 		(gpointer) f);  //CHECKME = correct value?
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (btn), state);
 	gtk_container_add (GTK_CONTAINER (container), btn);
-	gtk_widget_show (btn);
-	return (btn);
+	return btn;
 }
 /**
 @brief create and show a radio btn in a specified container
@@ -280,8 +276,7 @@ static GtkWidget *_e2pc_create_radio_button (GtkWidget *container,
 	g_signal_connect (G_OBJECT (btn), "toggled", G_CALLBACK (_e2pc_toggle_cb),
 		(gpointer) f);
 	gtk_container_add (GTK_CONTAINER (container), btn);
-	gtk_widget_show (btn);
-	return (btn);
+	return btn;
 }
 /**
 @brief create and show a radio btn in a specified container
@@ -304,8 +299,7 @@ static GtkWidget *_e2pc_create_radio_grouped_button (GtkWidget *container, GtkWi
 		(gpointer) f);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (btn), state);
   gtk_container_add (GTK_CONTAINER (container), btn);
-  gtk_widget_show (btn);
-  return (btn);
+  return btn;
 }
 /**
 @brief check whether a tree option is to be imported
@@ -321,7 +315,7 @@ static gboolean _e2pc_match_tree (gchar *setname)
 			i < treeset_names->len;
 			i++, iterator++)
 	{
-		if (g_str_equal (*iterator, setname))
+		if (!strcmp (*iterator, setname))
 		{
 			g_ptr_array_remove_index_fast (treeset_names, i);
 			return TRUE;
@@ -355,7 +349,7 @@ static void _e2pc_filter_options (gchar *contents, bitflag_t flags)
 		lsplit = g_strsplit (line, "=", 2);
 		if (lsplit[1] != NULL)
 		{
-			if (!g_str_equal (lsplit[1], "<")) //not a tree set
+			if (strcmp (lsplit[1], "<")) //not a tree set
 			{
 				if (flags & (E2PC_NONTREE | E2PC_ALL))
 				{
@@ -419,7 +413,7 @@ static void _e2pc_filter_options (gchar *contents, bitflag_t flags)
 
 @return
 */
-static void _e2pc_import_cb (GtkWidget *button, E2P_ConfigData *rt)
+static void _e2pc_import_cb (GtkButton *button, E2P_ConfigData *rt)
 {
 	//convert import flags to bitflags that we can play with
 	bitflag_t import_flags = 0;
@@ -563,9 +557,9 @@ static void _e2pc_import_cb (GtkWidget *button, E2P_ConfigData *rt)
 				gchar *path1 = g_build_filename (e2_cl_options.config_dir,
 					default_config_file, NULL);
 				gchar *local1 = F_FILENAME_TO_LOCALE (path1);
-				gchar *path2 = g_strconcat (default_config_file, "-before-import", NULL);
-				gchar *path3 = g_build_filename (e2_cl_options.config_dir, path2, NULL);
-				gchar *local2 = F_FILENAME_TO_LOCALE (path3);
+				gchar *name2 = g_strconcat (default_config_file, "-before-import", NULL);
+				gchar *path2 = g_build_filename (e2_cl_options.config_dir, name2, NULL);
+				gchar *local2 = F_FILENAME_TO_LOCALE (path2);
 				gchar *savepath = e2_utils_get_tempname (local2);
 				gdk_threads_leave ();	//downstream errors invoke local mutex locking
 #ifdef E2_VFS
@@ -577,11 +571,11 @@ static void _e2pc_import_cb (GtkWidget *button, E2P_ConfigData *rt)
 #endif
 				gdk_threads_enter ();
 				g_free (path1);
+				F_FREE (local1, path1);
+				g_free (name2);
 				g_free (path2);
-				g_free (path3);
+				F_FREE (local2, path2);
 				g_free (savepath);
-				F_FREE (local1);
-				F_FREE (local2);
 				//save updated config file
 				e2_option_file_write (NULL);
 				//do any upgrades needed
@@ -615,7 +609,7 @@ static void _e2pc_import_cb (GtkWidget *button, E2P_ConfigData *rt)
 			//recreate window and runtime data structs as appropriate
 			//this is like e2_option_refresh (FALSE, TRUE) but omits some fatal
 			//parts of that
-			app.rebuild_enabled = FALSE;	//block recursion
+			e2_option_disable_config_checks ();	//block recursion
 
 			if (import_flags & E2PC_KEYS)
 				e2_keybinding_clean ();
@@ -639,7 +633,7 @@ static void _e2pc_import_cb (GtkWidget *button, E2P_ConfigData *rt)
 						if (member->data != NULL)
 						{
 
-							if (! g_str_equal(((Plugin *)member->data)->signature, ANAME VERSION))
+							if (strcmp(((Plugin *)member->data)->signature, ANAME VERSION))
 							//FIXME handle plugins that are not allowed to be unloaded
 							{
 								e2_plugins_unload1 ((Plugin *)member->data, TRUE);
@@ -653,7 +647,8 @@ static void _e2pc_import_cb (GtkWidget *button, E2P_ConfigData *rt)
 			//clear relevant current information
 //			e2_option_clear_data ();
 			//now re-create things, in the same order as at session-start
-		//	e2_option_default_register ();
+//			e2_option_default_register ();
+//			e2_option_date_style (); //customise date-format option
 
 //			if (reload)
 //			{
@@ -661,7 +656,7 @@ static void _e2pc_import_cb (GtkWidget *button, E2P_ConfigData *rt)
 //				e2_option_file_read ();
 //			}
 
-		//	e2_option_tree_install_defaults ();
+//			e2_option_tree_install_defaults ();
 
 			if (import_flags & E2PC_PLUGINS)
 				e2_plugins_load_all (); // load plugins (if any) CHECKME what if this plugin is reloaded ?
@@ -670,23 +665,27 @@ static void _e2pc_import_cb (GtkWidget *button, E2P_ConfigData *rt)
 			e2_pane_create_option_data (&app.pane1);
 			e2_pane_create_option_data (&app.pane2);
 
-			e2_toolbar_initialise (E2_BAR_PANE1);
-			e2_toolbar_initialise (E2_BAR_PANE2);
-			e2_toolbar_initialise (E2_BAR_TASK);
-			e2_toolbar_initialise (E2_BAR_COMMAND);
-
-		//	if (recreate)
+//			if (recreate)
 				e2_window_recreate (&app.window); //this also recreates key bindings
-		//	else
-		//	{
-		//		e2_keybinding_register_all ();
-		//		e2_mousebinding_register_all ();
-		//	}
+/*			else
+			{
+				e2_toolbar_create (&app.pane1.toolbar);
+				e2_toolbar_rebadge (&app.pane1.toolbar);
+				e2_toolbar_create (&app.pane2.toolbar);
+				e2_toolbar_rebadge (&app.pane2.toolbar);
+				e2_toolbar_create (&app.toolbar);
+				e2_toolbar_create (&app.commandbar);
 
+				e2_keybinding_register_all ();
+#ifdef E2_MOUSECUSTOM
+				e2_mousebinding_register_all ();
+#endif
+			}
+*/
 			if (import_flags & E2PC_FILETYPES)
 				e2_filetype_add_all ();
 
-			app.rebuild_enabled = TRUE;	//unblock
+			e2_option_enable_config_checks ();
 		}
 	}
 	else
@@ -696,7 +695,7 @@ static void _e2pc_import_cb (GtkWidget *button, E2P_ConfigData *rt)
 	}
 	if (*filepath == '\0')
 		g_free (realpath);
-	F_FREE (localpath);
+	F_FREE (localpath, realpath);
 	g_ptr_array_free (treeset_names, TRUE);
 }
 /**
@@ -707,7 +706,7 @@ static void _e2pc_import_cb (GtkWidget *button, E2P_ConfigData *rt)
 
 @return
 */
-static void _e2pc_select_config_cb (GtkWidget *button, E2P_ConfigData *rt)
+static void _e2pc_select_config_cb (GtkButton *button, E2P_ConfigData *rt)
 {
 	GtkWidget *dialog = gtk_file_chooser_dialog_new (NULL,
 		GTK_WINDOW (rt->dialog), GTK_FILE_CHOOSER_ACTION_OPEN,
@@ -733,7 +732,7 @@ static void _e2pc_select_config_cb (GtkWidget *button, E2P_ConfigData *rt)
 		gchar *openpath = F_FILENAME_FROM_LOCALE (local);
 		gtk_entry_set_text (GTK_ENTRY (rt->open_entry), openpath);
 		g_free (local);
-		F_FREE (openpath);
+		F_FREE (openpath, local);
 	}
 	gtk_widget_destroy (dialog);
 }
@@ -745,7 +744,7 @@ static void _e2pc_select_config_cb (GtkWidget *button, E2P_ConfigData *rt)
 
 @return
 */
-static void _e2pc_save_cb (GtkWidget *button, E2P_ConfigData *rt)
+static void _e2pc_save_cb (GtkButton *button, E2P_ConfigData *rt)
 {
 	const gchar *savepath = gtk_entry_get_text (GTK_ENTRY (rt->save_entry));
 	if (*savepath != '\0')
@@ -761,7 +760,15 @@ static void _e2pc_save_cb (GtkWidget *button, E2P_ConfigData *rt)
 			if (e2_fs_access2 (dlocal E2_ERR_NONE()) == 0)
 #endif
 			{
-				DialogButtons choice = e2_dialog_ow_check (NULL, dlocal, NONE);
+				gdk_threads_leave ();
+				DialogButtons choice = e2_dialog_ow_check (NULL,
+#ifdef E2_VFS
+					&ddata,
+#else
+					dlocal,
+#endif
+					NONE);
+				gdk_threads_enter ();
 				if (choice != OK)
 				{
 					g_free (dlocal);
@@ -781,7 +788,7 @@ static void _e2pc_save_cb (GtkWidget *button, E2P_ConfigData *rt)
 
 @return
 */
-static void _e2pc_saveas_cb (GtkWidget *button, E2P_ConfigData *rt)
+static void _e2pc_saveas_cb (GtkButton *button, E2P_ConfigData *rt)
 {
 	GtkWidget *dialog = gtk_file_chooser_dialog_new (NULL,
 		GTK_WINDOW (rt->dialog), GTK_FILE_CHOOSER_ACTION_SAVE,
@@ -816,7 +823,15 @@ static void _e2pc_saveas_cb (GtkWidget *button, E2P_ConfigData *rt)
 		&& e2_fs_access2 (local E2_ERR_NONE()) == 0)
 #endif
 		{
-			DialogButtons choice = e2_dialog_ow_check (NULL, local, NONE);
+			gdk_threads_leave ();
+			DialogButtons choice = e2_dialog_ow_check (NULL,
+#ifdef E2_VFS
+				&ddata,
+#else
+				local,
+#endif
+				NONE);
+			gdk_threads_enter ();
 			if (choice != OK)
 			{
 				gtk_widget_destroy (dialog);
@@ -828,19 +843,19 @@ static void _e2pc_saveas_cb (GtkWidget *button, E2P_ConfigData *rt)
 		gchar *savepath = F_FILENAME_FROM_LOCALE (local);
 		gtk_entry_set_text (GTK_ENTRY (rt->save_entry), savepath);
 		g_free (local);
-		F_FREE (savepath);
+		F_FREE (savepath, local);
 	}
 	gtk_widget_destroy (dialog);
 }
 /**
 @brief select directory containing custom icons to be used
 
-@param widget clicked button, UNUSED
+@param button clicked button, UNUSED
 @param rt dialog runtime struct
 
 @return
 */
-static void _e2pc_select_icondir_cb (GtkWidget *widget, E2P_ConfigData *rt)
+static void _e2pc_select_icondir_cb (GtkButton *button, E2P_ConfigData *rt)
 {
 	GtkWidget *dialog = gtk_file_chooser_dialog_new (NULL,
 		GTK_WINDOW (rt->dialog), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
@@ -867,7 +882,7 @@ static void _e2pc_select_icondir_cb (GtkWidget *widget, E2P_ConfigData *rt)
 //		{
 			gchar *opendir = F_FILENAME_FROM_LOCALE (local);
 			gtk_entry_set_text (GTK_ENTRY (rt->icondir_entry), opendir);
-			F_FREE (opendir);
+			F_FREE (opendir, local);
 //		}
 		g_free (local);
 	}
@@ -876,19 +891,19 @@ static void _e2pc_select_icondir_cb (GtkWidget *widget, E2P_ConfigData *rt)
 /**
 @brief apply icon directory
 
-@param widget clicked button, UNUSED
+@param button clicked button, UNUSED
 @param rt dialog runtime struct
 
 @return
 */
-static void _e2pc_apply_icondir_cb (GtkWidget *widget, E2P_ConfigData *rt)
+static void _e2pc_apply_icondir_cb (GtkButton *button, E2P_ConfigData *rt)
 {
 	gchar *utfpath = g_strdup (gtk_entry_get_text (GTK_ENTRY (rt->icondir_entry)));
 	if (g_str_has_suffix (utfpath, G_DIR_SEPARATOR_S))
 		*(utfpath + strlen (utfpath) - sizeof(gchar)) = '\0';
 
 	gchar *localpath = F_FILENAME_TO_LOCALE (utfpath);
-	if (g_str_equal (localpath, ICON_DIR))
+	if (!strcmp (localpath, ICON_DIR))
 		e2_option_bool_set ("use-icon-dir", FALSE);
 	else
 	{
@@ -897,18 +912,18 @@ static void _e2pc_apply_icondir_cb (GtkWidget *widget, E2P_ConfigData *rt)
 		e2_option_str_set_direct (set, utfpath);
 		e2_toolbar_recreate_all ();
 	}
-	F_FREE (localpath);
 	g_free (utfpath);
+	F_FREE (localpath, utfpath);
 }
 /**
 @brief select directory in which to save custom icons now in use
 
-@param widget clicked button, UNUSED
+@param button clicked button, UNUSED
 @param rt dialog runtime struct
 
 @return
 */
-static void _e2pc_select_iconsavedir_cb (GtkWidget *widget, E2P_ConfigData *rt)
+static void _e2pc_select_iconsavedir_cb (GtkButton *button, E2P_ConfigData *rt)
 {
 	GtkWidget *dialog = gtk_file_chooser_dialog_new (NULL,
 		GTK_WINDOW (rt->dialog), GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER,
@@ -943,7 +958,15 @@ static void _e2pc_select_iconsavedir_cb (GtkWidget *widget, E2P_ConfigData *rt)
 			&& e2_fs_access2 (local E2_ERR_NONE()) == 0)
 #endif
 		{
-			DialogButtons choice = e2_dialog_ow_check (NULL, local, NONE);
+			gdk_threads_leave ();
+			DialogButtons choice = e2_dialog_ow_check (NULL,
+#ifdef E2_VFS
+				&ddata,
+#else
+				local,
+#endif
+				NONE);
+			gdk_threads_enter ();
 			if (choice != OK)
 			{
 				gtk_widget_destroy (dialog);
@@ -955,19 +978,19 @@ static void _e2pc_select_iconsavedir_cb (GtkWidget *widget, E2P_ConfigData *rt)
 		gchar *openpath = F_FILENAME_FROM_LOCALE (local);
 		gtk_entry_set_text (GTK_ENTRY (rt->iconsavedir_entry), openpath);
 		g_free (local);
-		F_FREE (openpath);
+		F_FREE (openpath, local);
 	}
 	gtk_widget_destroy (dialog);
 }
 /**
 @brief copy icons to specified directory
 If the dir is not absolute, curr_view->dir is prepended
-@param widget clicked button, UNUSED
+@param button clicked button, UNUSED
 @param rt dialog runtime struct
 
 @return
 */
-static void _e2pc_apply_iconsavedir_cb (GtkWidget *widget, E2P_ConfigData *rt)
+static void _e2pc_apply_iconsavedir_cb (GtkButton *button, E2P_ConfigData *rt)
 {
 	const gchar *path;
 	gchar *slocal, *dest, *dlocal;
@@ -995,7 +1018,20 @@ static void _e2pc_apply_iconsavedir_cb (GtkWidget *widget, E2P_ConfigData *rt)
 #else
 		&& e2_fs_access2 (dlocal E2_ERR_NONE()) == 0)
 #endif
-		result = e2_dialog_ow_check (slocal, dlocal, NONE);
+	{
+#ifdef E2_VFS
+		VPATH sdata = { slocal, NULL };
+#endif
+		gdk_threads_leave ();
+		result = e2_dialog_ow_check (
+#ifdef E2_VFS
+			&sdata, &ddata,
+#else
+			slocal, dlocal,
+#endif
+			NONE);
+		gdk_threads_enter ();
+	}
 	else
 		result = OK;
 
@@ -1015,7 +1051,7 @@ static void _e2pc_apply_iconsavedir_cb (GtkWidget *widget, E2P_ConfigData *rt)
 
 	g_free (slocal);
 	g_free (dest);
-	F_FREE (dlocal);
+	F_FREE (dlocal, dest);
 }
 
   /***************************/
@@ -1033,14 +1069,13 @@ static void _e2pc_apply_iconsavedir_cb (GtkWidget *widget, E2P_ConfigData *rt)
 static void	_e2pc_make_export_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 {
 	GtkWidget *vbox = gtk_vbox_new (FALSE, 0);
-	gtk_widget_show (vbox);
 #ifdef E2_ASSISTED
 	GtkWidget *label =
 #endif
 	e2_widget_add_label (vbox, _("Save configuration data in"),
 		0.5, 0.5, FALSE, E2_PADDING);
 
-	const gchar *savepath;
+	const gchar *savedir;
 	gchar *local = F_FILENAME_TO_LOCALE (e2_cl_options.config_dir);
 	//default export to config dir if it's usable
 #ifdef E2_VFS
@@ -1049,13 +1084,13 @@ static void	_e2pc_make_export_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 #else
 	if (e2_fs_is_dir3 (local E2_ERR_NONE()) && !e2_fs_access (local, R_OK | W_OK | X_OK E2_ERR_NONE()))
 #endif
-		savepath = e2_cl_options.config_dir;
+		savedir = e2_cl_options.config_dir;
 	else
-		savepath = g_get_home_dir ();
-	F_FREE (local);
+		savedir = g_get_home_dir ();
+	F_FREE (local, e2_cl_options.config_dir);
 
-	gchar *savename = g_build_filename (savepath, default_config_file, NULL);
-	local = F_FILENAME_TO_LOCALE (savename);
+	gchar *savepath = g_build_filename (savedir, default_config_file, NULL);
+	local = F_FILENAME_TO_LOCALE (savepath);
 	gchar *temppath, *tempext, *local2;
 	guint i = 0;
 	while (TRUE)
@@ -1064,10 +1099,10 @@ static void	_e2pc_make_export_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 		local2 = F_FILENAME_TO_LOCALE (tempext);
 		temppath = e2_utils_strcat (local, local2);
 		g_free (tempext);
-		F_FREE (local2);
+		F_FREE (local2, tempext);
 		E2_ERR_DECLARE
 #ifdef E2_VFS
-		ddata.localpath = temppath;
+		ddata.path = temppath;
 		if (e2_fs_access2 (&ddata E2_ERR_PTR())	//checkme need for vfs ?
 #else
 		if (e2_fs_access2 (temppath E2_ERR_PTR())	//checkme need for vfs ?
@@ -1081,21 +1116,22 @@ static void	_e2pc_make_export_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 		g_free (temppath);
 		i++;
 	}
-	F_FREE (local);
-	g_free (savename);
+	g_free (savepath);
+	F_FREE (local, savepath);
+
 	savepath = F_FILENAME_FROM_LOCALE (temppath);
-	if (savepath != temppath)
-		g_free (temppath);
 
 	rt->save_entry = e2_widget_add_entry (vbox, (gchar *) savepath, TRUE, TRUE);
 #ifdef E2_ASSISTED
-	e2_widget_set_label_relations (GTK_LABEL (label), rt->save_entry);
+	e2_widget_set_label_relations (label, rt->save_entry);
 #endif
 	gtk_widget_set_size_request (rt->save_entry, 400, -1);
 
+	g_free (temppath);
+	F_FREE (savepath, temppath);
+
 	GtkWidget *hbox = gtk_hbutton_box_new ();
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, E2_PADDING);
-	gtk_widget_show (hbox);
 
 	gtk_box_set_spacing (GTK_BOX (hbox), E2_PADDING_LARGE);
 	gtk_button_box_set_layout (GTK_BUTTON_BOX (hbox), GTK_BUTTONBOX_END);
@@ -1110,7 +1146,6 @@ static void	_e2pc_make_export_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 	GtkWidget *
 #endif
 	label = gtk_label_new (_("export"));
-	gtk_widget_show (label);
 	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox, label);
 }
 /**
@@ -1124,7 +1159,6 @@ static void	_e2pc_make_export_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 static void	_e2pc_make_import_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 {
 	GtkWidget *vbox = gtk_vbox_new (FALSE, 0);
-	gtk_widget_show (vbox);
 #ifdef E2_ASSISTED
 	GtkWidget *label =
 #endif
@@ -1143,19 +1177,18 @@ static void	_e2pc_make_import_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 		openpath = e2_cl_options.config_dir;
 	else
 		openpath = g_get_home_dir ();
-	F_FREE (local);
+	F_FREE (local, e2_cl_options.config_dir);
 
 	gchar *openname = g_build_filename (openpath, default_config_file, NULL);
 
 	rt->open_entry = e2_widget_add_entry (vbox, (gchar *) openname, TRUE, TRUE);
 #ifdef E2_ASSISTED
-	e2_widget_set_label_relations (GTK_LABEL (label), rt->open_entry);
+	e2_widget_set_label_relations (label, rt->open_entry);
 #endif
 	gtk_widget_set_size_request (rt->open_entry, 400, -1);
 
 	GtkWidget *hbox = gtk_hbutton_box_new ();
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, E2_PADDING);
-	gtk_widget_show (hbox);
 
 	gtk_box_set_spacing (GTK_BOX (hbox), E2_PADDING_LARGE);
 	gtk_button_box_set_layout (GTK_BUTTON_BOX (hbox), GTK_BUTTONBOX_END);
@@ -1184,10 +1217,8 @@ static void	_e2pc_make_import_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 		FALSE, _("_specific group option(s)"));
 	rt->expander = gtk_expander_new_with_mnemonic (_("_groups"));
 	gtk_box_pack_start (GTK_BOX (vbox), rt->expander, FALSE, FALSE, 0);
-	gtk_widget_show (rt->expander);
 	GtkWidget *expvbox = gtk_vbox_new (FALSE, 0);
 	gtk_container_add (GTK_CONTAINER (rt->expander), expvbox);
-	gtk_widget_show (expvbox);
 	hbox = e2_widget_add_box (expvbox, TRUE, E2_PADDING_SMALL,
 		FALSE, TRUE, E2_PADDING);
 	gchar *labeltxt = _e2pc_get_setlabel (PANEBAR1_P);
@@ -1248,7 +1279,6 @@ static void	_e2pc_make_import_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 	GtkWidget *
 #endif
 	label = gtk_label_new (_("import"));
-	gtk_widget_show (label);
 	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox, label);
 }
 /**
@@ -1262,7 +1292,6 @@ static void	_e2pc_make_import_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 static void	_e2pc_make_icons_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 {
 	GtkWidget *vbox = gtk_vbox_new (FALSE, 0);
-	gtk_widget_show (vbox);
 #ifdef E2_ASSISTED
 	GtkWidget *label =
 #endif
@@ -1285,14 +1314,13 @@ static void	_e2pc_make_icons_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 
 	rt->icondir_entry = e2_widget_add_entry (vbox, openpath, TRUE, TRUE);
 #ifdef E2_ASSISTED
-	e2_widget_set_label_relations (GTK_LABEL (label), rt->icondir_entry);
+	e2_widget_set_label_relations (label, rt->icondir_entry);
 #endif
 	g_free (openpath);
 	gtk_widget_set_size_request (rt->icondir_entry, 400, -1);
 
 	GtkWidget *hbox = gtk_hbutton_box_new ();
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, E2_PADDING);
-	gtk_widget_show (hbox);
 
 	gtk_box_set_spacing (GTK_BOX (hbox), E2_PADDING_LARGE);
 	gtk_button_box_set_layout (GTK_BUTTON_BOX (hbox), GTK_BUTTONBOX_END);
@@ -1316,14 +1344,13 @@ static void	_e2pc_make_icons_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 		_("icons"), NULL);
 	rt->iconsavedir_entry = e2_widget_add_entry (vbox, openpath, TRUE, TRUE);
 #ifdef E2_ASSISTED
-	e2_widget_set_label_relations (GTK_LABEL (label), rt->iconsavedir_entry);
+	e2_widget_set_label_relations (label, rt->iconsavedir_entry);
 #endif
 	g_free (openpath);
 	gtk_widget_set_size_request (rt->iconsavedir_entry, 400, -1);
 
 	hbox = gtk_hbutton_box_new ();
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, E2_PADDING);
-	gtk_widget_show (hbox);
 
 	gtk_box_set_spacing (GTK_BOX (hbox), E2_PADDING_LARGE);
 	gtk_button_box_set_layout (GTK_BUTTON_BOX (hbox), GTK_BUTTONBOX_END);
@@ -1336,7 +1363,6 @@ static void	_e2pc_make_icons_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 	gtk_box_pack_start (GTK_BOX (hbox), btn, FALSE, FALSE, 0);
 
 	GtkWidget *pagelabel = gtk_label_new (_("icons"));
-	gtk_widget_show (pagelabel);
 	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox, pagelabel);
 }
 /**
@@ -1349,29 +1375,37 @@ static void	_e2pc_make_icons_tab (GtkWidget *notebook, E2P_ConfigData *rt)
 */
 static gboolean _e2p_config_dialog_create (gpointer from, E2_ActionRuntime *art)
 {
-	E2P_ConfigData rt;
+	if (srt == NULL)
+	{
+		E2P_ConfigData rt;
 
-	rt.dialog = e2_dialog_create (NULL, NULL, _("manage configuration data"),
-		DUMMY_RESPONSE_CB, NULL);
-	GtkWidget *vbox =
+		srt = &rt; //toggle-callback needs extra data
+
+		rt.dialog = e2_dialog_create (NULL, NULL, _("manage configuration data"),
+			DUMMY_RESPONSE_CB, NULL);
+		GtkWidget *vbox =
 #ifdef USE_GTK2_14
-		gtk_dialog_get_content_area (GTK_DIALOG (rt.dialog));
+			gtk_dialog_get_content_area (GTK_DIALOG (rt.dialog));
 #else
-		GTK_DIALOG (rt.dialog)->vbox;
+			GTK_DIALOG (rt.dialog)->vbox;
 #endif
-	//populate it with widgets
-	GtkWidget *notebook = e2_widget_add_notebook
-		(vbox, TRUE, 0, NULL, NULL);
-	_e2pc_make_export_tab (notebook, &rt);	//page 0
-	_e2pc_make_import_tab (notebook, &rt);	//page 1
-	_e2pc_make_icons_tab (notebook, &rt);	//page 2
-	gtk_widget_show (notebook);
+		//populate it with widgets
+		GtkWidget *notebook = e2_widget_add_notebook
+			(vbox, TRUE, 0, NULL, NULL);
+		_e2pc_make_export_tab (notebook, srt);	//page 0
+		_e2pc_make_import_tab (notebook, srt);	//page 1
+		_e2pc_make_icons_tab (notebook, srt);	//page 2
 
-	e2_dialog_set_negative_response (rt.dialog, GTK_RESPONSE_CLOSE);
-	//block until user selects
-	e2_dialog_show (rt.dialog, app.main_window,
-		E2_DIALOG_BLOCKED | E2_DIALOG_DONT_SHOW_ALL | E2_DIALOG_FREE,
-		&E2_BUTTON_CLOSE, NULL);	//FIXME close btn is not suitable here
+		e2_dialog_set_negative_response (rt.dialog, GTK_RESPONSE_CLOSE);
+		//block until user selects
+		e2_dialog_show (rt.dialog, app.main_window,
+			E2_DIALOG_BLOCKED /*| E2_DIALOG_DONT_SHOW_ALL*/ | E2_DIALOG_FREE,
+			&E2_BUTTON_CLOSE, NULL);	//FIXME close btn is not suitable here
+
+		srt = NULL;
+	}
+	else
+		gtk_window_present (GTK_WINDOW (srt->dialog));
 
 	return TRUE;
 }
@@ -1395,11 +1429,13 @@ gboolean init_plugin (Plugin *p)
 
   if (p->action == NULL)
   {
-	//no need to free this
-	gchar *action_name = g_strconcat (_A(3),".",aname,NULL);
-	p->action = e2_plugins_action_register
-	  (action_name, E2_ACTION_TYPE_ITEM, _e2p_config_dialog_create, NULL, FALSE, 0, NULL);
-	return TRUE;
+	//don't free name string here
+	E2_Action plugact =
+	{g_strconcat (_A(3),".",aname,NULL),_e2p_config_dialog_create,FALSE,E2_ACTION_TYPE_ITEM,0,NULL,NULL};
+	p->action = e2_plugins_action_register (&plugact);
+	if (p->action != NULL)
+		return TRUE;
+	g_free (plugact.name);
   }
   return FALSE;
 }
