@@ -1,14 +1,9 @@
-/* $Id: e2p_cpbar.c 1344 2008-11-02 06:53:19Z tpgww $
-**************************************
+/* $Id: e2p_cpbar.c 2469 2012-03-16 00:19:24Z tpgww $
 
-Emelcopybar - a plugin for emelFM2
-by Florian Zaehringer
-with help from tooar and tom to port it to Gtk+2
+Copyright (C) 2004-2012 tooar <tooar@emelfm2.net>
+Portions copyright (C) 2004 Florian Zaehringer <flo.zaehringer@web.de>
+Portions copyright (C) 1999 Michael Clark
 
-source of source-code and inspiration:
-emelFM by Michael Clark
-
-**************************************
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by
@@ -21,8 +16,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 /**
@@ -145,7 +139,7 @@ _e2p_cpbar_progress (E2_ProgressData *data)
 	E2_BarData pdata;
 	pdata.totalsize = 0;
 #ifdef E2_VFS
-	ddata.localpath = localpath;
+	ddata.path = localpath;
 	if (!e2_fs_lstat (&ddata, &sb E2_ERR_NONE()))
 #else
 	if (!e2_fs_lstat (localpath, &sb E2_ERR_NONE()))
@@ -294,7 +288,11 @@ _e2p_cpbar_exec (VPATH *slocal, VPATH *dlocal, E2_FileTaskMode flags,
 			shortsrc, shortdest, num1, num2);
 		gdk_threads_enter ();
 		gtk_label_set_text (GTK_LABEL (wdata->label), labeltext);
+#ifdef USE_GTK2_18
+		if (!gtk_widget_get_visible (wdata->dialog))
+#else
 		if (!GTK_WIDGET_VISIBLE (wdata->dialog))
+#endif
 			gtk_widget_show (wdata->dialog);
 		gdk_threads_leave ();
 		g_free (shortsrc);
@@ -361,7 +359,12 @@ _e2p_cpbar_exec (VPATH *slocal, VPATH *dlocal, E2_FileTaskMode flags,
 			gdk_threads_leave ();
 
 			if (wdata->bflags & E2_BARTASK_PAUSEREQ //pause was requested
-				&& GTK_IS_DIALOG (wdata->dialog) && GTK_WIDGET_VISIBLE (wdata->dialog))
+				&& GTK_IS_DIALOG (wdata->dialog) &&
+#ifdef USE_GTK2_18
+				gtk_widget_get_visible (wdata->dialog))
+#else
+				GTK_WIDGET_VISIBLE (wdata->dialog))
+#endif
 			{
 				wdata->bflags &= ~E2_BARTASK_PAUSEREQ;
 				wdata->loop = e2_main_loop_new (FALSE);
@@ -370,7 +373,10 @@ _e2p_cpbar_exec (VPATH *slocal, VPATH *dlocal, E2_FileTaskMode flags,
 					kill (pid, SIGSTOP);
 					wdata->bflags |= E2_BARTASK_PAUSED;
 					e2_filelist_enable_refresh ();
+					pthread_cleanup_push ((gpointer)gdk_threads_leave, NULL);
+					gdk_threads_enter ();
 					e2_main_loop_run (wdata->loop);
+					pthread_cleanup_pop (1);
 					kill (pid, SIGCONT);	//don't get to here if stopped or aborted
 				}
 			}
@@ -380,7 +386,12 @@ _e2p_cpbar_exec (VPATH *slocal, VPATH *dlocal, E2_FileTaskMode flags,
 
 		//show the full-time score ASAP
 		if (results[0]	//copy succeeded
-			&& GTK_IS_WIDGET (wdata->dialog) && GTK_WIDGET_VISIBLE (wdata->dialog))
+			&& GTK_IS_WIDGET (wdata->dialog) &&
+#ifdef USE_GTK2_18
+			gtk_widget_get_visible (wdata->dialog))
+#else
+			GTK_WIDGET_VISIBLE (wdata->dialog))
+#endif
 		{
 			progress = pdata.totalsize + bdata->totalsize;
 			g_snprintf (progresstext, sizeof (progresstext), progress_format,
@@ -495,7 +506,7 @@ static gboolean
 _e2p_cpbarQ (E2_ActionTaskData *qed)
 {
 	//handle case of specific data instead of selection??
-	if (g_str_equal (qed->currdir, qed->othrdir))
+	if (!strcmp (qed->currdir, qed->othrdir))
 	{
 		//display some message ??
 		return FALSE;
@@ -531,12 +542,15 @@ _e2p_cpbarQ (E2_ActionTaskData *qed)
 	windowdata.loop = NULL;	//no pause yet
 	windowdata.dialog = NULL;	//setup for abort
 	pthread_cleanup_push ((gpointer)_e2p_cpbar_abort_clean_dialog, &windowdata.dialog);
+	gdk_threads_enter ();
 	windowdata.dialog = e2_dialog_create (NULL, NULL, _("copying"),
-		_e2p_cpbar_response_cb, &windowdata);
+		(ResponseFunc)_e2p_cpbar_response_cb, &windowdata);
 	e2_dialog_setup (windowdata.dialog, app.main_window);
+	gdk_threads_leave ();
 
+#ifndef USE_GTK3_0
 	gtk_dialog_set_has_separator (GTK_DIALOG (windowdata.dialog), FALSE);
-
+#endif
 	dialog_vbox =
 #ifdef USE_GTK2_14
 		gtk_dialog_get_content_area (GTK_DIALOG (windowdata.dialog));
@@ -550,7 +564,6 @@ _e2p_cpbarQ (E2_ActionTaskData *qed)
 
 	windowdata.progbar = gtk_progress_bar_new ();
 	gtk_box_pack_start (GTK_BOX (dialog_vbox), windowdata.progbar, TRUE, TRUE, E2_PADDING_LARGE);
-	gtk_widget_show (windowdata.progbar);
 
 	//buttons
 	E2_Button local_btn = { _("_Pause"), GTK_STOCK_MEDIA_PAUSE,
@@ -570,6 +583,8 @@ _e2p_cpbarQ (E2_ActionTaskData *qed)
 	local_btn = E2_BUTTON_CANCEL;
 	local_btn.showflags |= E2_BTN_DEFAULT;
 	windowdata.stop_btn = e2_dialog_add_defined_button (windowdata.dialog, &local_btn);
+	//show everything but the dialog itself
+	gtk_widget_show_all (dialog_vbox);
 
 	e2_dialog_set_negative_response (windowdata.dialog, E2_BUTTON_CANCEL.response);
 
@@ -581,7 +596,7 @@ _e2p_cpbarQ (E2_ActionTaskData *qed)
 	{
 		g_string_printf (src, "%s%s", qed->currdir, (*iterator)->filename);  //separator comes with dir
 #ifdef E2_VFS
-		sdata.localpath = src->str;
+		sdata.path = src->str;
 #endif
 		//if (!
 			e2_fs_tw (
@@ -619,20 +634,22 @@ _e2p_cpbarQ (E2_ActionTaskData *qed)
 
 		DialogButtons result;
 #ifdef E2_VFS
-		sdata.localpath = src->str;
-		ddata.localpath = dest->str;
+		sdata.path = src->str;
+		ddata.path = dest->str;
 		if (check && e2_fs_access2 (&ddata E2_ERR_NONE()) == 0)
 #else
 		if (check && e2_fs_access2 (dest->str E2_ERR_NONE()) == 0)
 #endif
 		{
-			e2_filelist_enable_refresh ();  //allow updates while we wait
-			gdk_threads_enter ();
 			*qed->status = E2_TASK_PAUSED;
-			result = e2_dialog_ow_check (src->str, dest->str, extras);
+			result = e2_dialog_ow_check (
+#ifdef E2_VFS
+				&sdata, &ddata,
+#else
+				src->str, dest->str,
+#endif
+				extras);
 			*qed->status = E2_TASK_RUNNING;
-			gdk_threads_leave ();
-			e2_filelist_disable_refresh ();
 			switch (result)
 			{
 				case YES_TO_ALL:
@@ -706,11 +723,11 @@ init_plugin (Plugin *p)
 	p->icon = "plugin_copy"E2ICONTB;  //use icon file pathname if appropriate
 
 	//child UI data
-	gchar *sig1 = "0-"ANAME;
+	gchar *sig1 = "0-"ANAME VERSION;
 	gchar *label1 = (gchar *)p->menu_name;
 //	gchar *icon1 = "";	//no icon "plugin_"ANAME E2ICONTB;  //use icon file pathname if appropriate
 	gchar *tip1 = _("Copy selected item(s), with displayed progress details");
-	gchar *sig2 = "1-"ANAME;
+	gchar *sig2 = "1-"ANAME VERSION;
 	gchar *label2 = _("Copy with _times");
 //	gchar *icon2 = "";
 	gchar *tip2 = _("Copy selected item(s), with preserved time-properties and displayed progress details");
@@ -721,7 +738,6 @@ init_plugin (Plugin *p)
 		Plugin *pc = e2_plugins_create_child (p);
 		if (pc != NULL)
 		{
-			retval = TRUE;
 			//for reconciling with config treestore data, signatures must reflect
 			//0-based index of each child's order in the list of children
 			pc->signature = sig1; //begin with "n-", n=0,1, ...
@@ -731,10 +747,13 @@ init_plugin (Plugin *p)
 //			pc->description = _("Copy selected item(s), with displayed progress details");
 			pc->description = tip1;	//or whatever
 //			pc->icon = "plugin_copy"E2ICONTB;  //use icon file pathname if appropriate
-			//no need to free this
-			gchar *action_name = g_strconcat (_A(6),".",aname,NULL);
-			pc->action = e2_plugins_action_register
-				(action_name, E2_ACTION_TYPE_ITEM, _e2p_cpbar, NULL, FALSE, 0, NULL);
+			//don't free name string here
+			E2_Action plugact =
+			{g_strconcat (_A(6),".",aname,NULL),_e2p_cpbar,FALSE,E2_ACTION_TYPE_ITEM,0,NULL,NULL};
+			pc->action = e2_plugins_action_register (&plugact);
+			retval = (pc->action != NULL);
+			if (!retval)
+				g_free (plugact.name);
 		}
 		else
 			retval = FALSE;
@@ -751,11 +770,11 @@ init_plugin (Plugin *p)
 			pc->menu_name = label2;
 			pc->description = tip2;
 //			pc->icon = "plugin_copy"E2ICONTB;  //use icon file pathname if appropriate
-			//no need to free this
-			gchar *action_name = g_strconcat (_A(6),".",aname2,NULL);
-			pc->action = e2_plugins_action_register
-				(action_name, E2_ACTION_TYPE_ITEM, _e2p_cpbar,
-					GINT_TO_POINTER (E2_FTM_SAMETIME), FALSE, 0, NULL);
+			//don't free name string here
+			E2_Action plugact =
+			{g_strconcat (_A(6),".",aname2,NULL),_e2p_cpbar,FALSE,E2_ACTION_TYPE_ITEM,0,GINT_TO_POINTER (E2_FTM_SAMETIME), NULL};
+			pc->action = e2_plugins_action_register (&plugact);
+			retval = (pc->action != NULL);
 		}
 		else
 			retval = FALSE;
