@@ -1,6 +1,6 @@
-/* $Id: e2p_rename.c 1917 2009-09-27 09:30:57Z tpgww $
+/* $Id: e2p_rename.c 2469 2012-03-16 00:19:24Z tpgww $
 
-Copyright (C) 2005-2009 tooar <tooar@emelfm2.net>
+Copyright (C) 2005-2012 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -14,8 +14,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 /**
@@ -1271,7 +1270,7 @@ static void _e2p_ren_toggle_cb (GtkToggleButton *button, gpointer flagnum)
 {
 	E2_RenDialogRuntime *rt = g_object_get_data (G_OBJECT (button), "e2-runtime");
 	//if this if this is during setup, before a widget is created ...
-#ifdef USE_GTK2_99
+#ifdef USE_GTK2_20
 	if (!gtk_widget_get_mapped (rt->dialog))
 #else
 	if (!GTK_WIDGET_MAPPED (rt->dialog))
@@ -1623,9 +1622,10 @@ static gboolean _e2p_renameQ (E2_ActionTaskData *qed)
 	//don't need to be considered active while simply showing the dialog
 	*qed->status = E2_TASK_PAUSED;
 	//create dialog
+	gdk_threads_enter ();
 	rt.dialog = e2_dialog_create (NULL, NULL, _("rename items"),
 		(ResponseFunc)_e2p_ren_response_cb, &rt);
-
+	gdk_threads_leave ();
 	//populate it with widgets
 	GtkWidget *dialog_vbox, *hbox;
 	dialog_vbox =
@@ -1676,9 +1676,13 @@ static gboolean _e2p_renameQ (E2_ActionTaskData *qed)
 	gtk_widget_set_sensitive (rt.chooser_button, state);
 #endif
 */
+	//cuz E2_COMBOBOX_MENU_STYLE flag is not set, on gtk2, downstream calls
+	//gtk_widget_set_name() which MAY? need BGL closed
+	gdk_threads_enter ();
 	rt.directory = e2_combobox_add (dialog_vbox, FALSE, E2_PADDING_XSMALL,
 		(ActivateFunc)_e2p_ren_activation_cb, &rt, &dir_history,
 		E2_COMBOBOX_HAS_ENTRY | E2_COMBOBOX_FOCUS_ON_CHANGE);
+	gdk_threads_leave ();
 //#ifndef E2_RENCHOOSER
 	gboolean state = _e2p_ren_get_flag (SEARCH_THIS_P);
 //#endif
@@ -1722,9 +1726,11 @@ static gboolean _e2p_renameQ (E2_ActionTaskData *qed)
 	GtkSizeGroup *same = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 	gtk_size_group_add_widget (same, wid);
 
+	gdk_threads_enter ();
 	rt.pattern = e2_combobox_add (hbox, TRUE, E2_PADDING_XSMALL,
 		(ActivateFunc)_e2p_ren_activation_cb, &rt, &pattern_history,
 		E2_COMBOBOX_HAS_ENTRY | E2_COMBOBOX_FOCUS_ON_CHANGE);
+	gdk_threads_leave ();
 	gtk_entry_set_text (
 #ifdef USE_GTK2_14
 		GTK_ENTRY (gtk_bin_get_child (GTK_BIN (rt.pattern))),
