@@ -1,6 +1,6 @@
-/* $Id: e2p_times.c 1118 2008-08-21 08:15:50Z tpgww $
+/* $Id: e2p_times.c 2455 2012-03-09 11:55:10Z tpgww $
 
-Copyright (C) 2006-2008 tooar <tooar@gmx.net>
+Copyright (C) 2006-2012 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -14,8 +14,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 /**
@@ -193,10 +192,10 @@ static E2_TwResult _e2_task_twcb_touch (VPATH *localpath,
 				dirfix = member->data;
 				if (dirfix != NULL)
 				{
-					if (g_str_equal (dirfix->path, localpath))
+					if (!strcmp (dirfix->path, localpath))
 					{
 #ifdef E2_VFS
-						ddata.localpath = dirfix->path;
+						ddata.path = dirfix->path;
 						if (!_e2pt_touch1 (&ddata, statptr, user_data))
 #else
 						if (!_e2pt_touch1 (dirfix->path, statptr, user_data))
@@ -342,7 +341,7 @@ static gboolean _e2p_touch (VPATH *localpath, E2_TouchData *data, gboolean recur
 				{
 					E2P_DirEnt *dirfix = member->data;
 #ifdef E2_VFS
-					ddata.localpath = dirfix->path;
+					ddata.path = dirfix->path;
 					if (e2_fs_lstat (&ddata, &statbuf E2_ERR_NONE())  //no link pass-thru
 						|| !_e2pt_touch1 (&ddata, &statbuf, data))
 #else
@@ -404,8 +403,20 @@ static gboolean _e2p_times_parse_time (GtkWidget *current, GtkWidget *date_combo
 	struct tm tm;
 	//clear result structure
 	memset (&tm, '\0', sizeof (tm));
-	const gchar *date = gtk_entry_get_text (GTK_ENTRY (GTK_BIN (date_combo)->child));
-	const gchar *time = gtk_entry_get_text (GTK_ENTRY (GTK_BIN (time_combo)->child));
+	const gchar *date = gtk_entry_get_text (
+#ifdef USE_GTK2_14
+		GTK_ENTRY (gtk_bin_get_child (GTK_BIN (date_combo)))
+#else
+		GTK_ENTRY (GTK_BIN (date_combo)->child)
+#endif
+	);
+	const gchar *time = gtk_entry_get_text (
+#ifdef USE_GTK2_14
+		GTK_ENTRY (gtk_bin_get_child (GTK_BIN (time_combo)))
+#else
+		GTK_ENTRY (GTK_BIN (time_combo)->child)
+#endif
+	);
 	const gchar *currdate = gtk_label_get_text (GTK_LABEL (current));
 	const gchar *currtime = strchr (currdate, ' ') + 1;	//ascii scanning for space char
 	const gchar *cp;
@@ -465,7 +476,13 @@ static void _e2p_times_apply_cb (GtkWidget *widget, E2_TimesDlgRuntime *rt)
 //		GtkWidget *entry;
 		const gchar *entrytext;
 		//store any valid new dates/times
-		if (GTK_TOGGLE_BUTTON (rt->set_mtime_button)->active)
+		if (
+#ifdef USE_GTK2_14
+			gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (rt->set_mtime_button))
+#else
+			GTK_TOGGLE_BUTTON (rt->set_mtime_button)->active
+#endif
+		)
 		{
 			success = _e2p_times_parse_time (rt->mcurrent, rt->mdate_combo,
 				rt->mtime_combo, &rt->data->mtime);
@@ -473,19 +490,41 @@ static void _e2p_times_apply_cb (GtkWidget *widget, E2_TimesDlgRuntime *rt)
 			{
 /*as dialog is destroyed each iteration, no point in deferring backup until later
 				//update combo history
-				entry = GTK_BIN (rt->mdate_combo)->child;
+				entry =
+#ifdef USE_GTK2_14
+				gtk_bin_get_child (GTK_BIN (rt->mdate_combo));
+#else
+				GTK_BIN (rt->mdate_combo)->child;
+#endif
 				entrytext = gtk_entry_get_text (GTK_ENTRY (entry));
 				if (*entrytext != '\0')
 					e2_combobox_activated_cb (entry, NULL);
-				entry = GTK_BIN (rt->mtime_combo)->child;
+				entry =
+#ifdef USE_GTK2_14
+				gtk_bin_get_child (GTK_BIN (rt->mtime_combo));
+#else
+				GTK_BIN (rt->mtime_combo)->child;
+#endif
 				entrytext = gtk_entry_get_text (GTK_ENTRY (entry));
 				if (*entrytext != '\0')
 					e2_combobox_activated_cb (entry, NULL);
 */
-				entrytext = gtk_entry_get_text (GTK_ENTRY (GTK_BIN (rt->mdate_combo)->child));
+				entrytext = gtk_entry_get_text (
+#ifdef USE_GTK2_14
+					GTK_ENTRY (gtk_bin_get_child (GTK_BIN (rt->mdate_combo)))
+#else
+					GTK_ENTRY (GTK_BIN (rt->mdate_combo)->child)
+#endif
+				);
 				if (*entrytext != '\0')
 					e2_list_update_history (entrytext, &mdate_history, NULL, 0, FALSE);
-				entrytext = gtk_entry_get_text (GTK_ENTRY (GTK_BIN (rt->mtime_combo)->child));
+				entrytext = gtk_entry_get_text (
+#ifdef USE_GTK2_14
+					GTK_ENTRY (gtk_bin_get_child (GTK_BIN (rt->mtime_combo)))
+#else
+					GTK_ENTRY (GTK_BIN (rt->mtime_combo)->child)
+#endif
+				);
 				if (*entrytext != '\0')
 					e2_list_update_history (entrytext, &mtime_history, NULL, 0, FALSE);
 			}
@@ -495,26 +534,54 @@ static void _e2p_times_apply_cb (GtkWidget *widget, E2_TimesDlgRuntime *rt)
 		else
 			rt->data->mtime = (time_t) -1;
 
-		if (success && GTK_TOGGLE_BUTTON (rt->set_atime_button)->active)
+		if (success &&
+#ifdef USE_GTK2_14
+			gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (rt->set_atime_button))
+#else
+			GTK_TOGGLE_BUTTON (rt->set_atime_button)->active
+#endif
+		)
 		{
 			success = _e2p_times_parse_time (rt->acurrent, rt->adate_combo,
 				rt->atime_combo, &rt->data->atime);
 			if (success)
 			{
 /*				//update combo history
-				entry = GTK_BIN (rt->adate_combo)->child;
+				entry =
+#ifdef USE_GTK2_14
+					gtk_bin_get_child (GTK_BIN (rt->adate_combo));
+#else
+					GTK_BIN (rt->adate_combo)->child;
+#endif
 				entrytext = gtk_entry_get_text (GTK_ENTRY (entry));
 				if (*entrytext != '\0')
 					e2_combobox_activated_cb (entry, NULL);
-				entry = GTK_BIN (rt->atime_combo)->child;
+				entry =
+#ifdef USE_GTK2_14
+					gtk_bin_get_child (GTK_BIN (rt->atime_combo));
+#else
+					GTK_BIN (rt->atime_combo)->child;
+#endif
 				entrytext = gtk_entry_get_text (GTK_ENTRY (entry));
 				if (*entrytext != '\0')
 					e2_combobox_activated_cb (entry, NULL);
 */
-				entrytext = gtk_entry_get_text (GTK_ENTRY (GTK_BIN (rt->adate_combo)->child));
+				entrytext = gtk_entry_get_text (
+#ifdef USE_GTK2_14
+					GTK_ENTRY (gtk_bin_get_child (GTK_BIN (rt->adate_combo)))
+#else
+					GTK_ENTRY (GTK_BIN (rt->adate_combo)->child)
+#endif
+				);
 				if (*entrytext != '\0')
 					e2_list_update_history (entrytext, &adate_history, NULL, 0, FALSE);
-				entrytext = gtk_entry_get_text (GTK_ENTRY (GTK_BIN (rt->atime_combo)->child));
+				entrytext = gtk_entry_get_text (
+#ifdef USE_GTK2_14
+					GTK_ENTRY (gtk_bin_get_child (GTK_BIN (rt->atime_combo)))
+#else
+					GTK_ENTRY (GTK_BIN (rt->atime_combo)->child)
+#endif
+				);
 				if (*entrytext != '\0')
 					e2_list_update_history (entrytext, &atime_history, NULL, 0, FALSE);
 			}
@@ -524,7 +591,13 @@ static void _e2p_times_apply_cb (GtkWidget *widget, E2_TimesDlgRuntime *rt)
 		else
 			rt->data->atime = (time_t) -1;
 
-		if (success && GTK_TOGGLE_BUTTON (rt->set_ctime_button)->active)
+		if (success &&
+#ifdef USE_GTK2_14
+			gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (rt->set_ctime_button))
+#else
+			GTK_TOGGLE_BUTTON (rt->set_ctime_button)->active
+#endif
+		)
 		{
 			success = _e2p_times_parse_time (rt->ccurrent, rt->cdate_combo,
 				rt->ctime_combo, &rt->data->ctime);
@@ -547,19 +620,41 @@ static void _e2p_times_apply_cb (GtkWidget *widget, E2_TimesDlgRuntime *rt)
 			if (success)
 			{
 /*				//update combo history
-				entry = GTK_BIN (rt->cdate_combo)->child;
+				entry =
+#ifdef USE_GTK2_14
+				gtk_bin_get_child (GTK_BIN (rt->cdate_combo));
+#else
+				GTK_BIN (rt->cdate_combo)->child;
+#endif
 				entrytext = gtk_entry_get_text (GTK_ENTRY (entry));
 				if (*entrytext != '\0')
 					e2_combobox_activated_cb (entry, NULL);
-				entry = GTK_BIN (rt->ctime_combo)->child;
+				entry =
+#ifdef USE_GTK2_14
+				gtk_bin_get_child (GTK_BIN (rt->ctime_combo));
+#else
+				GTK_BIN (rt->ctime_combo)->child;
+#endif
 				entrytext = gtk_entry_get_text (GTK_ENTRY (entry));
 				if (*entrytext != '\0')
 					e2_combobox_activated_cb (entry, NULL);
 */
-				entrytext = gtk_entry_get_text (GTK_ENTRY (GTK_BIN (rt->cdate_combo)->child));
+				entrytext = gtk_entry_get_text (
+#ifdef USE_GTK2_14
+					GTK_ENTRY (gtk_bin_get_child (GTK_BIN (rt->cdate_combo)))
+#else
+					GTK_ENTRY (GTK_BIN (rt->cdate_combo)->child)
+#endif
+				);
 				if (*entrytext != '\0')
 					e2_list_update_history (entrytext, &cdate_history, NULL, 0, FALSE);
-				entrytext = gtk_entry_get_text (GTK_ENTRY (GTK_BIN (rt->ctime_combo)->child));
+				entrytext = gtk_entry_get_text (
+#ifdef USE_GTK2_14
+					GTK_ENTRY (gtk_bin_get_child (GTK_BIN (rt->ctime_combo)))
+#else
+					GTK_ENTRY (GTK_BIN (rt->ctime_combo)->child)
+#endif
+				);
 				if (*entrytext != '\0')
 					e2_list_update_history (entrytext, &ctime_history, NULL, 0, FALSE);
 			}
@@ -577,7 +672,12 @@ static void _e2p_times_apply_cb (GtkWidget *widget, E2_TimesDlgRuntime *rt)
 			{
 				rt->real_choice = OK; //the entered choice stands confirmed
 				if (rt->recurse_button != NULL)
-					*rt->recurse = GTK_TOGGLE_BUTTON (rt->recurse_button)->active;
+					*rt->recurse =
+#ifdef USE_GTK2_14
+						gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (rt->recurse_button));
+#else
+						GTK_TOGGLE_BUTTON (rt->recurse_button)->active;
+#endif
 			}
 //			return;
 		}
@@ -598,19 +698,26 @@ static void _e2p_times_apply_cb (GtkWidget *widget, E2_TimesDlgRuntime *rt)
 */
 static GtkWidget *_e2p_times_create_combo (GList *history)
 {
-	GtkWidget *combo = e2_combobox_get (NULL, NULL, &history,
+	//func MAY need BGL closed on gtk2
+	GtkWidget *combo = e2_combobox_get ((ActivateFunc)NULL, NULL, &history,
 		E2_COMBOBOX_HAS_ENTRY | E2_COMBOBOX_NO_AUTO_HISTORY |
 		E2_COMBOBOX_FOCUS_ON_CHANGE | E2_COMBOBOX_CYCLE_HISTORY);
 	//show the last-used value, if any
 	if (e2_combobox_has_history (GTK_COMBO_BOX (combo)))
 		e2_combobox_set_active (combo, 0);
-//	gtk_entry_set_activates_default (GTK_ENTRY (GTK_BIN (combo)->child), TRUE);
+//	gtk_entry_set_activates_default (GTK_ENTRY
+//#ifdef USE_GTK2_14
+//	(gtk_bin_get_child (GTK_BIN (combo))),
+//#else
+//	(GTK_BIN (combo)->child),
+//#endif
+//	TRUE);
 //	gtk_widget_set_size_request (combo,  40, -1);
 	return combo;
 }
 /**
 @brief create and run time-change dialog
-
+Expects BGL open/off.
 @param local_dir path of dir containing item to process, localised string with trailer
 @param spacedata pointer to space data for active pane
 @param info pointer to selection data struct for the item being processed
@@ -667,7 +774,9 @@ static DialogButtons _e2p_times_dialog_run (gchar *local_dir,
 	rt.data = data;
 	rt.recurse = recurse_ret;
 
+	gdk_threads_enter ();
 	times_dialog = e2_dialog_create (NULL, NULL, _("times"), DUMMY_RESPONSE_CB, NULL);
+	gdk_threads_leave ();
 	dialog_vbox =
 #ifdef USE_GTK2_14
 		gtk_dialog_get_content_area (GTK_DIALOG (times_dialog));
@@ -682,13 +791,11 @@ static DialogButtons _e2p_times_dialog_run (gchar *local_dir,
 	gchar *utf = F_DISPLAYNAME_FROM_LOCALE (info->filename);
 	gchar *label_text = g_strdup_printf ("%s: <b>%s</b>", labeltype, utf);
 	e2_widget_add_mid_label (hbox, label_text, 0, TRUE, E2_PADDING); //L, R padding
-	gtk_widget_show (hbox);
-	F_FREE (utf);
+	F_FREE (utf, info->filename);
 	g_free (label_text);
 
 	sub_vbox = gtk_vbox_new (FALSE, E2_PADDING);
 	gtk_box_pack_start (GTK_BOX (dialog_vbox), sub_vbox, TRUE, TRUE, E2_PADDING_LARGE);
-	gtk_widget_show (sub_vbox);
 
 	gint rows = (thisis_dir) ? 5 : 4;
 	table = e2_widget_add_table (sub_vbox, rows, 5, FALSE, TRUE, E2_PADDING);  //4 rows, 5 cols, !homogen, expand
@@ -757,7 +864,7 @@ static DialogButtons _e2p_times_dialog_run (gchar *local_dir,
 		gtk_widget_set_sensitive (rt.set_ctime_button, FALSE);
 		gtk_widget_set_sensitive (rt.recurse_button, FALSE);
 
-		e2_ownership_dialog_warn (dialog_vbox);
+		e2_dialog_setup_auth (dialog_vbox);
 	}
 	else if (getuid () != 0)
 	{
@@ -766,11 +873,13 @@ static DialogButtons _e2p_times_dialog_run (gchar *local_dir,
 		gtk_widget_set_sensitive (rt.cdate_combo, FALSE);
 		gtk_widget_set_sensitive (rt.set_ctime_button, FALSE);
 	}
+
+	//block until user responds
+	E2_DialogFlags flags = E2_DIALOG_BLOCKED;
+
 	// add buttons in the order that they will appear
 	GtkWidget *btn;
 	E2_Button no_btn;
-	//block until user responds
-	E2_DialogFlags flags = E2_DIALOG_BLOCKED;
 	if (multi)
 	{
 		e2_dialog_set_negative_response (times_dialog, E2_RESPONSE_NOTOALL);
@@ -794,12 +903,12 @@ static DialogButtons _e2p_times_dialog_run (gchar *local_dir,
 	if (!rt.permission)
 		gtk_widget_set_sensitive (btn, FALSE);
 
-	e2_dialog_setup (times_dialog, app.main_window);
-
 	gdk_threads_enter ();
+
+	e2_dialog_setup (times_dialog, app.main_window);
 	do
 	{
-		choice = e2_dialog_run (times_dialog, app.main_window, flags);
+		choice = e2_dialog_run (times_dialog, NULL, flags);
 	} while (rt.real_choice == NO); //after date/time parse error, keep trying
 /*as dialog is destroyed each iteration, no gain in deferring this from the
   response callback
@@ -868,7 +977,7 @@ static gboolean _e2p_task_timesQ (E2_ActionTaskData *qed)
 		//".." entries filtered when names compiled
 		g_string_printf (path, "%s%s", curr_local, (*iterator)->filename); //separator comes with dir
 #ifdef E2_VFS
-		sdata.localpath = path->str;
+		sdata.path = path->str;
 #endif
 		if (all)
 		{
@@ -882,7 +991,7 @@ static gboolean _e2p_task_timesQ (E2_ActionTaskData *qed)
 			if (!permission)
 			{
 #ifdef E2_VFS
-				sdata.localpath = (*iterator)->filename;
+				sdata.path = (*iterator)->filename;
 #endif
 				e2_fs_error_simple (
 					_("You do not have authority to change time(s) for %s"),
@@ -947,7 +1056,7 @@ static gboolean _e2p_task_timesQ (E2_ActionTaskData *qed)
 #  ifdef E2_VFS
 			if (_e2p_touch (&sdata, &data, recurse))
 			{
-				sdata.localpath = curr_local;
+				sdata.path = curr_local;
 				//make the file-list refresher notice successful change
 				e2_fs_utime (&sdata, NULL E2_ERR_NONE());
 			}
@@ -968,7 +1077,6 @@ static gboolean _e2p_task_timesQ (E2_ActionTaskData *qed)
 			break;
 	}
 	g_string_free (path, TRUE);
-	F_FREE (curr_local);
 
 	//FIXME E2_INCLIST - just do other view if it's same
 	e2_window_clear_status_message ();
@@ -1000,11 +1108,13 @@ gboolean init_plugin (Plugin *p)
 
 	if (p->action == NULL)
 	{
-		//no need to free this
-		gchar *action_name = g_strconcat (_A(6),".",aname,NULL);
-		p->action = e2_plugins_action_register (action_name, E2_ACTION_TYPE_ITEM,
-			_e2p_task_times, NULL, FALSE, 0, NULL);
-		return TRUE;
+		//don't free name string here
+		E2_Action plugact =
+		{g_strconcat (_A(6),".",aname,NULL),_e2p_task_times,FALSE,E2_ACTION_TYPE_ITEM,0,NULL,NULL};
+		p->action = e2_plugins_action_register (&plugact);
+		if G_LIKELY((p->action != NULL))
+			return TRUE;
+		g_free (plugact.name);
 	}
 	return FALSE;
 }
