@@ -1,6 +1,6 @@
-/* $Id: e2p_unpack.c 1917 2009-09-27 09:30:57Z tpgww $
+/* $Id: e2p_unpack.c 2309 2011-01-17 00:12:22Z tpgww $
 
-Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
+Copyright (C) 2003-2011 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -14,8 +14,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 /**
@@ -30,7 +29,7 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #include "e2_task.h"
 #include "e2_filelist.h"
 //same enum as in pack plugin, though it need not be so
-enum { TAR_GZ, TAR_BZ2, TAR_LZMA, TAR, /*DEB, RPM, */ZIP, Z7Z, RAR, ARJ, ZOO, MAXTYPES };
+enum { TAR_GZ, TAR_BZ2, TAR_LZMA, TAR_XZ, TAR, /*DEB, RPM, */ZIP, Z7Z, RAR, ARJ, ZOO, MAXTYPES };
 
 typedef struct _E2P_Unpackdata
 {
@@ -95,6 +94,7 @@ static gint _e2p_unpack_match_type (VPATH *localpath)
 			"x-lzma-compressed-tar",
 			//this may be version-specific i.e. NOT for LZMA_Alone tool in LZMA SDK <= v.4.43
 			"x-lzma",
+			"x-xz"
 			"x-tar",
 //			"x-deb",
 //			"x-rpm",
@@ -110,6 +110,7 @@ static gint _e2p_unpack_match_type (VPATH *localpath)
 			TAR_BZ2,
 			TAR_LZMA,	//2 alternates for lzma
 			TAR_LZMA,
+			TAR_XZ,
 			TAR,
 	//		DEB,
 	//		RPM,
@@ -149,6 +150,7 @@ static gint _e2p_unpack_match_type (VPATH *localpath)
 			".tar.gz", ".tgz", //compress(1) does ".taz", ".tar.z",
 			".tar.bz2", ".tbz2",
 			".tar.lzma", ".tlz",
+			".tar.xz",
 			".tar",
 	//		".deb",
 	//		".rpm".
@@ -164,6 +166,7 @@ static gint _e2p_unpack_match_type (VPATH *localpath)
 			TAR_GZ, TAR_GZ,
 			TAR_BZ2, TAR_BZ2,
 			TAR_LZMA, TAR_LZMA,
+			TAR_XZ,
 			TAR,
 	//		DEB,
 	//		RPM,
@@ -215,8 +218,8 @@ static gboolean _e2p_unpack_delete_dir (E2P_Unpackdata *data)
 {
 	LISTS_LOCK
 	//conservative approach - no deletion while any chance of the temp-dir being used
-	if (curr_view->listcontrols.cd_working ||
-		curr_view->listcontrols.refresh_working || (
+	if (g_atomic_int_get (&curr_view->listcontrols.cd_working) ||
+		g_atomic_int_get (&curr_view->listcontrols.refresh_working) || (
 #ifdef E2_VFS
 		curr_view->spacedata == NULL &&	//local temp dirs
 #endif
@@ -227,8 +230,8 @@ static gboolean _e2p_unpack_delete_dir (E2P_Unpackdata *data)
 		data->pack_id = g_timeout_add (500, (GSourceFunc)_e2p_unpack_pause, data);
 		return FALSE;
 	}
-	if (other_view->listcontrols.cd_working ||
-		other_view->listcontrols.refresh_working || (
+	if (g_atomic_int_get (&other_view->listcontrols.cd_working) ||
+		g_atomic_int_get (&other_view->listcontrols.refresh_working) || (
 #ifdef E2_VFS
 		other_view->spacedata == NULL &&	//local temp dirs
 #endif
@@ -331,6 +334,7 @@ static void _e2p_unpack_repack (E2P_Unpackdata *data, gpointer from)
 		">tar cf - . | gzip - > %s",
 		">tar cf - . | bzip2 - > %s",	//default block size -9 is not significantly better, maybe use -3 instead ?
 		">tar cf - . | lzma - > %s",	//default compresssion -7, maybe use -2 instead ?
+		">tar cf - . | xz - > %s",
 		"tar cf %s .",
 //		deb 'pack' command ?
 //		rpm 'pack' command does not exist
@@ -374,6 +378,7 @@ install/uninstall scripts, the other the files to be installed.
 		case TAR_GZ:
 		case TAR_BZ2:
 		case TAR_LZMA:
+		case TAR_XZ:
 		case TAR:
 		case ZIP:
 		case Z7Z:
@@ -516,9 +521,11 @@ static gboolean _e2p_unpack_change_dir_hook (gchar *path, E2P_Unpackdata *data)
 	printd (DEBUG, "ready to cleanup unpack dir");
 
 	//ask user what to do with the unpacked items
+	gdk_threads_enter ();
 	GtkWidget *dialog = e2_dialog_create (GTK_STOCK_DIALOG_QUESTION,
 		_("What do you want to do with the unpacked item(s) ?"), NULL,
 		(ResponseFunc)_e2p_unpack_response_decode_cb, data);
+	gdk_threads_leave ();
 
 	e2_dialog_add_simple_button (dialog, GTK_STOCK_CLEAR,
 		_("Re_pack"), E2_RESPONSE_USER1);
@@ -560,6 +567,7 @@ static gboolean _e2p_unpack (gpointer from, E2_ActionRuntime *art)
 		">gzip -cd %s | tar -xpf -",
 		">bzip2 -cd %s | tar -xpf -",
 		">lzma -cd %s | tar -xpf -",
+		">xz -cd %s | tar -xpf -",
 		"tar -xpf %s",
 #else
 		//note: an --atime-preserve in these tar commands prevents the
@@ -568,6 +576,7 @@ static gboolean _e2p_unpack (gpointer from, E2_ActionRuntime *art)
 		"tar -xpzf %s",
 		"tar -xpjf %s",
 		"tar --lzma -xpf %s",
+		"tar --xz -xpf %s",
 		"tar -xpf %s",
 #endif
 		//.deb unpack command by Martin Zelaia (has GNU-specific tar options)
