@@ -1,6 +1,6 @@
-/* $Id: e2p_thumbs.c 1999 2009-11-05 23:20:07Z tpgww $
+/* $Id: e2p_thumbs.c 2404 2011-10-30 23:55:23Z tpgww $
 
-Copyright (C) 2007-2009 tooar <tooar@emelfm2.net>
+Copyright (C) 2007-2011 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -14,9 +14,9 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
+
 /**
 @file plugins/optional/e2p_thumbs.c
 @brief image-viewer plugin
@@ -37,15 +37,14 @@ DnD ?
 #include "emelfm2.h"
 #include <string.h>
 
-//static gboolean FIXME; //check that app.libgimp_connected is gone
-
-//setup relevant #defines in included gimp-library headers
-#define GIMP_THUMB_DISABLE_DEPRECATED
-
-#ifdef E2_THUMBLIBSTATIC //from Makefile via build.h
+#ifdef E2_THUMBLIB //from Makefile via build.h
+//work with shared lib
+# define THUMB_OBJECT
+//setup relevant #defines in included library headers
+# define GIMP_THUMB_DISABLE_DEPRECATED
+# include <libthumbs/gimpthumb.h>
+#else
 # include "gimpthumb.c"
-#else //work with shared lib
-# include <libgimpthumbsafe/gimpthumb.h>
 #endif
 #include "e2_context_menu.h"
 #include "e2_filelist.h"
@@ -63,7 +62,7 @@ DnD ?
  different or non-uses for many of them */
 enum
 {
-//	FILENAME = 0 ... FINFO are enumerated elswhere
+//	FILENAME = 0 ... FINFO are enumerated elswhere, THUMBNAIL mmaybe unused
 	PIXBUF = 1, THUMBNAIL, THUMBCOLCOUNT = FINFO+1
 };
 
@@ -79,7 +78,8 @@ typedef struct _E2_ThumbDialogRuntime
 	gint sort_type;			//to determine the correct sort func, corresponds to column in original store
 	gint filtercount;		//for matching whether parent filelist has been re-filtered
 	GtkSortType sort_order;
-	gboolean blocked;		//flag to prevent recursive refreshed
+	gint blocked;			//0 when no refresh in progress, prevents recursive refreshing
+	guint timer_id;			//id of refresh-deferral timer
 #ifdef E2_VFSTMP
 	//FIXME path when dir not mounted local
 #else
@@ -112,12 +112,9 @@ static void	_e2p_thumbs_mousegestures (E2_OptionSet *set);
 #endif
 static void _e2p_thumbs_selection_change_cb (GtkIconView *iconview,
 	E2_ThumbDialogRuntime *rt);
-static gboolean _e2p_thumbs_refresh_hook (ViewInfo *view,
-	E2_ThumbDialogRuntime *rt);
 static void _e2p_thumbs_response_cb (GtkDialog *dialog, gint response,
 	E2_ThumbDialogRuntime *rt);
 
-
   /*********************/
  /***** utilities *****/
 /*********************/
@@ -135,7 +132,15 @@ static GtkListStore *_e2p_thumbs_make_store (void)
 	GtkListStore *store = gtk_list_store_new (THUMBCOLCOUNT,
 		G_TYPE_STRING,  //FILENAME displayed
 		GDK_TYPE_PIXBUF, //PIXBUF displayed in place of SIZE
+#if 0 //thumbnail data is needed
+# ifdef THUMB_OBJECT
 		GIMP_TYPE_THUMBNAIL, //THUMBNAIL in place of PERM
+# else
+		G_TYPE_POINTER,  //THUMBNAIL in place of PERM
+# endif
+#else
+		G_TYPE_POINTER,  //PERM unused
+#endif
 		G_TYPE_POINTER,  //OWNER unused
 		G_TYPE_POINTER,  //GROUP unused
 		G_TYPE_POINTER,  //MODIFIED unused
@@ -164,16 +169,28 @@ static void _e2p_thumbs_clear_store (GtkListStore *store)
 		//CHECKME need to clear anything else?
 		FileInfo *info;
 		GdkPixbuf *pxb;
-		GimpThumbnail *thm;
+#if 0	//thumbnail data is needed
+		GimpThumbnail *thumbnail;
+#endif
 		do
 		{
-			gtk_tree_model_get (mdl, &iter, FINFO, &info, PIXBUF, &pxb, THUMBNAIL, &thm, -1);
+			gtk_tree_model_get (mdl, &iter, PIXBUF, &pxb,
+#if 0
+			THUMBNAIL, &thumbnail,
+#endif
+			FINFO, &info, -1);
 			DEALLOCATE (FileInfo, info);
 //			if (pxb != NULL)
 				g_object_unref (G_OBJECT (pxb));
-//			if (thm != NULL)
 			//CHECKME cache the pixbufs here instead of when filling the store
-				g_object_unref (G_OBJECT (thm));
+#if 0
+			if (thumbnail != NULL)
+# ifdef THUMB_OBJECT
+				g_object_unref (G_OBJECT (thumbnail));
+# else
+				gimp_thumbnail_destroy (thumbnail);
+# endif
+#endif
 		} while (gtk_tree_model_iter_next (mdl, &iter));
 		gtk_list_store_clear (store);	//NEEDED ??
 	}
@@ -205,29 +222,28 @@ static gboolean _e2p_thumbs_clear_old_stores (GSList *oldstores)
 Rows are added for each relevant non-dir item in the corresponding filelist's
 liststore.
 The existing liststore is replaced, and queued for cleanup.
-BGL is expected to be open
+Expects BGL open/off
 @param rt pointer to data struct for dialog
 
 @return pointer to the liststore, or NULL if there's a problem
 */
 static GtkListStore *_e2p_thumbs_replace_store (E2_ThumbDialogRuntime *rt)
 {
-	rt->blocked = TRUE;	//prevent re-entrance
+	if (!g_atomic_int_compare_and_exchange (&rt->blocked, 0, 1)) //prevent re-entrance
+		return NULL;
 //	printd (DEBUG, "start store fill");
 	GtkListStore *store = _e2p_thumbs_make_store ();
 	if (store == NULL)
 	{
-		rt->blocked = FALSE;
+		g_atomic_int_add (&rt->blocked, -1);
 		return NULL;
 	}
 
 	E2_ListChoice pane = (rt->view == &app.pane1.view) ? PANE1 : PANE2;
 	e2_filelist_disable_one_refresh (pane);
-//	gboolean ref = rt->view->refresh_requested;
 
 	//transfer data from filelist store to iconview store
 	GtkTreeIter itert;
-//	GtkTreeModel *mdlt = GTK_TREE_MODEL (rt->view->store);	//the child model (everything)
 	GtkTreeModel *mdlt = rt->view->model;	//the filter-model
 
 	if (gtk_tree_model_get_iter_first (mdlt, &itert))
@@ -240,7 +256,7 @@ static GtkListStore *_e2p_thumbs_replace_store (E2_ThumbDialogRuntime *rt)
 		gchar *filename, *namekey, *dlocal, *localpath;
 		GtkTreeIter iteri;
 		gboolean doscale = e2_option_bool_get_direct (rt->clampset);
-		gint threshold = (doscale) ? e2_option_int_get ("thumb-limit") : 0;
+		GimpThumbSize threshold = (doscale) ? e2_option_int_get ("thumb-limit") : 0;
 
 		GtkTreeSortable *sortablei = GTK_TREE_SORTABLE (store);
 
@@ -311,6 +327,9 @@ static GtkListStore *_e2p_thumbs_replace_store (E2_ThumbDialogRuntime *rt)
 
 			if (makepxb)
 			{
+				GimpThumbState status;
+				gint width, height;
+
 				thumbnail = gimp_thumbnail_new ();
 				error = NULL;
 				if (!gimp_thumbnail_set_filename (thumbnail, localpath, &error))
@@ -322,16 +341,16 @@ static GtkListStore *_e2p_thumbs_replace_store (E2_ThumbDialogRuntime *rt)
 					}
 					makepxb = FALSE;
 				}
+
 				if (makepxb)
 				{
-					gint width, height;
-					if (gdk_pixbuf_get_file_info (localpath, &width, &height) != NULL)
+					if (doscale)
 					{
-						gboolean scaled;
-						gint scaleto;
-recreate:
-						if (doscale)
+						if (gdk_pixbuf_get_file_info (localpath, &width, &height) != NULL)
 						{
+							gboolean scaled;
+							GimpThumbSize scaleto;
+recreate:
 /*							if (width < E2_SCALEUP_SIZE && height < E2_SCALEUP_SIZE)
 							{
 								scaled = TRUE;
@@ -349,38 +368,41 @@ recreate:
 								scaled = FALSE;
 								scaleto = MAX (width, height);
 							}
-						}
-						else
-						{
-							scaled = FALSE;
-							scaleto = MAX (width, height);
-						}
-//						if (gimp_thumbnail_peek_thumb (handle, thumbnail, scaleto) != GIMP_THUMB_STATE_EXISTS)
-						if (gimp_thumbnail_check_thumb (handle, thumbnail, scaleto) != GIMP_THUMB_STATE_OK)
-						{
-rescale:
-							if (scaled)
-								pxb = gdk_pixbuf_new_from_file_at_scale (localpath,
-									scaleto, scaleto, TRUE, &error);
-							else
-								pxb = gdk_pixbuf_new_from_file (localpath, &error);
 
-							if (pxb != NULL)
+							status = gimp_thumbnail_check_thumb (handle, thumbnail, scaleto); //ensure thumb is current
+							if (status != GIMP_THUMB_STATE_OK)
 							{
-#ifdef DEBUG_MESSAGES
 								if (scaled)
+									pxb = gdk_pixbuf_new_from_file_at_scale (localpath,
+										scaleto, scaleto, TRUE, &error);
+								else
+									pxb = gdk_pixbuf_new_from_file (localpath, &error);
+
+								if (pxb != NULL)
 								{
-									gint wide = gdk_pixbuf_get_width (pxb);
-									printd (DEBUG, "width of scaled pixbuf %s is %u", localpath, wide);
-									gint high = gdk_pixbuf_get_height (pxb);
-									printd (DEBUG, "and width of scaled pixbuf %s is %u", localpath, high);
-								}
+#ifdef DEBUG_MESSAGES
+									if (scaled)
+									{
+										gint wide = gdk_pixbuf_get_width (pxb);
+										printd (DEBUG, "width of scaled pixbuf %s is %u", localpath, wide);
+										gint high = gdk_pixbuf_get_height (pxb);
+										printd (DEBUG, "and width of scaled pixbuf %s is %u", localpath, high);
+									}
 #endif
-//								local save >> into CWD/.thumblocal/normal, maybe not creatable ?! BUG?
-//								if (!gimp_thumbnail_save_thumb_local (handle, thumbnail, pxb,
-								//CHECKME save pixbuf later, so we can startup more quickly
-								if (!gimp_thumbnail_save_thumb (handle, thumbnail, pxb,
-									E2_SOFTWARE_ID, &error))
+	//								local save >> into CWD/.thumblocal/normal, maybe not creatable ?! BUG?
+	//								if (!gimp_thumbnail_save_thumb_local (handle, thumbnail, pxb,
+									//CHECKME save pixbuf later, so we can startup more quickly
+									if (!gimp_thumbnail_save_thumb (handle, thumbnail, pxb,
+										E2_SOFTWARE_ID, &error))
+									{
+										//warn the user
+										if (error != NULL)
+											g_error_free (error);
+										g_object_unref (G_OBJECT (pxb));
+										makepxb = FALSE;
+									}
+								}
+								else	//pixbuf creation failed
 								{
 									//warn the user
 									if (error != NULL)
@@ -388,50 +410,101 @@ rescale:
 									makepxb = FALSE;
 								}
 							}
-							else	//pixbuf creation failed
+							else	//a probably-valid cached image found
 							{
-								//warn the user
-								if (error != NULL)
-									g_error_free (error);
-								makepxb = FALSE;
-							}
-						}
-						else	//a maybe-valid cached image found
-						{
-							//some hacky stuff to work around the lib's approach to reporting
-							//non-standard-size thumbnails ??
-							if (gimp_thumbnail_peek_image (thumbnail) == GIMP_THUMB_STATE_EXISTS)
-							{
-								width = MAX (thumbnail->image_width, thumbnail->image_height);
-								if (width == 0)	//0 is reported for non-standard-size ?
+								pxb = gimp_thumbnail_load_thumb (handle, thumbnail, scaleto, &error);
+								if (pxb != NULL)
 								{
-									height = width = scaleto;
-									goto rescale;
-								}
-								else if (//width < E2_SCALEUP_SIZE ||
+									width = gdk_pixbuf_get_width (pxb);
+									height = gdk_pixbuf_get_height (pxb);
+									if (height > width)
+										width = height;
+									if (//width < E2_SCALEUP_SIZE ||
 										(doscale && width > threshold)
 									  || width != scaleto)
+									{
+										g_object_unref (G_OBJECT (pxb));
+										height = width;
+										goto recreate;
+									}
+								}
+								else
 								{
-									height = width;
-									goto recreate;
+									//warn the user
+									if (error != NULL)
+										g_error_free (error);
+									makepxb = FALSE;
 								}
 							}
-							pxb = gimp_thumbnail_load_thumb (handle, thumbnail, scaleto, &error);
-							if (pxb == NULL)
-							{
-								//warn the user
-								if (error != NULL)
-									g_error_free (error);
-								makepxb = FALSE;
-							}
+						}
+						else //no file info
+						{
+	//						printd (DEBUG, "skipping %s, no pixbuf file info", localpath);
+							//FIXME warn the user if it does seem to be an image
+							makepxb = FALSE;
 						}
 					}
-					else //no file info
+					else //un-scaled image wanted
 					{
-						//FIXME warn the user
-						makepxb = FALSE;
+						pxb = gimp_thumbnail_load_thumb (handle, thumbnail,
+														GIMP_THUMB_SIZE_LARGE, NULL); /* try for any size */
+						if (pxb)
+						{
+							// check for unscaled
+							if (G_LIKELY(gdk_pixbuf_get_file_info
+								(thumbnail->thumb_filename, &width, &height) != NULL))
+							{
+								if (thumbnail->image_width == width && thumbnail->image_height == height)
+									goto create;
+							}
+							g_object_unref (G_OBJECT (pxb));
+						}
+
+						error = NULL;
+						pxb = gdk_pixbuf_new_from_file (localpath, &error);
+						if (pxb == NULL)	// pixbuf creation failed
+						{
+							//warn the user
+							if (error != NULL)
+								g_error_free (error);
+							makepxb = FALSE;
+						}
+						else if (G_LIKELY(gdk_pixbuf_get_file_info
+								(localpath, &width, &height) != NULL))
+						{
+							if (width <= GIMP_THUMB_SIZE_LARGE && height <= GIMP_THUMB_SIZE_LARGE)
+								gimp_thumbnail_save_thumb (handle, thumbnail, pxb,
+										E2_SOFTWARE_ID, NULL);
+#if 0 //thumbnail is needed
+# ifdef THUMB_OBJECT
+							//set non-standard thumbnail details
+							g_object_set (G_OBJECT (thumbnail),
+										  "image-state",      GIMP_THUMB_STATE_OK,
+										  "image-filesize",   0,
+										  "image-mtime",      0,
+										  "image-mimetype",   NULL,
+										  "image-width",      width,
+										  "image-height",     height,
+										  "image-type",       NULL,
+										  "image-num-layers", 0,
+										  "thumb-state",      GIMP_THUMB_STATE_FAILED, //?
+										  NULL);
+# else
+							thumbnail->image_state = GIMP_THUMB_STATE_OK;
+							thumbnail->image_filesize = 0;
+							thumbnail->image_mtime = 0;
+							thumbnail->image_mimetype = NULL;
+							thumbnail->image_width = width;
+							thumbnail->image_height = height;
+							thumbnail->image_type = NULL;
+							thumbnail->image_num_layers = 0;
+							thumbnail->thumb_state = GIMP_THUMB_STATE_FAILED;
+# endif
+#endif //0
+						}
 					}
 				}
+create:
 				if (makepxb)
 				{
 					//copy the FileInfo, in case a refresh clobbers the original
@@ -441,15 +514,23 @@ rescale:
 					//store the data
 					gtk_list_store_insert_with_values (store, &iteri, -1,
 						FILENAME, filename,
+						PIXBUF, pxb,
+#if 0 //thumbnail is needed
+						THUMBNAIL, thumbnail,
+#endif
 						NAMEKEY, namekey,
 						FINFO, info2,
-						PIXBUF, pxb,
-						THUMBNAIL, thumbnail,
 						-1);
 				}
+#if 0 //thumbnail is needed
 				else
-					g_object_unref (G_OBJECT (thumbnail));	//CHECKME any specific contents to clean ?
-
+#endif
+					if (thumbnail != NULL)
+#ifdef THUMB_OBJECT
+					g_object_unref (G_OBJECT (thumbnail));
+#else
+					gimp_thumbnail_destroy (thumbnail);
+#endif
 				g_free (dlocal);
 				g_free (localpath);
 			}
@@ -498,14 +579,14 @@ rescale:
 #endif */
 	e2_filelist_enable_one_refresh (pane);
 
-	rt->blocked = FALSE;
-//	printd (DEBUG, "finish icons store fill");
+	g_atomic_int_set (&rt->blocked, 0);
+	//	printd (DEBUG, "finish icons store fill");
 	return store;
 }
 /**
 @brief refresh icons-view liststore
 The existing liststore is replaced, and queued for cleanup.
-BGL is expected to be open
+Expects BGL open/off
 @param rt pointer to data struct for dialog
 
 @return
@@ -532,9 +613,20 @@ static void _e2p_thumbs_refresh_store (E2_ThumbDialogRuntime *rt)
 		g_list_free (selpaths);
 	}
 
+	//disable dialog close-button while replacing the thumbnails liststore
+	//Closing during this process, which may take a noticeable period, will likely
+	//crash
+	gdk_threads_enter ();
+	gtk_dialog_set_response_sensitive (GTK_DIALOG(rt->dialog), GTK_RESPONSE_CLOSE, FALSE);
+	gdk_threads_leave ();
+
 	//update the store
 	_e2p_thumbs_replace_store (rt);
 
+	gdk_threads_enter ();
+	gtk_dialog_set_response_sensitive (GTK_DIALOG(rt->dialog), GTK_RESPONSE_CLOSE, TRUE);
+	gdk_threads_leave ();
+
 	//reselect things FIXME do this smarter
 	if (g_list_length (selnames) > 0)
 	{
@@ -588,8 +680,8 @@ static void _e2p_thumbs_refresh_store (E2_ThumbDialogRuntime *rt)
 	//FIXME when dir is not mounted local
 #endif
 		if ((rt->dir_mtime < rt->view->dir_mtime || rt->filtercount != rt->view->filtercount)
-			&& !rt->blocked
-			&& !strcmp (rt->path, rt->view->dir))
+			&& g_atomic_int_get (&rt->blocked) == 0
+			&& strcmp (rt->path, rt->view->dir) == 0)
 		{
 			busy = TRUE;
 			_e2p_thumbs_refresh_store (rt);
@@ -600,74 +692,63 @@ static void _e2p_thumbs_refresh_store (E2_ThumbDialogRuntime *rt)
 }
 */
 /**
+@brief timer callback to do a store refresh after blockage is removed
+@param rt pointer to dialog's data struct
+@return TRUE if the 'blockage' from the main-window process is still present, or a local busy flag is set
+*/
+static gboolean _e2p_thumbs_wait_to_refresh (E2_ThumbDialogRuntime *rt)
+{
+	if (GTK_IS_WIDGET (rt->dialog))
+	{
+		static gboolean working = FALSE;	//simple blocker instead of killing timer
+
+		if (working
+		 || g_atomic_int_get (&rt->blocked) //another blocker, set downstream
+		 || g_atomic_int_get (&rt->view->listcontrols.refresh_working)
+		 || g_atomic_int_get (&rt->view->listcontrols.cd_working))
+			return TRUE; //wait more
+		working = TRUE;
+		printd (DEBUG, "deferred replication of icons liststore");
+		_e2p_thumbs_refresh_store (rt);
+		working = FALSE;
+	}
+	rt->timer_id = 0;
+	return FALSE; //kill the timer
+}
+/**
 @brief hook function to do a store refresh after filelist for @a view is refreshed
 This is initiated from an idle-callback, so BGL is off
-@param view pointer to data struct for refreshed pane
+@param view UNUSED pointer to data struct for refreshed pane
 @param rt pointer to dialog's data struct
 @return TRUE unless the dialog is destroyed already
 */
 static gboolean _e2p_thumbs_refresh_hook (ViewInfo *view,
 	E2_ThumbDialogRuntime *rt)
 {
+	static gboolean working = FALSE;
+
 	if (!GTK_IS_WIDGET (rt->dialog))
 		return FALSE; //kill the callback
-	static gboolean working = FALSE;
-	if (!working)
+	//see API text for _e2_fileview_more_refresh() for advice about race-checks here
+	if (g_atomic_int_get (&rt->view->listcontrols.refresh_working)
+	 || g_atomic_int_get (&rt->view->listcontrols.cd_working))
+	//something which will affect the display has started
+		return TRUE;	//ignore this refresh, wait for the new process to finish
+	if (working)
+	{
+		printd (DEBUG, "defer refresh until current refresh is finished");
+		if (rt->timer_id == 0)
+			rt->timer_id = g_timeout_add (300, (GSourceFunc)_e2p_thumbs_wait_to_refresh, rt);
+	}
+	else
 	{
 		working = TRUE;
-//		if (!rt->view->norefresh)	//CHECKME ->listing ?
-//		{
-			printd (DEBUG, "replicating REFRESH of icons liststore");
-			_e2p_thumbs_refresh_store (rt);
-/*		}
-		else
-		{
-			printd (DEBUG, "when the current filelist refresh is finished");
-			g_timeout_add (200, (GSourceFunc)_e2p_thumbs_wait_to_refresh, rt);
-		}
-*/
+		printd (DEBUG, "replicating REFRESH of icons liststore");
+		_e2p_thumbs_refresh_store (rt);
 		working = FALSE;
 	}
 	return TRUE;
 }
-/* *
-@brief timer callback to turn auto-refresh back on
-@param rt pointer to dialog's data struct
-@return FALSE
-*/
-/*static gboolean _e2p_thumbs_enable_refresh (E2_ThumbDialogRuntime *rt)
-{
-	if (GTK_IS_WIDGET (rt->dialog))
-		e2_hook_register (&rt->view.hook_refresh, _e2p_thumbs_refresh_hook, rt);
-	return FALSE;
-} */
-/**
-@brief timer callback to do a store replacement after refreshing is unblocked
-@param rt pointer to dialog's data struct
-@return TRUE if the associated-filelist refresh is still in progress
-*/
-static gboolean _e2p_thumbs_wait_to_replace (E2_ThumbDialogRuntime *rt)
-{
-	if (GTK_IS_WIDGET (rt->dialog) && !rt->blocked)
-	{
-		LISTS_LOCK
-		gboolean busy = rt->view->listcontrols.refresh_working
-					 || rt->view->listcontrols.cd_working;
-		LISTS_UNLOCK
-		if (busy)
-			return TRUE;
-
-		static gboolean working = FALSE;	//simple blocker instead of killing timer
-		if (!working)
-		{	//any previous timer callback has finished
-//			printd (DEBUG, "doing deferred replication of CHANGE-DIR for icons liststore");
-			working = TRUE;
-			_e2p_thumbs_refresh_store (rt);
-			working = FALSE;
-		}
-	}
-	return FALSE;
-}
 /**
 @brief hook function for app.paneX.hook_change_dir
 This is initiated from cd thread, with BGL off/open
@@ -678,50 +759,24 @@ This is initiated from cd thread, with BGL off/open
 static gboolean _e2p_thumbs_change_dir_hook (gchar *newpath, E2_ThumbDialogRuntime *rt)
 {
 	printd (DEBUG, "replicating CHANGE-DIR for icons liststore");
+	//grab new path
 	g_free (rt->path);
 	rt->path = g_strdup (newpath);
-	LISTS_LOCK
-	gboolean busy = rt->view->listcontrols.refresh_working
-				 || rt->view->listcontrols.cd_working;
-	LISTS_UNLOCK
-
-	if (!busy)
-		_e2p_thumbs_refresh_store (rt);
-	else
+	/* Hooklist is run towards end of _e2_fileview_change_dir() thread-func
+	   In principle, another cd could have beein initiated while the current one
+	   is being processed, but such can be ignored here */
+	if (g_atomic_int_get (&rt->view->listcontrols.refresh_working)) //this is probably impossible here
 	{
-		printd (DEBUG, "when the current filelist refresh is finished");
-		g_timeout_add (500, (GSourceFunc)_e2p_thumbs_wait_to_replace, rt);
+		printd (DEBUG, "defer cd until the current filelist refresh is finished");
+		if (rt->timer_id == 0)
+			rt->timer_id = g_timeout_add (300, (GSourceFunc)_e2p_thumbs_wait_to_refresh, rt);
 	}
+	else
+		_e2p_thumbs_refresh_store (rt);
 
 	return TRUE;
 }
 /**
-@brief timer callback to do a store refresh after refreshing is unblocked
-@param rt pointer to dialog's data struct
-@return TRUE if the associated-filelist refresh is still in progress
-*/
-static gboolean _e2p_thumbs_wait_to_refresh (E2_ThumbDialogRuntime *rt)
-{
-	if (GTK_IS_WIDGET (rt->dialog) && !rt->blocked)
-	{
-		LISTS_LOCK
-		gboolean disabled = rt->view->listcontrols.norefresh;	//CHECKME ->listing ?
-		LISTS_UNLOCK
-		if (disabled)
-			return TRUE;
-
-		static gboolean working = FALSE;	//simple blocker instead of killing timer
-		if (!working)
-		{	//any previous timer callback has finished
-			working = TRUE;
-			printd (DEBUG, "deferred replication of REFRESH of icons liststore");
-			_e2p_thumbs_refresh_store (rt);
-			working = FALSE;
-		}
-	}
-	return FALSE;
-}
-/**
 @brief migrate iconview selection to associated filelist
 
 @param rt pointer to dialog's data struct
@@ -732,28 +787,31 @@ static gboolean _e2p_thumbs_transfer_selection (E2_ThumbDialogRuntime *rt)
 {
 	gboolean retval = FALSE;
 	GtkTreeSelection *listsel = gtk_tree_view_get_selection (GTK_TREE_VIEW (rt->view->treeview));
+	//TODO detect whatever needs to be unselected, without clearing the lot,
+	// to protect selections created via other dialog(s)
 	gtk_tree_selection_unselect_all (listsel);
 	GList *sel = gtk_icon_view_get_selected_items (GTK_ICON_VIEW (rt->iconview));
 	if (sel != NULL && g_list_length (sel) > 0)
 	{
-		gchar *name;
-		GList *member;
-		GtkTreePath *tp;
-		GtkTreeModel *mdl = GTK_TREE_MODEL (rt->store);
+		//walk the file-list iters, selecting each one that matches
+		GtkTreeIter listiter;
 		GtkTreeModel *listmdl = gtk_tree_view_get_model (GTK_TREE_VIEW (rt->view->treeview)); //filter model
-		GtkTreeIter iter, listiter;
-
 		if (gtk_tree_model_get_iter_first (listmdl, &listiter))
 		{
+			GtkTreeIter iter;
+			GList *member;
+			GtkTreeModel *mdl = GTK_TREE_MODEL (rt->store);
 			for (member = sel; member != NULL; member = member->next)
 			{
+				gchar *name;
+				GtkTreePath *tp;
 				tp = (GtkTreePath *)member->data;
 				gtk_tree_model_get_iter (mdl, &iter, tp);
 				gtk_tree_model_get (mdl, &iter, FILENAME, &name, -1);
 				if (e2_tree_find_iter_from_str_same (listmdl, FILENAME, name, &listiter))
 				{
 					gtk_tree_selection_select_iter (listsel, &listiter);
-					gtk_tree_model_get_iter_first (listmdl, &listiter);
+					gtk_tree_model_get_iter_first (listmdl, &listiter); //restart searching, order may be different
 				}
 				g_free (name);
 				gtk_tree_path_free (tp);
@@ -962,7 +1020,7 @@ static void _e2p_thumbs_unselect_all_cb (GtkMenuItem *widget, E2_ThumbDialogRunt
 	gtk_icon_view_unselect_all (GTK_ICON_VIEW (rt->iconview));
 }
 /**
-@brief iconview refresh callback
+@brief iconview manual-refresh callback
 
 @param widget UNUSED the menu item widget which activated the callback
 @param rt pointer to dialog data struct
@@ -971,17 +1029,18 @@ static void _e2p_thumbs_unselect_all_cb (GtkMenuItem *widget, E2_ThumbDialogRunt
 */
 static void _e2p_thumbs_refresh_cb (GtkMenuItem *widget, E2_ThumbDialogRuntime *rt)
 {
-	LISTS_LOCK
-	gboolean busy = rt->view->listcontrols.norefresh;
-	LISTS_UNLOCK
-	if (!busy)
+	if (g_atomic_int_get (&rt->view->listcontrols.refresh_working))
+	{
+		if (rt->timer_id == 0)
+			rt->timer_id = g_timeout_add (300, (GSourceFunc)_e2p_thumbs_wait_to_refresh, rt);
+	}
+	else if (!g_atomic_int_get (&rt->view->listcontrols.cd_working))
 	{
 		gdk_threads_leave ();
 		_e2p_thumbs_refresh_store (rt);
 		gdk_threads_enter ();
 	}
-	else
-		g_timeout_add (500, (GSourceFunc)_e2p_thumbs_wait_to_refresh, rt);
+	//if cd is happening already, ignore the refresh request
 }
 /**
 @brief rotate selected thumbnails 90 degrees clockwise
@@ -1179,6 +1238,7 @@ static void _e2p_thumbs_show_context_menu (GtkWidget *iconview,
 			e2_option_int_get ("thumb-limit"));
 	e2_widget_set_safetip (item, tip);
 	g_free (tip);
+	//handle change of "thumb-limit" value while menu is displayed ?
 /*	item =
 	e2_menu_add_check (menu, _("_Hidden items"), rt->show_hidden,
 		_e2_treedlg_toggle_strict_cb, rt);
@@ -1260,7 +1320,7 @@ static gboolean _e2p_thumbs_button_press_cb (GtkWidget *iconview,
 static gboolean _e2p_thumbs_popup_menu_cb (GtkWidget *iconview,
 	E2_ThumbDialogRuntime *rt)
 {
-	gint event_time = gtk_get_current_event_time ();
+	guint32 event_time = gtk_get_current_event_time ();
 	_e2p_thumbs_show_context_menu (iconview, 0, event_time, rt);
 	return TRUE;
 }
@@ -1385,8 +1445,8 @@ set @a push_in to TRUE for menu completely inside the screen,
 FALSE for menu clamped to screen size
 
 @param menu the GtkMenu to be positioned
-@param x	place to store gint representing the menu left
-@param y  place to store gint representing the menu top
+@param x place to store gint representing the menu left
+@param y place to store gint representing the menu top
 @param push_in place to store pushin flag
 @param button the activated dialog button
 
@@ -1398,8 +1458,12 @@ static void _e2p_thumbs_set_sortmenu_position (GtkMenu *menu, gint *x, gint *y,
 	gint button_y;
 	e2_utils_get_abs_pos (button, x, &button_y);
 	GtkRequisition menu_size;
+#ifdef USE_GTK3_0
+	//TODO only height ? gtk_widget_get_preferred_height()
+	gtk_widget_get_preferred_size (GTK_WIDGET (menu), NULL, &menu_size);
+#else
 	gtk_widget_size_request (GTK_WIDGET (menu), &menu_size);
-
+#endif
 	//place below or above button, left-aligned
 	if (button_y - menu_size.height <= 2)	//> gdk_screen_height ())
 #ifdef USE_GTK2_18
@@ -1421,7 +1485,12 @@ static void _e2p_thumbs_set_sortmenu_position (GtkMenu *menu, gint *x, gint *y,
 @param rt pointer to data struct for the dialog
 @return
 */
-static void _e2p_thumbs_destroy_cb (GtkObject *object,
+static void _e2p_thumbs_destroy_cb (
+#ifdef USE_GTK3_0
+	GtkWidget *object,
+#else
+	GtkObject *object,
+#endif
 	E2_ThumbDialogRuntime *rt)
 {
 	g_signal_handlers_disconnect_by_func ((gpointer)rt->dialog,
@@ -1451,13 +1520,22 @@ static void _e2p_thumbs_response_cb (GtkDialog *dialog, gint response,
 		case E2_RESPONSE_USER2: //sort-button click
 		{
 			GtkWidget *menu = _e2p_thumbs_create_sorting_menu (rt);
-			guint32 time = gtk_get_current_event_time ();
+			guint32 event_time = gtk_get_current_event_time ();
 			gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
-				(GtkMenuPositionFunc) _e2p_thumbs_set_sortmenu_position, rt->sortbtn, 1, time);
+				(GtkMenuPositionFunc) _e2p_thumbs_set_sortmenu_position,
+					rt->sortbtn, 1, event_time);
 		}
 			break;
 		default:
-		{
+			if (g_atomic_int_get (&rt->blocked))
+			{
+				if (response != GTK_RESPONSE_CANCEL)
+					break;
+					//TODO support a "cancel" UI and refresh-cancel-mechanism
+			}
+
+			if (rt->timer_id != 0)
+				g_source_remove (rt->timer_id);
 #ifdef USE_GTK2_18
 			GtkAllocation alloc;
 			gtk_widget_get_allocation (rt->dialog, &alloc);
@@ -1490,7 +1568,6 @@ static void _e2p_thumbs_response_cb (GtkDialog *dialog, gint response,
 			//NOTE crash if the plugin is unloaded before this cb happens !
 			g_idle_add ((GSourceFunc) _e2p_thumbs_clear_old_stores, rt->oldstores);
 			DEALLOCATE (E2_ThumbDialogRuntime, rt);
-		}
 			break;
 	}
 }
@@ -1522,7 +1599,7 @@ static gpointer _e2p_thumbs_dialog_run (ViewInfo *view)
 	rt->sort_type = view->sort_column;
 	rt->sort_order = view->sort_order;
 	rt->filtercount = view->filtercount;
-	rt->blocked = FALSE;
+	rt->blocked = 0;
 /* no need for this, before the store is initially filled
 	GtkTreeSortable *sortable = GTK_TREE_SORTABLE (rt->store);
 	gtk_tree_sortable_set_sort_func (sortable, FILENAME,
@@ -1536,8 +1613,12 @@ static gpointer _e2p_thumbs_dialog_run (ViewInfo *view)
 	rt->oldstores = NULL;
 
 	gchar *title = (view == &app.pane1.view) ? _("pane 1 images") : _("pane 2 images") ;
+	gdk_threads_enter ();
 	rt->dialog = e2_dialog_create (NULL, NULL, title,
 		(ResponseFunc)_e2p_thumbs_response_cb, rt);
+	gdk_threads_leave ();
+
+	e2_dialog_set_negative_response (rt->dialog, GTK_RESPONSE_CLOSE); //override default
 
 	//scrolled window for the treeview
 	GtkWidget *sw = e2_widget_add_sw (
@@ -1578,7 +1659,13 @@ static gpointer _e2p_thumbs_dialog_run (ViewInfo *view)
 	const PangoMatrix *matrix = pango_context_get_matrix (context);
 	PangoGravity grav = pango_gravity_get_for_matrix (matrix);
 	if PANGO_GRAVITY_IS_VERTICAL(grav)
+	{
+# ifdef USE_GTK3_0
+		gtk_orientable_set_orientation (GTK_ORIENTABLE (iconview), GTK_ORIENTATION_HORIZONTAL);
+# else
 		gtk_icon_view_set_orientation (iconview, GTK_ORIENTATION_HORIZONTAL); //VERTICAL is the default
+# endif
+	}
 #endif
 //	gtk_icon_view_set_columns (iconview, gint columns);
 //	gtk_icon_view_set_item_width (iconview, gint item_width);
@@ -1661,8 +1748,13 @@ static gpointer _e2p_thumbs_dialog_run (ViewInfo *view)
 		G_CALLBACK (_e2_fileview_cursor_change_cb), view);
 */
 	//cleanup if the dialog rt data becomes invalid
-	g_signal_connect (GTK_OBJECT(view->treeview), "destroy",
-		G_CALLBACK (_e2p_thumbs_destroy_cb), rt);
+	g_signal_connect (
+#ifdef USE_GTK3_0
+	G_OBJECT (view->treeview),
+#else
+	GTK_OBJECT (view->treeview),
+#endif
+		"destroy", G_CALLBACK (_e2p_thumbs_destroy_cb), rt);
 
 	//relate initial size to last-used, or if first, to filepanes size
 #ifdef USE_GTK2_18
@@ -1700,16 +1792,18 @@ static gpointer _e2p_thumbs_dialog_run (ViewInfo *view)
 	e2_dialog_show (rt->dialog, app.main_window, E2_DIALOG_CLOSELOCK,
 		&E2_BUTTON_CLOSE, NULL);
 
-	//get the actual data for the iconview
-	LISTS_LOCK
-	gboolean busy = rt->view->listcontrols.refresh_working
-				 || rt->view->listcontrols.cd_working;
-	LISTS_UNLOCK
-	if (!busy)
-		_e2p_thumbs_replace_store (rt);
+	//populate the actual data for the iconview, as soon as we can
+	gboolean busy =
+		g_atomic_int_get (&rt->view->listcontrols.refresh_working)
+	 || g_atomic_int_get (&rt->view->listcontrols.cd_working);
+	if (busy)
+		rt->timer_id = g_timeout_add_full (G_PRIORITY_HIGH, 100,
+			(GSourceFunc)_e2p_thumbs_wait_to_refresh, rt, NULL);
 	else
-		g_timeout_add_full (G_PRIORITY_HIGH, 100,
-			(GSourceFunc)_e2p_thumbs_wait_to_replace, rt, NULL);
+	{
+		rt->timer_id = 0;
+		_e2p_thumbs_replace_store (rt);
+	}
 
 	//show and select the startup row corresponding to displayed dir
 //	_e2_treedlg_show_path (rt->view.dir, TRUE, rt);
@@ -1732,13 +1826,23 @@ can be slow
 @param from UNUSED the button, menu item etc which was activated
 @param art UNUSED action runtime data
 
-@return TRUE
+@return TRUE if a gimpthumb handle is available
 */
 static gboolean _e2p_thumbs_show_action (gpointer from, E2_ActionRuntime *art)
 {
-	E2_PaneRuntime *rt = e2_pane_get_runtime (from, art->data, NULL);
-	g_thread_create ((GThreadFunc) _e2p_thumbs_dialog_run, &rt->view, FALSE, NULL);
-	return TRUE;
+	if (handle == NULL)
+		handle = gimp_thumbconnection_new (E2_SOFTWARE_ID, NULL);
+	if (handle != NULL)
+	{
+		E2_PaneRuntime *rt = e2_pane_get_runtime (from, art->data, NULL);
+#ifdef USE_GLIB2_32
+		g_thread_new ("", (GThreadFunc) _e2p_thumbs_dialog_run, &rt->view);
+#else
+		g_thread_create ((GThreadFunc) _e2p_thumbs_dialog_run, &rt->view, FALSE, NULL);
+#endif
+		return TRUE;
+	}
+	return FALSE;
 }
 
 #ifdef E2_TRANSIENTBINDINGS
@@ -1824,41 +1928,38 @@ gboolean init_plugin (Plugin *p)
 
 	if (p->action == NULL)
 	{
-		handle = gimp_thumbconnection_new (E2_SOFTWARE_ID, NULL);
-		if (handle != NULL)
+		//don't free name string here
+		E2_Action plugact =
+		{g_strconcat (_A(7),".",aname,NULL),_e2p_thumbs_show_action,TRUE,E2_ACTION_TYPE_ITEM,0,NULL,NULL};
+		p->action = e2_plugins_action_register (&plugact);
+		if G_UNLIKELY((p->action == NULL))
 		{
-			//don't free name string here
-			E2_Action plugact =
-			{g_strconcat (_A(7),".",aname,NULL),_e2p_thumbs_show_action,TRUE,E2_ACTION_TYPE_ITEM,0,NULL,NULL};
-			p->action = e2_plugins_action_register (&plugact);
-			if G_UNLIKELY((p->action == NULL))
-			{
-				g_free (plugact.name);
-				return FALSE;
-			}
-			E2_OptionSet *set;
-			E2_OptionSetupExtra ex;
-			gchar *group = g_strconcat(_C(34),".",_C(27),":",aname,NULL); //_("plugins.options:thumbs"
-			ex.exbool = TRUE;
-			set = e2_plugins_option_register (E2_OPTION_TYPE_BOOL, "thumb-scale",
-				group, _("limit thumbnail size"),
-				_("If enabled, images larger than the size specified below will be scaled down"),
-				NULL, &ex, E2_OPTION_FLAG_FREEGROUP | E2_OPTION_FLAG_ADVANCED);
-			//because plugins are loaded after config data, config options need to
-			//get any data from unknown-options data
-			e2_option_transient_value_get (set);
-
-			ex.exint.def = GIMP_THUMB_SIZE_NORMAL;
-			ex.exint.min = 16;
-			ex.exint.max = 256;
-			set = e2_plugins_option_register (E2_OPTION_TYPE_INT, "thumb-limit",
-				group, _("largest thumbnail size"),
-				_("Pixel-size threshold for images scaling"),
-				"thumb-scale", &ex, E2_OPTION_FLAG_ADVANCED);
-			e2_option_transient_value_get (set);
-
-			return TRUE;
+			g_free (plugact.name);
+			return FALSE;
 		}
+		E2_OptionSet *set;
+		E2_OptionSetupExtra ex;
+		gchar *group = g_strconcat(_C(34),".",_C(27),":",aname,NULL); //_("plugins.options:thumbs"
+		memset (&ex, 0, sizeof (E2_OptionSetupExtra));
+		ex.exbool = TRUE;
+		set = e2_plugins_option_register (E2_OPTION_TYPE_BOOL, "thumb-scale",
+			group, _("limit thumbnail size"),
+			_("If enabled, images larger than the size specified below will be scaled down"),
+			NULL, &ex, E2_OPTION_FLAG_FREEGROUP | E2_OPTION_FLAG_ADVANCED);
+		//because plugins are loaded after config data, config options need to
+		//get any data from unknown-options data
+		e2_option_transient_value_get (set);
+
+		ex.exint.def = GIMP_THUMB_SIZE_NORMAL;
+		ex.exint.min = 16;
+		ex.exint.max = 256;
+		set = e2_plugins_option_register (E2_OPTION_TYPE_INT, "thumb-limit",
+			group, _("largest thumbnail size"),
+			_("Pixel-size threshold for images scaling"),
+			"thumb-scale", &ex, E2_OPTION_FLAG_ADVANCED);
+		e2_option_transient_value_get (set);
+
+		return TRUE;
 	}
 	return FALSE;
 }
