@@ -1,21 +1,19 @@
-/* $Id: e2_action.c 2008 2009-11-12 01:25:40Z tpgww $
+/* $Id: e2_action.c 2439 2012-02-21 23:23:53Z tpgww $
 
-Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
+Copyright (C) 2003-2012 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
-emelFM2 is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3, or (at your option)
-any later version.
+emelFM2 is free software; you can redistribute it and/or modify it under the
+terms of the GNU General Public License as published by the Free Software
+Foundation; either version 3, or (at your option) any later version.
 
 emelFM2 is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+You should have received a copy of the GNU General Public License along with
+emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 /**
@@ -223,8 +221,7 @@ based on data stored in column 2
 
 @return TRUE if the row is visible
 */
-static gboolean _e2_action_visible_cb (GtkTreeModel *model,
-	GtkTreeIter *iter, gpointer data)
+static gboolean _e2_action_visible_cb (GtkTreeModel *model, GtkTreeIter *iter, gpointer data)
 {
 	gboolean retval;
 	E2_ACTION_EXCLUDE ex1 = GPOINTER_TO_INT (data);
@@ -242,24 +239,128 @@ static gboolean _e2_action_visible_cb (GtkTreeModel *model,
 //	g_free (child);
 	return retval;
 }
+static GtkWidget *selitem = NULL;
+/**
+@brief "button-press-event" signal callback for a hover-menu widget
+@param item the menu-item where the button was pressed
+@param event button-event data
+@param data pointer to data struct for the hover action
 
-static gboolean _e2_action_enter_menu_cb (GtkWidget *widget, GdkEventCrossing *event,
-		E2HoverData *data)
+@return	TRUE if the event was processed
+*/
+static gboolean _e2_action_hover_buttonpress_cb (GtkWidget *item, GdkEventButton *event,
+	E2HoverData *data)
+{
+	if (event->button == 1 && event->type == GDK_BUTTON_PRESS)
+	{
+		gtk_menu_shell_activate_item (GTK_MENU_SHELL (data->menu), item, TRUE);
+		data->menu = NULL;	//data->menu will have been destroyed as consequence of activation
+		selitem = NULL;
+		return TRUE;
+	}
+	return FALSE;
+}
+/**
+@brief foreach func to apply button-press callback to each item in a hover menu
+@param menuitem item in hover menu
+@param data pointer to data struct for the hover action
+@return
+*/
+static void _e2_action_hover_setup (GtkWidget *menuitem, E2HoverData *data)
+{
+	g_signal_connect (G_OBJECT (menuitem), "button-press-event",
+		G_CALLBACK (_e2_action_hover_buttonpress_cb), data);
+}
+/**
+@brief "motion-notify" event signal for entered history menu
+@param window UNUSED the parent-window-widget of the hover menu
+@param event motion-event data
+@param data pointer to data struct for the hover action
+@return FALSE always
+*/
+static gboolean _e2_action_hover_pointermove_cb (GtkWidget *window,
+	GdkEventMotion *event, E2HoverData *data)
+{
+	GtkWidget *menuitem = gtk_get_event_widget ((GdkEvent*) event);
+	if (selitem != menuitem)
+	{
+		//show selected item
+#ifdef USE_GTK3_0
+		//without activating the item on gtk3
+		if (selitem != NULL)
+			gtk_menu_item_deselect (GTK_MENU_ITEM(selitem));
+		gtk_menu_item_select (GTK_MENU_ITEM(menuitem));
+#endif
+		selitem = menuitem;
+#ifndef USE_GTK3_0
+		gtk_menu_shell_select_item (GTK_MENU_SHELL (data->menu), menuitem);
+#endif
+	}
+	return FALSE;
+}
+/**
+@brief "enter-notify-event" signal callback for a hover-menu parent-window widget
+Kills any hover-timer, and sets up for menu selection
+@param window the parent-window-widget of the hover menu
+@param event crossing-event data
+@param data pointer to data struct for the hover action
+
+@return	FALSE always so the event is propagated
+*/
+static gboolean _e2_action_enter_menu_cb (GtkWidget *window, GdkEventCrossing *event,
+	E2HoverData *data)
+{
+	//kill timer initiated when button was exited, prevent the popup menu from closing
+	if (data->timer_id > 0)
+	{
+		g_source_remove (data->timer_id);
+		data->timer_id = 0;
+	}
+#ifdef USE_GTK2_18
+	if (G_LIKELY(!gtk_widget_has_grab (window)))
+#else
+	if (G_LIKELY(!GTK_WIDGET_HAS_GRAB (window)))
+#endif
+		gtk_grab_add (window); //apply keyboard and mouse focus
+	g_signal_connect (G_OBJECT (window), "motion-notify-event",
+		G_CALLBACK (_e2_action_hover_pointermove_cb), data);
+
+	return FALSE;
+}
+/**
+@brief "leave-notify-event" signal callback for a hover-menu parent-window widget
+@param window the parent-window-widget of the hover menu
+@param event crossing-event data
+@param data pointer to data struct for the hover action
+
+@return	FALSE always so the event is propagated
+*/
+static gboolean _e2_action_leave_menu_cb (GtkWidget *window, GdkEventCrossing *event,
+	E2HoverData *data)
 {
 	if (data->timer_id > 0)
 	{
 		g_source_remove (data->timer_id);
 		data->timer_id = 0;
 	}
+	printd (DEBUG, "In _e2_action_leave_menu_cb()");
+#ifdef USE_GTK2_18
+	if (G_LIKELY (gtk_widget_has_grab (window)))
+#else
+	if (G_LIKELY (GTK_WIDGET_HAS_GRAB (window)))
+#endif
+		gtk_grab_remove (window);
+	gtk_widget_destroy (window);
+	data->menu = NULL;
+	selitem = NULL;
 	return FALSE;
 }
-
 /**
-@brief timer callback to implement a hover callback
+@brief timer callback to implement a hover menu
 @brief data pointer to hover data
 @return FALSE to cancel the timer
 */
-static gboolean _e2_action_do_hover_cb (E2HoverData *data)
+static gboolean _e2_action_do_hover_timeout (E2HoverData *data)
 {
 	//get rid of this ASAP in case of pending departure from hover-widget
 	if (data->timer_id > 0)
@@ -268,39 +369,39 @@ static gboolean _e2_action_do_hover_cb (E2HoverData *data)
 		data->timer_id = 0;
 	}
 
-	gdk_threads_enter ();
-	(*data->callback) (data->hovered, data);
-
-	if (GTK_IS_MENU (data->popup) &&
-#ifdef USE_GTK2_18
-		!gtk_widget_get_visible (data->popup)
-#else
-		!GTK_WIDGET_VISIBLE (data->popup)
-#endif
-		)
+	(*data->callback) (data->hovered, data); //e.g. _e2_pane_forward_hover
+	if (GTK_IS_MENU (data->menu))
 	{
-		GtkWidget *popwin = gtk_widget_get_ancestor (data->popup, GTK_TYPE_WINDOW); //would be parent, actually
-		gtk_window_set_transient_for (GTK_WINDOW (popwin), GTK_WINDOW (app.main_window));
-		g_signal_connect (G_OBJECT (popwin), "enter-notify-event",
-			G_CALLBACK (_e2_action_enter_menu_cb), data);
-		g_signal_connect (G_OBJECT (popwin), "leave-notify-event",
-		G_CALLBACK (e2_action_end_hover_cb), data);
-
-		gtk_widget_show (data->popup); //must do this here, not later - dunno why
-
-		GtkRequisition requisition;
-		gtk_widget_size_request (data->popup, &requisition);
-		gint x, y;
-		gboolean push = TRUE;
-		e2_toolbar_set_menu_position (GTK_MENU (data->popup), &x, &y, &push, data->hovered);
-		gtk_widget_size_request (popwin, &requisition);
-		gtk_widget_set_size_request (popwin, requisition.width, requisition.height);
-		gtk_window_move (GTK_WINDOW (popwin), x, y); //this might be ignored by WM
-		gtk_widget_show_all (popwin); //doesn't work, dunno why
-		gtk_window_move (GTK_WINDOW (popwin), x, y); //fallback
+		GtkWidget *popwin = gtk_widget_get_ancestor (data->menu, GTK_TYPE_WINDOW); //menu's parent, actually
+		if (popwin != NULL)
+		{
+			gtk_window_set_transient_for (GTK_WINDOW (popwin), GTK_WINDOW (app.main_window));
+			//setup to handle button-presses too
+			gtk_container_foreach (GTK_CONTAINER(data->menu),
+				(GtkCallback)_e2_action_hover_setup, data);
+			g_signal_connect (G_OBJECT (popwin), "enter-notify-event",
+				G_CALLBACK (_e2_action_enter_menu_cb), data);
+			g_signal_connect (G_OBJECT (popwin), "leave-notify-event",
+				G_CALLBACK (_e2_action_leave_menu_cb), data);
+#ifdef USE_GTK2_18
+			if (!gtk_widget_get_visible (data->menu))
+#else
+			if (!GTK_WIDGET_VISIBLE (data->menu))
+#endif
+			{
+				gtk_widget_show (data->menu); //setup size for use when positioning ?
+				gint x, y;
+				gboolean push = TRUE;
+				e2_toolbar_set_menu_position (GTK_MENU (data->menu), &x, &y, &push, data->hovered);
+				gtk_window_move (GTK_WINDOW (popwin), x, y); //this might be ignored by WM ?
+				gdk_threads_enter ();
+				gtk_widget_show_all (popwin);
+				gdk_threads_leave ();
+			}
+		}
+		else
+			printd (DEBUG, "No parent window for history menu");
 	}
-	gdk_threads_leave ();
-
 	return FALSE;
 }
 /**
@@ -310,28 +411,38 @@ static gboolean _e2_action_do_hover_cb (E2HoverData *data)
 */
 static gboolean _e2_action_clear_hover_cb (E2HoverData *data)
 {
-	if (GTK_IS_WIDGET (data->popup))
+	if (GTK_IS_WIDGET (data->menu))
 	{
+		GtkWidget *popwin = gtk_widget_get_toplevel (data->menu);
 		gdk_threads_enter ();
-		gtk_widget_destroy (data->popup);
+		gtk_widget_destroy (popwin);
 		gdk_threads_leave ();
 	}
-	data->popup = NULL;
+	data->menu = NULL;
 
 	return FALSE;
 }
-
+/**
+@brief "clicked" signal callback on a button which supports a hover-menu
+If the button is clicked, we should respond to that, and abort any hover activity
+@param button the clicked item
+@param data hover data struct
+@return
+*/
 static void _e2_action_abort_hover_cb (GtkButton *button, E2HoverData *data)
 {
-	printd (DEBUG, "in function _e2_action_abort_hover_cb");
+//	printd (DEBUG, "in function _e2_action_abort_hover_cb");
 	if (data->timer_id > 0)
 	{
 		g_source_remove (data->timer_id);
 		data->timer_id = 0;
 	}
-	if (GTK_IS_WIDGET (data->popup))
-		gtk_widget_destroy (data->popup);
-	data->popup = NULL;
+	if (GTK_IS_WIDGET (data->menu))
+	{
+		GtkWidget *popwin = gtk_widget_get_toplevel (data->menu);
+		gtk_widget_destroy (popwin);
+	}
+	data->menu = NULL;
 }
 
   /******************/
@@ -349,25 +460,26 @@ a E2_ACTION_TYPE_HOVER action.
 
 @return	FALSE always so the event is propagated
 */
-gboolean e2_action_start_hover_cb (GtkWidget *widget, GdkEventCrossing *event,
-		E2HoverData *data)
+gboolean e2_action_start_hover_cb (GtkWidget *button, GdkEventCrossing *event, E2HoverData *data)
 {
-	printd (DEBUG, "in function e2_action_start_hover_cb");
+//	printd (DEBUG, "in function e2_action_start_hover_cb");
 	if (data->timer_id > 0)
 	{
 		//we're re-entering the buttton from a menu
 		g_source_remove (data->timer_id);
 		data->timer_id = 0;
 	}
-	if (data->popup == NULL)
+	if (data->menu == NULL)
 	{
 		//no popup if button is clicked
-		g_signal_connect (G_OBJECT (widget), "clicked", //not "button_press-event", that's blocked
+		g_signal_connect (G_OBJECT (button), "clicked", //not "button_press-event", that's blocked
 			G_CALLBACK (_e2_action_abort_hover_cb), data);
-		//FIXME relate interval to gtk's tooltip popup time
-		data->timer_id = g_timeout_add (1000, (GSourceFunc)_e2_action_do_hover_cb, data);
-	}
 
+		gint delay = 1000;
+		GtkSettings* defs = gtk_settings_get_default ();
+        g_object_get (G_OBJECT (defs), "gtk-menu-popup-delay", &delay, NULL);
+		data->timer_id = g_timeout_add (delay, (GSourceFunc)_e2_action_do_hover_timeout, data);
+	}
 	return FALSE;
 }
 /**
@@ -381,20 +493,26 @@ a E2_ACTION_TYPE_HOVER action, and for a popped-up menu widget.
 
 @return	FALSE always so the event is propagated
 */
-gboolean e2_action_end_hover_cb (GtkWidget *widget, GdkEventCrossing *event,
-		E2HoverData *data)
+gboolean e2_action_end_hover_cb (GtkWidget *widget, GdkEventCrossing *event, E2HoverData *data)
 {
-	printd (DEBUG, "in function e2_action_end_hover_cb");
+//	printd (DEBUG, "in function e2_action_end_hover_cb");
 	if (data->timer_id > 0)
 	{
 		g_source_remove (data->timer_id);
 		data->timer_id = 0;
 	}
-	if (GTK_IS_WIDGET (data->popup))
+	if (GTK_IS_WIDGET (data->menu))
+	{
 		//allow short interval to move pointer from button into the menu or vice-versa, before killing it
-		data->timer_id = g_timeout_add (100, (GSourceFunc)_e2_action_clear_hover_cb, data);
+		gint delay = 1000;
+		GtkSettings* defs = gtk_settings_get_default ();
+        g_object_get (G_OBJECT (defs), "gtk-menu-popdown-delay", &delay, NULL);
+		if (delay < 1000)
+			delay += 500;
+		data->timer_id = g_timeout_add (delay, (GSourceFunc)_e2_action_clear_hover_cb, data);
+	}
 	else
-		data->popup = NULL;
+		data->menu = NULL;
 
 	g_signal_handlers_disconnect_by_func ((gpointer)widget, _e2_action_abort_hover_cb, data);
 
@@ -404,11 +522,11 @@ gboolean e2_action_end_hover_cb (GtkWidget *widget, GdkEventCrossing *event,
 /**
 @brief option helper for action columns
 
-This function is used to source a model for config-dialog cell renderers
-in E2_OPTION_TREE_TYPE_SEL columns of tree-options
+This function is used to source a model for cell renderers in columns of type
+E2_OPTION_TREE_TYPE_SEL in tree-option config-dialog treeviews
 
 @param ex pointerised flags which specify categories of actions
- *		that should not be visible
+		that should not be visible
 
 @return GtkTreeFilterModel of all actions
 */
@@ -666,27 +784,27 @@ Runtime data @a rt may contain additional user data.
 @a from may be used by the action handler to determine the context.
 
 @param from the item activated to trigger the action (eg toolbar GtkButton, maybe NULL)
-@param rt runtime data for the action
+@param art runtime data for the action
 
 @return TRUE on success of the action, FALSE on failure
 */
-gboolean e2_action_run (gpointer from, E2_ActionRuntime *rt)
+gboolean e2_action_run (gpointer from, E2_ActionRuntime *art)
 {
 	//some quick checks
-	if (rt == NULL || rt->action == NULL || rt->action->func == NULL)
+	if (art == NULL || art->action == NULL || art->action->func == NULL)
 	{
 		printd (DEBUG, "cannot run action, it doesn't exist");
 		return FALSE;
 	}
-	printd (DEBUG, "e2_action_run (from:, rt:%s)", rt->action->name);
-	rt->state = e2_action_get_current_state ();
+	printd (DEBUG, "e2_action_run (from:, art:%s)", art->action->name);
 	if (from == NULL)
 		from = app.main_window;
-	gboolean (*func) (gpointer, E2_ActionRuntime *) = rt->action->func;
+	art->state = e2_action_get_current_state (from);
+	gboolean (*func) (gpointer, E2_ActionRuntime *) = art->action->func;
 #ifndef DEBUG_MESSAGES
-	return func (from, rt);
+	return func (from, art);
 #else
-	gboolean res = func (from, rt);
+	gboolean res = func (from, art);
 	printd (DEBUG,"e2_action_run () ends");
 	return res;
 #endif
@@ -707,13 +825,13 @@ E2_ActionRuntime *e2_action_pack_runtime (E2_Action *action, gpointer data,
 	void(*data_free)(gpointer))
 {
 //	printd (DEBUG, "action pack runtime for %s", action->name);
-	E2_ActionRuntime *rt = ALLOCATE (E2_ActionRuntime);
-	CHECKALLOCATEDFATAL (rt);
-	rt->action = action;
-	rt->data = data;
-	rt->data_free = (data == NULL) ? NULL : data_free;
-	rt->state = 0;	//usually reset when action is run
-	return rt;
+	E2_ActionRuntime *art = ALLOCATE (E2_ActionRuntime);
+	CHECKALLOCATEDFATAL (art);
+	art->action = action;
+	art->data = data;
+	art->data_free = (data == NULL) ? NULL : data_free;
+	art->state = 0;	//usually updated when action is run
+	return art;
 }
 /**
 @brief free action runtime object
@@ -739,10 +857,11 @@ void e2_action_free_runtime (E2_ActionRuntime *rt)
 }
 /**
 @brief determine state flags for an action about to run
+@param from pointer to the widget where the action was initiated
 
 @return tne state flags
 */
-E2_ActionState e2_action_get_current_state (void)
+E2_ActionState e2_action_get_current_state (gpointer from)
 {
 	E2_ActionState state = 0;
 	guint btn;
@@ -750,7 +869,17 @@ E2_ActionState e2_action_get_current_state (void)
 	if (event != NULL)
 	{
 		if (!gdk_event_get_state (event, &state))
+#ifdef USE_GTK3_0
+		{
+			GtkWidget *win = gtk_widget_get_toplevel (from);
+			if (gtk_widget_is_toplevel (win))
+				state = e2_utils_get_savedstate (win);
+			else
+				state = 0;
+		}
+#else
 			state = e2_utils_get_modifiers ();
+#endif
 
 		switch (event->type)
 		{
@@ -771,7 +900,15 @@ E2_ActionState e2_action_get_current_state (void)
 	}
 	else
 	{
+#ifdef USE_GTK3_0
+		GtkWidget *win = gtk_widget_get_toplevel (from);
+		if (gtk_widget_is_toplevel (win))
+			state = e2_utils_get_savedstate (win);
+		else
+			state = 0;
+#else
 		state = e2_utils_get_modifiers ();
+#endif
 		btn = 0;
 	}
 	ACTION_SETSTATE (state, btn);
@@ -992,6 +1129,7 @@ void e2_action_setup_labels (void)
 	action_labels[30] = _("add");
 	action_labels[31] = _("adjust_ratio");
 	action_labels[32] = _("application");
+	//see 127 _("change");
 	action_labels[33] = _("clear");
 	action_labels[34] = _("clear_history");
 	action_labels[35] = _("complete");
@@ -1100,8 +1238,9 @@ void e2_action_setup_labels (void)
 #ifdef WITH_BUTTONFAKE
 	action_labels[126] = _("button");
 #endif
+	action_labels[127] = _("change");
 }
-#define LAST_NAME_INDEX 126
+#define LAST_NAME_INDEX 127
 #if LAST_NAME_INDEX > ALLOCATED_NAMES - 1
 # warning must increase size of action names array
 #endif
