@@ -1,6 +1,6 @@
-/* $Id: e2_option.c 1952 2009-10-02 01:43:23Z tpgww $
+/* $Id: e2_option.c 2469 2012-03-16 00:19:24Z tpgww $
 
-Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
+Copyright (C) 2003-2012 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -14,8 +14,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 /**
@@ -47,6 +46,33 @@ ToDo - description of how options work
 #endif
 
 static void _e2_option_clean1 (E2_OptionSet *set);
+//disable-refresh-counter for config file
+volatile gint cfg_refresh_refcount = 0;
+#ifdef E2_FAM_KERNEL
+volatile
+#endif
+ gint cfgdirty; //<>0 when some change to config-data file has been detected
+extern time_t last_work_time;
+
+static gboolean _e2_option_repoll (GtkWidget *widget, GdkEvent *event,
+	gpointer userdata)
+{
+	g_signal_handlers_disconnect_by_func ((gpointer)app.main_window,
+		_e2_option_repoll, userdata);
+	last_work_time = time (NULL);
+	printd (DEBUG, "Resume operation of timer CONFIG_T");
+	app.timers[CONFIG_T] =
+#ifdef USE_GLIB2_14
+		g_timeout_add_seconds (E2_CONFIGCHECK_INTERVAL_S,
+#else
+		g_timeout_add (E2_CONFIGCHECK_INTERVAL,
+#endif
+		(GSourceFunc) e2_option_check_config_files, NULL);
+#ifdef E2_FAM_KERNEL
+	e2_fs_FAM_resume ();
+#endif
+	return FALSE;
+}
 
   /******************/
  /***** public *****/
@@ -100,22 +126,46 @@ gboolean e2_option_set_config_dir (void)
 	return retval;
 }
 /**
-@brief Initialize the default trash directory and a pointer to it
-It is intended that this default be native, not virtual
+@brief Helper for making chilren of trash dir
+Assumes the user will be the owner of any created dir
+@param localpath fully-qualified path of dir to create, or parent of dir to create
+@param childdir name of subdir in @a localpath, or NULL
+
+@return TRUE if dir created successfully
+*/
+static gboolean _e2_option_make_trash (const gchar *localpath, const gchar *childdir)
+{
+	gboolean success;
+	gchar *dir = (childdir != NULL) ? g_build_filename (localpath, childdir, NULL) : (gchar*)localpath;
+	printd (INFO, "creating trash directory '%s'", dir);
+#ifdef E2_VFS
+	VPATH ddata = { dir, NULL };	//only local config data
+	success = !e2_fs_recurse_mkdir (&ddata, 0755 E2_ERR_NONE());
+#else
+	success = !e2_fs_recurse_mkdir (dir, 0755 E2_ERR_NONE());
+#endif
+	if (childdir != NULL)
+		g_free (dir);
+	return success;
+}
+/**
+@brief Initialize the default trash directories and a pointer to the topmost
+It is intended that this default be native, not virtual, and owned by the current user
 Uses command-line option if that was provided.
-Otherwise, tries to use XDG standard, or if that is not
-available, tries the e2 config dir.
-Creates dir if doesn't exist, or print error message.
-Sets e2_cl_options.trash_dir to the path (a trailing '/'),
-or it stays NULL if dir not available.
+Otherwise, tries to use XDG standard, or if that is not available, tries
+the e2 config dir.
+Creates dir(s) that don't exist, or print error message.
+Sets e2_cl_options.trash_dir to the path (with a trailing '/'), or it stays NULL
+if dir not available.
 Needs to be called after the config dir is checked/created
 Expects BGL off/open
 @return
 */
 void e2_option_set_trash_dir (void)
 {
-	//make the trash dir if it doesn't exist
+	gboolean success = TRUE;
 	gchar *local = F_FILENAME_TO_LOCALE (e2_cl_options.trash_dir);
+	//make the trash dir if it doesn't exist
 #ifdef E2_VFS
 	VPATH ddata = { local, NULL };	//only local config data
 	if (!e2_fs_is_dir3 (&ddata E2_ERR_NONE()))
@@ -123,55 +173,82 @@ void e2_option_set_trash_dir (void)
 	if (!e2_fs_is_dir3 (local E2_ERR_NONE()))
 #endif
 	{
-		gchar *subdir = g_build_filename (local, "files", NULL);
-		printd (INFO, "creating trash directory '%s'", subdir);
-#ifdef E2_VFS
-		ddata.path = subdir;
-		if (e2_fs_recurse_mkdir (&ddata, 0755 E2_ERR_NONE()))
-#else
-		if (e2_fs_recurse_mkdir (local, 0755 E2_ERR_NONE()))
-#endif
+		success = _e2_option_make_trash (local, "files");
+		if (success)
+			success = _e2_option_make_trash (local, "info");
+		if (!success)
 		{
 			gchar *message = g_strdup_printf (_("Cannot create trash directory %s"),
 				e2_cl_options.trash_dir);
 			gdk_threads_enter ();
 			e2_output_print_error (message, TRUE);
 			gdk_threads_leave ();
-			g_free (e2_cl_options.trash_dir);
-			e2_cl_options.trash_dir = NULL;
+		}
+	}
+	//it's there, but can we use it ?
+#ifdef E2_VFS
+	else if (e2_fs_access (&ddata, R_OK | W_OK | X_OK E2_ERR_NONE()))
+#else
+	else if (e2_fs_access (local, R_OK | W_OK | X_OK E2_ERR_NONE()))
+#endif
+	{	//nope
+		success = FALSE;
+		printd (WARN, "cannot use trash directory '%s'",
+			e2_cl_options.trash_dir);
+	}
+	else
+	{	//yup, the 'parent' is ok
+		gchar *subdir = g_build_filename (local, "files", NULL);
+#ifdef E2_VFS
+		ddata.path = subdir;
+		if (e2_fs_access (&ddata, F_OK E2_ERR_NONE()))
+#else
+		if (e2_fs_access (subdir, F_OK E2_ERR_NONE()))
+#endif
+		{
+			success = _e2_option_make_trash (subdir, NULL);
+		}
+		else
+		{
+#ifdef E2_VFS
+			success = !e2_fs_access (&ddata, R_OK | W_OK | X_OK E2_ERR_NONE());
+#else
+			success = !e2_fs_access (subdir, R_OK | W_OK | X_OK E2_ERR_NONE());
+#endif
 		}
 		g_free (subdir);
 		subdir = g_build_filename (local, "info", NULL);
 #ifdef E2_VFS
 		ddata.path = subdir;
-		if (e2_fs_recurse_mkdir (&ddata, 0755 E2_ERR_NONE()))
+		if (e2_fs_access (&ddata, F_OK E2_ERR_NONE()))
 #else
-		if (e2_fs_recurse_mkdir (local, 0755 E2_ERR_NONE()))
+		if (e2_fs_access (subdir, F_OK E2_ERR_NONE()))
 #endif
 		{
-			g_free (e2_cl_options.trash_dir);
-			e2_cl_options.trash_dir = NULL;
+			success = success && _e2_option_make_trash (subdir, NULL);
+		}
+		else
+		{
+#ifdef E2_VFS
+			success = success && !e2_fs_access (&ddata, R_OK | W_OK | X_OK E2_ERR_NONE());
+#else
+			success = success && !e2_fs_access (subdir, R_OK | W_OK | X_OK E2_ERR_NONE());
+#endif
 		}
 		g_free (subdir);
 	}
-	//it's there, but can we use it ?
-#ifdef E2_VFS
-	else if (e2_fs_access (&ddata, R_OK | X_OK E2_ERR_NONE()))
-#else
-	else if (e2_fs_access (local, R_OK | X_OK E2_ERR_NONE()))
-#endif
-	{	//nope
-		printd (WARN, "cannot access trash directory '%s'",
-			e2_cl_options.trash_dir);
-		F_FREE (local, e2_cl_options.trash_dir);
+
+	F_FREE (local, e2_cl_options.trash_dir);
+	if (success)
+	{
+		//cleanup, add trailer
+		e2_cl_options.trash_dir = e2_utils_path_clean (e2_cl_options.trash_dir);
+	}
+	else
+	{
 		g_free (e2_cl_options.trash_dir);
 		e2_cl_options.trash_dir = NULL;
-		return;
 	}
-	F_FREE (local, e2_cl_options.trash_dir);
-	//cleanup, add trailer
-	if (e2_cl_options.trash_dir != NULL)
-		e2_cl_options.trash_dir = e2_utils_path_clean (e2_cl_options.trash_dir);
 }
 /**
 @brief dump all current configuration data and recreate it, with window content updates as appropriate
@@ -186,7 +263,7 @@ Probably needs BGL closed upon arrival here.
 */
 void e2_option_refresh (gboolean reload, gboolean recreate)
 {
-	app.rebuild_enabled = FALSE;	//block recursion
+	e2_option_disable_config_checks (); //block recursion
 	//this is the one option that we want to keep ...
 	gboolean advanced = e2_option_bool_get ("advanced-config");
 	e2_keybinding_clean ();
@@ -199,6 +276,7 @@ void e2_option_refresh (gboolean reload, gboolean recreate)
 	e2_option_clear_data ();
 	//now re-create things, in the same order as at session-start
 	e2_option_default_register ();
+	e2_option_date_style (); //customise date-format option
 
 	if (reload)
 	{
@@ -235,38 +313,77 @@ void e2_option_refresh (gboolean reload, gboolean recreate)
 
 	e2_option_bool_set ("advanced-config", advanced);
 
-	app.rebuild_enabled = TRUE;	//unblock
+	e2_option_enable_config_checks (); //unblock
 }
 /**
 @brief disable checking for changed config file, if that is in effect
+As this is suspension, not "setup", the poll-timer is not handled here, nor is
+the backend monitoring if any
 
 @return
 */
 void e2_option_disable_config_checks (void)
 {
-	if (e2_option_bool_get ("auto-refresh-config") && app.timers[CONFIG_T] != 0)
-	{
-#ifdef E2_FAM
-		if (app.monitor_type != E2_MONITOR_DEFAULT)
-			e2_fs_FAM_cancel_monitor_config ();	//don't trigger a config file reload due to this write
-#endif
-		g_source_remove (app.timers[CONFIG_T]);
-		app.timers[CONFIG_T] = 0;
-	}
+	if (e2_option_bool_get ("auto-refresh-config"))
+		g_atomic_int_add (&cfg_refresh_refcount, 1);
 }
 /**
 @brief enable checking for changed config file, if that is in effect
+As this is end of suspension, the poll-timer is not handled here, nor is
+the backend monitoring if any
 
 @return
 */
 void e2_option_enable_config_checks (void)
 {
-	if (e2_option_bool_get ("auto-refresh-config") && app.timers[CONFIG_T] == 0)
+	if (e2_option_bool_get ("auto-refresh-config"))
 	{
-#ifdef E2_FAM
-		if (app.monitor_type != E2_MONITOR_DEFAULT)
-			e2_fs_FAM_monitor_config ();
+		if (g_atomic_int_exchange_and_add (&cfg_refresh_refcount, -1) < 1)
+			g_atomic_int_set (&cfg_refresh_refcount, 0);
+	}
+}
+/**
+@brief stop change-monitoring
+Stop the timer which polls for config file 'dirty' indicator, and if relevant
+stop the 'kernel-backend' process(es)
+No check here for "auto-refresh-config" option status
+
+@return
+*/
+void e2_option_stop_config_checks (void)
+{
+	if (app.timers[CONFIG_T] != 0)
+	{
+# ifdef E2_REFRESH_DEBUG
+		printd (DEBUG, "stop timer CONFIG_T");
+# endif
+		g_source_remove (app.timers[CONFIG_T]);
+		app.timers[CONFIG_T] = 0;
+	}
+#ifdef E2_FAM_KERNEL
+	e2_fs_FAM_cancel_monitor_config (); //stop backend
 #endif
+	g_atomic_int_set (&cfg_refresh_refcount, 0);
+}
+/**
+@brief start change-monitoring
+Start a timer which polls for config file 'dirty' indicator, and if relevant
+start the 'kernel-backend' process(es).
+No check here for "auto-refresh-config" option status
+The timer may later be stopped permanently if change-monitoring is abandoned,
+probably via a config dialog. But it won't be stopped merely if
+change-monitoring is disabled i.e. suspended
+
+@return
+*/
+void e2_option_start_config_checks (void)
+{
+	g_atomic_int_set (&cfg_refresh_refcount, 0);
+	if (app.timers[CONFIG_T] == 0)
+	{
+# ifdef E2_REFRESH_DEBUG
+		printd (DEBUG, "start timer CONFIG_T");
+# endif
 		app.timers[CONFIG_T] =
 #ifdef USE_GLIB2_14
 			g_timeout_add_seconds (E2_CONFIGCHECK_INTERVAL_S,
@@ -275,14 +392,15 @@ void e2_option_enable_config_checks (void)
 #endif
 			(GSourceFunc) e2_option_check_config_files, NULL);
 	}
+#ifdef E2_FAM_KERNEL
+	e2_fs_FAM_resume ();	//ensure it's polled there too
+	e2_fs_FAM_monitor_config (); //start backend
+#endif
 }
-
-gboolean cfgdirty;
-
 /**
 @brief check for and respond to changed config file
 
-This is the fn called by the check-config-files timer.
+This is called by the check-config-files timer, and may be called directly.
 With E2_FAM, update is normally detected and flagged as part of the filepane
 monitoring, as all change-data arrives via the same stream.
 Otherwise, here we check config file mtime against a cached value.
@@ -292,44 +410,100 @@ If update is indicated, re-create window after re-reading config data.
 */
 gboolean e2_option_check_config_files (gpointer user_data)
 {
+	static gboolean cfgdeferred = FALSE;
+	printd (DEBUG, "In e2_option_check_config_files()");
 #ifdef E2_FAM
-	if (app.monitor_type == E2_MONITOR_DEFAULT)
+	if (app.monitor_type != E2_MONITOR_DEFAULT)
 	{
+		if (cfgdeferred || g_atomic_int_get (&cfgdirty))
+		{
+			if (g_atomic_int_get (&cfg_refresh_refcount))
+			{
+				cfgdeferred = TRUE;
+				//don't clear the main flag, if set
+				printd (DEBUG, "config refresh deferred");
+			}
+			else
+			{
+				//force default settings
+				cfgdeferred = FALSE;
+				g_atomic_int_set (&cfgdirty, 1);
+				printd (DEBUG, "config refresh flag set");
+			}
+		}
+	}
+	else
+	{	//only polling available
 #endif
 		struct stat stat_buf;
 		gchar *cfg_file = g_build_filename (e2_cl_options.config_dir, default_config_file, NULL);
 		gchar *local = F_FILENAME_TO_LOCALE (cfg_file);
+		cfgdirty = 0;
 #ifdef E2_VFS
 		VPATH ddata = { local, NULL }; //config data always local
 		if (!e2_fs_stat (&ddata, &stat_buf E2_ERR_NONE())
 #else
 		if (!e2_fs_stat (local, &stat_buf E2_ERR_NONE())
 #endif
-			&& stat_buf.st_size >= 20)	//first line of config file should be "# emelFM2 (v x.y.z)"
-				cfgdirty = (stat_buf.st_mtime != app.config_mtime);
-		else
-			cfgdirty = FALSE; //no point in reloading if the file is gone
-		if (cfgdirty && app.rebuild_enabled)
-			app.config_mtime = stat_buf.st_mtime;
+			&& stat_buf.st_size >= 20	//first line of config file should be "# emelFM2 (v x.y.z)"
+			&& !S_ISDIR (stat_buf.st_mode))
+		{
+			if (stat_buf.st_mtime != app.config_mtime)
+			{
+				if (cfg_refresh_refcount == 0)
+				{
+					cfgdirty = 1;
+					cfgdeferred = FALSE;
+					app.config_mtime = stat_buf.st_mtime;
+				}
+				else
+					cfgdeferred = TRUE;
+			}
+		}
 		g_free (cfg_file);
 		F_FREE (local, cfg_file);
 #ifdef E2_FAM
 	}
+	if (!cfgdeferred && g_atomic_int_get (&cfgdirty))
+#else
+	if (!cfgdeferred && cfgdirty)
 #endif
-	if (cfgdirty && app.rebuild_enabled)
 	{
 		//CHECKME e2_option_refresh() probably needs BGL closed
+//		printd (DEBUG, "At e2_option_check_config_files 1, goto e2_option_refresh");
 		gdk_threads_enter ();
 		e2_option_refresh (TRUE, TRUE);
 		e2_output_print (&app.tab, _("Configuration data re-loaded"), NULL, TRUE, NULL);
 		gdk_threads_leave ();
-		cfgdirty = FALSE;
+		g_atomic_int_set (&cfgdirty, 0);
+		last_work_time = time (NULL);
+	}
+	else
+#ifdef E2_FAM
+		if (!cfgdeferred)
+#endif
+	{
+		if (time (NULL) >= last_work_time + QUIET_SECONDS)
+		{
+			printd (DEBUG, "Suspend operation of timer CONFIG_T");
+			g_source_remove (app.timers[CONFIG_T]);
+			app.timers[CONFIG_T] = 0;
+#ifdef E2_FAM_KERNEL
+			//backend shutdown too
+			e2_fs_FAM_suspend ();
+#endif
+			//arrange to restart upon activity
+			g_signal_connect (G_OBJECT (app.main_window), "event",
+				G_CALLBACK (_e2_option_repoll), GUINT_TO_POINTER (CONFIG_T));
+		}
 	}
 	return TRUE;
 }
 /**
 @brief register a config option
-This does not return if there's an allocation failure
+String-paramteters are not copied! Upstream needs to ensure continuing validity
+(careful with plugins!)
+This does not return if there's an allocation failure.
 @param type flag for the type of set that it is
 @param name name of the option
 @param group group the option belongs to, used in config dialog
@@ -347,10 +521,10 @@ E2_OptionSet *e2_option_register (E2_OptionType type, gchar *name, gchar *group,
 	if (set == NULL)
 	{
 #ifdef USE_GLIB2_10
-		set = (E2_OptionSet *) g_slice_alloc (sizeof (E2_OptionSet));
-//		set = ALLOCATE (E2_OptionSet);
+		set = (E2_OptionSet *) g_slice_alloc0 (sizeof (E2_OptionSet));
+//		set = ALLOCATE0 (E2_OptionSet);
 #else
-		set = ALLOCATE (E2_OptionSet);
+		set = ALLOCATE0 (E2_OptionSet);
 #endif
 #if (CHECKALLOCATEDFATAL)
 		CHECKALLOCATEDFATAL (set)
@@ -363,8 +537,8 @@ E2_OptionSet *e2_option_register (E2_OptionType type, gchar *name, gchar *group,
 			//non-freeable parameters set only once
 			set->type = type;
 			set->flags = flags;
-			set->hook_freezed = FALSE;
-			set->widget = NULL;
+//			set->hook_freezed = FALSE; zero'd struct
+//			set->widget = NULL;
 #if !(CHECKALLOCATEDFATAL)
 		}
 #endif
@@ -544,6 +718,31 @@ void e2_option_clear_data (void)
 		(GDestroyNotify) _e2_option_clean1);
 #endif
 }
+/**
+@brief setup date-format array value
+@return
+*/
+void e2_option_date_style (void)
+{
+	//replicate part of e2_option_sel_register()
+	E2_OptionSet *set = e2_option_get_simple ("date-string");
+	gchar **title = &set->ex.sel.def[5];
+	g_free (*title);
+
+	if (e2_option_bool_get ("advanced-config"))
+	{
+		extern const gchar *custom_date_label;
+		*title = g_strdup (custom_date_label);
+		set->ex.sel.def_count = 6;
+	}
+	else
+	{
+		if (e2_option_sel_get_direct (set) == 5)
+			e2_option_sel_set_direct (set, 4); //revert to localised
+		*title = NULL;
+		set->ex.sel.def_count = 5;
+	}
+}
 
 /*void e2_option_tree_stores_clear (void)
 {
@@ -666,6 +865,8 @@ void e2_option_file_write (const gchar *utfpath)
 
 		e2_fs_close_stream (f);
 
+		printd (DEBUG, "before renaming replacement config file, suspend monitoring");
+		e2_option_disable_config_checks ();	//suspend monitoring to prevent iterated reloads
 #ifdef E2_VFS
 		tdata.path = tempname;
 		tdata.spacedata = NULL;
@@ -681,6 +882,9 @@ void e2_option_file_write (const gchar *utfpath)
 		e2_fs_chmod (local, 0600 E2_ERR_NONE());
 #endif
 		gdk_threads_enter ();
+		printd (DEBUG, "after renaming replacement config file, resume monitoring");
+		e2_option_enable_config_checks ();	//resume monitoring
+
 		goto cleanup;
 	}
 error_handler:
@@ -889,10 +1093,20 @@ static gboolean _e2_option_config_file_read (gpointer *contents)
 #else
 		e2_fs_stat (local, &stat_buf E2_ERR_NONE());
 #endif
-		app.config_mtime = stat_buf.st_mtime;
+		if (stat_buf.st_size > 5)
+		{
+			app.config_mtime = stat_buf.st_mtime;
 //#endif
-		printd (DEBUG, "config file '%s' read", cfg_file);
-		retval = TRUE;
+			printd (DEBUG, "config file '%s' read", cfg_file);
+			retval = TRUE;
+		}
+		else
+		{
+			printd (WARN, "bad config file '%s'", cfg_file);
+			if (contents != NULL)
+				g_free (*contents);
+			retval = FALSE;
+		}
 	}
 	else
 	{
@@ -981,7 +1195,18 @@ static void _e2_option_clean1 (E2_OptionSet *set)
 			g_strfreev (set->ex.tree.def.tree_strings);
 			set->ex.tree.def.tree_strings = NULL;
 		}
-		e2_list_free_with_data (&set->ex.tree.columns);
+		GList *member;
+		for (member = set->ex.tree.columns; member != NULL; member = member->next)
+		{
+			E2_OptionTreeColumn *opt = member->data;
+			if (opt->flags & E2_OPTION_TREE_COL_FREENAME)
+				g_free (opt->label);
+			if (opt->flags & E2_OPTION_TREE_COL_FREESTRING)
+				g_free (opt->sdef);
+//			if (opt->data_cleaner != NULL)
+//				(opt->data_cleaner) (opt->visible_check_data);
+			DEALLOCATE (E2_OptionTreeColumn, opt);
+		}
 		g_object_unref (G_OBJECT (set->ex.tree.model));
 	}
 	//CHECKME set-related hooks are un-registered when config dialog is destroyed
@@ -1035,6 +1260,7 @@ writing */
 		lang = "C";
 	default_config_file = g_strconcat ("config-", lang, NULL);  //DO NOT TRANSLATE
 	options_array = g_ptr_array_new ();
+	//any non-constant key (option name) is cleared in _e2_option_clean1()
 	options_hash = g_hash_table_new_full (g_str_hash, g_str_equal, NULL,
 		(GDestroyNotify) _e2_option_clean1);
 	options_queue = g_hash_table_new_full (g_str_hash, g_str_equal, g_free,
@@ -1045,7 +1271,6 @@ writing */
 #ifndef E2_FAM
 	e2_fs_FAM_config_stamp ();	//set baseline time, for config refreshing
 #endif
-	app.rebuild_enabled = TRUE;	//set signal which allows config refreshes
 
 #ifdef E2_IMAGECACHE
 	e2_cache_icons_init ();
