@@ -1,6 +1,6 @@
-/* $Id: e2_option.h 1973 2009-10-11 07:49:48Z tpgww $
+/* $Id: e2_option.h 2344 2011-04-11 22:46:24Z tpgww $
 
-Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
+Copyright (C) 2003-2011 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -14,8 +14,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 #ifndef __E2_OPTION_H__
@@ -162,6 +161,7 @@ struct _E2_OptionSet
 typedef	union _E2_OptionSetupExtra
 {
 	gboolean exbool;
+	gchar *exstr;
 //	GdkColor excol;
 	struct
 	{
@@ -191,6 +191,8 @@ GHashTable *options_queue;	//for not-yet-registed options
 void e2_option_setup_labels (void);
 gboolean e2_option_set_config_dir (void);
 gboolean e2_option_check_config_files (gpointer user_data);
+void e2_option_stop_config_checks (void);
+void e2_option_start_config_checks (void);
 void e2_option_disable_config_checks (void);
 void e2_option_enable_config_checks (void);
 void e2_option_refresh (gboolean reload, gboolean recreate);
@@ -217,6 +219,7 @@ gboolean e2_option_file_read (void);
 void e2_option_file_write (const gchar *utfpath);
 
 //void e2_option_destroy_config ();
+void e2_option_date_style (void);
 
 #include "e2_option_bool.h"
 #include "e2_option_int.h"
