@@ -1,6 +1,6 @@
-/* $Id: e2_option__default.c 1902 2009-09-25 05:07:11Z tpgww $
+/* $Id: e2_option__default.c 2344 2011-04-11 22:46:24Z tpgww $
 
-Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
+Copyright (C) 2003-2011 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -14,8 +14,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 /**
@@ -39,6 +38,9 @@ The order of registration determines the order of the config dialog
 #include "e2_mousebinding.h"
 #endif
 
+const gchar *custom_date_label = N_("strftime() format provided below");
+gboolean custom_date_label_translated = FALSE;
+
 /**
 @brief setup and register all options, plus a few other initialisation tasks
 
@@ -149,6 +151,9 @@ void e2_option_default_register (void)
 		_("The delay (in milliseconds) after the mouse pointer leaves a menu item, before popping down an activated submenu"),
 		NULL, 400, 0, 10000,
 		E2_OPTION_FLAG_ADVANCED); //no rebuild
+    //option-values for this must conform to GtkCornerType enum, but for scrolled
+	//windows, the enum determines where the child is positioned, the opposite of
+	//where the bar(s) are positioned
 	const gchar *opt_scrollbar_position[] =
 		{_("bottom-right"), _("top-right"), _("bottom-left"), _("top-left"), NULL};
 	e2_option_sel_register ("scrollbar-position", group_name, _("scrollbar position"),
@@ -196,18 +201,24 @@ void e2_option_default_register (void)
 
 	group_name = g_strconcat(_C(33) ,":",_C(26),NULL);  //_("panes:miscellaneous"
 	e2_option_bool_register ("auto-refresh", group_name, _("auto refresh"),
-		_("This enables automatic checking for changes to the content of displayed directories. Otherwise you need to refresh manually"), NULL, TRUE,
+		_("Automatically check for and display file-list changes"), NULL, TRUE,
 		E2_OPTION_FLAG_ADVANCED | E2_OPTION_FLAG_FREEGROUP ); //no rebuild
 	e2_option_bool_register ("dir-line-focus-after-activate",
 		group_name, _("focus the pane after completing a directory entry"),
-		_("If activated, after return/enter is pressed in a directory line, the pane containing that dir line will become the active one"),
+		_("After a directory-line is activated, make the pane containing it the active one"),
 		NULL, TRUE,
 		E2_OPTION_FLAG_ADVANCED);	//no rebuild
-	e2_option_bool_register ("select-first-item",
-		group_name, _("select first item in newly-opened directories"),
-		_("If activated, when a directory is first displayed in a session, the first item there will be selected"),
+	e2_option_bool_register ("select-first-item", group_name,
+		_("select first item in newly-opened directories"),
+		_("Select the first item in a file-list when a directory is first displayed in a session"),
 		NULL, FALSE,
 		E2_OPTION_FLAG_ADVANCED);	//no rebuild
+	e2_option_bool_register ("cache-history", group_name,
+		_("persistent opened-directories history"),
+		_("Save opened-directories data for use in the next session"),
+		NULL, TRUE,
+		E2_OPTION_FLAG_ADVANCED);	//no rebuild, and if changed in-session,
+									//will not affect cacheing of that session
 
 	group_name = g_strconcat(_C(33) ,":",_C(39),NULL); //_("panes:style"
 	e2_option_bool_register ("panes-horizontal", group_name, _("use horizontal panes"),
@@ -238,14 +249,25 @@ void e2_option_default_register (void)
 		_("This is the font used for flle pane text"), "custom-list-font", "Sans 10",//_I( font name
 		E2_OPTION_FLAG_BASIC | E2_OPTION_FLAG_BUILDLISTS);
 
+	if (!custom_date_label_translated)
+	{
+		custom_date_label = _(custom_date_label);
+		custom_date_label_translated = TRUE;
+	}
+	//HACK - custom/last format-string is set to its real value in e2_option_date_style()
 	const gchar *opt_filelist_date_string[] = {_("Default: May 20 09:11"),
 		_("Standard: 20/05/04 09:11"), _("American: 05/20/04 09:11"),
-		"ISO8601: 2004-05-20T09:11", _("LC_TIME locale specified"), NULL};	//no need to translate ISO string
+		"ISO8601: 2004-05-20T09:11", _("LC_TIME locale specified"), custom_date_label, NULL};	//no need to translate ISO string
 	e2_option_sel_register ("date-string", group_name, _("date format"),
 		_("This determines the format of all dates displayed in the panes"),
 			NULL, 0, opt_filelist_date_string,
 			E2_OPTION_FLAG_BASIC | E2_OPTION_FLAG_BUILDLISTS);
 
+	e2_option_str_register ("custom-date-format", group_name, _("custom date/time format string:"),
+		_("Format string interpreted by strftime() to present file date/time"),
+			NULL, "%c",
+			E2_OPTION_FLAG_ADVANCED | E2_OPTION_FLAG_BUILDLISTS);
+
 	const gchar *opt_filelist_size_string[] = {_("condensed"), _("exact"), NULL};
 	e2_option_sel_register ("size-string", group_name, _("size format"),
 		_("Displayed item-sizes can be 'condensed' to show kB, MB where appropriate"),
