@@ -1,6 +1,6 @@
-/* $Id: e2_option_tree.c 1949 2009-10-01 13:26:52Z tpgww $
+/* $Id: e2_option_tree.c 2469 2012-03-16 00:19:24Z tpgww $
 
-Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
+Copyright (C) 2003-2012 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -14,8 +14,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 /**
@@ -437,8 +436,8 @@ static gboolean _e2_option_tree_btnchange_check (E2_BtnChangeData *data)
 		}
 	}
 	//CHECKME cancel the original btn press cb on the renderer
-//	g_signal_handlers_disconnect_by_func (
-//		data->entry, _e2_option_tree_btnchange_cb, data);
+//	g_signal_handlers_disconnect_by_func ((gpointer)data->entry,
+//		_e2_option_tree_btnchange_cb, data);
 	g_free (data->path_string);
 	g_free (data->newtext);
 	DEALLOCATE (BtnChangeData, data);
@@ -627,13 +626,13 @@ static void _e2_option_tree_btn_edit_start_cb (GtkCellRenderer *renderer,
 }
 #endif //def E2_MOUSECUSTOM
 /**
-@brief timer callback to check for and warn about duplicate keybinding
-This is outside other cell-related cb's to prevent crash
-@param data pointer to keyvhange data struct
+@brief check for and warn about a duplicate keybinding
+Called only from within callbacks
+@param data pointer to keychange data struct
 
-@return FALSE always to cancel timer
+@return
 */
-static gboolean _e2_option_tree_keychange_check (E2_KeyChangeData *data)
+static void _e2_option_tree_keychange_check (E2_KeyChangeData *data)
 {
 	GtkTreeView *treeview = GTK_TREE_VIEW (data->set->widget);
 	GtkTreeSelection *sel = gtk_tree_view_get_selection (treeview);
@@ -668,9 +667,9 @@ static gboolean _e2_option_tree_keychange_check (E2_KeyChangeData *data)
 				else
 					prompt = g_markup_printf_escaped (fmt, keyname, action);
 
-				gdk_threads_enter ();
+//in cb			gdk_threads_enter ();
 				result = e2_dialog_warning (prompt, NULL);
-				gdk_threads_leave ();
+//				gdk_threads_leave ();
 				g_free (prompt);
 				g_free (storedkey);
 				g_free (action);
@@ -692,7 +691,6 @@ static gboolean _e2_option_tree_keychange_check (E2_KeyChangeData *data)
 				_e2_option_tree_key_edit_start_cb, data->set);
 		}
 	}
-	return FALSE;
 }
 /**
 @brief interpret key name determined from key press event @a event
@@ -702,7 +700,7 @@ Stored value is not changed until after check in timer callback
 @param event pointer to event data struct for the pressed key
 @param data pointer to data needed to update the cell the renderer
 
-@return TRUE always, to prevent any other key-press handler
+@return TRUE always, to prevent any other key-press-event handler
 */
 static gboolean _e2_option_tree_keychange_cb (GtkEntry *entry,
 	GdkEventKey *event, E2_KeyChangeData *data)
@@ -716,8 +714,8 @@ static gboolean _e2_option_tree_keychange_cb (GtkEntry *entry,
 		return FALSE;	//keep waiting
 
 	//finished with this cb now that we have a real key
-	g_signal_handlers_disconnect_by_func (
-		entry, _e2_option_tree_keychange_cb, data);
+	g_signal_handlers_disconnect_by_func ((gpointer)entry,
+		_e2_option_tree_keychange_cb, data);
 
 	guint keyval = event->keyval;
 	GdkModifierType mask = event->state;
@@ -741,34 +739,28 @@ static gboolean _e2_option_tree_keychange_cb (GtkEntry *entry,
 	if (keyval < 0xF000 || keyval > 0xFFFF)
 	{
 		if ((mask & E2_MODIFIER_MASK) == GDK_SHIFT_MASK)
-		{	//control alt etc not pressed
+		{	//shft only, no ctrl alt etc
 			if (mask & GDK_LOCK_MASK)
-				//lcase already, we don't want <Shift>
-				keyname = gdk_keyval_name (keyval);
-			else
-/*			{	//toggle case
-				if (gdk_keyval_is_lower (keyval))
-					keyname = gdk_keyval_name (gdk_keyval_to_upper (keyval));
-				else
-					keyname = gdk_keyval_name (gdk_keyval_to_lower (keyval));
-			} */
 			{
-				if (gdk_keyval_is_lower (keyval))
-					keyval = gdk_keyval_to_upper (keyval);
-				keyname = gdk_keyval_name (keyval);
+				//lcase already, we don't want <Shift>
 			}
-			keyname = g_strdup (keyname);
+			else
+			{	//nocaps + shift >> ucase
+				keyval = gdk_keyval_to_upper (keyval);
+			}
+			keyname = g_strdup (gdk_keyval_name (keyval));
 		}
 		else if ((mask & GDK_LOCK_MASK) && (mask & E2_MODIFIER_MASK) == 0)
 			//only lock is active
-			keyname = g_strdup (gdk_keyval_name (gdk_keyval_to_upper (keyval)));
+			keyname = g_strdup (gdk_keyval_name (gdk_keyval_to_upper (keyval))); //3
 		else
-			keyname = gtk_accelerator_name (keyval, mask);
+			keyname = gtk_accelerator_name (keyval, mask); //e.g. <Shift><Control>g for keyval 0x67 4
 	}
 	else
 		keyname = gtk_accelerator_name (keyval, mask);
 
 	data->newtext = keyname;
+	//NOTE: keyval changed as appropriate in _e2_keybinding_sync_one() ?
 	_e2_option_tree_keychange_check (data);
 	//CHECKME also unblock bindings if the user cancels, changes focus etc
 	e2_keybinding_unblock (config_dialog, NULL);
@@ -780,12 +772,12 @@ static gboolean _e2_option_tree_keychange_cb (GtkEntry *entry,
 
 typedef struct _E2_KeyBlock
 {
-	gboolean blocked;
+	gint blocked; //flag for preventing re-entrance
 	E2_KeyChangeData *data;
 } E2_KeyBlock;
 //static data used cuz only one config dialog can exist
 //Otherwise need to send variable data to callback via each relevant button
-E2_KeyBlock blocknow = {FALSE, NULL};
+E2_KeyBlock blocknow = {0, NULL};
 
 #define KEYCHANGEKEY "_e2-binding-key_"
 /**
@@ -806,7 +798,7 @@ static gboolean _e2_option_tree_mnemonic_cb (GtkWidget *button, gboolean group_c
 {
 	printd (DEBUG, "_e2_option_tree_mnemonic_cb");
 //	E2_KeyBlock *bdata = g_object_get_data (G_OBJECT(button), KEYCHANGEKEY);
-	if (bdata != NULL && bdata->blocked && bdata->data != NULL)
+	if (bdata != NULL && bdata->data != NULL && g_atomic_int_get (&bdata->blocked) != 0)
 	{
 		GList *child, *all = gtk_widget_list_mnemonic_labels (button);
 		for (child = all; child != NULL; child = child->next)
@@ -822,7 +814,7 @@ static gboolean _e2_option_tree_mnemonic_cb (GtkWidget *button, gboolean group_c
 				//CHECKME also unblock bindings if the user cancels, changes focus etc
 				e2_keybinding_unblock (config_dialog, NULL);
 				//unblock all button-mnemonics
-				bdata->blocked = FALSE;
+				g_atomic_int_set (&bdata->blocked, 0);
 				bdata->data = NULL; //will have been cleared downstream
 				break;
 			}
@@ -831,7 +823,7 @@ static gboolean _e2_option_tree_mnemonic_cb (GtkWidget *button, gboolean group_c
 		return TRUE;
 	}
 	else if (bdata != NULL)
-		return bdata->blocked;
+		return (g_atomic_int_get (&bdata->blocked) != 0);
 	else
 		return FALSE;
 }
@@ -871,7 +863,7 @@ static void _e2_option_tree_set_mnemonics_data (GtkWidget **buttons, guint count
 static void _e2_option_tree_unblock_mnemonics (void)
 {
 //	printd (DEBUG, "_e2_option_tree_unblock_mnemonics");
-	blocknow.blocked = FALSE;
+	g_atomic_int_set (&blocknow.blocked, 0);
 	if (blocknow.data != NULL)
 	{
 		g_free (blocknow.data->path_string);
@@ -913,7 +905,7 @@ static void _e2_option_tree_key_edit_start_cb (GtkCellRenderer *renderer,
 		//prevent operation of any aberrant current binding
 		e2_keybinding_block (config_dialog, NULL);
 		//or of any dialog-button mnemonic
-		blocknow.blocked = TRUE;
+		g_atomic_int_set (&blocknow.blocked, 1);
 		blocknow.data = data;
 /* if static data not viable
 		//for main dialog buttons
@@ -1125,7 +1117,7 @@ static gboolean _e2_option_tree_popup_menu_cb (GtkWidget *treeview,
 	//set appropriate item sensitivities
 	e2_option_tree_menu_set_sensitive (menu, treeview);
 
-	gint event_time = gtk_get_current_event_time ();
+	guint32 event_time = gtk_get_current_event_time ();
 	gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
 		(GtkMenuPositionFunc) e2_confdlg_menu_set_position,
 		treeview, 0, event_time);	//button code = 0 as this was a menu-button press
@@ -1178,7 +1170,7 @@ static gboolean _e2_option_tree_button_press_cb (GtkWidget *treeview,
 		gtk_tree_model_get_iter (set->ex.tree.model, &iter, path);
 		gtk_tree_model_get (set->ex.tree.model, &iter, column, &icon, -1);
 		gchar *dialog_name = g_strdup_printf (_("select icon for %s"), set->name);
-		GtkWidget *dialog = e2_sid_create (config_dialog, dialog_name, icon);
+		GtkWidget *dialog = e2_sid_create (config_dialog, dialog_name, icon, event);
 		g_free (icon);
 		g_free (dialog_name);
 		if (dialog != NULL)
@@ -1595,6 +1587,8 @@ void e2_option_tree_add_widget (GtkWidget *parent, gboolean single, GtkWidget *b
 		gboolean editable = !(opt->flags & E2_OPTION_TREE_COL_NOT_EDITABLE);
 		switch (opt->type)
 		{
+//			case E2_OPTION_TREE_TYPE_INT:
+//				break;
 			case E2_OPTION_TREE_TYPE_BOOL:
 				renderer = gtk_cell_renderer_toggle_new ();
 				g_signal_connect (G_OBJECT (renderer), "toggled",
@@ -1698,9 +1692,12 @@ void e2_option_tree_add_widget (GtkWidget *parent, gboolean single, GtkWidget *b
 				gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
 				break;
 			default:
+				printd (WARN, "don't know how to render column %d of %s", j, set->name);
+//			case E2_OPTION_TREE_TYPE_HIDDENINT:
+			case E2_OPTION_TREE_TYPE_HIDDENBOOL:
+			case E2_OPTION_TREE_TYPE_HIDDENSTR:
 				renderer = NULL;
 				column = NULL;  //warning prevention
-				printd (WARN, "don't know how to render column %d of %s", j, set->name);
 				break;
 		}
 		if (renderer != NULL)
@@ -1840,9 +1837,8 @@ void e2_option_tree_add_widget (GtkWidget *parent, gboolean single, GtkWidget *b
 }
 /**
 @brief begin registration of a tree option
-Complete registraion involves this func, then e2_option_tree_add_column()
-for each column in the treestore, tnen e2_option_tree_create_store() to
-finish things
+Complete registration involves this func, then e2_option_tree_add_column() for
+each column in the treestore, tnen e2_option_tree_create_store() to finish things
 @param name name of the option, generally a constant string but sometimes runtime-created
 @param group group the option belongs to, used in config dialog, can be a r-t string FREEME or a _()
 @param desc textual description of the option used in config dialog, a r-t _() string
@@ -1874,25 +1870,25 @@ E2_OptionSet *e2_option_tree_register (gchar *name, gchar *group, gchar *desc,
 }
 /**
 @brief setup data for a column of a tree-set
-This establishes some parameters for regular use, and others for
-config-dialog use.
+This establishes some parameters for regular use, and others for config-dialog
+use. String-parameters (@a name, @a sdef) are NOT COPIED
 @param set pointer to data struct for the tree-set
 @param name column label string
 @param type indicator of the type of data in the column
-@param idef default value if the column has a bool value
-@param sdef default value if the column has a non-bool value
+@param idef default value expessed as an integer (0 for string-options)
+@param sdef default value expressed as a string
 @param flags flags relating to data editability, cleanup
 @param visible_check_func function to check whether a cell in the column is displayed in a config dialog
 @param visible_check_data data to provide to @a visible_check_func
 
 @return
 */
-void e2_option_tree_add_column (E2_OptionSet *set, const gchar *name,
-	E2_OptionTreeType type, gint idef, const gchar *sdef,
+void e2_option_tree_add_column (E2_OptionSet *set, gchar *name,
+	E2_OptionTreeType type, gint idef, gchar *sdef,
 	E2_OptionTreeColFlags flags,
 	gpointer visible_check_func, gpointer visible_check_data)
 {
-	E2_OptionTreeColumn *col = ALLOCATE (E2_OptionTreeColumn); //FIXME no deallocation
+	E2_OptionTreeColumn *col = ALLOCATE (E2_OptionTreeColumn);
 	CHECKALLOCATEDFATAL (col);
 	set->ex.tree.columns = g_list_append (set->ex.tree.columns, col);
 	col->label = name;
@@ -1902,6 +1898,7 @@ void e2_option_tree_add_column (E2_OptionSet *set, const gchar *name,
 	col->flags = flags;
 	col->visible_check_func = visible_check_func;
 	col->visible_check_data = visible_check_data;
+//	col->data_cleaner = ?;
 }
 /**
 @brief create a treestore for the data of a tree-set
@@ -1924,10 +1921,14 @@ void e2_option_tree_create_store (E2_OptionSet *set)
 		E2_OptionTreeColumn *tcol = member->data;
 		switch (tcol->type)
 		{
+//			case E2_OPTION_TREE_TYPE_INT:
+//			case E2_OPTION_TREE_TYPE_HIDDENINT:
 			case E2_OPTION_TREE_TYPE_BOOL:
+			case E2_OPTION_TREE_TYPE_HIDDENBOOL:
 				t[i] = G_TYPE_INT;
 				break;
 			case E2_OPTION_TREE_TYPE_STR:
+			case E2_OPTION_TREE_TYPE_HIDDENSTR:
 			case E2_OPTION_TREE_TYPE_ICON:
 			case E2_OPTION_TREE_TYPE_SEL:
 			case E2_OPTION_TREE_TYPE_KEY:
@@ -1978,7 +1979,11 @@ gchar *e2_option_tree_row_write_to_string (E2_OptionSet *set,
 		E2_OptionTreeColumn *opt = column->data;
 		switch (opt->type)
 		{
+//			case E2_OPTION_TREE_TYPE_INT:
+//			case E2_OPTION_TREE_TYPE_HIDDENINT:
+//				break;
 			case E2_OPTION_TREE_TYPE_BOOL:
+			case E2_OPTION_TREE_TYPE_HIDDENBOOL:
 			{
 				gboolean int_data;
 				gtk_tree_model_get (set->ex.tree.model, iter, j, &int_data, -1);
@@ -1989,6 +1994,7 @@ gchar *e2_option_tree_row_write_to_string (E2_OptionSet *set,
 			}
 			break;
 			case E2_OPTION_TREE_TYPE_STR:
+			case E2_OPTION_TREE_TYPE_HIDDENSTR:
 			case E2_OPTION_TREE_TYPE_ICON:
 			case E2_OPTION_TREE_TYPE_SEL:
 			case E2_OPTION_TREE_TYPE_KEY:
@@ -2358,11 +2364,15 @@ static void _e2_option_tree_add_default (E2_OptionSet *set, GtkTreeIter *iter, G
 		E2_OptionTreeColumn *opt = column->data;
 		switch (opt->type)
 		{
+//			case E2_OPTION_TREE_TYPE_INT:
+//			case E2_OPTION_TREE_TYPE_HIDDENINT:
 			case E2_OPTION_TREE_TYPE_BOOL:
+			case E2_OPTION_TREE_TYPE_HIDDENBOOL:
 				arg[i] = GINT_TO_POINTER (opt->idef);
 				break;
 			case E2_OPTION_TREE_TYPE_SEL:
 			case E2_OPTION_TREE_TYPE_STR:
+			case E2_OPTION_TREE_TYPE_HIDDENSTR:
 			case E2_OPTION_TREE_TYPE_ICON:
 			case E2_OPTION_TREE_TYPE_KEY:
 #ifdef E2_MOUSECUSTOM
@@ -2687,12 +2697,14 @@ void *_e2_option_tree_get_void_simple (E2_OptionTreeColumn *col, gchar *option)
 	switch (col->type)
 	{
 		case E2_OPTION_TREE_TYPE_BOOL:
+		case E2_OPTION_TREE_TYPE_HIDDENBOOL:
 			if (!strcmp (option, "true"))
 				return (void *) TRUE;
 			else
 				return (void *) FALSE;
 			break;
 		case E2_OPTION_TREE_TYPE_STR:
+		case E2_OPTION_TREE_TYPE_HIDDENSTR:
 		case E2_OPTION_TREE_TYPE_ICON:
 		case E2_OPTION_TREE_TYPE_SEL:
 		case E2_OPTION_TREE_TYPE_KEY:
@@ -2709,4 +2721,3 @@ void *_e2_option_tree_get_void_simple (E2_OptionTreeColumn *col, gchar *option)
 	}
 	return NULL;
 }
-
