@@ -1,6 +1,6 @@
-/* $Id: e2_config_dialog.c 2018 2009-11-19 20:50:44Z tpgww $
+/* $Id: e2_config_dialog.c 2469 2012-03-16 00:19:24Z tpgww $
 
-Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
+Copyright (C) 2003-2012 tooar <tooar@emelfm2.net>
 Portions copyright (C) 1999 Michael Clark
 
 This file is part of emelFM2.
@@ -15,8 +15,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 #include "emelfm2.h"
@@ -45,6 +44,7 @@ typedef struct _E2_TreeCfgData
 //common vars we need in several functions
 GtkWidget *config_dialog = NULL;	//this is accessed by other files
 static gchar *page_last_name = NULL;	//the group name for the notebook page last opened
+extern time_t last_work_time;
 
 // =========== functions supporting single-page config dialogs ===============
 
@@ -136,12 +136,12 @@ static void _e2_cd1_response_cb (GtkDialog *dialog, gint response,
 			_e2_cd1_cancel (rt, FALSE);
 			//perform post-dialog data processing, via caller-specified 'apply'
 			apply_new ();
-			e2_option_disable_config_checks ();
+//			e2_option_disable_config_checks (); in write func
 			//update stored config data
 			e2_option_file_write (NULL);
 			//update stamp, to prevent auto-refresh
 			e2_fs_touch_config_dir ();
-			e2_option_enable_config_checks ();
+//			e2_option_enable_config_checks ();
 		}
 			break;
 		case E2_RESPONSE_MORE:
@@ -180,7 +180,7 @@ E2_SpecificConfDialogRuntime *e2_config_dialog_single (gchar *set_name,
 	config_dialog =
 	rt->dialog = e2_dialog_create (NULL, NULL, _("configuration"),
 		(ResponseFunc)_e2_cd1_response_cb, rt);
-	gtk_widget_set_size_request (rt->dialog, 600, 400);
+	gtk_widget_set_size_request (rt->dialog, 150, 50);
 	GtkWidget *dialog_vbox =
 #ifdef USE_GTK2_14
 		gtk_dialog_get_content_area (GTK_DIALOG (rt->dialog));
@@ -216,6 +216,7 @@ E2_SpecificConfDialogRuntime *e2_config_dialog_single (gchar *set_name,
 	discard_btn = E2_BUTTON_DISCARD;
 	discard_btn.showflags |= E2_BTN_DEFAULT;
 
+	gtk_window_set_default_size (GTK_WINDOW (rt->dialog), 450, 350);
 	E2_BUTTON_APPLY.showflags &= ~E2_BTN_DEFAULT;	//CHECKME local copy ?
 	e2_dialog_show (rt->dialog, app.main_window, 0,
 		&E2_BUTTON_MORE, &discard_btn, &E2_BUTTON_APPLY, NULL);
@@ -408,13 +409,15 @@ static void _e2_confdlg_clean (void)
 	g_hash_table_destroy (tree_view_buffer_hash);
 }
 /**
+
 @brief process commit or apply button-click
 
 @param close TRUE if 'commit' button was pressed, FALSE if 'apply'
+@param rt pointer to runtime data for the current dialog
 
 @return
 */
-static gboolean _e2_confdlg_apply (gboolean close)
+static gboolean _e2_confdlg_apply (gboolean close, E2_ConfigDialogRuntime *rt)
 {
 	gboolean bchoice;
 	gint ichoice;
@@ -425,7 +428,7 @@ static gboolean _e2_confdlg_apply (gboolean close)
 	E2_OptionSet *set;
 
 	printd (DEBUG, "config dialog callback: %s", (close) ? "commit" : "apply");
-
+	//prevent interruptions - disable all refreshing
 	e2_filelist_disable_refresh ();
 	e2_option_disable_config_checks ();
 
@@ -659,8 +662,13 @@ static gboolean _e2_confdlg_apply (gboolean close)
 		if (buildflags & E2_OPTION_FLAG_BUILDLISTS)
 		{
 			e2_fileview_set_font ();	//FIXME do this only if needed
-			//filelists will be refreshed only if dirty, so cd instead
+			//filelists will be refreshed only if dirty, so cd instead, but we
+			//must trick the cd manager into thinking that it's a real change
+			gchar *e = curr_view->dir + strlen (curr_view->dir) - 1;
+			*e = '\0'; //strip trailing /
 			e2_pane_change_dir (curr_pane, curr_pane->path);
+			e = other_view->dir + strlen (other_view->dir) - 1;
+			*e = '\0'; //strip trailing /
 			e2_pane_change_dir (other_pane, other_pane->path);
 		}
 	}
@@ -693,33 +701,44 @@ static gboolean _e2_confdlg_apply (gboolean close)
 		}
 */
 	}
-
 	//set filelist monitoring according to current option
 	if (e2_option_bool_get ("auto-refresh"))
 	{
-		if (app.timers[DIRTYCHECK_T] == 0)
+		if (!rt->refresh_files)
 		{
-			e2_filelist_start_refresh_polling ();
-#ifdef E2_FAM
-			//e2_fs_FAM_connect ();
-			//CHECKME what things to start monitoring ?
-			//e2_fs_FAM_change (gchar *olddir, E2_PaneRuntime *rt);
-#endif
+			//option was not set before
+			last_work_time = time (NULL);
+			e2_filelist_start_refresh_checks ();
+			rt->refresh_files = TRUE;	//in case we're continuing
 		}
+		else
+			e2_filelist_enable_refresh ();
 	}
-	else if (app.timers[DIRTYCHECK_T] != 0)
+	else if (rt->refresh_files)
 	{
-		g_source_remove (app.timers[DIRTYCHECK_T]);
-		app.timers[DIRTYCHECK_T] = 0;
-#ifdef E2_FAM
-		//FIXME don't want FAM reports to pile up due to lack of polling
-		//e2_fs_FAM_cancel_monitor_dir (gchar *path)
-		//e2_fs_FAM_clean_reports (gchar *path)
-#endif
+		//option was set before
+		e2_filelist_stop_refresh_checks ();
+		rt->refresh_files = FALSE;	//in case we're continuing
+	}
+	//after the config file update, set config monitoring according to current option
+	if (e2_option_bool_get ("auto-refresh-config"))
+	{
+		if (!rt->refresh_cfg)
+		{
+			//option was not set before
+			last_work_time = time (NULL);
+			e2_option_start_config_checks ();
+			rt->refresh_cfg = TRUE;	//in case we're continuing
+		}
+		else
+			e2_option_enable_config_checks ();
+	}
+	else if (rt->refresh_cfg)
+	{
+		//option was set before
+		e2_option_stop_config_checks ();
+		rt->refresh_cfg = FALSE;	//in case we're continuing
 	}
-	//after config file update, set config monitoring according to current option
-	e2_option_enable_config_checks ();
-	e2_filelist_enable_refresh ();	//clear settings changed at start of func
 
 	return TRUE;
 }
@@ -757,11 +776,11 @@ static void _e2_confdlg_response_cb (GtkDialog *dialog, gint response,
 	switch (response)
 	{
 		case E2_RESPONSE_APPLY:
-			_e2_confdlg_apply (TRUE);	//does some cleanup
+			_e2_confdlg_apply (TRUE, rt);	//does some cleanup
 			DEALLOCATE (E2_ConfigDialogRuntime, rt);//CHECKME more cleanup ?
 			break;
 		case E2_RESPONSE_MORE:
-			_e2_confdlg_apply (FALSE);
+			_e2_confdlg_apply (FALSE, rt);
 			break;
 		case E2_RESPONSE_USER1:	//revert to default options
 		{
@@ -771,6 +790,7 @@ static void _e2_confdlg_response_cb (GtkDialog *dialog, gint response,
 			{
 				_e2_confdlg_clean ();	//MUST kill dialog before clearing option data
 //				app.keytrans = TRUE;	//translate the re-created keybindings, later on
+				printd (DEBUG, "At cfgdlg1, goto e2_option_refresh");
 				e2_option_refresh (FALSE, TRUE);
 				DEALLOCATE (E2_ConfigDialogRuntime, rt);
 				WAIT_FOR_EVENTS
@@ -783,6 +803,7 @@ static void _e2_confdlg_response_cb (GtkDialog *dialog, gint response,
 			e2_option_bool_toggle ("advanced-config");
 			_e2_confdlg_clean ();
 			DEALLOCATE (E2_ConfigDialogRuntime, rt);
+			e2_option_date_style (); //this option depends on advanced/basic
 			e2_config_dialog_create (NULL);
 			break;
 		default:
@@ -818,8 +839,8 @@ gboolean e2_confdlg_key_press_cb (GtkWidget *widget, GdkEventKey *event,
 	return FALSE;
 } */
 
-//this flag is ok because only 1 config dialog, and only 1 edit, at any time
-static gboolean cancel_blocked = FALSE;
+//this static flag is ok because only 1 config dialog, and only 1 edit, at any time
+static gint cancel_blocked = 0;
 /**
 @brief allow <Esc> press to cancel editing of the entry text without closing the dialog
 
@@ -832,11 +853,11 @@ static gboolean cancel_blocked = FALSE;
 static gboolean _e2_confdlg_key_press_cb (GtkWidget *widget, GdkEventKey *event,
 	E2_OptionSet *set)
 {
-	if (!cancel_blocked)
+	if (g_atomic_int_get (&cancel_blocked) == 0)
 	{
 		g_signal_handlers_block_by_func (G_OBJECT (config_dialog),
 			e2_dialog_key_neg_cb, config_dialog);
-		cancel_blocked = TRUE;
+		g_atomic_int_set (&cancel_blocked, 1);
 	}
 	else if (event->keyval == GDK_Escape
 		  || event->keyval == GDK_Return
@@ -852,7 +873,7 @@ static gboolean _e2_confdlg_key_press_cb (GtkWidget *widget, GdkEventKey *event,
 		//revert <Esc> key handling when cell editing is finsished
 		g_signal_handlers_unblock_by_func (G_OBJECT (config_dialog),
 			e2_dialog_key_neg_cb, config_dialog);
-		cancel_blocked = FALSE;
+		g_atomic_int_set (&cancel_blocked, 0);
 		return (event->keyval == GDK_Escape);
 	}
 	return FALSE;
@@ -871,11 +892,11 @@ This is a callback for entry "focus-out-event",
 static gboolean _e2_confdlg_focus_out_cb (GtkWidget *widget, GdkEventFocus *event,
 	gpointer data)
 {
-	if (cancel_blocked)
+	if (g_atomic_int_get (&cancel_blocked) != 0)
 	{
 		g_signal_handlers_unblock_by_func (G_OBJECT (config_dialog),
 			e2_dialog_key_neg_cb, config_dialog);
-		cancel_blocked = FALSE;
+		g_atomic_int_set (&cancel_blocked, 0);
 	}
 	return FALSE;
 }
@@ -912,7 +933,7 @@ static gboolean _e2_confdlg_button_press_cb (GtkWidget *treeview,
 */
 static gboolean _e2p_confdlg_popup_menu_cb (GtkWidget *treeview, gpointer data)
 {
-	gint event_time = gtk_get_current_event_time ();
+	guint32 event_time = gtk_get_current_event_time ();
 	_e2_confdlg_show_context_menu (treeview, 0, event_time, data);
 	return TRUE;
 }
@@ -1124,13 +1145,15 @@ static void _e2_confdlg_font_select_cb (GtkButton *button, E2_OptionSet *set)
 	{
 		//CHECKME the normal dialog properties like close on Esc press ?
 		dialog = GTK_FONT_SELECTION_DIALOG (gtk_font_selection_dialog_new (title));
+#ifndef USE_GTK3_0
 		gtk_widget_destroy (
-#ifdef USE_GTK2_14
+# ifdef USE_GTK2_14
 			gtk_font_selection_dialog_get_apply_button (dialog)
-#else
+# else
 			dialog->apply_button
-#endif
+# endif
 		);
+#endif
 		g_object_set_data (
 #ifdef USE_GTK2_14
 			G_OBJECT (gtk_font_selection_dialog_get_ok_button (dialog)),
@@ -1396,13 +1419,13 @@ static void _e2_confdlg_color_select_cb (GtkButton *button, E2_OptionSet *set)
 	{
 		//CHECKME close on Esc press ?
 		dialog = GTK_COLOR_SELECTION_DIALOG (gtk_color_selection_dialog_new (title));
-		gtk_widget_destroy (
-#ifdef USE_GTK2_99
-			gtk_color_selection_dialog_get_help_button (dialog)
+#ifdef USE_GTK2_20
+		GtkWidget *btn;
+		g_object_get (G_OBJECT (dialog), "help-button", &btn, NULL);
+		gtk_widget_destroy (btn);
 #else
-			dialog->help_button
+		gtk_widget_destroy (dialog->help_button);
 #endif
-		);
 
 		GdkColor color;
 		gchar *color_str = gtk_editable_get_chars (GTK_EDITABLE (set->widget), 0, -1);
@@ -1415,19 +1438,21 @@ static void _e2_confdlg_color_select_cb (GtkButton *button, E2_OptionSet *set)
 #endif
 			&color);
 		g_free (color_str);
-		g_signal_connect (
-#ifdef USE_GTK2_99
-			G_OBJECT (gtk_color_selection_dialog_get_ok_button (dialog)),
+
+#ifdef USE_GTK2_20
+		g_object_get (G_OBJECT (dialog), "ok-button", &btn, NULL);
+		g_signal_connect (G_OBJECT (btn),
 #else
-			G_OBJECT (dialog->ok_button),
+		g_signal_connect (G_OBJECT (dialog->ok_button),
 #endif
 			"clicked",
 			G_CALLBACK (_e2_confdlg_color_select_ok_cb), set->widget);
-		g_signal_connect (
-#ifdef USE_GTK2_99
-			G_OBJECT (gtk_color_selection_dialog_get_cancel_button (dialog)),
+
+#ifdef USE_GTK2_20
+		g_object_get (G_OBJECT (dialog), "cancel-button", &btn, NULL);
+		g_signal_connect (G_OBJECT (btn),
 #else
-			G_OBJECT (dialog->cancel_button),
+		g_signal_connect (G_OBJECT (dialog->cancel_button),
 #endif
 			"clicked",
 			G_CALLBACK (_e2_confdlg_color_select_cancel_cb), dialog);
@@ -1650,7 +1675,7 @@ static GtkWidget *_e2_confdlg_get_page (E2_OptionSet *set, E2_ConfigDialogRuntim
 }
 /**
 @brief create and show configuration dialog
-Expects BGL closed upon arrival here
+Expects BGL closed
 @param page name of page to show when dialog is started, or "" or NULL to use the last-used page
 
 @return
@@ -1677,20 +1702,22 @@ void e2_config_dialog_create (gchar *page)
 
 	e2_option_connect (config_dialog, FALSE);
 
+	GtkWidget *dialog_vbox =
+#ifdef USE_GTK2_14
+		gtk_dialog_get_content_area (GTK_DIALOG (config_dialog));
+#else
+		GTK_DIALOG (config_dialog)->vbox;
+#endif
+	gtk_container_set_border_width (GTK_CONTAINER (dialog_vbox), E2_PADDING);
+
 	//the main hpane - contains the categories treeview and the frame
 	GtkWidget *hpane = gtk_hpaned_new ();
-	gtk_container_add (GTK_CONTAINER (
-#ifdef USE_GTK2_14
-		gtk_dialog_get_content_area (GTK_DIALOG (config_dialog))
-#else
-		GTK_DIALOG (config_dialog)->vbox
-#endif
-		), hpane);
+	gtk_box_pack_start (GTK_BOX (dialog_vbox), hpane, TRUE, TRUE, 0);
 
 	//scrolled window for the categories treeview
 	GtkWidget *sw = e2_widget_get_sw (GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC,
 		GTK_SHADOW_IN);
-	gtk_widget_set_size_request (sw, 200, 0);
+	gtk_widget_set_size_request (sw, 150, 50);
 	gtk_paned_pack1 (GTK_PANED (hpane), sw, FALSE, TRUE);
 
 	//categories treestore has single node, 2 columns
@@ -1702,19 +1729,22 @@ void e2_config_dialog_create (gchar *page)
 		gtk_tree_view_new_with_model (GTK_TREE_MODEL (rt->store));
 	g_object_unref (rt->store);
 	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (catsview), TRUE);
+//#ifdef USE_GTK3_0
+//CHECKME	gtk_scrollable_set_vscroll_policy (GTK_SCROLLABLE (catsview), GTK_SCROLL_NATURAL);
+//#endif
 //#ifdef USE_GTK2_10
 //	gtk_tree_view_set_enable_tree_lines (GTK_TREE_VIEW (catstore), TRUE);
 //#endif
 	GtkTreeSelection *selection =
-	gtk_tree_view_get_selection (GTK_TREE_VIEW (catsview));
+		gtk_tree_view_get_selection ((GtkTreeView*)catsview);
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
 
 	GtkCellRenderer *renderer = gtk_cell_renderer_text_new ();
-	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW
-		(catsview), -1, _("Categories"), renderer, "text", 0, NULL);
-//	GtkTreeViewColumn *column = gtk_tree_view_get_column (GTK_TREE_VIEW
-//		(treeview), 0);
-//	gtk_tree_view_column_set_clickable (GTK_TREE_VIEW_COLUMN (column), FALSE);
+	gtk_tree_view_insert_column_with_attributes ((GtkTreeView*)catsview, -1,
+		_("Categories"), renderer, "text", 0, NULL);
+	GtkTreeViewColumn *col = gtk_tree_view_get_column ((GtkTreeView*)catsview, 0);
+	gtk_tree_view_column_set_sizing (col, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+//	gtk_tree_view_column_set_clickable (col, FALSE);
 	gtk_container_add (GTK_CONTAINER (sw), catsview);
 
 	//the notebook
@@ -1747,6 +1777,9 @@ void e2_config_dialog_create (gchar *page)
 
 	rt->opthash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free,
 		(GDestroyNotify)_e2_hashfree);
+	//log these in case they're changed via the dialog
+	rt->refresh_files = e2_option_bool_get ("auto-refresh");
+	rt->refresh_cfg = e2_option_bool_get ("auto-refresh-config");
 
 	guint i;
 	gpointer *walker;
