@@ -1,6 +1,6 @@
-/* $Id: e2_file_info_dialog.c 1321 2008-10-13 09:17:38Z tpgww $
+/* $Id: e2_file_info_dialog.c 2309 2011-01-17 00:12:22Z tpgww $
 
-Copyright (C) 2003-2008 tooar <tooar@gmx.net>
+Copyright (C) 2003-2011 tooar <tooar@emelfm2.net>
 Portions copyright (C) 1999 Michael Clark.
 
 This file is part of emelFM2.
@@ -15,8 +15,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 #include "emelfm2.h"
@@ -26,6 +25,10 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #include <time.h>
 #include <pwd.h>
 #include <grp.h>
+#ifdef E2_MAGIC
+# include <magic.h>
+# include <dlfcn.h>
+#endif
 #include "e2_dialog.h"
 
 //max length of the file type string
@@ -47,6 +50,30 @@ typedef struct _E2_FileInfoRuntime
 	GtkWidget *changedate;
 } E2_FileInfoRuntime;
 
+/**
+@brief add thousand-separators to the number-string contained in @a number
+@param number Gstring containing the number to be processed
+@return
+*/
+static void _e2_file_info_dialog_prettify_number (GString *number)
+{
+	//expand number to add separator char(s)
+	const gchar *comma = nl_langinfo (THOUSEP);
+	if (comma == NULL || *comma == '\0')
+		comma = ",";
+	guint len = number->len;
+	guint ths = len-1;  //0-based index
+	guint i;
+	while (ths > 2 && len < number->allocated_len)
+	{
+		for (i = len-1; i > ths-3; i--)
+			number->str[i+1] = number->str[i];
+		number->str[i+1] = *comma;
+		number->str[++len] = '\0';
+		ths = i;
+	}
+	number->len = len;
+}
 /**
 @brief get description and mimetype of @a filepath, using the 'file' command
 @a readable_type and/or @a mime_type are set to NULL if nothing valid is available
@@ -63,44 +90,110 @@ static gboolean _e2_file_info_dialog_get_file_type (VPATH *localpath,
 	if (e2_fs_item_is_mounted (localpath))
 	{
 #endif
-	gpointer results;
-//tag E2_BADQUOTES
-	gchar *qp = e2_utils_quote_string (VPCSTR (localpath));
-	gchar *command = e2_utils_strcat ("file -bhnprs ", qp);
-	if (!e2_fs_get_command_output (command, &results))
-	{
-		g_free (qp);
-		g_free (command);
+#ifdef E2_MAGIC
+		MagicIface ifc;
+
+		if (e2_utils_fill_magic_iface (&ifc))
+		{
+			magic_t handle = ifc.open (
+				MAGIC_PRESERVE_ATIME | MAGIC_RAW | MAGIC_ERROR | MAGIC_DEVICES);
+			if (handle != NULL)
+			{
+				gchar *results;
+				const gchar *msg;
+
+				ifc.load (handle, NULL); //load failure will result in NULL msg
+				msg = ifc.file (handle, VPCSTR (localpath));
+				if (msg != NULL)
+				{
+					results = g_strdup (msg);
+					g_strstrip (results);
+					*readable_type = results;
+				}
+				else
+				{
+					msg = ifc.error (handle);
+					if (msg != NULL)
+						e2_output_print_error ((gchar *)msg, FALSE);
+				}
+
+				*mime_type = e2_utils_get_mimetype (localpath);
+
+				if (*mime_type != NULL && g_str_has_prefix (*mime_type, "text/"))
+				{
+					//try to get encoding as well
+					ifc.setflags (handle, MAGIC_PRESERVE_ATIME | MAGIC_RAW | MAGIC_ERROR
+						| MAGIC_MIME_TYPE | MAGIC_MIME_ENCODING);
+					msg = ifc.file (handle, VPCSTR (localpath));
+					if (msg != NULL)
+					{
+						results = g_strdup (msg);
+						g_strstrip (results);	//get rid of \n etc
+						if (*(gchar *)results != '\0')
+						{
+							g_free (*mime_type);
+							*mime_type = results;
+						}
+						else
+							g_free (results);
+					}
+					else
+					{
+						msg = ifc.error (handle);
+						if (msg != NULL)
+							e2_output_print_error ((gchar *)msg, FALSE);
+					}
+				}
+
+				ifc.close (handle);
+				return TRUE;
+			}
+			dlclose (ifc.libhandle);
+		}
+//		e2_output_print_error ("?", FALSE);
 		*readable_type = NULL;
 		*mime_type = NULL;
 		return FALSE;
-	}
-	g_free (command);
-	g_strstrip ((gchar *)results);	//get rid of \n etc
-	*readable_type = results;
-
-	*mime_type = e2_utils_get_mimetype (localpath);
-
-	if (*mime_type != NULL && g_str_has_prefix (*mime_type, "text/"))
-	{
-		//try to get encoding as well
-		command = e2_utils_strcat ("file -bhnpri ", qp);
-		if (e2_fs_get_command_output (command, &results))
+#else
+//tag E2_BADQUOTES
+		gchar *qp = e2_utils_quote_string (VPCSTR (localpath));
+		gchar *command = e2_utils_strcat ("file -bhnprs ", qp);
+		gpointer results;
+		if (!e2_fs_get_command_output (command, &results))
 		{
-			g_strstrip ((gchar *)results);	//get rid of \n etc
-			if (*(gchar *)results != '\0')
-			{
-				g_free (*mime_type);
-				*mime_type = results;
-			}
-			else
-				g_free (results);
+			g_free (qp);
+			g_free (command);
+			*readable_type = NULL;
+			*mime_type = NULL;
+			return FALSE;
 		}
 		g_free (command);
-	}
-	g_free (qp);
+		g_strstrip ((gchar *)results);	//get rid of \n etc
+		*readable_type = results;
 
-	return TRUE;
+		*mime_type = e2_utils_get_mimetype (localpath);
+
+		if (*mime_type != NULL && g_str_has_prefix (*mime_type, "text/"))
+		{
+			//try to get encoding as well
+			command = e2_utils_strcat ("file -bhnpri ", qp);
+			if (e2_fs_get_command_output (command, &results))
+			{
+				g_strstrip ((gchar *)results);	//get rid of \n etc
+				if (*(gchar *)results != '\0')
+				{
+					g_free (*mime_type);
+					*mime_type = results;
+				}
+				else
+					g_free (results);
+			}
+			g_free (command);
+		}
+		g_free (qp);
+
+		return TRUE;
+#endif
 #ifdef E2_VFS
 	}
 	else	//not a local item
@@ -124,9 +217,9 @@ set @a push_in to TRUE for menu completely inside the screen,
 FALSE for menu clamped to screen size
 
 @param menu the GtkMenu to be positioned
-@param x	place to store gint representing the menu left
-@param y  place to store gint representing the menu top
-@param push_in place to store pushin flag
+@param x pointer to store for left-position of @a menu
+@param y pointer to store for top-position of @a menu
+@param push_in pointer to store for pushin flag
 @param dialog the dialog widget in focus when the menu key was pressed
 
 @return
@@ -136,11 +229,11 @@ static void _e2_file_info_dialog_set_menu_position (GtkWidget *menu,
 {
 	gint left, top;
 	gtk_window_get_position (GTK_WINDOW (dialog), &left, &top);
-	GtkAllocation alloc =
-#ifdef USE_GTK2_99
-		gtk_widget_get_allocation (dialog);
+	GtkAllocation alloc;
+#ifdef USE_GTK2_18
+	gtk_widget_get_allocation (dialog, &alloc);
 #else
-		dialog->allocation;
+	alloc = dialog->allocation;
 #endif
 	*x = left + alloc.x + alloc.width/2;
 	*y = top + alloc.y + alloc.height/2;
@@ -149,12 +242,12 @@ static void _e2_file_info_dialog_set_menu_position (GtkWidget *menu,
 /**
 @brief copy dialog data to clipboard
 
-@param menuitem UNUSED the selected widget
+@param menuitem UNUSED the activated widget
 @param rt runtime struct to work on
 
 @return
 */
-static void _e2_file_info_dialog_copy_cb (GtkWidget *menuitem,
+static void _e2_file_info_dialog_copy_cb (GtkMenuItem *menuitem,
 	E2_FileInfoRuntime *rt)
 {
 	//some format massaging is needed ...
@@ -196,18 +289,18 @@ static void _e2_file_info_dialog_copy_cb (GtkWidget *menuitem,
 		_("Group:"), gtk_label_get_text (GTK_LABEL(rt->group)),
 		_("Permissions:"), gtk_label_get_text (GTK_LABEL(rt->permissions)),
 		_("Accessed:"), gtk_label_get_text (GTK_LABEL(rt->accessdate)),
-		_("Modified:"), gtk_label_get_text (GTK_LABEL(rt->moddate)),
-		_("Changed:"), gtk_label_get_text (GTK_LABEL(rt->changedate))
+		_("Content modified:"), gtk_label_get_text (GTK_LABEL(rt->moddate)),
+		_("Property changed:"), gtk_label_get_text (GTK_LABEL(rt->changedate))
 	);
 	GtkClipboard *cb = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
 	gtk_clipboard_set_text (cb, text->str, text->len);
-	F_FREE (utf);
+	F_FREE (utf, rt->localpath);
 	if (*typelabel != '\0')
 		g_free (typelabel);
 	g_string_free (text, TRUE);
 }
 /**
-@brief construct and show dialog context menu
+@brief construct and pop up destroyable context-menu for @a dialog
 
 @param textview the textview widget where the click happened
 @param event_button which mouse button was clicked (0 for a menu key)
@@ -223,7 +316,7 @@ static void _e2_file_info_dialog_show_context_menu (GtkWidget *dialog,
 	e2_menu_add (menu, _("_Copy"), GTK_STOCK_COPY,
 		_("Copy displayed data"), _e2_file_info_dialog_copy_cb, rt);
 	g_signal_connect (G_OBJECT (menu), "selection-done",
-		G_CALLBACK (e2_menu_destroy_cb), NULL);
+		G_CALLBACK (e2_menu_selection_done_cb), NULL);
 	if (event_button == 0)
 		gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
 			(GtkMenuPositionFunc) _e2_file_info_dialog_set_menu_position,
@@ -244,7 +337,7 @@ static void _e2_file_info_dialog_show_context_menu (GtkWidget *dialog,
 static gboolean _e2_file_info_dialog_popup_menu_cb (GtkWidget *dialog,
 	E2_FileInfoRuntime *rt)
 {
-	gint event_time = gtk_get_current_event_time ();
+	guint32 event_time = gtk_get_current_event_time ();
 	_e2_file_info_dialog_show_context_menu (dialog, 0, event_time, rt);
 	return TRUE;
 }
@@ -272,8 +365,83 @@ static gboolean _e2_file_info_dialog_button_press_cb (GtkWidget *dialog,
 	return FALSE;
 }
 /**
-@brief create and run file info dialog
+@brief update total count of items in a directory
+This is a callback for a treewalk function
+@param localpath absolute path of item reported by the walker, localised string
+@param statptr pointer to struct stat with data about @a localpath
+@param status code from the walker, indicating what type of report it is
+@param twdata pointer to integer to update
 
+@return E2TW_CONTINUE always
+*/
+static E2_TwResult
+_e2_file_info_dialog_count_twcb (VPATH *localpath, const struct stat *statptr,
+	E2_TwStatus status, guint64 *twdata)
+{
+	switch (status)
+	{
+		case E2TW_DL:	//dir, not opened due to tree-depth limit (reported upstream)
+		case E2TW_DM:	//dir, not opened due to different file system (reported upstream)
+		case E2TW_D:	//dir (don't care about its reported size)
+		case E2TW_DRR:	//dir now readable
+		case E2TW_DNR:	//unreadable dir (reported upstream)
+		{
+			gchar *fullpath = VPSTR (localpath);
+			gchar *base = strrchr (fullpath, G_DIR_SEPARATOR);
+			if (base != NULL)
+				base++;
+			else
+				base = fullpath;
+			if (*base == '.' && *(base+1) == '\0') //".." items are filtered out upstream
+				break;
+		}
+		case E2TW_F:	//not dir or link
+		case E2TW_SL:	//symbolic link
+		case E2TW_SLN:	//symbolic link naming non-existing file
+		case E2TW_NS:	//un-statable item (reported upstream)
+			*twdata += 1;
+//		case E2TW_DP:	//dir, finished
+		default:
+			break;
+	}
+	return E2TW_CONTINUE;
+}
+/**
+@brief update total size of items in a directory
+This is a callback for a treewalk function
+@param localpath absolute path of item reported by the walker, localised string
+@param statptr pointer to struct stat with data about @a localpath
+@param status code from the walker, indicating what type of report it is
+@param twdata pointer to integer to update
+
+@return E2TW_CONTINUE always
+*/
+static E2_TwResult
+_e2_file_info_dialog_usage_twcb (VPATH *localpath, const struct stat *statptr,
+	E2_TwStatus status, guint64 *twdata)
+{
+	switch (status)
+	{
+		//don't bother filtering out "." directories, if any
+		case E2TW_DL:	//dir, not opened due to tree-depth limit (reported upstream)
+		case E2TW_DM:	//dir, not opened due to different file system (reported upstream)
+		case E2TW_D:	//dir (don't care about its reported size)
+		case E2TW_DRR:	//dir now readable
+		case E2TW_DNR:	//unreadable dir (reported upstream)
+		case E2TW_F:	//not dir or link
+		case E2TW_SL:	//symbolic link
+		case E2TW_SLN:	//symbolic link naming non-existing file
+			*twdata += statptr->st_size;
+//		case E2TW_NS:	//un-statable item (reported upstream)
+//		case E2TW_DP:	//dir, finished
+		default:
+			break;
+	}
+	return E2TW_CONTINUE;
+}
+/**
+@brief create and run file info dialog
+Expects BGL off/open
 @param localpath localised path string of item to be processed
 @param multi TRUE when the dialog needs buttons consistent with >1 item being processed
 
@@ -293,13 +461,24 @@ DialogButtons e2_file_info_dialog_run (VPATH *localpath, gboolean multi)
 	struct group *grp_buf;
 	E2_FileInfoRuntime rt;
 
+//tag DEBUGfreeze;
+	//maybe locking gtk out of X during dialog setup will help
+	//CHECKME effects of child-signals received due to file-command pipe ?
+//	gdk_threads_enter ();
+
 	if (e2_fs_lstat (localpath, &statbuf E2_ERR_NONE()))
+	{
+//tag DEBUGfreeze;
+		gdk_threads_leave ();
 		//CHECKME explain to user ?
 		return CANCEL;
+	}
 
 	//must prevent the default dialog response, it stuffs up Q cleanups
+	gdk_threads_enter ();
 	rt.dialog = e2_dialog_create (NULL, NULL, _("file info"),
 		DUMMY_RESPONSE_CB, NULL);
+	gdk_threads_leave ();
 	dialog_vbox =
 #ifdef USE_GTK2_14
 		gtk_dialog_get_content_area (GTK_DIALOG (rt.dialog));
@@ -419,21 +598,37 @@ DialogButtons e2_file_info_dialog_run (VPATH *localpath, gboolean multi)
 	//links to dirs reported as dirs by (e2_fs_is_dir3 (localpath))
 	else if (S_ISDIR (statbuf.st_mode))
 	{
-		GtkIconTheme *icon_theme = gtk_icon_theme_get_for_screen
-				(gtk_widget_get_screen (app.main_window));	//app.main_window->screen;
-		GdkPixbuf *pixbuf = gtk_icon_theme_load_icon (icon_theme, "stock_open",
-				GTK_ICON_SIZE_LARGE_TOOLBAR, 0, NULL);
-		s2 =  _("Directory");
-		if (pixbuf != NULL)
+		guint64 total;
+
+		total = 0;
+		e2_fs_tw (localpath, _e2_file_info_dialog_count_twcb, &total, 1, E2TW_QT | E2TW_PHYS E2_ERR_NONE());
+		s1 =  _("Directory");
+		if (total > 1)
+		{
+			g_string_printf (label_text, "%"PRIu64, total - 1);
+			_e2_file_info_dialog_prettify_number (label_text);
+			gchar *countstr = g_strdup (label_text->str);
+			total = 0;
+			e2_fs_tw (localpath, _e2_file_info_dialog_usage_twcb, &total, -1, E2TW_PHYS E2_ERR_NONE());
+			g_string_printf (label_text, "%"PRIu64, total - statbuf.st_size);
+			_e2_file_info_dialog_prettify_number (label_text);
+			s2 = g_strdup_printf (
+				_("%s containing %s item(s) which occupy %s byte(s)"), s1, countstr, label_text->str);
+			g_free (countstr);
+		}
+		else
+			s2 = g_strdup_printf ("%s - %s", s1, _("empty"));
+
+		GtkWidget *image = e2_widget_get_icon (GTK_STOCK_DIRECTORY,
+						GTK_ICON_SIZE_LARGE_TOOLBAR);
+		if (image != NULL)
 		{
-			GtkWidget *image = gtk_image_new_from_pixbuf (pixbuf);
 			GtkWidget *hbox = e2_widget_add_box (dialog_vbox, TRUE, 0, FALSE,
 				FALSE, E2_PADDING_LARGE);
 			gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
 			//CHECKME escape label ?
 			rt.description =
 			e2_widget_add_mid_label (hbox, s2, 0, TRUE, 0);
-			g_object_unref (G_OBJECT (pixbuf));
 		}
 		else
 		{
@@ -441,7 +636,8 @@ DialogButtons e2_file_info_dialog_run (VPATH *localpath, gboolean multi)
 			rt.description =
 			e2_widget_add_mid_label (dialog_vbox, label_text->str, 0, TRUE, 0);
 		}
-		//FIXME report the device holding the dir
+		g_free (s2);
+		//TODO report the device holding the dir
 	}
 	else // not regular or directory
 	{
@@ -478,17 +674,17 @@ DialogButtons e2_file_info_dialog_run (VPATH *localpath, gboolean multi)
 #endif
 		if (len > 0)
 		{
-			target[len] = '\0';
 			s1 = F_DISPLAYNAME_FROM_LOCALE (target);
 			s2 = g_markup_escape_text (s1, -1);
 			label_text = g_string_assign (label_text, " ");
 			g_string_append_printf (label_text, _("to %s"), s2);
-			F_FREE (s1);
+			F_FREE (s1, target);
 			g_free (s2);
-			//statting of relative targets fails, so ...
+			//stat() fails for relative targets, so ...
 			if (g_str_has_prefix (target, "."G_DIR_SEPARATOR_S))
 			{
-				memcpy (target, (target + 2 * sizeof (gchar)), len);
+				memmove (target, (target + 2 * sizeof (gchar)), len - 1);
+				len -= 2;
 			}
 			if (g_str_has_prefix (target, ".."G_DIR_SEPARATOR_S))
 			{
@@ -498,13 +694,13 @@ DialogButtons e2_file_info_dialog_run (VPATH *localpath, gboolean multi)
 #else
 				s2 = e2_utils_translate_relative_path (curr_view->dir, s1);
 #endif
-				F_FREE (s1);
+				F_FREE (s1, target);
 				s1 = F_FILENAME_TO_LOCALE (s2);
 				g_strlcpy (target, s1, sizeof(target));
 				//the translation adds a trailing separator, which lstat assumes to be a dir match ..
 				len = strlen (target);
 				target[len-1] = '\0';
-				F_FREE (s1);
+				F_FREE (s1, s2);
 				g_free (s2);
 			}
 			//FIXME get intelligent absolute path
@@ -514,41 +710,45 @@ DialogButtons e2_file_info_dialog_run (VPATH *localpath, gboolean multi)
 #ifdef E2_VFSTMP
 				local = ; FIXME
 #else
-				local = F_FILENAME_TO_LOCALE (curr_view->dir);
+				local = D_FILENAME_TO_LOCALE (curr_view->dir); //always dup, to avoid dirchange race
 #endif
 				gchar *t2 = g_build_filename (local, target, NULL);
 				g_strlcpy (target, t2, sizeof (target));
-				F_FREE (local);
+				g_free (local);
 				g_free (t2);
 			}
 			E2_ERR_DECLARE
 			//check the real target, without looking thru chained links
 			struct stat statbuf2;	//don't disturb the size etc data for the link itelf
 #ifdef E2_VFS
-			ddata.localpath = target;
+			ddata.path = target;
 			if (e2_fs_lstat (&ddata, &statbuf2 E2_ERR_PTR()) && E2_ERR_IS (ENOENT))
 #else
 			if (e2_fs_lstat (target, &statbuf2 E2_ERR_PTR()) && E2_ERR_IS (ENOENT))
 #endif
 				g_string_append_printf (label_text, "\n %s",_("(which is <b>missing</b>)"));
-			else if (g_str_equal (localpath, target))
+			else if (!strcmp (localpath, target))
 				g_string_append_printf (label_text, "\n %s",_("(which is <b>itself</b>)"));
 			else
 			{
 				if (e2_fs_walk_link (&filename E2_ERR_NONE()))
 				{
 					printd (DEBUG, "final target is %s", filename);
-					if (!g_str_equal (filename, target))
+					if (strcmp (filename, target))
 					{
-						s1 = F_FILENAME_FROM_LOCALE (filename);
-#ifdef E2_VFSTMP
-						//FIXME which CWD for the baseline
+						s1 = D_FILENAME_TO_LOCALE (curr_view->dir); //always dup, to avoid dirchange race
+#ifdef E2_VFS
+						VPATH data2 = { s1, localpath->spacedata };
+						ddata.path = filename;
+						s2 = e2_utils_create_relative_path (&ddata, &data2);
 #else
-						s2 = e2_utils_create_relative_path (s1, curr_view->dir);
+						s2 = e2_utils_create_relative_path (filename, s1);
 #endif
+						g_free (s1);
+						s1 = F_FILENAME_FROM_LOCALE (s2);
 						label_text = g_string_append (label_text, "\n ");
-						g_string_append_printf (label_text, _("and ultimately to %s"), s2);
-						F_FREE (s1);
+						g_string_append_printf (label_text, _("and ultimately to %s"), s1);
+						F_FREE (s1, s2);
 						g_free (s2);
 					}
 				}
@@ -609,23 +809,7 @@ DialogButtons e2_file_info_dialog_run (VPATH *localpath, gboolean multi)
 	e2_widget_add_mid_label_to_table (table, _("Size:"), 0, 0,1,1,2);
 
 	g_string_printf (label_text, "%"PRIu64, statbuf.st_size);
-	//expand number to add separator char(s)
-	const gchar *comma = nl_langinfo (THOUSEP);
-	if (comma == NULL || *comma == '\0')
-		comma = ",";
-	guint len = label_text->len;
-	guint ths = len-1;  //0-based index
-	guint i;
-	while (ths > 2 && len < label_text->allocated_len)
-	{
-		for (i = len-1; i > ths-3; i--)
-			label_text->str[i+1] = label_text->str[i];
-		label_text->str[i+1] = *comma;
-		label_text->str[++len] = '\0';
-		ths = i;
-	}
-	label_text->len = len;
-
+	_e2_file_info_dialog_prettify_number (label_text);
 	label_text = g_string_append_c (label_text, ' ');
 	label_text = g_string_append (label_text, _("bytes"));
 	rt.size =
@@ -667,49 +851,39 @@ DialogButtons e2_file_info_dialog_run (VPATH *localpath, gboolean multi)
 	e2_widget_add_mid_label_to_table (table, _("Accessed:"), 0, 0,1,7,8);
 	rt.accessdate =
 	e2_widget_add_mid_label_to_table (table, s2, 0, 1,2,7,8);
-#ifdef USE_GTK2_12TIPS
-	gtk_widget_set_tooltip_text (
-#else
-	e2_widget_set_tooltip (NULL,
-#endif
-		rt.accessdate, _("Item was last opened, run, read etc"));
+	e2_widget_set_safetip (rt.accessdate, _("Item was last opened, run, read etc"));
 	g_free (s2);
 
 	tm_ptr = localtime (&statbuf.st_mtime);
 	strftime (date_string, sizeof (date_string), s1, tm_ptr);
 	s2 = e2_utf8_from_locale (date_string);
-	e2_widget_add_mid_label_to_table (table, _("Modified:"), 0, 0,1,8,9);
+	e2_widget_add_mid_label_to_table (table, _("Content modified:"), 0, 0,1,8,9);
 	rt.moddate =
 	e2_widget_add_mid_label_to_table (table, s2, 0, 1,2,8,9);
-#ifdef USE_GTK2_12TIPS
-	gtk_widget_set_tooltip_text (
-#else
-	e2_widget_set_tooltip (NULL,
-#endif
-		rt.moddate, _("Content of the item was last altered"));
+	e2_widget_set_safetip (rt.moddate, _("Item content was last changed"));
 	g_free (s2);
 
 	tm_ptr = localtime (&statbuf.st_ctime);
 	strftime (date_string, sizeof (date_string), s1, tm_ptr);
 	s2 = e2_utf8_from_locale (date_string);
-	e2_widget_add_mid_label_to_table (table, _("Changed:"), 0, 0,1,9,10);
+	e2_widget_add_mid_label_to_table (table, _("Property changed:"), 0, 0,1,9,10);
 	rt.changedate =
 	e2_widget_add_mid_label_to_table (table, s2, 0, 1,2,9,10);
-#ifdef USE_GTK2_12TIPS
-	gtk_widget_set_tooltip_text (
-#else
-	e2_widget_set_tooltip (NULL,
-#endif
-		rt.changedate, _("Property of the item (name, permission, owner etc) was last altered"));
+	e2_widget_set_safetip (rt.changedate,
+		_("Item name, permission, owner, etc was last changed"));
 	g_free (s2);
 
 	g_string_free (label_text,TRUE);
 	if (mime != NULL)
 		g_free (mime);
 
-	E2_Button local_btn;
+//tag DEBUGfreeze2;
+//	gdk_threads_leave ();
+
 	//block until user responds
 	E2_DialogFlags flags = E2_DIALOG_BLOCKED | E2_DIALOG_CLOSELOCK | E2_DIALOG_FREE;
+
+	E2_Button local_btn;
 	if (multi)
 	{
 		e2_dialog_set_negative_response (rt.dialog, E2_RESPONSE_NOTOALL);
@@ -727,7 +901,10 @@ DialogButtons e2_file_info_dialog_run (VPATH *localpath, gboolean multi)
 	g_signal_connect (G_OBJECT (rt.dialog), "button-press-event",
 		G_CALLBACK (_e2_file_info_dialog_button_press_cb), &rt);
 
+//tag DEBUGfreeze
+	gdk_threads_enter ();
 	e2_dialog_setup (rt.dialog, app.main_window);
-	return (e2_dialog_run (rt.dialog, app.main_window, flags));
-}
+	gdk_threads_leave ();
 
+	return (e2_dialog_run (rt.dialog, NULL, flags));
+}
