@@ -1,6 +1,6 @@
-/* $Id: e2_filetype_dialog.c 1277 2008-09-27 01:10:48Z tpgww $
+/* $Id: e2_filetype_dialog.c 2469 2012-03-16 00:19:24Z tpgww $
 
-Copyright (C) 2003-2008 tooar <tooar@gmx.net>
+Copyright (C) 2003-2012 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -14,8 +14,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 #include "emelfm2.h"
@@ -93,7 +92,7 @@ static void _e2_edftdlg_save_current_category (E2_FileTypeDlgRuntime *rt)
 		{  // extension or command loop = level 2
 			cfgchildren = gtk_tree_model_iter_n_children (cfgmdl, &cfg_iter2);
 			gtk_tree_model_get (cfgmdl, &cfg_iter2, 1, &extension, -1);
-			if (g_str_equal (extension, _C(13)))  //extensions node found
+			if (!strcmp (extension, _C(13)))  //extensions node found
 			{
 				g_free (extension);
 				mdl = GTK_TREE_MODEL (rt->stores[E2_EXTPANE]);
@@ -145,7 +144,7 @@ static void _e2_edftdlg_save_current_category (E2_FileTypeDlgRuntime *rt)
 					}
 				}
 			}
-			else if (g_str_equal (extension, _C(6)))  //commands node found
+			else if (!strcmp (extension, _C(6)))  //commands node found
 			{
 				g_free (extension);
 				mdl = GTK_TREE_MODEL (rt->stores[E2_CMDPANE]);
@@ -261,7 +260,7 @@ static void _e2_edftdlg_update_category (GtkTreePath *path,
 				gtk_tree_model_get (cfgmdl, &iter2, 1, &extension, -1);
 				if (gtk_tree_model_iter_children (cfgmdl, &iter3, &iter2))
 				{
-					if (g_str_equal (extension, _C(13)))  //extensions node found
+					if (!strcmp (extension, _C(13)))  //extensions node found
 					{
 						g_free (extension);
 						do
@@ -280,7 +279,7 @@ static void _e2_edftdlg_update_category (GtkTreePath *path,
 							g_free (extension);
 						} while (gtk_tree_model_iter_next (cfgmdl, &iter3));
 					}
-					else if (g_str_equal (extension, _C(6)))  //commands node found
+					else if (!strcmp (extension, _C(6)))  //commands node found
 					{
 						g_free (extension);
 						do
@@ -380,12 +379,12 @@ static gboolean _e2_edftdlg_find_selected
 		return (gtk_tree_selection_get_selected (sel, &mdl, iter));
 
 	gboolean retval;
-	GList *items = gtk_tree_selection_get_selected_rows (sel, &mdl);
-	if (items != NULL)
+	GList *selpaths = gtk_tree_selection_get_selected_rows (sel, &mdl);
+	if (selpaths != NULL)
 	{
-		retval = gtk_tree_model_get_iter (mdl, iter, (GtkTreePath *)items->data);
-		g_list_foreach (items, (GFunc) gtk_tree_path_free, NULL);
-		g_list_free (items);
+		retval = gtk_tree_model_get_iter (mdl, iter, (GtkTreePath *)selpaths->data);
+		g_list_foreach (selpaths, (GFunc) gtk_tree_path_free, NULL);
+		g_list_free (selpaths);
 	}
 	else
 		retval = FALSE;
@@ -582,12 +581,12 @@ static void _e2_edftdlg_response_cb (GtkDialog *dialog, gint response,
 /**
 series of context-menu callback functions
 
-@param item the selected menu item widget
+@param item the activated menu item widget
 @param rt pointer to dialog data struct
 
 @return
 */
-static void _e2_edftdlg_menu_copy_cb (GtkWidget *item, E2_FileTypeDlgRuntime *rt)
+static void _e2_edftdlg_menu_copy_cb (GtkMenuItem *item, E2_FileTypeDlgRuntime *rt)
 {
 	GtkTreeView *treeview = GTK_TREE_VIEW (rt->views[rt->currentID]);
 	GtkTreeSelection *sel = gtk_tree_view_get_selection (treeview);
@@ -600,7 +599,7 @@ static void _e2_edftdlg_menu_copy_cb (GtkWidget *item, E2_FileTypeDlgRuntime *rt
 		g_hash_table_replace (rt->buffer_hash, id_string, copied);
 	}
 }
-static void _e2_edftdlg_menu_cut_cb (GtkWidget *item, E2_FileTypeDlgRuntime *rt)
+static void _e2_edftdlg_menu_cut_cb (GtkMenuItem *item, E2_FileTypeDlgRuntime *rt)
 {
 	GtkTreeView *treeview = GTK_TREE_VIEW (rt->views[rt->currentID]);
 	GtkTreeSelection *sel = gtk_tree_view_get_selection (treeview);
@@ -610,7 +609,7 @@ static void _e2_edftdlg_menu_cut_cb (GtkWidget *item, E2_FileTypeDlgRuntime *rt)
 		e2_tree_delete (treeview);
 	}
 }
-static void _e2_edftdlg_menu_paste_cb (GtkWidget *item, E2_FileTypeDlgRuntime *rt)
+static void _e2_edftdlg_menu_paste_cb (GtkMenuItem *item, E2_FileTypeDlgRuntime *rt)
 {
 	//as a special case, if the store is empty, setup to add the data
 	gboolean empty = (gtk_tree_model_iter_n_children
@@ -634,15 +633,15 @@ static void _e2_edftdlg_menu_paste_cb (GtkWidget *item, E2_FileTypeDlgRuntime *r
 	if (empty)
 		e2_tree_delete (treeview);
 }
-static void _e2_edftdlg_menu_add_below_cb (GtkWidget *item, E2_FileTypeDlgRuntime *rt)
+static void _e2_edftdlg_menu_add_below_cb (GtkMenuItem *item, E2_FileTypeDlgRuntime *rt)
 {
 	_e2_edftdlg_insert_row (rt);
 }
-static void _e2_edftdlg_menu_move_up_cb (GtkWidget *item, E2_FileTypeDlgRuntime *rt)
+static void _e2_edftdlg_menu_move_up_cb (GtkMenuItem *item, E2_FileTypeDlgRuntime *rt)
 {
 	_e2_edftdlg_response_cb (GTK_DIALOG (filetypes_dialog), E2_RESPONSE_USER1, rt);
 }
-static void _e2_edftdlg_menu_move_down_cb (GtkWidget *item, E2_FileTypeDlgRuntime *rt)
+static void _e2_edftdlg_menu_move_down_cb (GtkMenuItem *item, E2_FileTypeDlgRuntime *rt)
 {
 	_e2_edftdlg_response_cb (GTK_DIALOG (filetypes_dialog), E2_RESPONSE_USER2, rt);
 }
@@ -667,18 +666,18 @@ static void _e2_edftdlg_set_menu_position (GtkMenu *menu,
 {
 	gint left, top;
 	gtk_window_get_position (GTK_WINDOW (filetypes_dialog), &left, &top);
-	GtkAllocation alloc =
-#ifdef USE_GTK2_99
-		gtk_widget_get_allocation (treeview);
+	GtkAllocation alloc;
+#ifdef USE_GTK2_18
+	gtk_widget_get_allocation (treeview, &alloc);
 #else
-		treeview->allocation;
+	alloc = treeview->allocation;
 #endif
 	*x = left+ alloc.x + alloc.width/2;
 	*y = top + alloc.y +alloc.height/2 - 30;
 	*push_in = FALSE;
 }
 /**
-@brief create treeview context menu
+@brief construct and pop up destroyable context-menu for the filetype-dialog
 
 @param treeview widget where the action is to occur
 @param button the clicked mouse button, or 0 for menu key
@@ -712,6 +711,9 @@ static void _e2_edftdlg_menu_create (GtkWidget *treeview,
 	e2_menu_add (menu, _("_Down"), GTK_STOCK_GO_DOWN,
 		_("Move selected row down"), _e2_edftdlg_menu_move_down_cb, rt);
 
+	g_signal_connect (G_OBJECT (menu), "selection-done",
+		G_CALLBACK (e2_menu_selection_done_cb), NULL);
+
 	if (button == 0)
 		//this was a menu-key press
 		gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
@@ -821,6 +823,7 @@ static gboolean _e2_edftdlg_key_press_cb (GtkWidget *widget, GdkEventKey *event,
 }
 /**
 @brief create and show edit filetypes dialog
+Assumes BGL is closed
 @param open_category name of category to focus when dialog opens, or NULL
 @return
 */
@@ -845,7 +848,7 @@ void e2_filetype_dialog_edit_create (gchar *open_category)
 		g_free, (GDestroyNotify) e2_option_tree_menu_hash_clean);
 
 	filetypes_dialog = e2_dialog_create (NULL, NULL, _("edit filetypes"),
-		_e2_edftdlg_response_cb, rt);
+		(ResponseFunc)_e2_edftdlg_response_cb, rt);
 
 	e2_option_connect (filetypes_dialog, FALSE);
 
@@ -858,13 +861,16 @@ void e2_filetype_dialog_edit_create (gchar *open_category)
 #endif
 		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC, TRUE, E2_PADDING_SMALL);
 	gtk_widget_set_size_request (sw, 0, 400);
-	gtk_widget_show (sw);
 
 	GtkWidget *hbox = gtk_hbox_new (FALSE, 0);
 	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (sw), hbox);
-	gtk_viewport_set_shadow_type (GTK_VIEWPORT (GTK_BIN (sw)->child),
+	gtk_viewport_set_shadow_type (
+#ifdef USE_GTK2_14
+		GTK_VIEWPORT (gtk_bin_get_child (GTK_BIN (sw))),
+#else
+		GTK_VIEWPORT (GTK_BIN (sw)->child),
+#endif
 		GTK_SHADOW_NONE);
-	gtk_widget_show (hbox);
 
 	//categories scrolled window
 	GtkWidget *cat_sw = e2_widget_get_sw (GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC,
@@ -896,7 +902,6 @@ void e2_filetype_dialog_edit_create (gchar *open_category)
 #endif
 	gtk_container_add (GTK_CONTAINER (cat_sw), rt->views[E2_CATPANE]);
 	gtk_widget_set_size_request (cat_sw, 150, 0);
-	gtk_widget_show_all (cat_sw);
 
 	//extensions scrolled window
 	GtkWidget *ext_sw = e2_widget_get_sw (GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC,
@@ -918,7 +923,6 @@ void e2_filetype_dialog_edit_create (gchar *open_category)
 		(GTK_TREE_VIEW (rt->views[E2_EXTPANE]), -1, _("Extensions"), renderer, "text", 0, NULL);
 	gtk_container_add (GTK_CONTAINER (ext_sw), rt->views[E2_EXTPANE]);
 	gtk_widget_set_size_request (ext_sw, 100, 0);
-	gtk_widget_show_all (ext_sw);
 	GtkTreeSelection *selection = gtk_tree_view_get_selection
 		(GTK_TREE_VIEW (rt->views[E2_EXTPANE]));
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
@@ -955,7 +959,6 @@ void e2_filetype_dialog_edit_create (gchar *open_category)
 
 	gtk_container_add (GTK_CONTAINER (cmd_sw), rt->views[E2_CMDPANE]);
 	gtk_widget_set_size_request (cmd_sw, 400, 0);
-	gtk_widget_show_all (cmd_sw);
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (rt->views[E2_CMDPANE]));
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
 
@@ -1071,6 +1074,7 @@ static void _e2_filetype_dialog_response_cb (GtkDialog *dialog, gint response,
 @brief create and show dialog for handling unknown or ambiguous filetype
 @a newtype should be TRUE when @a filename does have an extension
 @a newtype and @a ambig should not both be TRUE
+Assumes BGL is closed
 @param localpath absolute or relative path of item to be processed, localised string
 @param text TRUE to include view or edit option in the dialog
 @param ambig TRUE to enable choice to execute or open item
@@ -1110,8 +1114,8 @@ gint e2_filetype_dialog_create (VPATH *localpath, gboolean text, gboolean ambig,
 	base = g_path_get_basename (public);
 	message = g_strdup_printf (
 		_("What would you like to do with\n<b>%s</b>\nin %s ?"), base, dir);
-	GtkWidget *dialog = e2_dialog_create (GTK_STOCK_DIALOG_QUESTION,
-		message, title, _e2_filetype_dialog_response_cb, &response);
+	GtkWidget *dialog = e2_dialog_create (GTK_STOCK_DIALOG_QUESTION, message, title,
+		(ResponseFunc)_e2_filetype_dialog_response_cb, &response);
 	g_free (public);
 	g_free (dir);
 	g_free (base);
@@ -1159,7 +1163,7 @@ gint e2_filetype_dialog_create (VPATH *localpath, gboolean text, gboolean ambig,
 	}
 
 	//block until user selects something
-	e2_dialog_show (dialog, NULL, E2_DIALOG_BLOCKED, &E2_BUTTON_CLOSE, NULL);
+	e2_dialog_show (dialog, NULL, E2_DIALOG_BLOCKED, &E2_BUTTON_CANCEL, NULL);
 
 	if (addpath)
 		g_free (usepath);
@@ -1181,23 +1185,17 @@ gint e2_filetype_dialog_create (VPATH *localpath, gboolean text, gboolean ambig,
 			break;
 		case E2_RESPONSE_USER1:	//edit
 			action = g_strconcat (_A(6),".",_A(43), NULL);
-			gdk_threads_leave ();	//open BGL
 			e2_action_run_simple_from (action, item, dialog);
-			gdk_threads_enter ();
 			g_free (action);
 			break;
 		case E2_RESPONSE_USER2:	//view
-			action = g_strconcat (_A(6),".",_A(105), NULL);
-			gdk_threads_leave ();	//open BGL
+			action = g_strconcat (_A(6),".",_A(106), NULL);
 			e2_action_run_simple_from (action, item, dialog);
-			gdk_threads_enter ();
 			g_free (action);
 			break;
 		case E2_RESPONSE_USER3:	//open with
-			action = g_strconcat (_A(6),".",_A(66), NULL);
-			gdk_threads_leave ();	//open BGL
+			action = g_strconcat (_A(6),".",_A(66), NULL);	//Q'd action
 			e2_action_run_simple_from (action, item, dialog);	//don't know if user cancelled this
-			gdk_threads_enter ();
 			g_free (action);
 			break;
 		case E2_RESPONSE_USER5:	//execute
