@@ -1,6 +1,6 @@
-/* $Id: e2_mkdir_dialog.c 2031 2009-12-21 02:35:14Z tpgww $
+/* $Id: e2_mkdir_dialog.c 2469 2012-03-16 00:19:24Z tpgww $
 
-Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
+Copyright (C) 2004-2012 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -14,8 +14,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 #include "emelfm2.h"
@@ -252,9 +251,13 @@ static void _e2_mkdirdlg_update_dialog_size (E2_MkdirDialogRuntime *rt)
 #else
 	alloc = rt->dialog->allocation;
 #endif
-#if 1
+#ifdef USE_GTK2_20
 	GtkRequisition req;
-	gtk_widget_size_request (rt->dialog, &req);
+# ifdef USE_GTK3_0
+	gtk_widget_get_preferred_size (rt->dialog, NULL, &req);
+# else
+	gtk_widget_get_requisition (rt->dialog, &req);
+# endif
 	if (alloc.width != req.width || alloc.height != req.height)
 #else
 	//this way is not so reliable
@@ -266,8 +269,12 @@ static void _e2_mkdirdlg_update_dialog_size (E2_MkdirDialogRuntime *rt)
 		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (rt->scrolled),
 			GTK_POLICY_NEVER, GTK_POLICY_NEVER);
 //		WAIT_FOR_EVENTS
-#if 1
+#ifdef USE_GTK2_20
+# ifdef USE_GTK3_0
+		gtk_widget_get_preferred_size (rt->dialog, NULL, &req);
+# else
 		gtk_widget_size_request (rt->dialog, &req);//CHECKME maybe different now ?
+# endif
 		//+20 wide seems needed for vertical scroller
 		if (req.width > 0 && req.height > 0)
 			gtk_window_resize (GTK_WINDOW (rt->dialog), req.width + 20, req.height);
@@ -283,7 +290,7 @@ static void _e2_mkdirdlg_update_dialog_size (E2_MkdirDialogRuntime *rt)
 #endif
 //		WAIT_FOR_EVENTS
 		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (rt->scrolled),
-			GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+			GTK_POLICY_AUTOMATIC, GTK_POLICY_NEVER);
 	}
 }
 /**
@@ -362,16 +369,17 @@ static void _e2_mkdirdlg_update_status (E2_MkdirDialogRuntime *rt)
 
 	rt->creation_possible = FALSE;
 
-	const gchar *dir = gtk_entry_get_text (
+	const gchar *dirname = gtk_entry_get_text (
 #ifdef USE_GTK2_14
 		GTK_ENTRY (gtk_bin_get_child (GTK_BIN (rt->combo)))
 #else
 		GTK_ENTRY (GTK_BIN (rt->combo)->child)
 #endif
 	);
-	//if the entry is empty, we'll only check if the parent directory is
+	printd (DEBUG, "dir name entry text is now '%s'", dirname);
+	//if the name-entry is empty, we'll only check if the parent directory is
 	//writable, so, set path to the currently active directory
-	if (*dir == '\0')
+	if (*dirname == '\0')
 	{
 		absolute = TRUE;
 		path = g_strdup (rt->path);
@@ -383,8 +391,8 @@ static void _e2_mkdirdlg_update_status (E2_MkdirDialogRuntime *rt)
 	else //otherwise we have to create an absolute path from the active
 		 //directory and the entry text
 	{
-		absolute = g_path_is_absolute ((gchar *) dir);
-		path = e2_utils_translate_relative_path (rt->path, (gchar *) dir);
+		absolute = g_path_is_absolute ((gchar *) dirname);
+		path = e2_utils_translate_relative_path (rt->path, (gchar *) dirname);
 		len = strlen (path);
 		if (len > 1 && *(path + len - 1) == G_DIR_SEPARATOR)
 			*(path + len - 1) = '\0';
@@ -399,7 +407,7 @@ static void _e2_mkdirdlg_update_status (E2_MkdirDialogRuntime *rt)
 	_e2_mkdirdlg_update_parent_label (parent, rt);
 
 	gchar *reason = NULL;
-	if (*dir == '\0')
+	if (*dirname == '\0')
 	{
 		_e2_mkdirdlg_update_creation_widgets (reason, rt);
 		g_free (path);
@@ -441,6 +449,7 @@ static void _e2_mkdirdlg_update_status (E2_MkdirDialogRuntime *rt)
 		_e2_mkdirdlg_update_creation_widgets (reason, rt);
 
 		E2_ERR_CLEAR
+		g_free (reason);
 		g_free (path);
 		g_free (parent);
 		F_FREE (local, parent);
@@ -460,21 +469,31 @@ static void _e2_mkdirdlg_update_status (E2_MkdirDialogRuntime *rt)
 #else
 		if  (e2_fs_is_dir3 (local E2_ERR_NONE()))
 #endif
-			reason = _("the directory already exists.");
+			reason = _("the directory already exists");
 		else
-			reason = _("something is in the way.");
+			reason = _("something is in the way");
 		_e2_mkdirdlg_update_creation_widgets (reason, rt);
+		F_FREE (local, path);
 		g_free (path);
 		g_free (parent);
-		F_FREE (local, path);
 		return;
 	}
+	gboolean warn = (strchr (local, '?') != NULL || strchr (local, '*') != NULL);
 	F_FREE (local, path);
 
+	if (warn)
+		reason = _("but it's unwise to use wildcard char(s)");
 	if (num > 1)
-		reason = g_strdup_printf ("%d %s",num,_("directories will be created"));
+	{
+		const gchar *r2 = _("directories will be created");
+		reason = (warn) ?
+			g_strdup_printf ("%s\n%d %s", reason, num, r2):
+			g_strdup_printf ("%d %s", num, r2);
+	}
 	rt->creation_possible = TRUE;
 	_e2_mkdirdlg_update_creation_widgets (reason, rt);
+	if (num > 1)
+		g_free (reason);
 	g_free (parent);
 	g_free (path);
 }
@@ -796,30 +815,33 @@ static void _e2_mkdirdlg_response_cb (GtkDialog *dialog, gint response,
 */
 static gboolean _e2_mkdirdlg_deferred_change_cb (E2_MkdirDialogRuntime *rt)
 {
+	rt->idle_id = 0;	//allow sequential changes ASAP
 	if (GTK_IS_DIALOG (rt->dialog))
 	{
 		gdk_threads_enter ();
 		_e2_mkdirdlg_update_status (rt);
 		_e2_mkdirdlg_update_dialog_size (rt);
 		gdk_threads_leave ();
-
-		rt->idle_id = 0;
 	}
 	return FALSE;
 }
 /**
-@brief directory-name combobox "changed" signal callback, initiates a GUI update
+@brief directory-name entry "changed" signal callback, initiates a GUI update
 This is also called every time a letter is added to or removed from the dialog's
 name-entry. So GUI update is deferred to idle time, to help eliminate keypress latency
-@param combo UNUSED the affected combobox
+@param editable UNUSED the editable of the affected combobox-entry
 @param rt pointer to dialog's data struct
 @return
 */
-static void _e2_mkdirdlg_changed_cb (GtkComboBox *combo, E2_MkdirDialogRuntime *rt)
+static void _e2_mkdirdlg_changed_cb (GtkEditable *editable, E2_MkdirDialogRuntime *rt)
 {
-	printd (DEBUG, "mkdir dialog changed cb");
+//	printd (DEBUG, "mkdir dialog changed cb");
 	if (rt->idle_id == 0)	//don't want re-entrance
 		rt->idle_id = g_idle_add ((GSourceFunc)_e2_mkdirdlg_deferred_change_cb, rt);
+//		rt->idle_id = g_idle_add_full (G_PRIORITY_HIGH,
+//			(GSourceFunc)_e2_mkdirdlg_deferred_change_cb, rt, NULL);
+//	else
+//		printd (DEBUG, " idle callback pending already");
 }
 /**
 @brief dialog "button-press-event" signal callback, to pop up a context menu
@@ -934,18 +956,14 @@ static void _e2_mkdirdlg_expander_toggled_cb (GtkWidget *expander,
 		gtk_widget_hide (rt->info_box);
 
 	_e2_mkdirdlg_update_dialog_size (rt);
-}
-static void _e2_mkdirdlg_expander_toggled_cb2 (GtkWidget *expander,
-	GParamSpec *spec, E2_OptionSet *boolset)
-{
-//	printd (DEBUG, "mkdir dialog expander toggle 2");
+
 	GtkWidget *controller = g_object_get_data (G_OBJECT (expander),
 			"e2-controller-widget");
 	if (!GPOINTER_TO_INT (g_object_get_data (G_OBJECT (controller),
 			"e2-controller-blocked")))
 	{
-		gboolean value = gtk_expander_get_expanded (GTK_EXPANDER (expander));
-		e2_option_bool_set_direct (boolset, value);
+        E2_OptionSet *showoption = g_object_get_data (G_OBJECT (expander), "e2-show-option");
+		e2_option_bool_set_direct (showoption, value);
 	}
 }
 /**
@@ -976,7 +994,11 @@ static gboolean _e2_mkdirdlg_set_expander_hook (gpointer state, GtkWidget *expan
 /******************/
 /**
 @brief create and show a mkdir dialog
-@return TRUE always
+
+@param from the button, menu item etc which was activated
+@param art action runtime data
+
+@return TRUE if task completed successfully, else FALSE
 */
 static gboolean _e2_mkdir_dialog_create (gpointer from, E2_ActionRuntime *art)
 {
@@ -995,23 +1017,26 @@ static gboolean _e2_task_mkdirQ (E2_ActionTaskData *qed)
 #endif
 	rt.status = qed->status;	//enable on-the-fly status changes
 	rt.creation_possible = TRUE;
+	rt.idle_id = 0;	//value allows initial idle to be applied
 
 //tag DEBUGfreeze;
-	//maybe this helps gtk 2.16 crashes and/or keypress latency ?
-	//gdk_threads_enter ();
 
 	//create dialog
+	gdk_threads_enter ();
 	rt.dialog = e2_dialog_create (GTK_STOCK_DIALOG_QUESTION,
 		_("What is the new directory's name?"), _("create directory"),
 		(ResponseFunc)_e2_mkdirdlg_response_cb, &rt);
 
 	GtkWidget *vbox = e2_dialog_add_sw (rt.dialog);
-	rt.scrolled = gtk_widget_get_ancestor (vbox, GTK_TYPE_SCROLLED_WINDOW);
 
+	//cuz E2_COMBOBOX_MENU_STYLE flag is not set, on gtk2, downstream calls
+	//gtk_widget_set_name() which MAY? need BGL closed
 	rt.combo = e2_combobox_add (vbox, FALSE, E2_PADDING,
 		(ActivateFunc)_e2_mkdirdlg_activated_cb, &rt, &mkdir_history,
 		E2_COMBOBOX_HAS_ENTRY | E2_COMBOBOX_FOCUS_ON_CHANGE);
+	gdk_threads_leave ();
 
+	rt.scrolled = gtk_widget_get_ancestor (vbox, GTK_TYPE_SCROLLED_WINDOW);
 	rt.info_expander = gtk_expander_new (_("info"));
 	gtk_box_pack_start (GTK_BOX (vbox), rt.info_expander, FALSE, FALSE, 0);
 
@@ -1020,13 +1045,12 @@ static gboolean _e2_task_mkdirQ (E2_ActionTaskData *qed)
 		gtk_expander_set_expanded (GTK_EXPANDER(rt.info_expander), TRUE);
 
 	g_object_set_data (G_OBJECT (rt.info_expander), "e2-controller-widget", rt.dialog);
+	g_object_set_data (G_OBJECT (rt.info_expander), "e2-show-option", infoshow);
 	e2_option_attach_value_changed_simple (infoshow, rt.info_expander,
 		(HookFunc)_e2_mkdirdlg_set_expander_hook, rt.info_expander);
 
 	g_signal_connect (G_OBJECT (rt.info_expander), "notify::expanded",
 		G_CALLBACK (_e2_mkdirdlg_expander_toggled_cb), &rt);
-	g_signal_connect (G_OBJECT (rt.info_expander), "notify::expanded",
-		G_CALLBACK (_e2_mkdirdlg_expander_toggled_cb2), infoshow);
 
 	//info box is packed into vbox, not the expander, so that dialog-resizing works
 	rt.info_box = e2_widget_get_box (TRUE, FALSE, 1);
@@ -1056,12 +1080,15 @@ static gboolean _e2_task_mkdirQ (E2_ActionTaskData *qed)
 	if (connected == NULL)
 		connected = e2_option_get ("dialog-mkdir-connected");
 
+	gdk_threads_enter ();
+	//CHECKME seems that this needs BGL closed
 	rt.menu = e2_menu_create_options_menu (rt.dialog, NULL,
 		follow_pane, _e2_mkdirdlg_toggled1_cb, &rt,
 		suggest_dir, _e2_mkdirdlg_toggled3_cb, &rt,
 		show_last, _e2_mkdirdlg_toggled2_cb, &rt,
 		connected, e2_menu_control_cb, rt.dialog,
 		NULL);
+	gdk_threads_leave ();
 
 	//arrange menu cleanup
 	g_object_set_data_full (G_OBJECT (rt.dialog), "menu", rt.menu,
@@ -1113,22 +1140,32 @@ static gboolean _e2_task_mkdirQ (E2_ActionTaskData *qed)
 	gtk_widget_show_all (vbox);
 	if (!e2_option_bool_get_direct (infoshow))
 		gtk_widget_hide (rt.info_box);
-	rt.idle_id = g_idle_add ((GSourceFunc)_e2_mkdirdlg_deferred_change_cb, &rt);
+
+	_e2_mkdirdlg_deferred_change_cb (&rt);	//init dialog widgets
 
 	gdk_threads_enter ();
+#ifdef GTK3_COMBO_FIX
+	gtk_widget_grab_focus (gtk_bin_get_child (GTK_BIN(rt.combo)));
+#endif
 	e2_dialog_setup (rt.dialog, app.main_window);
 	e2_dialog_run (rt.dialog, NULL, E2_DIALOG_DONT_SHOW_ALL);
 	gdk_threads_leave ();
 
-	//do this after showing, to reduce repetition
-	g_signal_connect (G_OBJECT (rt.combo), "changed",
+	//connect signal after showing, to reduce repetition during setup
+	GtkWidget *entry =
+#ifdef USE_GTK2_14
+		gtk_bin_get_child (GTK_BIN (rt.combo));
+#else
+		GTK_BIN (rt.combo)->child;
+#endif
+	g_signal_connect (GTK_EDITABLE(entry), "changed",
 		G_CALLBACK (_e2_mkdirdlg_changed_cb), &rt);
 
 	*qed->status = E2_TASK_PAUSED;
 	e2_dialog_wait (rt.dialog, FALSE, TRUE, FALSE, FALSE);  //CHECKME TRUE maincontext ?
 	*qed->status = E2_TASK_RUNNING;
 
-	if (GTK_IS_DIALOG(rt.dialog))
+	if (GTK_IS_DIALOG (rt.dialog))
 	{	//not explicitly closed by the user
 		gdk_threads_enter ();
 		gtk_widget_destroy (rt.dialog);
