@@ -1,6 +1,6 @@
-/* $Id: e2_select_image_dialog.c 1987 2009-10-14 00:41:33Z tpgww $
+/* $Id: e2_select_image_dialog.c 2377 2011-06-11 00:16:34Z tpgww $
 
-Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
+Copyright (C) 2003-2011 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -14,8 +14,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 /**
@@ -153,8 +152,13 @@ static void _e2_sidlg_activated_cb (GtkIconView *iconview, GtkTreePath *path,
 
 @return
 */
-static void _e2_sidlg_page_switch_cb (GtkNotebook *notebook, GtkNotebookPage *page,
-		guint page_num, E2_SID_Runtime *rt)
+static void _e2_sidlg_page_switch_cb (GtkNotebook *notebook,
+#ifdef USE_GTK3_0
+	GtkWidget *page,
+#else
+	GtkNotebookPage *page,
+#endif
+	guint page_num, E2_SID_Runtime *rt)
 {
 	rt->page = page_num;
 	gtk_widget_set_sensitive (rt->dir_chooser, (page_num == 0));
@@ -375,26 +379,42 @@ static gboolean _e2_sidlg_fill_stock_store (GtkListStore *store)
 	GSList *ids = gtk_stock_list_ids ();
 	if (ids != NULL)
 	{
+#ifdef USE_GTK3_0
+		GtkStyleContext *context = gtk_widget_get_style_context (app.main_window);
+#else
 		GtkStyle *style = gtk_widget_get_style (app.main_window);
 		GtkTextDirection dir = gtk_widget_get_direction (app.main_window);
+#endif
 		GSList *member;
 
 		ids = g_slist_sort (ids, (GCompareFunc) strcmp);
 		for (member = ids; member != NULL; member = g_slist_next (member))
 		{
-			GtkIconSet *iset = gtk_style_lookup_icon_set (style, (const gchar*) member->data);
+			GtkIconSet *iset;
+#ifdef USE_GTK3_0
+			iset = gtk_style_context_lookup_icon_set (context,
+									(const gchar*) member->data);
+#else
+			iset = gtk_style_lookup_icon_set (style, (const gchar*) member->data);
+#endif
 			if (iset != NULL)
 			{
-				GdkPixbuf *pix = gtk_icon_set_render_icon (iset, style, dir,
-							GTK_STATE_NORMAL, GTK_ICON_SIZE_LARGE_TOOLBAR, NULL, NULL);
-				if (pix != NULL)
+				GdkPixbuf *pxb;
+#ifdef USE_GTK3_0
+				pxb = gtk_icon_set_render_icon_pixbuf (iset, context,
+					GTK_ICON_SIZE_LARGE_TOOLBAR);
+#else
+				pxb = gtk_icon_set_render_icon (iset, style, dir, GTK_STATE_NORMAL,
+					GTK_ICON_SIZE_LARGE_TOOLBAR, NULL, NULL);
+#endif
+				if (pxb != NULL)
 				{
 					GtkTreeIter iter;
 					gtk_list_store_insert_with_values (store, &iter, -1,
-						PIXBUF_COL, pix,
+						PIXBUF_COL, pxb,
 						TEXT_COL, (gchar*) member->data + 4,	//skip leading "gtk-"
 						-1);
-					g_object_unref (G_OBJECT (pix));
+					g_object_unref (G_OBJECT (pxb));
 				}
 			}
 			g_free (member->data);
@@ -450,8 +470,12 @@ static GtkWidget *_e2_sidlg_create_stock_icon_browser (E2_SID_Runtime *rt)
 	PangoGravity grav = pango_gravity_get_for_matrix (matrix);
 	if PANGO_GRAVITY_IS_VERTICAL (grav)
 	{
+# ifdef USE_GTK3_0
+		gtk_orientable_set_orientation (GTK_ORIENTABLE (rt->stockview), GTK_ORIENTATION_HORIZONTAL);
+# else
 		gtk_icon_view_set_orientation (rt->stockview, GTK_ORIENTATION_HORIZONTAL);
 		//CHECKME how to set item height ?
+# endif
 	}
 	else
 	{
@@ -507,8 +531,12 @@ static GtkWidget *_e2_sidlg_create_custom_icon_browser (E2_SID_Runtime *rt)
 	PangoGravity grav = pango_gravity_get_for_matrix (matrix);
 	if PANGO_GRAVITY_IS_VERTICAL (grav)
 	{
+# ifdef USE_GTK3_0
+		gtk_orientable_set_orientation (GTK_ORIENTABLE (rt->customview), GTK_ORIENTATION_HORIZONTAL);
+# else
 		gtk_icon_view_set_orientation (rt->customview, GTK_ORIENTATION_HORIZONTAL);
 		//CHECKME how to set item height ?
+# endif
 	}
 	else
 	{
@@ -603,14 +631,15 @@ static gint _e2_sidlg_show_current (gboolean checkboth, E2_SID_Runtime *rt)
 
 /**
 @brief create icon-selection dialog
-
+Assumes BGL closed
 @param parent the main config-dialog widget
 @param name dialog title string
 @param icon gtk-stock-icon name, or path to icon file, taken from tree-option store
+@param event data struct for the event which initiated dialog creation
 
 @return the dialog widget, or NULL if error occurred
 */
-GtkWidget *e2_sid_create (GtkWidget *parent, const gchar *name, gchar *icon)
+GtkWidget *e2_sid_create (GtkWidget *parent, const gchar *name, gchar *icon, GdkEventButton *event)
 {
 	//get or create and inititialize select image dialog runtime object and
 	//ensure that only one runtime object (and only one dialog) exists
@@ -660,8 +689,13 @@ GtkWidget *e2_sid_create (GtkWidget *parent, const gchar *name, gchar *icon)
 #endif
 //	gtk_container_set_border_width (GTK_CONTAINER (dialog_vbox), E2_PADDING);
 	//when the dialog is destroyed, free the runtime object
-	g_signal_connect (GTK_OBJECT(rt->dialog), "destroy",
-		G_CALLBACK (_e2_sidlg_destroy_cb), rt);
+	g_signal_connect (
+#ifdef USE_GTK3_0
+	G_OBJECT (rt->dialog),
+#else
+	GTK_OBJECT (rt->dialog),
+#endif
+		"destroy", G_CALLBACK (_e2_sidlg_destroy_cb), rt);
 	//ensure this dialog is destroyed with the parent widget
 	if (parent != NULL)
 		g_object_set_data_full (G_OBJECT (parent), rt->name, rt,
@@ -673,7 +707,7 @@ GtkWidget *e2_sid_create (GtkWidget *parent, const gchar *name, gchar *icon)
 #else
 		initial_dir != NULL
 #endif
-		&& (e2_utils_get_modifiers () & GDK_CONTROL_MASK))
+		&& (event->state & GDK_CONTROL_MASK))
 	{
 		//<Ctrl> press always opens default icons dir
 #ifdef E2_VFSTMP
