@@ -1,6 +1,6 @@
-/* $Id: e2_tree_dialog.c 1256 2008-09-20 10:59:13Z tpgww $
+/* $Id: e2_tree_dialog.c 2404 2011-10-30 23:55:23Z tpgww $
 
-Copyright (C) 2007-2008 tooar <tooar@gmx.net>
+Copyright (C) 2007-2011 tooar <tooar@emelfm2.net>
 Portions copyright (C) 1996-2007 Steve Baker <ice@mama.indstate.edu>
 
 This file is part of emelFM2.
@@ -15,8 +15,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 #include "e2_tree_dialog.h"
@@ -79,8 +78,10 @@ typedef struct _E2_TreeDialogRuntime
 #ifndef STRICT_HIDE
 	gboolean strict_hide;	//TRUE to prevent display of hidden ancestors
 #endif
-	GHashTable *dirhash;	//for contructing dependcies when store is filled
+	GHashTable *dirhash;	//for contructing dependencies when store is filled
 	ViewInfo *view;			//data for displayed file pane
+	gchar *opendir;			//path of dir from which to shallow-scan
+	pthread_t threadID;		//ID of cancellable thread doing a treewalk
 //	inodata *itable[256];	//NULL-terminated circular array of data structs, for preventing symlink circularity
 } E2_TreeDialogRuntime;
 
@@ -102,6 +103,8 @@ static gint window_height = -1;
 static void _e2_treedlg_row_expanded_cb (GtkTreeView *treeview, GtkTreeIter *iter,
 	GtkTreePath *path, E2_TreeDialogRuntime *rt);
 #endif
+static void _e2_treedlg_response_cb (GtkDialog *dialog, gint response,
+	E2_TreeDialogRuntime *rt);
 gchar *_e2_treedlg_make_path (GtkTreeModel *model, GtkTreePath *path);
 
   /*********************/
@@ -653,6 +656,7 @@ static E2_TwResult _e2_treedlg_twcb_scan (VPATH *dirpath,
 				if (tp != NULL)
 				{
 					gtk_tree_model_get_iter (GTK_TREE_MODEL (rt->store), &iter, tp);
+					gtk_tree_path_free (tp);
 					guint iterflags;
 					gtk_tree_model_get (GTK_TREE_MODEL (rt->store), &iter, DIRFLAGS, &iterflags, -1);
 					iterflags |= E2_TRDLG_SCANNED;	//CHECKME used ??
@@ -691,7 +695,7 @@ static E2_TwResult _e2_treedlg_twcb_scan (VPATH *dirpath,
 	if (retval == E2TW_CONTINUE)
 	{
 		//at dialog startup, the root path is not yet scanned FIXME more elegant way to handle this
-		if (g_str_equal (dirpath, G_DIR_SEPARATOR_S))
+		if (!strcmp (dirpath, G_DIR_SEPARATOR_S))
 			return retval;
 
 		gchar *lastseg = strrchr (VPSTR (dirpath), G_DIR_SEPARATOR);
@@ -718,6 +722,7 @@ static E2_TwResult _e2_treedlg_twcb_scan (VPATH *dirpath,
 					if (tp != NULL)
 					{
 						gtk_tree_model_get_iter (GTK_TREE_MODEL (rt->store), &iter, tp);
+						gtk_tree_path_free (tp);
 						gtk_tree_model_get (GTK_TREE_MODEL (rt->store), &iter, DIRFLAGS, &iterflags, -1);
 						if (iterflags & E2_TRDLG_STORED)
 							return retval;
@@ -756,9 +761,10 @@ static E2_TwResult _e2_treedlg_twcb_scan (VPATH *dirpath,
 				GtkTreeModel *model = GTK_TREE_MODEL (rt->store);
 				if (gtk_tree_model_get_iter (model, &parent, tp))
 				{
-					gchar *utf, *key;
+					gchar *local, *utf, *key;
 					//store data for the item
-					utf = F_FILENAME_FROM_LOCALE (lastseg+sizeof(gchar));
+					local = lastseg + sizeof(gchar);
+					utf = F_FILENAME_FROM_LOCALE (local);
 #ifdef USE_GTK2_8
 					key = g_utf8_collate_key_for_filename (utf, -1);
 #else
@@ -781,7 +787,7 @@ static E2_TwResult _e2_treedlg_twcb_scan (VPATH *dirpath,
 #endif
 						-1);
 
-					F_FREE (utf);
+					F_FREE (utf, local);
 					g_free (key);
 
 					//remember this new addition
@@ -834,6 +840,7 @@ static E2_TwResult _e2_treedlg_twcb_scan (VPATH *dirpath,
 						gtk_tree_store_set (rt->store, &parent,
 						TXTSET, TRUE, TXTCOLOR, hidden_color, -1);
 				}
+				gtk_tree_path_free (tp);
 				retval = E2TW_SKIPSUB;
 			}
 #endif
@@ -849,6 +856,63 @@ static E2_TwResult _e2_treedlg_twcb_scan (VPATH *dirpath,
 		printd (DEBUG, "Error code for %s is %d", dirpath, errno);
 	return retval;  //must report (some/all?) errors, else it keeps trying & hangs
 }
+
+#ifdef ENABLE_FULL_TREE
+/**
+@brief thread-function to initiate a full treewalk
+@param rt pointer to dialog data
+@return pointerised TRUE/FALSE reflecting the result of the treewalk
+*/
+static gpointer _e2_treedlg_do_full_scan (E2_TreeDialogRuntime *rt)
+{
+	e2_utils_block_thread_signals ();	//block all allowed signals to this thread
+	pthread_setcanceltype (PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
+#ifdef E2_VFS
+	VPATH ddata = { G_DIR_SEPARATOR_S, NULL };
+#endif
+	gboolean success =
+#ifdef E2_VFS
+		e2_fs_tw (&ddata, _e2_treedlg_twcb_scan, rt, -1,
+#else
+		e2_fs_tw (G_DIR_SEPARATOR_S, _e2_treedlg_twcb_scan, rt, -1,
+#endif
+#ifdef WARNBADS
+			E2TW_QT |
+#else
+			E2TW_XQT |
+#endif
+			E2TW_ONLYDIR E2_ERR_NONE());
+	return GINT_TO_POINTER (success);
+}
+#endif
+/**
+@brief thread-function to initiate a shallow (2-level) treewalk
+@param rt pointer to dialog data
+@return pointerised TRUE/FALSE reflecting the result of the treewalk
+*/
+static gpointer _e2_treedlg_do_shallow_scan (E2_TreeDialogRuntime *rt)
+{
+	e2_utils_block_thread_signals ();	//block all allowed signals to this thread
+	pthread_setcanceltype (PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
+#ifdef E2_VFS
+	VPATH ddata = { rt->opendir, rt->view->spacedata };
+#endif
+	gboolean success =
+#ifdef E2_VFS
+		e2_fs_tw (&ddata, _e2_treedlg_twcb_scan, rt, 2,
+#else
+		e2_fs_tw (rt->opendir, _e2_treedlg_twcb_scan, rt, 2,
+#endif
+#ifdef WARNBADS
+			E2TW_QT |
+#else
+			E2TW_XQT |
+#endif
+			E2TW_ONLYDIR E2_ERR_NONE());
+
+	return GINT_TO_POINTER (success);
+}
+
 /**
 @brief populate empty treestore for directories-tree dialog
 Directories data are obtained and stored for all dirs in @a startpath
@@ -881,25 +945,29 @@ static void	_e2_treedlg_fill_store (gchar *startpath, E2_TreeDialogRuntime *rt)
 # ifdef E2_VFSTMP
 	//FIXME vfs, and other namespaces
 # endif
+	gpointer result;
 	gdk_threads_leave ();
-	//if (!
-		e2_fs_tw (G_DIR_SEPARATOR_S, _e2_treedlg_twcb_scan, rt, -1,
-#ifdef WARNBADS
-			E2TW_QT |
-#else
-			E2TW_XQT |
-#endif
-			E2TW_ONLYDIR E2_ERR_NONE())
-	//)
-	//{
-		//FIXME handle error
-	//}
-	//}
+	if (pthread_create (&rt->threadID, NULL, _e2_treedlg_do_full_scan, rt) == 0)
+	{
+		pthread_join (thisID, &result);
+		rt->threadID = 0;
+		if (result == NULL)
+			printd (WARN, "Error in tree-scan process");
+	}
+	else
+	{
+		result = NULL;
+		printd (WARN, "Failed to create tree-scan thread");
+	}
 	gdk_threads_enter ();
+	if (result == NULL)
+	{
+		//FIXME do what ?
+	}
 #else
 # ifndef STRICT_HIDE
 	//when doing this expansion, we don't want to cleanup each level individually
-	gboolean wasnot_hidden =  (rt->strict_hide) ? FALSE : !rt->show_hidden;
+	gboolean wasnot_hidden = (rt->strict_hide) ? FALSE : !rt->show_hidden;
 	if (wasnot_hidden)
 		rt->show_hidden = TRUE;
 # endif //ndef STRICT_HIDE
@@ -920,13 +988,14 @@ static void	_e2_treedlg_fill_store (gchar *startpath, E2_TreeDialogRuntime *rt)
 	guint i, count = g_strv_length (split);
 	if (count > 1)	//should always pass for a valid startpath string
 	{
-		gchar *freeme, *madepath = g_strdup ("");
+		gchar *madepath = g_strdup ("");
 		for (i = 1; i < count; i++)
 		{
 			if (*split[i] != '\0')
 			{
-				freeme = madepath;
+				gchar *freeme = madepath;
 				madepath = g_strconcat (madepath, G_DIR_SEPARATOR_S, split[i], NULL);
+				g_free (freeme);
 				GtkTreeRowReference *ref = g_hash_table_lookup (rt->dirhash, madepath);
 				tp = (ref == NULL) ?	//should pass this test only in first loop
 					_e2_treedlg_infill (rt->store, madepath, rt->dirhash):
@@ -935,7 +1004,7 @@ static void	_e2_treedlg_fill_store (gchar *startpath, E2_TreeDialogRuntime *rt)
 				{
 					gtk_tree_model_get_iter (GTK_TREE_MODEL (rt->store), &iter, tp);
 					_e2_treedlg_row_expanded_cb (GTK_TREE_VIEW (rt->treeview), &iter, tp, rt);
-					g_free (freeme);
+					gtk_tree_path_free (tp);
 				}
 				else
 				{
@@ -967,6 +1036,7 @@ static void	_e2_treedlg_fill_store (gchar *startpath, E2_TreeDialogRuntime *rt)
 	g_object_unref (G_OBJECT (rt->store));
 #ifdef ENABLE_FULL_TREE
 	g_hash_table_destroy (rt->dirhash);
+	rt->dirhash = NULL;
 #endif
 }
 
@@ -995,11 +1065,11 @@ static void _e2_treedlg_set_menu_position (GtkMenu *menu,
 {
 	gint left, top;
 	gtk_window_get_position (GTK_WINDOW (rt->dialog), &left, &top);
-	GtkAllocation alloc =
-#ifdef USE_GTK2_99
-		gtk_widget_get_allocation (rt->treeview);
+	GtkAllocation alloc;
+#ifdef USE_GTK2_18
+	gtk_widget_get_allocation (rt->treeview, &alloc);
 #else
-		rt->treeview->allocation;
+	alloc = rt->treeview->allocation;
 #endif
 	*x = left + alloc.x + alloc.width/2;
 	*y = top + alloc.y +alloc.height/2 - 30;
@@ -1013,7 +1083,7 @@ static void _e2_treedlg_set_menu_position (GtkMenu *menu,
 
 @return
 */
-static void _e2_treedlg_copy_cb (GtkWidget *widget, GtkTreeView *treeview)
+static void _e2_treedlg_copy_cb (GtkMenuItem *widget, GtkTreeView *treeview)
 {
 	GtkTreeIter iter;
 	GtkTreeModel *model;
@@ -1032,12 +1102,12 @@ static void _e2_treedlg_copy_cb (GtkWidget *widget, GtkTreeView *treeview)
 /**
 @brief tree collapse-all callback
 
-@param widget UNUSED the menmu-item widget which was selected
+@param widget UNUSED the menu-item widget which was selected
 @param treeview treeview to which the menu belongs
 
 @return
 */
-static void _e2_treedlg_collapse_all_cb (GtkWidget *widget, GtkTreeView *treeview)
+static void _e2_treedlg_collapse_all_cb (GtkMenuItem *widget, GtkTreeView *treeview)
 {
 	gtk_tree_view_collapse_all (treeview);
 	GtkTreePath *tp = gtk_tree_path_new_first ();
@@ -1052,7 +1122,7 @@ This can also be called directly, in which case BGL must be closed
 
 @return
 */
-static void _e2_treedlg_refresh_cb (GtkWidget *widget, E2_TreeDialogRuntime *rt)
+static void _e2_treedlg_refresh_cb (GtkMenuItem *widget, E2_TreeDialogRuntime *rt)
 {
 	//FIXME use filter-model and just re-filter ?
 	GtkTreeModel *model;
@@ -1082,19 +1152,19 @@ static void _e2_treedlg_refresh_cb (GtkWidget *widget, E2_TreeDialogRuntime *rt)
 /**
 @brief toggle display of hidden ancestors of visible dirs
 This is performed when the context-menu item 'strict hiding' is selected
-@param menuitem the clicked menu widget
+@param menuitem the activated menu widget
 @param rt ptr to data struct for the dialog
 
 @return
 */
-static void _e2_treedlg_toggle_strict_cb (GtkWidget *menuitem, E2_TreeDialogRuntime *rt)
+static void _e2_treedlg_toggle_strict_cb (GtkMenuItem *menuitem, E2_TreeDialogRuntime *rt)
 {
 	rt->strict_hide = gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (menuitem));
 	_e2_treedlg_refresh_cb (NULL, rt);
 }
 #endif
 /**
-@brief construct and show dialog context menu
+@brief construct and pop up destroyable context-menu for the dialog
 
 @param treeview the widget where the click happened
 @param event_button which mouse button was clicked (0 for a menu key)
@@ -1123,17 +1193,12 @@ static void _e2_treedlg_show_context_menu (GtkWidget *treeview,
 	item = e2_menu_add_check (menu, _("_Strict hiding"), rt->strict_hide,
 		_e2_treedlg_toggle_strict_cb, rt);
 	if (!rt->show_hidden)
-#ifdef USE_GTK2_12TIPS
-		gtk_widget_set_tooltip_text (
-#else
-		e2_widget_set_tooltip (NULL,
-#endif
-		item,
+		e2_widget_set_safetip (item,
 		_("If active, hidden ancestors of visible directories will not be displayed"));
 	gtk_widget_set_sensitive (item, !rt->show_hidden);
 #endif
 	g_signal_connect (G_OBJECT (menu), "selection-done",
-		G_CALLBACK (e2_menu_destroy_cb), NULL);
+		G_CALLBACK (e2_menu_selection_done_cb), NULL);
 
 	if (event_button == 0)
 		gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
@@ -1199,7 +1264,7 @@ static gboolean _e2_treedlg_button_press_cb (GtkWidget *treeview,
 static gboolean _e2_treedlg_popup_menu_cb (GtkWidget *treeview,
 	E2_TreeDialogRuntime *rt)
 {
-	gint event_time = gtk_get_current_event_time ();
+	guint32 event_time = gtk_get_current_event_time ();
 	_e2_treedlg_show_context_menu (treeview, 0, event_time, rt);
 	return TRUE;
 }
@@ -1230,7 +1295,7 @@ static void _e2_treedlg_row_activated_cb (
 		if (newpath != NULL)
 		{
 			//show that in associated filepane
-			E2_PaneRuntime *rt = (view == &app.pane1_view) ? &app.pane1 : &app.pane2 ;
+			E2_PaneRuntime *rt = (E2_PaneRuntime *)view;
 			e2_pane_change_dir (rt, newpath);
 			g_free (newpath);
 		}
@@ -1238,9 +1303,9 @@ static void _e2_treedlg_row_activated_cb (
 }
 #ifndef ENABLE_FULL_TREE
 /**
-@brief callback for treeview "row-expanded" signal
-If the expanded row is not 'ready', check each child dir there. Any child
-not 'scanned', will be scanned and markes as such.
+@brief callback for treeview "row-expanded" signal, also called from _e2_treedlg_fill_store()
+If the expanded row is not 'ready', check each child dir there. Any child not
+'scanned', will be scanned and marked as such.
 Downstream code expects @a treeview to be connected to a treemodel
 This expects BGL to be closed, and the dialog window must exist
 @param treeview the object on which the signal was emitted
@@ -1266,6 +1331,10 @@ static void _e2_treedlg_row_expanded_cb (GtkTreeView *treeview, GtkTreeIter *ite
 			e2_dialog_set_cursor (rt->dialog, GDK_WATCH);
 		//almost certainly ok without reference, as any addtions are later in the store, but ...
 		GtkTreeRowReference *ref = gtk_tree_row_reference_new (model, path);
+#ifdef DEBUG_MESSAGES
+		if (ref == NULL)
+			printd (WARN, "NULL rowref for path");
+#endif
 		//check each child dir there
 		gchar *thisdir = _e2_treedlg_make_path (model, path);	//should never be NULL
 //FIXME disconnect view from store without collapsing view
@@ -1286,51 +1355,61 @@ static void _e2_treedlg_row_expanded_cb (GtkTreeView *treeview, GtkTreeIter *ite
 		if (wasnot_hidden)
 			rt->show_hidden = TRUE;
 #endif
-#ifdef E2_VFS
-		VPATH ddata = { thisdir, rt->view->spacedata };
-#endif
+		rt->opendir = thisdir;
+		gpointer result;
 		gdk_threads_leave ();
-		//if (!
-#ifdef E2_VFS
-			e2_fs_tw (&ddata, _e2_treedlg_twcb_scan, rt, 2,
-#else
-			e2_fs_tw (thisdir, _e2_treedlg_twcb_scan, rt, 2,
-#endif
-#ifdef WARNBADS
-			E2TW_QT |
-#else
-			E2TW_XQT |
-#endif
-			E2TW_ONLYDIR E2_ERR_NONE());
-		//)
-		//{
-			//FIXME handle error
-		//}
-		gdk_threads_enter ();
-		GtkTreePath *tp = gtk_tree_row_reference_get_path (ref);
-		if (tp != NULL)
+		if (pthread_create (&rt->threadID, NULL,
+			(void*(*)(void*))_e2_treedlg_do_shallow_scan, rt) == 0)
 		{
-			gtk_tree_model_get_iter (model, iter, tp);
-			//mark expanded iter as 'ready'
-			gtk_tree_model_get (model, iter, DIRFLAGS, &iterflags, -1);
-			iterflags |= E2_TRDLG_GRANDSCANNED;
-			gtk_tree_store_set (rt->store, iter, DIRFLAGS, iterflags, -1);
-#ifndef STRICT_HIDE
-			if (wasnot_hidden)
-			{	//cleanup erroneous hidden leaves
-				//FIXME handle .hidden1/.hidden2/displayed, .hidden2 is removed but then .hidden1 is kept
-				_e2_treedlg_clean_hidden (GTK_TREE_MODEL (rt->store), iter, rt->dirhash);
-				rt->show_hidden = FALSE;
-			}
-#endif
+			pthread_join (rt->threadID, &result);
+			rt->threadID = 0;
+			if (result == NULL)
+				printd (WARN, "Error in tree-scan process");
 		}
 		else
 		{
-			printd (DEBUG, "oops, at 4 failed to get iter for %s", thisdir);
+			result = NULL;
+			printd (WARN, "Failed to create tree-scan thread");
+		}
+		gdk_threads_enter ();
+		if (result == NULL)
+		{
+			//FIXME do what ?
+		}
+
+		if (G_LIKELY(ref != NULL))
+		{
+			GtkTreePath *tp = gtk_tree_row_reference_get_path (ref);
+			if (G_LIKELY(tp != NULL))
+			{
+				gtk_tree_model_get_iter (model, iter, tp);
+				gtk_tree_path_free (tp);
+				//mark expanded iter as 'ready'
+				gtk_tree_model_get (model, iter, DIRFLAGS, &iterflags, -1);
+				iterflags |= E2_TRDLG_GRANDSCANNED;
+				gtk_tree_store_set (rt->store, iter, DIRFLAGS, iterflags, -1);
+#ifndef STRICT_HIDE
+				if (wasnot_hidden)
+				{	//cleanup erroneous hidden leaves
+					//FIXME handle .hidden1/.hidden2/displayed, .hidden2 is removed but then .hidden1 is kept
+					_e2_treedlg_clean_hidden (GTK_TREE_MODEL (rt->store), iter, rt->dirhash);
+					rt->show_hidden = FALSE;
+				}
+#endif
+			}
+			else
+			{
+				printd (DEBUG, "oops, at 4 failed to get iter for %s", thisdir);
+				g_hash_table_remove (rt->dirhash, thisdir);
+			}
+			gtk_tree_row_reference_free (ref);
+		}
+		else
+		{
+			printd (DEBUG, "oops, at 5 failed to get iter for %s", thisdir);
 			g_hash_table_remove (rt->dirhash, thisdir);
 		}
 		g_free (thisdir);
-		gtk_tree_row_reference_free (ref);
 		//when filling whole store this will leave the sort column unsorted ??
 		gtk_tree_sortable_set_sort_column_id (sortable, sortcol, GTK_SORT_ASCENDING);
 
@@ -1347,6 +1426,24 @@ static void _e2_treedlg_row_expanded_cb (GtkTreeView *treeview, GtkTreeIter *ite
 }
 #endif //ndef ENABLE_FULL_TREE
 /**
+@brief backup and cleanup during the destruction of the view related to a dialog
+@param object UNUSED the object being destroyed
+@param rt pointer to data struct for the dialog
+@return
+*/
+static void _e2_treedlg_destroy_cb (
+#ifdef USE_GTK3_0
+	GtkWidget *object,
+#else
+	GtkObject *object,
+#endif
+	E2_TreeDialogRuntime *rt)
+{
+	g_signal_handlers_disconnect_by_func ((gpointer)rt->dialog,
+		_e2_treedlg_response_cb, rt); //no double-handling
+	_e2_treedlg_response_cb (GTK_DIALOG (rt->dialog), 0, rt);
+}
+/**
 @brief handle button click, window-close etc for directory-tree dialog
 This is the callback for response signals emitted from @a dialog
 @param dialog UNUSED the dialog where the response was generated
@@ -1368,22 +1465,34 @@ static void _e2_treedlg_response_cb (GtkDialog *dialog, gint response,
 			break;
 		default:
 			gtk_widget_hide (rt->dialog);
+			if (rt->threadID != 0)
+			{
+				pthread_cancel (rt->threadID);
+				usleep (20000);
+			}
+
 #ifndef ENABLE_FULL_TREE
-			g_hash_table_destroy (rt->dirhash);
+			if (rt->dirhash != NULL)
+				g_hash_table_destroy (rt->dirhash);
 #endif
 			show_hidden = rt->show_hidden;	//backups for later used this session
 #ifndef STRICT_HIDE
 			strict_hide = rt->strict_hide;
 #endif
-#ifdef USE_GTK2_99
-			GtkAllocation alloc = gtk_widget_get_allocation (rt->dialog);
+#ifdef USE_GTK2_18
+			GtkAllocation alloc;
+			gtk_widget_get_allocation (rt->dialog, &alloc);
 			window_width = alloc.width;
 			window_height = alloc.height;
 #else
 			window_width = rt->dialog->allocation.width;
 			window_height = rt->dialog->allocation.height;
 #endif
+			//FIXME use relevant container box
+			g_signal_handlers_disconnect_by_func ((gpointer)rt->view->treeview,
+				_e2_treedlg_destroy_cb, rt);
 			gtk_widget_destroy (rt->dialog);
+
 			DEALLOCATE (E2_TreeDialogRuntime, rt);
 			break;
 	}
@@ -1408,8 +1517,11 @@ static gpointer _e2_tree_dialog_run (ViewInfo *view)
 #endif
 //	memset (rt->itable, 0, sizeof(rt->itable));
 
-	gchar *title = (view == &app.pane1_view) ? _("pane 1 navigator") : _("pane 2 navigator") ;
-	rt->dialog = e2_dialog_create (NULL, NULL, title, _e2_treedlg_response_cb, rt);
+	gchar *title = (view == &app.pane1.view) ? _("pane 1 navigator") : _("pane 2 navigator") ;
+	gdk_threads_enter ();
+	rt->dialog = e2_dialog_create (NULL, NULL, title,
+		(ResponseFunc)_e2_treedlg_response_cb, rt);
+	gdk_threads_leave ();
 	//scrolled window for the treeview
 	GtkWidget *sw = e2_widget_add_sw (
 #ifdef USE_GTK2_14
@@ -1458,26 +1570,27 @@ static gpointer _e2_tree_dialog_run (ViewInfo *view)
 		NULL); //GtkDestroyNotify destroy
 
 	//relate initial size to last-used, or if first, to filepanes size
-#ifdef USE_GTK2_99
+#ifdef USE_GTK2_18
 	if (window_width == -1 || window_height == -1)
 	{
-		GtkAllocation alloc = gtk_widget_get_allocation (app.window.panes_paned);
+		GtkAllocation alloc;
+		gtk_widget_get_allocation (app.window.panes_paned, &alloc);
 #endif
 		if (window_width == -1)
 			window_width =
-#ifdef USE_GTK2_99
+#ifdef USE_GTK2_18
 			MIN(300, alloc.width/2);
 #else
 			MIN(300, app.window.panes_paned->allocation.width/2);
 #endif
 		if (window_height == -1)
 			window_height =
-#ifdef USE_GTK2_99
+#ifdef USE_GTK2_18
 			alloc.height;
 #else
 			app.window.panes_paned->allocation.height;
 #endif
-#ifdef USE_GTK2_99
+#ifdef USE_GTK2_18
 	}
 #endif
 	gtk_window_resize (GTK_WINDOW(rt->dialog), window_width, window_height);
@@ -1497,6 +1610,14 @@ static gpointer _e2_tree_dialog_run (ViewInfo *view)
 	g_signal_connect (G_OBJECT (rt->treeview),  "row-expanded",
 		G_CALLBACK (_e2_treedlg_row_expanded_cb), rt);
 #endif
+	//dialog depends on valid ViewInfo* data
+	g_signal_connect (
+#ifdef USE_GTK3_0
+	G_OBJECT (view->treeview), //FIXME use relevant container box
+#else
+	GTK_OBJECT (view->treeview),
+#endif
+		"destroy", G_CALLBACK(_e2_treedlg_destroy_cb), rt);
 
 	rt->hiddenbtn = e2_dialog_add_custom_button_full
 		(rt->dialog, FALSE, E2_RESPONSE_USER1,
@@ -1504,10 +1625,10 @@ static gpointer _e2_tree_dialog_run (ViewInfo *view)
 		(rt->show_hidden) ? "hidden_noshow"E2ICONTB : "hidden_show"E2ICONTB,
 		_("Toggle display of hidden directories"), NULL, NULL);
 	E2_BUTTON_CLOSE.showflags |= E2_BTN_DEFAULT; //set default button
-	e2_dialog_show (rt->dialog, app.main_window, E2_DIALOG_CLOSELOCK,
-		&E2_BUTTON_CLOSE, NULL);
 
 	gdk_threads_enter ();
+	e2_dialog_show (rt->dialog, app.main_window, 0,	&E2_BUTTON_CLOSE, NULL);
+
 	//get the dir names
 	_e2_treedlg_fill_store (NULL, rt);
 	//show and select the startup row corresponding to displayed dir
@@ -1534,7 +1655,11 @@ can be slow
 gboolean e2_tree_dialog_show_action (gpointer from, E2_ActionRuntime *art)
 {
 	E2_PaneRuntime *rt = e2_pane_get_runtime (from, art->data, NULL);
-	g_thread_create ((GThreadFunc) _e2_tree_dialog_run, rt->view, FALSE, NULL);
+#ifdef USE_GLIB2_32
+	g_thread_new ("", (GThreadFunc) _e2_tree_dialog_run, &rt->view);
+#else
+	g_thread_create ((GThreadFunc) _e2_tree_dialog_run, &rt->view, FALSE, NULL);
+#endif
 	return TRUE;
 }
 
