@@ -1,6 +1,6 @@
-/* $Id: e2_view_dialog.c 1950 2009-10-01 22:38:16Z tpgww $
+/* $Id: e2_view_dialog.c 2478 2012-03-25 22:23:11Z tpgww $
 
-Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
+Copyright (C) 2004-2012 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -14,8 +14,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 #include "emelfm2.h"
@@ -24,6 +23,7 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #include "e2_view_dialog.h"
 #include "e2_textiter.h"
 #include "e2_task.h"
+#include "e2_print.h"
 #ifdef E2_MOUSECUSTOM
 # include "e2_mousebinding.h"
 #endif
@@ -449,13 +449,13 @@ static void _e2_view_dialog_lang_toggle (GtkToggleButton *button,
 #endif
 
 /**
-@brief read text file to be viewed
-This is used by both view and edit dialogs. Assumes BGL open/off
+@brief read text file to be viewed, and put it to created @a rt ->textbuffer
+This is used by both view and edit dialogs.
 Assumes the item being read is in curr_view i.e. no cd race from queueing
 Character encoding is performed if necessary and possible.
 @a rt ->filepath has the localised name of the file to be read. It will be
 replaced by a newly-allocated string, to be freed elsewhere
-Error messages and any macro-instigated dialog expect BGL open
+Assumes BGL is closed
 @param localfile data for item to read, localised path string, not neccessarily absolute
 @param rt dialog runtime data struct
 
@@ -476,12 +476,14 @@ gboolean e2_view_dialog_read_text (VPATH *localfile, E2_ViewDialogRuntime *rt)
 	if (e2_fs_access (localpath, R_OK E2_ERR_PTR()))	//traverse a link
 #endif
 	{
+		gdk_threads_leave ();
 		e2_fs_error_local (_("Cannot read '%s'"),
 #ifdef E2_VFS
 		&ddata E2_ERR_MSGL());
 #else
 		localpath E2_ERR_MSGL());
 #endif
+		gdk_threads_enter ();
 		E2_ERR_CLEAR
 		g_free (localpath);
 		return FALSE;
@@ -543,9 +545,7 @@ gboolean e2_view_dialog_read_text (VPATH *localfile, E2_ViewDialogRuntime *rt)
 			{	//we need to do a conversion
 				gchar *utfpath = F_FILENAME_FROM_LOCALE (localpath);
 				//substitute file for any name/path macro in command string, get language etc
-				gdk_threads_enter ();	//in case there's a dialog-macro in the command
 				gchar *utfcmd = e2_utils_expand_macros (utfconverter, utfpath);
-				gdk_threads_leave ();
 				F_FREE (utfpath, localpath);
 
 				if (utfcmd == NULL || utfcmd == GINT_TO_POINTER(1))
@@ -570,9 +570,7 @@ gboolean e2_view_dialog_read_text (VPATH *localfile, E2_ViewDialogRuntime *rt)
 					{
 						gchar *msg = g_strdup_printf
 						(_("Encoding conversion command '%s' failed"), utfcmd);
-						gdk_threads_enter ();
 						e2_output_print_error (msg, TRUE);
-						gdk_threads_leave ();
 						utfconverter = NULL;	//set flag for internal conversion fallback
 					}
 					g_free (utfcmd);
@@ -706,18 +704,12 @@ rewait:
 						default:
 //						case NO_TO_ALL: //cancel
 							if (GTK_IS_DIALOG (dialog)) //not explicitly closed by the user
-							{
-								gdk_threads_enter ();
 								gtk_widget_destroy (dialog);
-								gdk_threads_leave ();
-							}
 							g_free (localpath);
 							g_free (contents);
 							return FALSE;
 						case YES_TO_ALL: //test
-							gdk_threads_enter ();
 							e2_dialog_set_cursor (dialog, GDK_WATCH);
-							gdk_threads_leave ();
 							//iterate through each selection, try to convert
 							gboolean match = FALSE;
 							//get selected charset(s)' name
@@ -762,11 +754,9 @@ rewait:
 													drt.first_coding = charset;
 												match = TRUE;
 												//update lower label in dialog
-												gdk_threads_enter ();
 												gtk_label_set_text (GTK_LABEL(drt.msg_label),
 													_("The file encoding may be any of the selected ones"));
 												gtk_widget_set_sensitive (drt.open_btn, TRUE);
-												gdk_threads_leave ();
 												break;
 											}
 											else
@@ -775,9 +765,7 @@ rewait:
 										if (type == ENCODING_TYPECOUNT)
 										{
 											//uncheck the corresponding button
-											gdk_threads_enter ();
 											gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(drt.chk_btns[row][col]), FALSE);
-											gdk_threads_leave ();
 										}
 									}
 								}
@@ -786,22 +774,16 @@ rewait:
 							}
 							if (!match) //no match, update label accordingly
 							{
-								gdk_threads_enter ();
 								gtk_label_set_text (GTK_LABEL(drt.msg_label),
 									_("File encoding cannot be converted from any selected type"));
 								gtk_widget_set_sensitive (drt.open_btn, FALSE);
-								gdk_threads_leave ();
 								drt.first_coding = NULL;
 							}
 
-							gdk_threads_enter ();
 							e2_dialog_set_cursor (dialog, GDK_LEFT_PTR);
-							gdk_threads_leave ();
 							goto rewait;
 						case OK: //open
-							gdk_threads_enter ();
 							gtk_widget_destroy (dialog);
-							gdk_threads_leave ();
 							//proceed to conversion
 							if (drt.first_coding == NULL)
 								drt.first_coding = "ISO-8859-15";	//default
@@ -830,9 +812,7 @@ rewait:
 						if (strcmp (rt->charset, "CP1252"))	//that's generally a default when we can't find the real value
 						{
 							msg = g_strdup_printf (format, rt->charset, utf_error->message);
-							gdk_threads_enter ();
 							e2_output_print_error (msg, TRUE);
-							gdk_threads_leave ();
 						}
 						g_error_free (utf_error);
 					}
@@ -853,9 +833,7 @@ rewait:
 				if (utf == NULL)
 				{
 					msg = g_strdup_printf (format, defset, utf_error->message);
-					gdk_threads_enter ();
 					e2_output_print_error (msg, TRUE);
-					gdk_threads_leave ();
 					g_error_free (utf_error);
 					g_free (contents);	//need free() if file buffer allocated by malloc()
 					return FALSE;
@@ -879,6 +857,14 @@ rewait:
 		contents = NULL;
 	}
 
+	rt->textbuffer = gtk_text_buffer_new (NULL); //always create a buffer
+	if (contents)
+	{
+		gtk_text_buffer_set_text (rt->textbuffer, contents, -1);
+		g_free (contents);
+		printd (DEBUG, "Read text file charset is %s", rt->charset);
+	}
+
 	if (rt->localpath != NULL)
 		g_free (rt->localpath);
 	rt->localpath = localpath;	//we're ok, so remember the real path
@@ -886,23 +872,6 @@ rewait:
 	rt->spacedata = localfile->spacedata;
 #endif
 
-	//for some reason, the buffer sometimes takes a while to be created
-	while (!GTK_IS_TEXT_BUFFER (rt->textbuffer))
-		rt->textbuffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (rt->textview));
-
-	if (contents != NULL)
-	{
-		gtk_text_buffer_set_text (rt->textbuffer, (gchar *)contents, -1);
-		g_free (contents);
-	}
-
-	//put cursor at start of buffer, for searching etc
-	GtkTextIter start;
-	gtk_text_buffer_get_start_iter (rt->textbuffer, &start);
-	gtk_text_buffer_place_cursor (rt->textbuffer, &start);
-
-	printd (DEBUG, "Read text file charset is %s", rt->charset);
-
 	return TRUE;
 }
 /**
@@ -969,243 +938,6 @@ void e2_view_dialog_set_menu_position (GtkWidget *menu,
 	*y = top + alloc.y + alloc.height/2;
 	*push_in = FALSE;
 }
-#ifdef USE_GTK2_10
-/**
-@brief callback for "begin-print" signal on print-operation for selected or all text
-
-@param op the print operation
-@param context print context for the operation
-@param rt runtime data struct for the dialog
-
-@return
-*/
-static void _e2_view_dialog_begin_print_cb (GtkPrintOperation *op,
-		GtkPrintContext *context, E2_ViewDialogRuntime *rt)
-{
-	GtkTextIter start, end;
-	if (gtk_text_buffer_get_has_selection (rt->textbuffer))
-		gtk_text_buffer_get_selection_bounds (rt->textbuffer, &start, &end);
-	else
-		gtk_text_buffer_get_bounds (rt->textbuffer, &start, &end);
-
-	gchar *seltext = gtk_text_buffer_get_text (rt->textbuffer, &start, &end, FALSE);
-	g_strstrip (seltext);
-	if (*seltext != '\0')
-	{
-		PangoLayout *layout = gtk_print_context_create_pango_layout (context);
-		//remember it for other uses
-		g_object_set_data_full (G_OBJECT (context), "e2-print-layout", layout,
-			(GDestroyNotify) g_object_unref);	//cleanup when context is destroyed
-		PangoFontDescription *font_desc = gtk_widget_get_style (rt->textview)->font_desc;
-		pango_layout_set_font_description (layout, font_desc);
-		gdouble page_width = gtk_print_context_get_width (context); //in pixels
-		//don't assume printer device-unit is one point
-		gdouble dpi = gtk_print_context_get_dpi_x (context);
-		page_width = page_width * 72 / dpi; //now in points
-		pango_layout_set_width (layout, (gint) page_width * PANGO_SCALE);
-		pango_layout_set_text (layout, seltext, -1);
-
-		gdouble page_height = gtk_print_context_get_height (context); //in pixels
-		if (!pango_font_description_get_size_is_absolute (font_desc))
-		{
-			dpi = gtk_print_context_get_dpi_y (context);
-			page_height = page_height * 72 / dpi; //now in points
-		}
-		gint text_height = pango_font_description_get_size (font_desc) / PANGO_SCALE; //in points or pixels
-		gint lines_per_page = page_height / text_height;
-		gint line_count = pango_layout_get_line_count (layout);
-		gint page_count = (line_count - 1) / lines_per_page + 1;	//round upwards
-		gtk_print_operation_set_n_pages (op, page_count);
-
-		//setup page header with filename
-		PangoLayout *layout_name = gtk_print_context_create_pango_layout (context);
-		g_object_set_data_full (G_OBJECT (context), "e2-print-name", layout_name,
-			(GDestroyNotify) g_object_unref);	//cleanup when context is destroyed
-		pango_layout_set_font_description (layout_name, font_desc);
-		pango_layout_set_width (layout_name, (gint) page_width * PANGO_SCALE);
-		gchar *base = g_path_get_basename (rt->localpath);
-		gchar *page_title = F_FILENAME_FROM_LOCALE (base);
-		pango_layout_set_text (layout_name, page_title, -1);
-		g_free (base);
-		F_FREE (page_title, base);
-		//and for page counter, if > 1 page
-		if (page_count > 1)
-		{
-			PangoLayout *layout_num = gtk_print_context_create_pango_layout (context);
-			pango_layout_set_font_description (layout_num, font_desc);
-//			pango_layout_set_alignment (layout_right, PANGO_ALIGN_RIGHT);
-			g_object_set_data_full (G_OBJECT (context), "e2-print-page", layout_num,
-				(GDestroyNotify) g_object_unref);	//cleanup when context is destroyed
-		}
-	}
-	g_free (seltext);
-}
-/* *
-@brief callback for "end-print" signal on print-operation for selected or all text
-
-@param op UNUSDED the print operation
-@param context print context for the operation
-@param data UNUSED data specified when callback was connected
-
-@return
-*/
-/*static void _e2_view_dialog_end_print_cb (GtkPrintOperation *op,
-	GtkPrintContext *context, gpointer data)
-{
-} */
-/**
-@brief callback for "draw-page" signal on print-operation for selected or all text
-
-@param op UNUSDED the print operation
-@param context print context for the operation
-@param page_num 0-based index of printed pages count
-@param rt runtime data struct for the dialog
-
-@return
-*/
-static void _e2_view_dialog_draw_page_cb (GtkPrintOperation *op,
-	GtkPrintContext *context, gint page_num, E2_ViewDialogRuntime *rt)
-{
-//	PangoLayoutLine *line, *layout_lh, *layout_rh;
-	cairo_t *cr = gtk_print_context_get_cairo_context (context);
-	if (cr != NULL)
-	{
-		gdouble dpi;
-		//print header, comprising filename, and page-number if > 1 page
-		PangoFontDescription *font_desc = gtk_widget_get_style (rt->textview)->font_desc;
-		//CHECKME dpi_v instead of 72 ?
-		cairo_move_to (cr, 0, - 10 * 72 / 25.4);	//left margin, up 10 mm from top of text
-		PangoLayout *layout = g_object_get_data (G_OBJECT (context), "e2-print-name");
-		pango_cairo_show_layout (cr, layout);
-
-		gint n_pages;
-		g_object_get (G_OBJECT (op), "n-pages", &n_pages, NULL);
-		if (n_pages > 1)
-		{
-			PangoLayout *layout_right = g_object_get_data (G_OBJECT (context),
-				"e2-print-page");
-			gchar *page_text = g_strdup_printf ("%d / %d", page_num + 1, n_pages);
-			pango_layout_set_text (layout_right, page_text, -1);
-			g_free (page_text);
-			gdouble page_width = gtk_print_context_get_width (context); //in pixels
-			dpi = gtk_print_context_get_dpi_x (context);
-			page_width = page_width * 72 / dpi; //in points
-			gint layout_width;
-			pango_layout_get_size (layout_right, &layout_width, NULL);
-			//CHECKME dpi_v instead of 72 ?
-			cairo_move_to (cr,
-				page_width - layout_width / PANGO_SCALE, - 10 * 72 / 25.4); //up 10 mm from top of text
-			pango_cairo_show_layout (cr, layout_right);
-		}
-
-		layout = g_object_get_data (G_OBJECT (context), "e2-print-layout");
-		gdouble page_height = gtk_print_context_get_height (context);	//in pixels
-		if (!pango_font_description_get_size_is_absolute (font_desc))
-		{
-			dpi = gtk_print_context_get_dpi_y (context);
-			page_height = page_height / dpi * 72; //now in points
-		}
-		gint text_height = pango_font_description_get_size (font_desc) / PANGO_SCALE; //in pixels or points
-		guint lines_per_page = page_height / text_height;
-		gint line_count = pango_layout_get_line_count (layout);
-		guint last_line = (line_count > lines_per_page * (page_num + 1)) ?
-			lines_per_page * (page_num + 1) : line_count;
-		guint pageline, fileline;
-		for (pageline = 1, fileline = lines_per_page * page_num;	//CHECKME pageline=0 (move to tops of lines?)
-				fileline < last_line; pageline++, fileline++)
-		{
-			PangoLayoutLine *linelay = pango_layout_get_line (layout, fileline);
-			cairo_move_to (cr, 0, pageline * text_height);
-			pango_cairo_show_layout_line (cr, linelay);
-		}
-	}
-}
-/**
-@brief callback for "status-changed" signal on print-operation for selected or all text
-
-@param op the print operation
-@param data UNUSED data specified when callback was connected
-
-@return
-*/
-static void _e2_view_dialog_print_status_cb (GtkPrintOperation *op,
-	gpointer data)
-{
-	GtkPrintStatus pstat = gtk_print_operation_get_status (op);
-	switch (pstat)
-	{
-		case GTK_PRINT_STATUS_PENDING_ISSUE:
-		case GTK_PRINT_STATUS_FINISHED:	//also for end of preview
-		case GTK_PRINT_STATUS_FINISHED_ABORTED: //BAD for sync, maybe OK for async
-			g_object_unref (G_OBJECT (op));	//cleanup, ref => 2
-		default:
-			break;
-	}
-}
-/**
-@brief asynchronously print currently-selected text or all text if no selection
-
-@param menuitem UNUSED the activated widget, or NULL
-@param rt runtime struct to work on
-
-@return
-*/
-void e2_view_dialog_print_cb (GtkMenuItem *menuitem, E2_ViewDialogRuntime *rt)
-{
-//	static GtkPrintSettings *settings = NULL;
-	static GtkPageSetup *page_setup = NULL;
-
-	GtkPrintOperation *op = gtk_print_operation_new ();
-/*	too hard to properly manage refcount in all usage cases
-	if (settings != NULL)
-		//re-use the settings from a former print operation
-		gtk_print_operation_set_print_settings (op, settings);
-*/
-
-	if (page_setup == NULL)
-	{
-		page_setup = gtk_page_setup_new ();	//never cleaned
-		gtk_page_setup_set_top_margin (page_setup, 25.0, GTK_UNIT_MM);
-		gtk_page_setup_set_bottom_margin (page_setup, 20.0, GTK_UNIT_MM);
-		gtk_page_setup_set_left_margin (page_setup, 20.0, GTK_UNIT_MM);
-		gtk_page_setup_set_right_margin (page_setup, 20.0, GTK_UNIT_MM);
-	}
-
-	gtk_print_operation_set_default_page_setup (op, page_setup);
-	gtk_print_operation_set_allow_async (op, TRUE);
-
-	g_signal_connect (G_OBJECT (op), "begin-print",
-		G_CALLBACK (_e2_view_dialog_begin_print_cb), rt);
-//	g_signal_connect (G_OBJECT (op), "end-print",
-//		G_CALLBACK(_e2_view_dialog_end_print_cb), NULL);
-	g_signal_connect (G_OBJECT (op), "draw-page",
-		G_CALLBACK (_e2_view_dialog_draw_page_cb), rt);
-	g_signal_connect (op, "status-changed",
-		G_CALLBACK (_e2_view_dialog_print_status_cb), NULL);
-	GError *error = NULL;
-	GtkPrintOperationResult result = gtk_print_operation_run (op,
-		GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG, GTK_WINDOW (rt->dialog), &error);
-
-	switch (result)
-	{
-//		GObject *debug2;
-		case GTK_PRINT_OPERATION_RESULT_ERROR:
-			e2_output_print_error (error->message, FALSE);
-			g_error_free (error);
-/*		case GTK_PRINT_OPERATION_RESULT_APPLY:	//e.g. after sync preview
-		case GTK_PRINT_OPERATION_RESULT_IN_PROGRESS:
-	TOO HARD TO PROPERLY MANAGE SETTINGS REFCOUNT IN ALL USAGE CASES
-*/
-		default:
-//			if (result != GTK_PRINT_OPERATION_RESULT_APPLY)//this for after sync preview
-//				debug2 = G_OBJECT (op);	//refcount 2 in async mode
-			if (!(result == GTK_PRINT_OPERATION_RESULT_IN_PROGRESS
-				|| result == GTK_PRINT_OPERATION_RESULT_APPLY))	//e.g. after a sync preview cancelled
-				g_object_unref (G_OBJECT (op));
-			break;
-	}
-}
-#endif //def USE_GTK2_10
 /**
 @brief perform copy
 
@@ -1262,7 +994,7 @@ static void _e2_view_dialog_show_context_menu (GtkWidget *textview,
 	gchar *tip = (gtk_text_buffer_get_has_selection (rt->textbuffer)) ?
 		_("Print selected text") : _("Print file");
 	e2_menu_add (menu, _("_Print.."), GTK_STOCK_PRINT, tip,
-		e2_view_dialog_print_cb, rt);
+		e2_dialog_print_cb, rt);
 #endif
 	e2_menu_add_separator (menu);
 	item_name = g_strconcat (_A(3),".",_A(32),NULL);
@@ -1285,7 +1017,7 @@ static void _e2_view_dialog_show_context_menu (GtkWidget *textview,
 }
 /**
 @brief construct and show hbox with search-bar items
-
+Downstream MAY need BGL closed (on gtk2)
 @param rt runtime struct to work on
 
 @return hbox for the searchbar
@@ -1298,6 +1030,8 @@ GtkWidget *e2_view_dialog_create_searchbar (E2_ViewDialogRuntime *rt)
 	GtkWidget *hbox = gtk_hbox_new (FALSE, 0);
 	//add search-string combo
 	//CHECKME close BGL to cut latency ?
+	//cuz E2_COMBOBOX_MENU_STYLE flag is not set, on gtk2, downstream calls
+	//gtk_widget_set_name() which MAY? need BGL closed
 	rt->combo = e2_combobox_add (hbox, TRUE, 0, NULL, NULL, &find_history,
 		E2_COMBOBOX_HAS_ENTRY | E2_COMBOBOX_FOCUS_ON_CHANGE);	//| E2_COMBOBOX_NO_AUTO_HISTORY);
 	gtk_widget_set_size_request (rt->combo, 230, -1);
@@ -1352,7 +1086,7 @@ GtkWidget *e2_view_dialog_create_searchbar (E2_ViewDialogRuntime *rt)
 @param localpath data including full path of item being re-opened
 @param rt runtime struct to work on
 
-@return hbox for the searchbar
+@return
 */
 void e2_view_dialog_show_atlast (VPATH *localpath, E2_ViewDialogRuntime *rt)
 {
@@ -1495,7 +1229,7 @@ void e2_view_dialog_destroy (E2_ViewDialogRuntime *rt)
 static gboolean _e2_view_dialog_popup_menu_cb (GtkWidget *textview,
 	E2_ViewDialogRuntime *rt)
 {
-	gint event_time = gtk_get_current_event_time ();
+	guint32 event_time = gtk_get_current_event_time ();
 	_e2_view_dialog_show_context_menu (textview, 0, event_time, rt);
 	return TRUE;
 }
@@ -1576,10 +1310,10 @@ static gboolean _e2_view_dialog_key_press_cb (GtkWidget *widget, GdkEventKey *ev
 {
 	printd (DEBUG, "_e2_view_dialog_key_press_cb widget: %x event: %x data: %x, key: %x",
 		widget, event, rt, event->keyval);
-	guint mask = gtk_accelerator_get_default_mod_mask () & event->state;
+	guint mask = event->state; //& gtk_accelerator_get_default_mod_mask ();
 	if (//g_ascii_isalpha (event->keyval) //&& (event->keyval < 0xF000 || event->keyval > 0xFFFF) //&&
 		(mask & GDK_CONTROL_MASK || mask == GDK_MOD1_MASK))
-	{	//the key is a letter and the modifier is Alt or a modifier is Ctrl (Shift+Ctrl reverses direction)
+	{	//the key is a letter and the modifier is Alt or a modifier is Ctrl (CHECKME support Shift+Ctrl to reverse direction?)
 		guint lower = (gdk_keyval_is_upper (event->keyval)) ?
 			gdk_keyval_to_lower (event->keyval) : event->keyval;
 		if (lower == find_keycode || lower == findnext_keycode)
@@ -1710,8 +1444,13 @@ static void _e2_view_dialog_response_cb (GtkDialog *dialog, gint response,
 				//as no callback happens then
 				//if a mod key is pressed, treat it specially
 				//Ctrl = first, Shift = temp reverse
-				GdkModifierType mask = e2_utils_get_modifiers () &
-					gtk_accelerator_get_default_mod_mask ();
+				GdkModifierType mask =
+#ifdef USE_GTK3_0
+					e2_utils_get_savedstate (rt->dialog)
+#else
+					e2_utils_get_modifiers ()
+#endif
+					& gtk_accelerator_get_default_mod_mask ();
 				if (mask & GDK_SHIFT_MASK)
 				{	//temporary direction change
 					rt->search_backward = !rt->search_backward;
@@ -1872,9 +1611,10 @@ static void _e2_view_dialog_mousegestures (E2_OptionSet *set)
 
 /**
 @brief create and show view dialog
-
-Expects BGL closed when we arrive here
-
+File content is loaded and converted if possible, but not here added to the
+textview, so that upstream can show the dialog before (possibly slowly)
+adding/displaying the content, and set appropriate cursor position
+Expects BGL closed
 @param localpath localised string which has path & name of item to be processed
 @param srt pointer to store for returning dialog data, or NULL
 
@@ -1895,13 +1635,9 @@ static GtkWidget *_e2_view_dialog_create (VPATH *localpath,
 		return NULL;
 	}
 #endif
-	rt->textview = gtk_text_view_new ();
-	// add the item's content
-	gdk_threads_leave ();	//any downstream error message does its own mutex management
+	//get the file content into rt->textbuf
 	if (!e2_view_dialog_read_text (localpath, rt))
 	{
-		gdk_threads_enter ();
-		gtk_widget_destroy (rt->textview);
 		DEALLOCATE (E2_ViewDialogRuntime, rt);
 		if (srt != NULL)
 			*srt = NULL;
@@ -1913,7 +1649,6 @@ static GtkWidget *_e2_view_dialog_create (VPATH *localpath,
 	//for a view dialog, we don't want to respond to replace keypresses
 	replace_keycode = 0;
 
-	gdk_threads_enter ();
 	rt->case_sensitive = case_sensitive;
 	rt->whole_words = whole_words;
 	rt->search_backward = search_backward;
@@ -1922,37 +1657,37 @@ static GtkWidget *_e2_view_dialog_create (VPATH *localpath,
 	rt->research = FALSE;
 	e2_view_dialog_init_hilites (rt);
 #endif
-
-	gchar *utf = F_FILENAME_FROM_LOCALE (rt->localpath);
-	rt->dialog = e2_dialog_create (NULL, utf, _("displaying file"),
+	gchar *utfpath = F_FILENAME_FROM_LOCALE (rt->localpath);
+	rt->dialog = e2_dialog_create (NULL, utfpath, _("displaying file"),
 		(ResponseFunc)_e2_view_dialog_response_cb, rt);
-	F_FREE (utf, rt->localpath);
+	F_FREE (utfpath, rt->localpath);
+//	gtk_window_set_type_hint (GTK_WINDOW (rt->dialog),
+//			    GDK_WINDOW_TYPE_HINT_NORMAL);
+	//override some default label properties
+	GtkWidget *label = g_object_get_data (G_OBJECT (rt->dialog),
+		"e2-dialog-label");
+	gtk_label_set_selectable (GTK_LABEL (label), TRUE);
+
 	GtkWidget *dialog_vbox =
 #ifdef USE_GTK2_14
 		gtk_dialog_get_content_area (GTK_DIALOG (rt->dialog));
 #else
 		GTK_DIALOG (rt->dialog)->vbox;
 #endif
-	//override some default label properties
-	GtkWidget *label = g_object_get_data (G_OBJECT (rt->dialog),
-		"e2-dialog-label");
-//	gtk_label_set_line_wrap (GTK_LABEL (label), FALSE);
-	gtk_label_set_selectable (GTK_LABEL (label), TRUE);
-
-//	gtk_window_set_type_hint (GTK_WINDOW (rt->dialog),
-//			    GDK_WINDOW_TYPE_HINT_NORMAL);
-
 	e2_widget_add_separator (dialog_vbox, FALSE, 0);
-	//create the view
 	GtkWidget *sw = e2_widget_add_sw (dialog_vbox,
 			GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC, TRUE, E2_PADDING);
-	gtk_container_add (GTK_CONTAINER (sw), rt->textview);
-	// set view defaults
+	//create the view
+	//BGL must be closed from textview-creation until dialog show, to block
+	//idle-callback happening too early (hence on gtk3 at least, no styling, crash)
+	rt->textview = gtk_text_view_new ();
+	//set view defaults
 	gtk_text_view_set_editable (GTK_TEXT_VIEW (rt->textview), FALSE);
-	gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (rt->textview), FALSE);
+	gtk_text_view_set_cursor_visible ((GtkTextView*)rt->textview, FALSE);
 	rt->textwrap = e2_option_bool_get ("dialog-view-wrap");
-	gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (rt->textview),
+	gtk_text_view_set_wrap_mode ((GtkTextView*)rt->textview,
 		rt->textwrap ? GTK_WRAP_WORD : GTK_WRAP_NONE);
+	gtk_container_add (GTK_CONTAINER (sw), rt->textview);
 	gint char_width, char_height;
 	e2_view_dialog_set_font (&char_width, &char_height, rt);
 	rt->window_width = e2_option_int_get ("dialog-view-width");
@@ -1979,28 +1714,19 @@ static GtkWidget *_e2_view_dialog_create (VPATH *localpath,
 
 	GtkWidget *hbox = e2_view_dialog_create_searchbar (rt);
 	gtk_container_add (GTK_CONTAINER (hndlbox), hbox);
-
-	//now add things to the action-area
+	//add things to the action-area
+	GtkWidget *hbbox =
+#ifdef USE_GTK2_14
+		gtk_dialog_get_action_area (GTK_DIALOG (rt->dialog));
+#else
+		GTK_DIALOG (rt->dialog)->action_area;
+#endif
 	//the "not found" warning is created but not displayed
 	gchar *labeltext = g_strconcat ("<span weight=\"bold\" foreground=\"",
 		e2_option_str_get ("color-negative"), "\">", _("not found"), "</span>", NULL);
-	rt->info_label =
-		e2_widget_add_mid_label (
-#ifdef USE_GTK2_14
-		gtk_dialog_get_action_area (GTK_DIALOG (rt->dialog)),
-#else
-		GTK_DIALOG (rt->dialog)->action_area,
-#endif
-		labeltext, 0.0, TRUE, 0);
+	rt->info_label = e2_widget_add_mid_label (hbbox, labeltext, 0.0, TRUE, 0);
 	//left-align the label
-	gtk_button_box_set_child_secondary (
-		GTK_BUTTON_BOX (
-#ifdef USE_GTK2_14
-		gtk_dialog_get_action_area (GTK_DIALOG (rt->dialog))
-#else
-		GTK_DIALOG (rt->dialog)->action_area
-#endif
-		), rt->info_label, TRUE);
+	gtk_button_box_set_child_secondary (GTK_BUTTON_BOX (hbbox), rt->info_label, TRUE);
 	//search and/or view buttons
 	labeltext = _("_Hide");
 	hide_keycode = e2_utils_get_mnemonic_keycode (labeltext);
@@ -2039,15 +1765,18 @@ static GtkWidget *_e2_view_dialog_create (VPATH *localpath,
 	//this prevents a check button from being activated by keyboard
 //	gtk_dialog_set_default_response (GTK_DIALOG (rt->dialog), E2_RESPONSE_FIND);
 
-	gtk_window_resize (GTK_WINDOW (rt->dialog), char_width * rt->window_width,
-				(char_height+3) * rt->window_height);
-
 	//highest priority - arrange for key-translations from locale
+#ifdef USE_GTK3_0
 	g_signal_connect (G_OBJECT (rt->dialog), "key-press-event",
-		G_CALLBACK (e2_utils_key_translate_cb), NULL);
+		G_CALLBACK (e2_window_key_cb), GINT_TO_POINTER(1));	//includes translation, non-NULL data to prevent disconnection by bindings
+	g_signal_connect (G_OBJECT (rt->dialog), "key-release-event",
+		G_CALLBACK (e2_window_key_cb), NULL);
+#else
+	g_signal_connect (G_OBJECT (rt->dialog), "key-press-event",
+		G_CALLBACK (e2_utils_key_translate_cb), GINT_TO_POINTER(1));
 	g_signal_connect (G_OBJECT (rt->dialog), "key-release-event",
 		G_CALLBACK (e2_utils_key_translate_cb), NULL);
-
+#endif
 #ifdef E2_TRANSIENTBINDINGS
 	//add dialog-specific key bindings, before the key-press callback
 	//see also dialog-general
@@ -2076,8 +1805,11 @@ static GtkWidget *_e2_view_dialog_create (VPATH *localpath,
 	gtk_widget_hide (rt->info_label);
 	gtk_widget_hide (rt->hidebtn);
 
+	gtk_window_resize (GTK_WINDOW (rt->dialog), char_width * rt->window_width,
+				(char_height+3) * rt->window_height);
+
 	if (srt != NULL)
-		*srt = rt;	//make rt available to openat func
+		*srt = rt;	//make rt available to caller
 
 	return rt->dialog;
 }
@@ -2150,21 +1882,20 @@ static gboolean _e2_view_dialog_reviewQ (E2_ActionTaskData *qed)
 #else
 			(localpath, &vrt);
 #endif
-		gdk_threads_leave ();
 		if (dialog != NULL)
 		{
-			WAIT_FOR_EVENTS_UNLOCKED_SLOWLY
-			gdk_threads_enter ();
+			gtk_text_view_set_buffer (GTK_TEXT_VIEW (vrt->textview), vrt->textbuffer);
+			g_object_unref (vrt->textbuffer); //destroy buffer with view
 #ifdef E2_VFS
 			e2_view_dialog_show_atlast (&ddata, vrt);
 #else
 			e2_view_dialog_show_atlast (localpath, vrt);
 #endif
-			gdk_threads_leave ();
 			retval = TRUE;
 		}
 		else
 			retval = FALSE;
+		gdk_threads_leave ();
 	}
 	g_free (localpath);
 	return retval;
@@ -2188,14 +1919,12 @@ static gboolean _e2_view_dialog_viewatQ (E2_ActionTaskData *qed)
 {
 	//for a help doc, has localised doc path, and a "[title]" arg in undefined encoding
 	gchar *view_this = (gchar *) qed->rt_data;
-//	gchar c;
+
 	//strip off any arg FIXME doc path may have whitespace
 	gchar *target = e2_utils_bare_strchr (view_this, ' ');	//always ascii ' ', don't need g_utf8_strchr()
 	if (target != NULL)
-	{
-//		c = *target;
 		*target = '\0';
-	}
+
 #ifdef E2_VFS
 	VPATH ddata = { view_this, qed->currspace };
 #endif
@@ -2210,35 +1939,41 @@ static gboolean _e2_view_dialog_viewatQ (E2_ActionTaskData *qed)
 	gdk_threads_leave ();
 
 	if (target != NULL)
-//		*target = c;	//revert the argument, ready for next time
 		*target = ' ';	//revert the argument, ready for next time
 
 	if (dialog != NULL)
 	{
+		gboolean found;
+		GtkTextIter match;
 		if (target != NULL)
 			target = e2_utils_pass_whitespace (target+1);
 		if (target != NULL && *(target+1) != '\0')
-		{	//find the target string and open there
-			GtkTextIter iter, start, end;
-			gtk_text_buffer_get_start_iter (vrt->textbuffer, &iter);
-			if (gtk_text_iter_forward_search (&iter, target, 0, &start, &end, NULL))
-			{	//need to add a mark to get the scroll to work properly
-				//FIXME sometimes it scrolls a bit too far
-				GtkTextMark *mark = gtk_text_buffer_create_mark
-					(vrt->textbuffer, NULL, &start, FALSE);
-				WAIT_FOR_EVENTS_UNLOCKED
-				gdk_threads_enter ();
-				//to support incremental searching, move the cursor
-				//to the start of the located string
-				gtk_text_buffer_place_cursor (vrt->textbuffer, &start);
-				gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW (vrt->textview), mark,
-					0.0, TRUE, 0.0, 0.2);
-				gdk_threads_leave ();
-			}
+		{	//try to find the target string and open there
+			GtkTextIter start;
+			gtk_text_buffer_get_start_iter (vrt->textbuffer, &start);
+			found = gtk_text_iter_forward_search (&start, target, 0, &match, NULL, NULL);
+//			if (found)
+//				gtk_text_buffer_place_cursor (vrt->textbuffer, &match); //probably needs BGL closed
 		}
+		else
+			found = FALSE;
 		gdk_threads_enter ();
-		e2_dialog_show (dialog, app.main_window, E2_DIALOG_DONT_SHOW_ALL, NULL);
-		gtk_window_present (GTK_WINDOW (dialog));
+//		e2_dialog_show (dialog, app.main_window, E2_DIALOG_DONT_SHOW_ALL, NULL);
+		e2_dialog_setup (dialog, app.main_window);
+//		gtk_window_present (GTK_WINDOW (dialog));
+		gtk_widget_show (dialog);
+		//insert the text now
+		gtk_text_view_set_buffer (GTK_TEXT_VIEW (vrt->textview), vrt->textbuffer);
+		g_object_unref (G_OBJECT(vrt->textbuffer)); //destroy buffer with view
+		if (found)
+		{	//FIXME sometimes it scrolls to wrong spot
+			//see API doco for gtk_text_view_scroll_to_iter() re idle-time processing
+			gtk_text_buffer_place_cursor (vrt->textbuffer, &match); //probably needs BGL closed
+			gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW (vrt->textview),
+				gtk_text_buffer_get_selection_bound (vrt->textbuffer),
+				0.0, TRUE, 0.0, 0.5);
+//			WAIT_FOR_EVENTS
+		}
 		gdk_threads_leave ();
 		return TRUE;
 	}
@@ -2254,20 +1989,28 @@ static gboolean _e2_view_dialog_viewatQ (E2_ActionTaskData *qed)
 @a localpath may not have any path, or an absolute path. In those cases,
 curr_view->dir will be prepended
 This returns immediately, so if run in a Q-thread, that will end
+Assumes BGL closed
 @param localpath localised string which has at least the name of item to be processed
 
 @return TRUE if the dialog was created
 */
 gboolean e2_view_dialog_create (VPATH *localpath)
 {
-	//there is no async thread protection here or downstream,, as this func can
-	//be called from different contexts. Such protection, if needed, must be done by the caller
-	GtkWidget *dialog = _e2_view_dialog_create (localpath, NULL);
+	E2_ViewDialogRuntime *vrt;
+	GtkWidget *dialog = _e2_view_dialog_create (localpath, &vrt);
 	if (dialog != NULL)
 	{
-		WAIT_FOR_EVENTS_SLOWLY
-		e2_dialog_show (dialog, app.main_window, E2_DIALOG_DONT_SHOW_ALL, NULL);
-		gtk_window_present (GTK_WINDOW (dialog));
+//		WAIT_FOR_EVENTS_SLOWLY
+//		e2_dialog_show (dialog, app.main_window, E2_DIALOG_DONT_SHOW_ALL, NULL);
+		e2_dialog_setup (dialog, app.main_window);
+//		gtk_window_present (GTK_WINDOW (dialog));
+		gtk_widget_show (dialog);
+		gtk_text_view_set_buffer (GTK_TEXT_VIEW (vrt->textview), vrt->textbuffer);
+		g_object_unref (G_OBJECT (vrt->textbuffer)); //destroy buffer with view
+		//put cursor at start of buffer, for searching etc
+		GtkTextIter start;
+		gtk_text_buffer_get_start_iter (vrt->textbuffer, &start);
+		gtk_text_buffer_place_cursor (vrt->textbuffer, &start);
 		return TRUE;
 	}
 	return FALSE;
