@@ -1,6 +1,6 @@
-/* $Id: e2_view_dialog.h 1917 2009-09-27 09:30:57Z tpgww $
+/* $Id: e2_view_dialog.h 2469 2012-03-16 00:19:24Z tpgww $
 
-Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
+Copyright (C) 2004-2012 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -14,8 +14,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 #ifndef __E2_VIEW_DIALOG_H__
@@ -135,9 +134,6 @@ void e2_view_dialog_set_menu_position (GtkWidget *menu,
 void e2_view_dialog_update_combo (GtkWidget *combo);
 gboolean e2_view_dialog_combokey_cb (GtkWidget *entry, GdkEventKey *event,
 	E2_ViewDialogRuntime *rt);
-#ifdef USE_GTK2_10
-void e2_view_dialog_print_cb (GtkMenuItem *menuitem, E2_ViewDialogRuntime *rt);
-#endif
 void e2_view_dialog_toggled_cb (GtkToggleButton *button, gboolean *store);
 GtkWidget *e2_view_dialog_create_searchbar (E2_ViewDialogRuntime *rt);
 gboolean e2_view_dialog_search (gboolean first, gboolean incremental,
