@@ -1,6 +1,6 @@
-/* $Id: e2_bookmark.c 1966 2009-10-08 02:28:27Z tpgww $
+/* $Id: e2_bookmark.c 2309 2011-01-17 00:12:22Z tpgww $
 
-Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
+Copyright (C) 2004-2011 tooar <tooar@emelfm2.net>
 
 This file is part of emelfm2.
 emelfm2 is free software; you can redistribute it and/or modify it
@@ -14,8 +14,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 /**
@@ -58,10 +57,14 @@ static void _e2_bookmark_recreate_toolbars (void)
 */
 static void _e2_bookmark_delete_mark (GtkTreeIter *iter)
 {
+	ViewInfo *view = curr_view;
 	e2_option_tree_del_direct (bookmarks_set, iter);
 	_e2_bookmark_recreate_toolbars ();
 	if (GTK_IS_MENU (app.context_menu))
 		gtk_menu_popdown (GTK_MENU (app.context_menu));
+	if (view != curr_view)
+		e2_pane_activate_other ();
+	gtk_widget_grab_focus (view->treeview);
 }
 /**
 @brief refresh the bookmarks menus
@@ -180,28 +183,51 @@ static void _e2_bookmark_confirm_response_cb (GtkDialog *dialog,
 @brief callback to add an item (directory) to the bookmarks list
 
 The activated item @a widget may be a menu item (eg in the panebar bookmarks
-sub-menu) or a context menu item (file pane or bookmarks widget). In the former
+sub-menu) or an actions-menu item (file pane or bookmarks widget). In the former
 case, it will have been processed as an action, and will have a string argument
 @a arg, with "pane1" or "pane2" or a specific UTF-8 path string, and possibly
 including one of "top", "child" (othewise the add is treated as "after").
-When @a widget is a context menu item, @a arg will be NULL.
+When @a widget is an actions-menu item, @a arg will be NULL.
 This function needs to know which pane to use for the path to add to the list.
 For menu callbacks, @a arg needs to include "1" or "2" as appropriate. For
-context-menu callbacks, @a widget will have associated data "bookmark-pane" =
-0 (active), 1 or 2.
+context-menu callbacks, @a widget will have associated E2_MarkData* which
+includes 0 (active), 1 or 2.
 
 @param widget NULL if a bookmarks menu item widget was selected or running an
  action directly, else a context menu item widget
-@param arg string which indicates where to put the new bookmark, or NULL
+@param arg NULL, or string which indicates mark-directory and/or where in the marks
+ list to put the new bookmark e.g. "child", mark directory may be like "N," where
+ N = ''|0|1|2 followed by a comma
 @return
 */
 static void _e2_bookmark_add_cb (GtkMenuItem *widget, const gchar *arg)
 {
-	guint pane;
+	gint32 pane;
 	gboolean before = FALSE;
 	gboolean parent = FALSE;
+	GtkTreePath *path;
 	gchar *s = NULL;
 
+	if (widget != NULL)
+	{
+		E2_MarkData *mdata = (E2_MarkData *) g_object_get_data (G_OBJECT (widget), "mark-data");
+		if (mdata != NULL)
+		{
+			pane = mdata->pane_num;
+			path = mdata->tpath;
+		}
+		else
+		{
+			pane = E2PANECUR;	//default to active pane
+			path = NULL;
+		}
+	}
+	else
+	{
+		pane = E2PANECUR;
+		path = NULL;
+	}
+	//override pane if caller wants
 	if (arg != NULL && *arg != '\0')
 	{
 		gchar *n;
@@ -214,27 +240,23 @@ static void _e2_bookmark_add_cb (GtkMenuItem *widget, const gchar *arg)
 		}
 
 		if (strchr (arg, G_DIR_SEPARATOR) != NULL)
-			pane = 9999;	//arg has a path string
+			pane = 9999;	//indicator that arg includes a path string
 		else if ((s = e2_utils_bare_strchr ((gchar *)arg, '1')) != NULL)
 		{
 			n = e2_utils_pass_whitespace (s+1);
-			pane = (n == NULL || *n == ',') ? E2PANE1 : 9999;
+			pane = (n == NULL || *n == ',') ? E2PANE1 : 9999; //"1" or "1,[place]" signals pane 1
 		}
 		else if ((s = e2_utils_bare_strchr ((gchar *)arg, '2')) != NULL)
 		{
 			n = e2_utils_pass_whitespace (s+1);
-			pane = (n == NULL || *n == ',') ? E2PANE2 : 9999;
+			pane = (n == NULL || *n == ',') ? E2PANE2 : 9999; //"2" or "2,[place]" signals pane 2
 		}
 		else
 		{
 			n = e2_utils_pass_whitespace ((gchar *)arg);
-			pane = (n == NULL) ? E2PANECUR : 9999;
+			pane = (n == NULL || *n == ',') ? E2PANECUR : 9999; //"" or ",[place]" signals current pane
 		}
 	}
-	else if (widget != NULL)
-		pane = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (widget), "bookmark-pane"));
-	else
-		pane = E2PANECUR;	//default to active pane
 
 	//get the relevant dir to add to the marks list
 	gchar *dirname;
@@ -282,7 +304,7 @@ static void _e2_bookmark_add_cb (GtkMenuItem *widget, const gchar *arg)
 	} */
 	void *options[4];
 	options[0] = basename;	//menu label
-	options[1] = "";  //no icon
+	options[1] = GTK_STOCK_DIRECTORY;  //generic icon in case it's in taskbar
 	//strip the trailing / for paths other than root
 	gint len = strlen (dirname);
 	if (len > 1)
@@ -305,9 +327,6 @@ static void _e2_bookmark_add_cb (GtkMenuItem *widget, const gchar *arg)
 //	E2_OptionSet *set = e2_option_get ("bookmarks");  //internal name USE THE GLOBAL VALUE
 	GtkTreeIter iter;
 	GtkTreeIter iter2;
-	GtkTreePath *path;
-	path = (widget != NULL) ?
-		g_object_get_data (G_OBJECT (widget), "parent-mark") : NULL;
 	if ((path != NULL) && gtk_tree_model_get_iter (bookmarks_set->ex.tree.model,
 			&iter2, path))
 		e2_option_tree_add (bookmarks_set, &iter, &iter2, !parent, before, 4, options);
@@ -315,7 +334,11 @@ static void _e2_bookmark_add_cb (GtkMenuItem *widget, const gchar *arg)
 		//with no parent, the behaviour of "before" is vice versa
 		e2_option_tree_add (bookmarks_set, &iter, NULL, FALSE, !before, 4, options);
 
+	ViewInfo *view = curr_view;
 	_e2_bookmark_recreate_toolbars (); //kludgy way to update bookmarks data in bars
+	if (view != curr_view)
+		e2_pane_activate_other ();
+	gtk_widget_grab_focus (view->treeview);
 
 	if (s != NULL)
 		g_free (s);
@@ -323,6 +346,7 @@ static void _e2_bookmark_add_cb (GtkMenuItem *widget, const gchar *arg)
 }
 /**
 @brief callback for deleting a bookmark (with any children)
+Assumes BGL closed
 @param widget UNUSED the widget which was activated
 @param path treestore path of bookmark to be deleted
 @return
@@ -366,7 +390,23 @@ static void _e2_bookmark_delete_cb (GtkMenuItem *widget, GtkTreePath *path)
 			g_free (name);
 		}
 		else	//no confirmation
-			_e2_bookmark_delete_mark (&iter);
+			_e2_bookmark_delete_mark (&iter); //changes active pane and focus-widget
+	}
+}
+/**
+@brief "selection-done" signal callback for items in bookmark-actions menu
+@param menu the actions-menu widget
+@param mdata data associated with the active imem of bookmarks (sub)menu
+@return
+*/
+static void _e2_bookmark_change_selected_cb (GtkWidget *menu, E2_MarkData *mdata)
+{
+	e2_menu_selection_done_cb (menu, NULL); //clean up actions-menu
+	if (mdata->top_menu != NULL)
+	{
+		gtk_menu_popdown (GTK_MENU (mdata->top_menu)); //hide marks-menu and all subs
+		if (mdata->mark_flags & E2_MARKFLAG_ONETIME)
+			e2_menu_selection_done_cb (mdata->top_menu, NULL); //clean up marks-menu
 	}
 }
 /**
@@ -381,76 +421,78 @@ use for the mark
 
 @param widget clicked button or menu item
 @param event gdk event data struct
-@param path pointer to bookmarks set treemodel path to the string which holds the bookmark directory path
+@param mdata pointer to bookmark data associated with a bookmark-menu item
 @return TRUE (block further handlers) if certain btn 2 or 3 conditions are satisfied
 */
 gboolean e2_bookmark_button_press_cb (GtkWidget *widget, GdkEventButton *event,
-	GtkTreePath* path)
+	E2_MarkData *mdata)
 {
 	printd (DEBUG, "e2_bookmark_button_press_cb");
 #ifdef E2_MOUSECUSTOM
 	if ((event->state & E2_MODIFIER_MASK) == 0)
 	{
 #endif
-		//get which pane's menu was clicked (NULL for active pane)
-		gpointer pane = g_object_get_data (G_OBJECT (widget), "bookmark-pane");
-	//	printd (DEBUG, "requested pane %d", GPOINTER_TO_INT (pane));
 		if (event->button == 2)
 		{
 			if (e2_option_bool_get ("bookmarks-button2-other"))
 			{
 				gint other = (curr_pane == &app.pane1) ? 2 : 1;
-				if (GPOINTER_TO_INT (pane) != other)
+				if (other != mdata->pane_num) //active pane's menu was clicked
 				{
-					gchar *mark = g_object_get_data (G_OBJECT (widget), "bookmark-path");
-					if (mark != NULL)
-					{
 #ifdef E2_VFSTMP
-			//FIXME bookmark parsing and setting for v-dirs
-						if (0)
-							e2_pane_change_space_byuri (other_pane, "some string");
+	//FIXME bookmark parsing and setting for v-dirs
+					if (0)
+						e2_pane_change_space_byuri (other_pane, "some string");
 #endif
-						e2_pane_change_dir (other_pane, mark);	//CHECKME is mark string UTF8 ?
-						if (e2_option_bool_get ("bookmarks-focus-after-open"))
-							e2_pane_activate_other ();
-						return TRUE;
-					}
+					e2_pane_change_dir (other_pane, mdata->mark_path);	//mark string is UTF-8
+					if (e2_option_bool_get ("bookmarks-focus-after-open"))
+						e2_pane_activate_other ();
+					return TRUE;
 				}
 			}
 		}
 		else if (event->button == 3)
 		{
-			//check whether context menu is valid for this widget
-			gpointer context = g_object_get_data (G_OBJECT (widget), "with-context");
-			if (context != NULL && GPOINTER_TO_INT (context) == 1)
-				return FALSE;
-			printd (DEBUG, "bookmark menu");
-			GtkWidget *menu = gtk_menu_new ();
-			GtkWidget *menuitem = e2_menu_add (menu, _("_Add after"), GTK_STOCK_ADD,
-				_("Bookmark the current directory after the selected bookmark"),
-				_e2_bookmark_add_cb, NULL);
-			g_object_set_data (G_OBJECT (menuitem), "parent-mark", path);
-			//pass through the pane no. (or NULL for default) to use when adding an item
-			g_object_set_data (G_OBJECT (menuitem), "bookmark-pane", pane);
+			if ((mdata->mark_flags & E2_MARKFLAG_ACTIONS) > 0)
+			{	//actions-menu allowed for this widget
+				gboolean parent;
+				GtkWidget *menu, *menuitem;
 
-			menuitem = e2_menu_add (menu, _("Add as _child"), GTK_STOCK_INDENT,
-				_("Bookmark the current directory a a child of the selected bookmark"),
-				_e2_bookmark_add_cb, _A(109));	//_(child
-			g_object_set_data (G_OBJECT (menuitem), "parent-mark", path);
-			//pass through the pane no. (or NULL for default) to use when adding an item
-			g_object_set_data (G_OBJECT (menuitem), "bookmark-pane", pane);
+				if (mdata->tpath != NULL)
+				{
+					GtkTreeIter iter;
+					gtk_tree_model_get_iter (bookmarks_set->ex.tree.model, &iter, mdata->tpath);
+					parent = gtk_tree_model_iter_has_child (bookmarks_set->ex.tree.model, &iter);
+				}
+				else
+					parent = FALSE;
 
-			e2_menu_add (menu, _("_Delete"), GTK_STOCK_DELETE,
-				_("Delete the selected bookmark, and its children if any"),
-				_e2_bookmark_delete_cb, path);
-/*  just rely on the edit item in the context menu, now
-			e2_menu_add_separator (menu);
-			e2_menu_add (menu, _("_Edit"), GTK_STOCK_PREFERENCES,
-				_("Open the bookmarks configuration dialog"),
-				_e2_bookmarks_configure, NULL);
-*/
-			e2_menu_popup (menu, event->button, event->time);
-			return TRUE;
+				menu = gtk_menu_new ();
+				//adding a child to an existing mark prevents its operation in the menu
+				if (parent)
+				{
+					//must preface callback arg with ',' to ensure correct parsing
+					gchar *arg = e2_utils_strcat (",", _A(109)); //_(child
+					menuitem = e2_menu_add (menu, _("_Add child"), GTK_STOCK_INDENT,
+						_("Bookmark the current directory a child of the selected item"),
+						_e2_bookmark_add_cb, arg);
+					g_object_set_data_full (G_OBJECT (menuitem), "__freeme", arg, g_free);
+				}
+				else
+					menuitem = e2_menu_add (menu, _("_Add after"), GTK_STOCK_ADD,
+						_("Bookmark the current directory after the selected item"),
+						_e2_bookmark_add_cb, NULL);
+				g_object_set_data (G_OBJECT (menuitem), "mark-data", mdata);
+
+				e2_menu_add (menu, _("_Delete"), GTK_STOCK_DELETE,
+					_("Delete the selected bookmark, and its children if any"),
+					_e2_bookmark_delete_cb, mdata->tpath);
+
+				g_signal_connect (G_OBJECT (menu), "selection-done",
+					G_CALLBACK (_e2_bookmark_change_selected_cb), mdata);
+				e2_menu_popup (menu, event->button, event->time);
+				return TRUE;
+			}
 		}
 #ifdef E2_MOUSECUSTOM
 	}
@@ -520,14 +562,17 @@ static gboolean _e2_bookmark_delete (gpointer from, E2_ActionRuntime *art)
 */
 static gboolean _e2_bookmark_show (gpointer from, E2_ActionRuntime *art)
 {
-	art->state &= ~GDK_MOD2_MASK;
-	if (!ACTION_CLICK(art))
-		return FALSE;
+	if (GTK_IS_BUTTON (from))
+	{
+		art->state &= ~GDK_MOD2_MASK;
+		if (!ACTION_CLICK(art))
+			return FALSE;
+	}
 	E2_OptionSet *set = e2_option_get ("bookmarks");
 	GtkTreeIter iter;
 	if (gtk_tree_model_get_iter_first (set->ex.tree.model, &iter))
 	{
-		gint pane;
+		gint32 pane;
 		E2_PaneRuntime *rt = e2_pane_get_runtime (from, art->data, NULL);
 		if (rt == curr_pane)	//CHECKME allow default regardless of actual pane ?
 			pane = E2PANECUR;
@@ -537,13 +582,14 @@ static gboolean _e2_bookmark_show (gpointer from, E2_ActionRuntime *art)
 		E2_Action *action = e2_action_get (action_name);
 		g_free (action_name);
 		GtkWidget *menu = gtk_menu_new ();
-		e2_menu_add_bookmark_items (menu, TRUE, action, pane, set->ex.tree.model, &iter);
-		g_signal_connect (G_OBJECT(menu), "selection-done",
+		e2_menu_add_bookmark_items (menu, menu, action,
+			E2_MARKFLAG_ACTIONS | E2_MARKFLAG_ONETIME, pane, set->ex.tree.model, &iter);
+		g_signal_connect (G_OBJECT (menu), "selection-done",
 			G_CALLBACK (e2_menu_selection_done_cb), NULL);
-		guint32 time = gtk_get_current_event_time ();
+		guint32 event_time = gtk_get_current_event_time ();
 		gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
 			(GtkMenuPositionFunc) e2_fileview_set_menu_position,
-			rt->view.treeview, 0, time);
+			rt->view.treeview, 0, event_time);
 		return TRUE;
 	}
 	return FALSE;
