@@ -1,6 +1,6 @@
-/* $Id: e2_context_menu.c 1694 2009-05-17 07:23:29Z tpgww $
+/* $Id: e2_context_menu.c 2469 2012-03-16 00:19:24Z tpgww $
 
-Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
+Copyright (C) 2003-2012 tooar <tooar@emelfm2.net>
 Portions copyright (C) 1999 Michael Clark.
 
 This file is part of emelFM2.
@@ -15,8 +15,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 /* 09-20-2000: Modified by Aurelien Gateau
             Added code for Named actions for filetypes
@@ -107,7 +106,8 @@ static gboolean _e2_context_menu_item_click_cb (GtkWidget *menuitem,
 */
 /**
 @brief context-menu "selection-done" signal callback
-This will be called _after_ running the action initiated by the selected item !
+This will be called _before_ running the action initiated by the selected item,
+but the big delay ensures menu persists until after any processing
 @param menu the menu widget (= app.context_menu)
 @param data data specified when the callback was connected
 
@@ -132,10 +132,10 @@ static void _e2_context_menu_selected_cb (GtkWidget *menu, gpointer data)
 /*******************/
 
 /**
-@brief populate @a menu with items for the deesktop actions for a file
+@brief populate @a menu with items for the desktop actions for a file
 
 @param menu the menu widget to which the action menu-items are to be added
-@param desktopfilepath path of .desktop file to parse
+@param desktopfilepath path of a .desktop file to parse
 
 @return
 */
@@ -454,15 +454,24 @@ static void _e2_context_menu_add_items (GtkWidget *menu, GtkTreeModel *model,
 							{
 								GtkTreeIter iter2;
 								if (gtk_tree_model_iter_children (set->ex.tree.model, &iter2, &iter))
-									e2_menu_add_bookmark_items (menu, TRUE, action,
-										pane, set->ex.tree.model, &iter2);
+									e2_menu_add_bookmark_items (menu, menu, action,
+										E2_MARKFLAG_ONETIME, pane, set->ex.tree.model, &iter2);
 							}
 						}
 						else if (gtk_tree_model_get_iter_first (set->ex.tree.model, &iter))
-								e2_menu_add_bookmark_items (menu, TRUE, action,
-									pane, set->ex.tree.model, &iter);
+								e2_menu_add_bookmark_items (menu, menu, action,
+									E2_MARKFLAG_ONETIME, pane, set->ex.tree.model, &iter);
 					}
 						break;
+					case E2_ACTION_TYPE_HISTORY:
+					{
+						GtkWidget *item = e2_menu_add (menu, label, icon, NULL, NULL, NULL);
+						E2_PaneRuntime *rt = e2_pane_get_runtime (curr_view->treeview,
+							arg, NULL);
+						GtkWidget *submenu = e2_pane_visited_menu (rt);
+						gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), submenu);
+					}
+					break;
 					case E2_ACTION_TYPE_FILE_ACTIONS:
 						e2_context_menu_add_filetype_item (menu);
 						break;
@@ -529,29 +538,19 @@ static void _e2_context_menu_add_items (GtkWidget *menu, GtkTreeModel *model,
 */
 void e2_context_menu_show (guint button, guint32 time, gint type)
 {
-//	FileInfo *info;
-//FIXME if we need to disable refresh during the popup,
+//CHECKME if we need to disable refresh during the popup,
 // make all menu choices depart via a fn that enables refresh again
-//#ifdef E2_REFRESH_DEBUG
-//	printd (DEBUG, "disable refresh, context menu show");
-//#endif
-//	e2_filelist_disable_refresh();
-
-//	if (curr_view->tagged != NULL)
-//		info = curr_view->tagged->data;
-//	else
-//	{
-		GtkTreeIter iter;
+	GtkTreeIter iter;
 /* show menu even with empty dir
 	if (gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (curr_view->store),
 			&iter, NULL, curr_view->row))
-			gtk_tree_model_get (GTK_TREE_MODEL (curr_view->store), &iter, FINFO, &info, -1);
-		else
-		{
-			printd (WARN, "missing model row");
-			return;
-		} */
-//	}
+		gtk_tree_model_get (GTK_TREE_MODEL (curr_view->store), &iter, FINFO, &info, -1);
+	else
+	{
+		printd (WARN, "missing model row");
+		return;
+	}
+*/
 	E2_OptionSet *set = e2_option_get ("context-menu");
 	if (gtk_tree_model_get_iter_first (set->ex.tree.model, &iter))
 	{
@@ -665,6 +664,7 @@ static void _e2_context_menu_tree_defaults (E2_OptionSet *set)
 	g_strconcat("\t",_("_Remove spaces"),"||false|true|>mv|%f `echo %f \\| sed -e 's/ //g'` 2>/dev/null &",NULL), 	//_A(20)
 	g_strconcat("\t",_("_Split file.."),"||false|true|split|-b %{",_("Enter the piece-size (in kB):"),"}k %f %f_",NULL), //_A(20)
 	g_strconcat("\t",_("Co_ncatenate files.."),"||false|true|cat|%f > '%{",_("Enter the name of the combined file:"),"}'",NULL), //_A(20)
+	g_strconcat("\t",_("Show _usage"),"||false|true|du|-bs %f",NULL),
 	g_strconcat("\t",_("_Free space"),"||false|true|>stat|-f %d \\| awk '/Blocks/ {printf \"%2.1f ", _("percent free"),"\",$5/$3*100}'",NULL),  //_A(20)
 	g_strconcat("\t||false|true|",_A(21),"|",NULL),
 	g_strconcat("\t",_("_Edit user commands.."),"|gtk-preferences|false|true|",_A(3),".",_A(32),"|",_C(8),NULL),
@@ -676,6 +676,7 @@ static void _e2_context_menu_tree_defaults (E2_OptionSet *set)
 	g_strconcat("\t",_("Add _bottom"),"|add_mark_bottom"E2ICONTB"|false|false|",_A(0),".",_A(30),"|",NULL),
 	g_strconcat("\t||false|false|",_A(21),"|",NULL),
 	g_strconcat("\t",_("_Edit bookmarks.."),"|gtk-preferences|false|false|",_A(3),".",_C(1),"|",NULL),
+	g_strconcat(_("_History"),"|gtk-jump-to|false|false|",_A(8),".",_A(27),"|",NULL),
 	g_strconcat("||false|false|",_A(21),"|",NULL),
 	g_strconcat(_("_Edit filetype.."),"|gtk-preferences|false|false|",_A(3),".",_A(45),"|",NULL),
 	g_strdup(">"),	//allow this to be freed
