@@ -1,6 +1,6 @@
-/* $Id: e2_dnd.c 1221 2008-09-11 13:45:23Z tpgww $
+/* $Id: e2_dnd.c 2377 2011-06-11 00:16:34Z tpgww $
 
-Copyright (C) 2006-2008 tooar <tooar@gmx.net>.
+Copyright (C) 2006-2011 tooar <tooar@emelfm2.net>.
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -14,8 +14,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 /**
@@ -99,7 +98,13 @@ void e2_dnd_drag_begin_cb (GtkWidget *treeview, GdkDragContext *context,
 	//remember where we began, for later drop-check
 //	dragged_sel = view->selection;
 #ifdef E2_ALTLEFTMOUSE
-	mask = e2_utils_get_modifiers ();
+	mask =
+# ifdef USE_GTK3_0
+		e2_utils_get_savedstate (app.main_window)
+# else
+		e2_utils_get_modifiers ()
+# endif
+		& gtk_accelerator_get_default_mod_mask ();
 	if (left_pressed && (mask & GDK_CONTROL_MASK) && !(mask & GDK_SHIFT_MASK))
 	{
 		drag_sel_now = TRUE;	//signal that a drag-sel has started
@@ -128,8 +133,12 @@ void e2_dnd_drag_begin_cb (GtkWidget *treeview, GdkDragContext *context,
 	return;
 }
 /**
-@brief pair of timer callback functions which together, scroll treeview as appropriate during a DnD operation
-@param view pointer to view data struct
+@brief 'chained' timer callback function which scrolls the filelist treeview for @a view
+
+The initial timer was initiated in the drag-motion callback.
+
+@param view rt data for the view where the drag-motion occurred
+
 @return TRUE to continue timer, FALSE to cancel it
 */
 static gboolean _e2_dnd_auto_scroll2 (ViewInfo *view)
@@ -139,21 +148,31 @@ static gboolean _e2_dnd_auto_scroll2 (ViewInfo *view)
 	gint y_scr; //,x_scr;
 	//must check mouse position here, as e2_dnd_drag_motion_cb() may not be called
 	//often enough when auto-scrolling a large treeview
-	//my and my are for bottom-left of pointer FIXME make it work for top-left
+	//is for bottom-left of pointer FIXME make it work for top-left
 	gint my;
-	gdk_window_get_pointer (
-#ifdef USE_GTK2_14
-		gtk_widget_get_window (view->treeview),
-#else
-		view->treeview->window,
+#ifdef USE_GTK3_0 //CHECKME pointer position
+	GdkDevice *device = (GdkDevice *)g_object_get_data (G_OBJECT (view->treeview),
+		"e2-drag-device");
+	if (device == NULL || gdk_window_get_device_position (
+			gtk_widget_get_window (view->treeview), device,
+			NULL, &my, NULL) == NULL)
+#else //gtk 2
+	if (gdk_window_get_pointer (
+# ifdef USE_GTK2_14
+			gtk_widget_get_window (view->treeview),
+# else
+			view->treeview->window,
+# endif
+			NULL, &my, NULL) == NULL)
 #endif
-		NULL, &my, NULL);
+		return FALSE;
 
-#ifdef USE_GTK2_99
-	GtkAllocation alloc = gtk_widget_get_allocation (view->treeview);
+#ifdef USE_GTK2_18
+	GtkAllocation alloc;
+	gtk_widget_get_allocation (view->treeview, &alloc);
 #endif
 	if (my >= AUTO_SCROLL_MARGIN && my <= (
-#ifdef USE_GTK2_99
+#ifdef USE_GTK2_18
 		alloc.height
 #else
 		view->treeview->allocation.height
@@ -168,18 +187,34 @@ static gboolean _e2_dnd_auto_scroll2 (ViewInfo *view)
 		y_scr = my - AUTO_SCROLL_MARGIN;	//-AUTO_SCROLL_MARGIN to -1
 	else
 		y_scr = my + AUTO_SCROLL_MARGIN -
-#ifdef USE_GTK2_99
+#ifdef USE_GTK2_18
 		alloc.height;
 #else
 		view->treeview->allocation.height; //1 to AUTO_SCROLL_MARGIN
 #endif
-	GtkAdjustment *vadj = gtk_tree_view_get_vadjustment
-		(GTK_TREE_VIEW (view->treeview));
+	GtkAdjustment *vadj;
+#ifdef USE_GTK3_0
+	vadj = gtk_scrollable_get_vadjustment (GTK_SCROLLABLE (view->treeview));
+#else
+	vadj = gtk_tree_view_get_vadjustment (GTK_TREE_VIEW (view->treeview));
+#endif
 	gdouble scrollnow = gtk_adjustment_get_value (vadj);
+	gdouble uppernow =
+#ifdef USE_GTK2_14
+		gtk_adjustment_get_upper (vadj);
+#else
+		vadj->upper;
+#endif
+	gdouble sizenow =
+#ifdef USE_GTK2_14
+		gtk_adjustment_get_page_size (vadj);
+#else
+		vadj->page_size;
+#endif
 	gdouble scroll_factor;
 	//accelerate the scroll by increasing the stepsize when we are further into
 	//the scroll margin. TrialnError fudge factor
-	scroll_factor = vadj->upper / 500.0
+	scroll_factor = uppernow / 500.0
 		* (exp (ABS (y_scr) * 2.0 / (gdouble) AUTO_SCROLL_MARGIN) - 0.8);
 	if (y_scr < 0)
 		scrollnow -= scroll_factor;
@@ -188,14 +223,15 @@ static gboolean _e2_dnd_auto_scroll2 (ViewInfo *view)
 	//clamp the new setting within relevant range
 	if (scrollnow < 0)
 		scrollnow = 0;
-	else if (scrollnow > vadj->upper - vadj->page_size)
-		scrollnow = vadj->upper - vadj->page_size - 0.1;
+	else if (scrollnow > uppernow - sizenow)
+		scrollnow = uppernow - sizenow - 0.1;
 	//no movement, turn off the scrolling CHECKME more iterations before turnoff ?
-	if (ABS (scrollnow-prior_scroll) < 0.1)	//0.001*vadj->upper)
+	if (ABS (scrollnow - prior_scroll) < 0.1)	//0.001*vadj->upper)
 	{
 		app.timers[ASCROLL_T] = 0;
 		return FALSE;
 	}
+
 	if (scrollnow >= 0)
 	{
 		gdk_threads_enter ();
@@ -206,13 +242,17 @@ static gboolean _e2_dnd_auto_scroll2 (ViewInfo *view)
 	return TRUE;
 }
 /**
-@brief first (longer-interval) timer callback for scrolling treeview
-@param view rt data for the view where the cursor is ?
-@return
+@brief first (longer-interval) timer callback for scrolling a filelist treeview
+
+Substitutes a shorter interval for the initial one which got to here, initiated
+in the drag-motion callback
+
+@param view rt data for the view where the drag-motion occurred
+
+@return FALSE always, so timer is cancelled
 */
 static gboolean _e2_dnd_auto_scroll (ViewInfo *view)
 {
-	//substitute shorter loop for the initial one
 	app.timers[ASCROLL_T] = g_timeout_add (AUTO_SCROLL_STEP_INTERVAL,
 		(GSourceFunc) _e2_dnd_auto_scroll2, view);
 	return FALSE;
@@ -220,10 +260,10 @@ static gboolean _e2_dnd_auto_scroll (ViewInfo *view)
 /**
 @brief "drag-motion" signal callback
 
-This is called when the user moves the cursor during a drag
-For a normal drag it handles the highlighting of drop-target rows,
-and manages treeview scrolling when cursor is near top or bottom of treeview
-For a drag-selection it selects item if needed and bumps counter
+This is called when the user moves the cursor during a drag.
+For a normal drag it handles the highlighting of drop-target rows, and manages
+treeview scrolling when cursor is near top or bottom of treeview.
+For a drag-selection it selects item if needed and bumps counter.
 
 @param treeview widget ptr
 @param context drag context
@@ -329,8 +369,24 @@ gboolean e2_dnd_drag_motion_cb (GtkWidget *treeview, GdkDragContext *context,
 		}
 	}
 	//arrange auto-scrolling when mouse pointer is near top or bottom of treeview
-	if (y < (pointer_height + AUTO_SCROLL_MARGIN) || y > (view->treeview->allocation.height - AUTO_SCROLL_MARGIN))
+#ifdef USE_GTK2_18
+	GtkAllocation alloc;
+	gtk_widget_get_allocation (view->treeview, &alloc);
+#endif
+	if (y < (pointer_height + AUTO_SCROLL_MARGIN)
+	 || y > (
+#ifdef USE_GTK2_18
+			 alloc.height
+#else
+	         view->treeview->allocation.height
+#endif
+			 - AUTO_SCROLL_MARGIN))
 	{
+#ifdef USE_GTK3_0 //CHECKME pointer position
+		//hacky way to send extra data to timer callback
+		g_object_set_data (G_OBJECT (view->treeview), "e2-drag-device",
+			gdk_drag_context_get_device (context));
+#endif
 		if (app.timers[ASCROLL_T] == 0)
 		{
 			//setup initial delay
@@ -441,7 +497,7 @@ void e2_dnd_drag_data_get_cb (GtkWidget *treeview, GdkDragContext *context,
 #ifdef E2_REFRESH_DEBUG
 	printd (DEBUG, "disable refresh, drag data get");
 #endif
-	e2_filelist_disable_refresh();
+	e2_filelist_disable_refresh ();
 
 	/* this is the emerging standard for uri list entries ?
 	<?xml version="1.0" encoding="UTF-8"?>
@@ -467,7 +523,7 @@ NOTE some apps omit the leading "//" from the first example, resulting in file:/
 #ifdef E2_VFSTMP
 	//FIXME dir when not mounted local
 #else
-		local_dir = F_FILENAME_TO_LOCALE (view->dir);
+		local_dir = D_FILENAME_TO_LOCALE (view->dir); //always dup, to avoid dirchange race
 #endif
 		GPtrArray *uri_array = g_ptr_array_sized_new (g_list_length (base) + 1);
 		//FIXME vfs
@@ -497,9 +553,12 @@ NOTE some apps omit the leading "//" from the first example, resulting in file:/
 		}
 		g_ptr_array_add (uri_array, NULL);	//NULL-terminate the array
 		gtk_selection_data_set_uris (data, (gchar **)uri_array->pdata);
-		F_FREE (local_dir);
+//#ifndef USE_GLIB2_22
+		g_ptr_array_foreach (uri_array, (GFunc)g_free, NULL);
+//#endif
 		g_ptr_array_free (uri_array, TRUE);
 		g_list_free (base);
+		g_free (local_dir);
 	}
 #ifdef E2_REFRESH_DEBUG
 	printd (DEBUG, "enable refresh, drag data get");
@@ -630,22 +689,21 @@ typedef struct _E2_MenuWait
 /**
 @brief get drag op menu item
 
-@param menuitem the clicked menu item widget
-@param chosen store for the chosen item
+@param menuitem the activated menu item widget
+@param w wait-data for this callback
 
 @return
 */
-static void _e2_dnd_drag_op_selected_cb (GtkWidget *menuitem, E2_MenuWait *w)
+static void _e2_dnd_drag_op_selected_cb (GtkMenuItem *menuitem, E2_MenuWait *w)
 {
-	w->chosen = menuitem;
+	w->chosen = GTK_WIDGET(menuitem);
 	e2_main_loop_quit (w->loopdata);
 }
 /**
 @brief execute drop
 
-This is the callback for "drag-data-received" signals, emitted on
-@a treeview when dragged data has been received.
-It initiates action for the dropped items
+This is the callback for "drag-data-received" signals, emitted on @a treeview
+when dragged data has been received. It initiates action for the dropped items.
 
 @param treeview widget for pane where the drop is to occur
 @param context the dragcontext
@@ -663,18 +721,10 @@ void e2_dnd_drag_data_received_cb (GtkWidget *treeview, GdkDragContext *context,
 	ViewInfo *view)
 {
 	printd (DEBUG, "callback: drag data received");
-/*	if (*(data->data) == '\0')
-		goto badexit;
-*/
 #ifdef E2_ALTLEFTMOUSE
 	left_pressed = FALSE;	//flag must be cleared at end of drag
 #endif
-	gchar **uris = gtk_selection_data_get_uris (data);
-	if (uris == NULL)
-		goto badexit;
-
 	gchar *dest_dir;
-	//create relevant drop-directory path string
 	//default is to just drop to the pane dir
 #ifdef E2_VFSTMP
 	//FIXME dir when not mounted local
@@ -682,6 +732,108 @@ void e2_dnd_drag_data_received_cb (GtkWidget *treeview, GdkDragContext *context,
 #else
 	dest_dir = view->dir; //utf-8 string with trailing /
 #endif
+	gboolean ask;
+#ifdef USE_GTK2_22
+	GdkDragAction action = gdk_drag_context_get_actions (context);
+#else
+	GdkDragAction action = context->actions;
+#endif
+	//get the user's choice on what to do with the dragged items
+	ask = (action & GDK_ACTION_ASK);
+	if (ask)
+	{	//a choice is offered
+		if (!btn2_released)
+		{
+			//need to get fresh mask here, the one set in drag-begin doesn't work
+#ifdef USE_GTK3_0
+			ask = (e2_utils_get_savedstate (app.main_window) & GDK_MOD1_MASK); //alt key pressed in main window
+#else
+			ask = (e2_utils_get_modifiers () & GDK_MOD1_MASK); //alt key pressed
+#endif
+		}
+	}
+	if (ask)
+	{
+		E2_MenuWait wdata;
+		//this loop for popup menu must be in default context and the menu will
+		//not survive any UI event
+		wdata.loopdata = e2_main_loop_new (TRUE);
+		if (wdata.loopdata != NULL)
+		{
+			GtkWidget *drag_op_menu = gtk_menu_new();
+			GtkWidget *item1, *item2, *item3, *item4;
+
+			if (action & GDK_ACTION_COPY)
+				item1 = e2_menu_add (drag_op_menu, _("_Copy"), NULL, NULL,
+					_e2_dnd_drag_op_selected_cb, &wdata);
+			else
+				item1 = NULL;
+
+			if (action & GDK_ACTION_MOVE)
+				item2 = e2_menu_add (drag_op_menu, _("_Move"), NULL, NULL,
+					_e2_dnd_drag_op_selected_cb, &wdata);
+			else
+				item2 = NULL;
+
+			if (action & GDK_ACTION_LINK)
+				item3 = e2_menu_add (drag_op_menu, _("_Link"), NULL, NULL,
+					_e2_dnd_drag_op_selected_cb, &wdata);
+			else
+				item3 = NULL;
+
+/*			if (?)	//drop target is an executable item
+				item4 = e2_menu_add (drag_op_menu, _("_SendTo"), NULL, NULL,
+					_e2_dnd_drag_op_selected_cb, &wdata);
+			else
+				item4 = NULL;
+*/
+			item4 = e2_menu_add (drag_op_menu, _("C_ancel"), NULL, NULL,
+				_e2_dnd_drag_op_selected_cb, &wdata);
+			wdata.chosen = item4; //default in case of abort == cancel
+
+			gtk_menu_popup (GTK_MENU (drag_op_menu), NULL, NULL, NULL, NULL, 0,
+				(guint32) time);
+
+			e2_main_loop_run (wdata.loopdata);
+
+			e2_menu_selection_done_cb (drag_op_menu, NULL);
+
+			if (wdata.chosen == item1)
+				action = GDK_ACTION_COPY;
+			else if (wdata.chosen == item2)
+				action = GDK_ACTION_MOVE;
+			else if (wdata.chosen == item3)
+				action = GDK_ACTION_LINK;
+/*			else if (wdata.chosen == item4)
+				setup to handle the sendto request
+*/
+			else //the user cancelled
+				action = 0;
+		}
+		else
+		{ //can't do anything
+			//FIXME warn user
+			action = 0;
+		}
+	}
+	else	//don't ask
+#ifdef USE_GTK2_22
+		action = gdk_drag_context_get_suggested_action (context);
+#else
+		action = context->suggested_action;
+#endif
+	//FIXME handle sendto
+	E2_TaskType droptask;
+	if (action & GDK_ACTION_COPY)
+		droptask = E2_TASK_COPY;
+	else if (action & GDK_ACTION_MOVE)
+		droptask = E2_TASK_MOVE;
+	else if (action & GDK_ACTION_LINK)
+		droptask = E2_TASK_LINK;
+	else
+		goto badexit;
+
+	//determine whether to drop the selection other than in view->dir
 	GtkTreePath *path;
 	if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (treeview),
 		x, y-pointer_height, &path, NULL, NULL, NULL))
@@ -711,10 +863,10 @@ void e2_dnd_drag_data_received_cb (GtkWidget *treeview, GdkDragContext *context,
 					g_free (filename);
 					gtk_tree_path_free (path);
 					gtk_tree_path_free (childpath);
-					goto cleanbadexit;
+					goto badexit;
 				}
 				//append target dir's name to the pane dir path
-				//view->dir, filename both utf, both have trailing /
+				//both view->dir and filename are UTF-8, with trailing /
 #ifdef E2_VFSTMP
 	//FIXME dir when not mounted local
 #else
@@ -726,185 +878,44 @@ void e2_dnd_drag_data_received_cb (GtkWidget *treeview, GdkDragContext *context,
 		gtk_tree_path_free (path);
 		gtk_tree_path_free (childpath);
 	}
-	gchar *dest_local = D_FILENAME_TO_LOCALE (dest_dir);
+
+	gboolean result, move;
+	gchar **uris = gtk_selection_data_get_uris (data);
+	if (uris != NULL)
+	{
+		result = e2_task_drop (droptask, dest_dir, uris); //do it
+		move = (result && (droptask == E2_TASK_MOVE));
+		g_strfreev (uris);
+	}
+	else
+	{
+badexit:
+		result = FALSE;
+		move = FALSE;
+		action = 0;
+	}
+#ifdef USE_GTK2_22
+	gdk_drag_status (context, action, (guint32) time);
+#else
+	context->action = action;
+#endif
+	gtk_drag_finish (context, result, move, time);
+//CHECKME filtermodel ...
+/* Must override the default 'drag-data-received' handler on GtkTreeView when
+using models that don't support the GtkTreeDragDest interface and enable
+drag-and-drop. The simplest way to do this is to connect to ::drag-data-received
+and call g_signal_stop_emission_by_name() in your signal handler to prevent the
+default handler from running. Look at the source code for the default handler in
+gtktreeview.c to get an idea what your handler should do.
+*/
+	//CHECKME nothing else needs doing here ?
+	g_signal_stop_emission_by_name ((gpointer)view->treeview,
+		"drag-data-received");
+
 #ifdef E2_VFSTMP
 	//FIXME dir when not mounted local
 #else
 	if (dest_dir != view->dir)
 #endif
 		g_free (dest_dir);
-
-	//get the sender's or user's choice on what to do with the dragged items
-	gboolean menu_wanted;
-	if (btn2_released)
-		menu_wanted = TRUE;
-	else
-	{
-		//need to get fresh mask here, the one set in drag-begin doesn't work
-		GdkModifierType mask = e2_utils_get_modifiers ();
-//		guint modifiers = gtk_accelerator_get_default_mod_mask ();
-//		menu = ((mask & modifiers) == GDK_MOD1_MASK); //only alt key pressed
-		menu_wanted = (mask & GDK_MOD1_MASK); //alt key pressed
-	}
-
-	GdkDragAction action = context->actions;
-	if (menu_wanted && (action & GDK_ACTION_ASK))
-	{
-		E2_MenuWait wdata;
-		//this loop for popup menu must be in default context and the menu will
-		//not survive any UI event
-		wdata.loopdata = e2_main_loop_new (TRUE);
-		if (wdata.loopdata != NULL)
-		{
-			GtkWidget *drag_op_menu = gtk_menu_new();
-			GtkWidget *item1, *item2, *item3, *item4;
-
-			if (action & GDK_ACTION_COPY)
-				item1 = e2_menu_add (drag_op_menu, _("_Copy"), NULL, NULL,
-					_e2_dnd_drag_op_selected_cb, &wdata);
-			else
-				item1 = NULL;
-
-			if (action & GDK_ACTION_MOVE)
-				item2 = e2_menu_add (drag_op_menu, _("_Move"), NULL, NULL,
-					_e2_dnd_drag_op_selected_cb, &wdata);
-			else
-				item2 = NULL;
-
-			if (action & GDK_ACTION_LINK)
-				item3 = e2_menu_add (drag_op_menu, _("_Link"), NULL, NULL,
-					_e2_dnd_drag_op_selected_cb, &wdata);
-			else
-				item3 = NULL;
-
-	/*		if (?)	//drop target is an executable item
-				item4 = e2_menu_add (drag_op_menu, _("_SendTo"), NULL, NULL,
-					_e2_dnd_drag_op_selected_cb, &wdata);
-			else
-				item4 = NULL;
-	*/
-			item4 = e2_menu_add (drag_op_menu, _("C_ancel"), NULL, NULL,
-				_e2_dnd_drag_op_selected_cb, &wdata);
-			wdata.chosen = item4; //default in case of abort == cancel
-
-			gtk_menu_popup (GTK_MENU (drag_op_menu), NULL, NULL, NULL, NULL, 0,
-				(guint32) time);
-
-			//thread cleanup not needed here, any UI action which might lead to
-			//aborting the loop already kills the menu and gets away from here
-//			pthread_cleanup_push (e2_main_cleanup_gdklock, GINT_TO_POINTER (TRUE));
-			gdk_threads_leave ();
-			e2_main_loop_run (wdata.loopdata);
-			gdk_threads_enter ();
-//			pthread_cleanup_pop (0);
-
-			gtk_widget_destroy (drag_op_menu);
-			if (wdata.chosen == item1)
-				action = GDK_ACTION_COPY;
-			else if (wdata.chosen == item2)
-				action = GDK_ACTION_MOVE;
-			else if (wdata.chosen == item3)
-				action = GDK_ACTION_LINK;
-/*			else if (wdata.chosen == item4)
-				setup to handle the sendto request
-*/
-			else //the user cancelled
-				goto cleanbadexit;
-		}
-		else
-			goto cleanbadexit;	//do nothing
-	}
-	else	//don't ask
-		action = context->suggested_action;
-
-	//convert uris vector to dest path and names array, for processing
-	//FIXME vfs
-	GPtrArray *names_array = g_ptr_array_new ();
-	gchar *basename, *path_local, *check;
-	gchar *src_local = NULL;	//path of each dropped item, localised with trailing /
-	gint path_len = -1;
-//	GError *error = NULL;
-	gchar **tmp = uris;
-	while (*tmp != NULL)
-	{
-		path_local = g_filename_from_uri (*tmp, NULL, NULL);	//&error);
-		if (path_local == NULL)
-		{	//check if we have a bad file URI
-			check = g_filename_to_uri (*tmp, NULL, NULL);
-			if (check != NULL)
-			{
-				path_local = g_filename_from_uri (check, NULL, NULL);
-				g_free (check);
-			}
-		}
-		if (path_local != NULL)
-		{
-			if (path_len == -1)
-			{	//once-only, setup some things
-				//assumes at least 1 separator and no trailing separator CHECKME ok ?
-				basename = strrchr (path_local, G_DIR_SEPARATOR);
-				basename++;
-				path_len = basename - path_local;
-				src_local = g_strndup (path_local, path_len);
-			}
-			//scrub the path, want only the basename in the array
-			g_strlcpy (path_local, path_local + path_len, strlen (path_local + path_len) + 1);
-			g_ptr_array_add (names_array, path_local);
-		}
-/*		else
-		{
-			//FIXME warn the user
-			g_error_free (error);
-		}
-*/
-		tmp++;
-	}
-	g_strfreev (uris);
-	//check for malformed uris
-	if (names_array->len == 0 || src_local == NULL)
-	{
-		g_ptr_array_free (names_array, TRUE);
-		goto badexit;
-	}
-
-	//do it
-	//FIXME handle sendto
-	E2_TaskType droptask;
-	if (action & GDK_ACTION_COPY)
-		droptask = E2_TASK_COPY;
-	else if (action & GDK_ACTION_MOVE)
-		droptask = E2_TASK_MOVE;
-	else if (action & GDK_ACTION_LINK)
-		droptask = E2_TASK_LINK;
-	else
-		droptask = -1;
-	gboolean result = (droptask != -1) ?
-		e2_task_drop (droptask, src_local, dest_local, names_array): FALSE;
-	context->action = action;
-	gboolean move = (action & GDK_ACTION_MOVE);
-	//paths and names are not copied in the handler, so may not be cleared here
-//	g_free (src_local);
-//	g_free (dest_local);
-//	g_ptr_array_free (names_array, TRUE);
-	goto exit;
-
-cleanbadexit:
-	g_strfreev (uris);
-badexit:
-	result = FALSE;
-	move = FALSE;
-exit:
-	gtk_drag_finish (context, result, move, time);
-//CHECKME filtermodel ...
-/* Must override the default 'drag_data_received' handler on GtkTreeView when
-using models that don't support the GtkTreeDragDest interface and enabling
-drag-and-drop. The simplest way to do this is to connect to 'drag_data_received'
-and call g_signal_stop_emission_by_name() in your signal handler to prevent the
-default handler from running. Look at the source code for the default handler in
-gtktreeview.c to get an idea what your handler should do.
-*/
-	//CHECKME nothing else needs doing here ?
-	g_signal_stop_emission_by_name (G_OBJECT (view->treeview),
-		"drag-data-received");
-	return;
 }
