@@ -1,6 +1,6 @@
-/* $Id: e2_filelist.c 2024 2009-12-05 07:12:47Z tpgww $
+/* $Id: e2_filelist.c 2486 2012-04-06 10:14:55Z tpgww $
 
-Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>.
+Copyright (C) 2004-2012 tooar <tooar@emelfm2.net>.
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -14,8 +14,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 /**
@@ -23,10 +22,84 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 @brief directory content liststore functions
 
 This file contains functions related to creation, filling and emptying a
-liststore of directory content data, and fns for the associated model.
+liststore of directory content data, and funcs for the associated model.
 Also filtering of store content, and other related utilties.
 */
 
+/**
+\page refresh filelist refreshing
+
+\section automatic automatic refreshing
+
+Automatic refreshing of filelists occurs if the value of configuration option
+"auto-refresh" is TRUE.
+
+Core infrastructure is a timer - app.timers[DIRTYCHECK_T]. This is used for either
+'default' change-monitoring (a polling process), or enhanced monitoring (using
+gamin, or a relevant kernel capability) if such is built-in and actually working.
+
+The default mechanism for this is periodic polling of the mtime and ctime of
+the directories currently displayed in the filelists.
+
+If enhanced monitoring is used, suitable polling is handled by the relevant backend.
+
+The kernel-backend, or the poller (via e2_fs_FAM_check_dirty()), sets dirty-flag(s)
+for filelist(s), as appropriate. Then for both panes:
+  If its dirty-flag is set:
+    Clear its dirty-flag, ready for downstream use again
+    If function-argument (userdata) is non-NULL (the usual for direct calls)
+		or a repeat-count (via _e2_filelist_refresh_ok()) is suitable:
+      Set refresh-requested flag for the pane
+Then if either refresh-requested flag is now set (maybe from elsewhere, see below):
+  If timer REFRESHBEGIN_T is active, abort it
+  Start timer REFRESHBEGIN_T with callback to _e2_filelist_refresh_manage()
+
+_e2_filelist_refresh_manage() polls both panes' refresh-requested flags, and, if
+and when not blocked by some other process that affects the pane's content,
+initiates (and joins to) a thread to refresh the relevant filelist. This process
+iterates until all requests have been handled, so there can be > 1 refresh before
+the function returns.
+
+With enhanced monitoring, the same interface is used to gather information about
+filelists and the config file, if the latter is being monitored. However any
+config-file-change is merely flagged, ready for processing by the corresponding
+process.
+
+\section manual manual refreshing
+
+Either or both panes' refresh-requested flag can be set at any time via a call to
+e2_filelist_request_refresh(). If the immediate flag is not set, that request will
+be acted upon next time e2_filelist_check_dirty() is called, manually or as a
+timer callback.
+
+e2_filelist_check_dirty() can be called manually, usually with a non-NULL argument.
+This will check for and refresh pane(s) flagged as 'dirty' or 'refresh-requested'.
+
+/section blocking refresh blocking
+
+Refreshing of either or both panes is contstrained by corresponding ref-counts,
+which can be increased or decreased by calls to e2_filelist_[dis|en]able_refresh()
+and/or to e2_filelist_[dis|en]able_one_refresh()
+
+Refreshing of either pane is blocked while either pane is still being refreshed
+as a result of a prior update
+
+\section config configuration file changes
+
+Automatic refreshing of config-file changes happens if the value of
+configuration option "auto-refresh-config" is TRUE.
+
+Core infrastructure is a timer - app.timers[CONFIG_T]. It has a longer intervals
+than the filelist timer. It is used for either 'default' change-monitoring, or
+enhanced monitoring (using gamin, or a relevant kernel capability) if such is
+built-in and actually working.
+
+The default mechanism is periodic polling of the mtime of the config file (the
+default or as specified in a startup parameter).
+
+If enhanced monitoring is used, suitable polling is handled by the relevant backend.
+*/
+
 #include "e2_filelist.h"
 #include <string.h>
 #include <sys/time.h>
@@ -47,17 +120,60 @@ extern GdkColor selectedtext;
 extern sigset_t dnotify_signal_set;
 #endif
 
-//disable-refresh counter for both filelists
-gint refresh_refcount;
-//disable-refresh counter for single filelists
-static gint refresh_active_refcount;
-static gint refresh_inactive_refcount;
+//filelist-needs-refresh flags, used by timers and threads, always do atomic access
+#ifdef E2_FAM_KERNEL
+volatile
+#endif
+ gint p1dirty = 0;
+#ifdef E2_FAM_KERNEL
+volatile
+#endif
+ gint p2dirty = 0;
 //refresh deferral counters
 static gint pane1repeats = 0;
 static gint pane2repeats = 0;
+//for managing timer suspension
+time_t last_work_time = 0;
 
-//static gboolean case_sensitive; //local copy of option value, for faster access
+extern
+#ifdef E2_FAM_KERNEL
+ volatile
+#endif
+ gint cfgdirty;
+extern volatile gint cfg_refresh_refcount;
+/**
+@brief "event" signal callback which resumes polling for whether a refresh is needed
+@param widget pointer to main-window widget
+@param UNUSED event pointer to event data
+@param userdata parameter specified when callback was connected, helps to identify the signal
 
+@return FALSE always, so the event will propogate
+*/
+gboolean e2_filelist_repoll (GtkWidget *widget, GdkEvent *event,
+	gpointer userdata)
+{
+	g_signal_handlers_disconnect_by_func ((gpointer)widget,
+		e2_filelist_repoll, userdata);
+	last_work_time = time (NULL);
+	printd (DEBUG, "Resume operation of DIRTYCHECK_T timer");
+#ifdef E2_FAM_KERNEL
+	if (app.monitor_type == E2_MONITOR_DEFAULT)
+	{
+#endif
+		app.timers[DIRTYCHECK_T] =
+#ifdef USE_GLIB2_14
+			g_timeout_add_seconds (E2_FILESCHECK_INTERVAL_S,
+#else
+			g_timeout_add (E2_FILESCHECK_INTERVAL,
+#endif
+				(GSourceFunc) e2_filelist_check_dirty, NULL);
+#ifdef E2_FAM_KERNEL
+	}
+	else
+		e2_fs_FAM_resume ();
+#endif
+	return FALSE;
+}
 /**
 @brief determine whether pane defined by @a pane is the active one
 @param pane enumerator for pane to be checked
@@ -93,25 +209,9 @@ working in a displayed directory
 void e2_filelist_disable_one_refresh (E2_ListChoice pane)
 {
 	if (_e2_filelist_check_active_pane (pane))
-	{
-		refresh_active_refcount++;
-		if (refresh_active_refcount == 1)
-		{
-			LISTS_LOCK
-			curr_view->listcontrols.norefresh = TRUE;
-			LISTS_UNLOCK
-		}
-	}
+		g_atomic_int_add (&curr_view->listcontrols.refresh_refcount, 1);
 	else
-	{
-		refresh_inactive_refcount++;
-		if (refresh_inactive_refcount == 1)
-		{
-			LISTS_LOCK
-			other_view->listcontrols.norefresh = TRUE;
-			LISTS_UNLOCK
-		}
-	}
+		g_atomic_int_add (&other_view->listcontrols.refresh_refcount, 1);
 }
 /**
 @brief enable refreshing of a single pane filelist
@@ -124,27 +224,13 @@ void e2_filelist_enable_one_refresh (E2_ListChoice pane)
 {
 	if (_e2_filelist_check_active_pane (pane))
 	{
-		refresh_active_refcount--;
-		if (refresh_active_refcount == 0 && refresh_refcount == 0)
-		{
-			LISTS_LOCK
-			curr_view->listcontrols.norefresh = FALSE;
-			LISTS_UNLOCK
-		}
-		if (refresh_active_refcount < 0)
-			refresh_active_refcount = 0;
+		if (g_atomic_int_exchange_and_add (&curr_view->listcontrols.refresh_refcount, -1) < 1)
+			g_atomic_int_set (&curr_view->listcontrols.refresh_refcount, 0);
 	}
 	else
 	{
-		refresh_inactive_refcount--;
-		if (refresh_inactive_refcount == 0 && refresh_refcount == 0)
-		{
-			LISTS_LOCK
-			other_view->listcontrols.norefresh = FALSE;
-			LISTS_UNLOCK
-		}
-		if (refresh_inactive_refcount < 0)
-			refresh_inactive_refcount = 0;
+		if (g_atomic_int_exchange_and_add (&other_view->listcontrols.refresh_refcount, -1) < 1)
+			g_atomic_int_set (&other_view->listcontrols.refresh_refcount, 0);
 	}
 }
 /**
@@ -180,11 +266,9 @@ gboolean e2_filelist_enable_refresh_action (gpointer from, E2_ActionRuntime *art
 */
 void e2_filelist_reset_refresh (void)
 {
-	if (refresh_refcount != 0)
-	{
-		refresh_refcount = 1;
-		e2_filelist_enable_refresh ();
-	}
+	g_atomic_int_set (&app.pane1.view.listcontrols.refresh_refcount, 0);
+	g_atomic_int_set (&app.pane2.view.listcontrols.refresh_refcount, 0);
+	g_atomic_int_set (&cfg_refresh_refcount, 0);
 }
 /* These functions must encapsulate any instructions that access the FileInfo
 structs produced when a selection is interrogated, because a refresh in the
@@ -192,147 +276,168 @@ middle of such an operation would free FileInfo structs as a side-effect.
 */
 /**
 @brief  disable refreshing of pane filelists and config data
-
-Increase ref count, if == 1, stop the timer process which calls the refresh fns
-Why is config data check bundled here ?
-Expects BGL open?
-
-return
+Increases all refresh ref-counts, which will prevent any refresh from happening
+@return
 */
 void e2_filelist_disable_refresh (void)
 {
-	refresh_refcount++;
-#ifdef E2_REFRESH_DEBUG
-	printd (DEBUG, "disable refresh, refresh ref now = %d", refresh_refcount);
-#endif
-	if (refresh_refcount == 1)
+	if (e2_option_bool_get ("auto-refresh"))
 	{
-		if (app.timers[REFRESHBEGIN_T] > 0)
-		{
-			//we're waiting for a pending refresh
-			g_source_remove (app.timers[REFRESHBEGIN_T]);
-			app.timers[REFRESHBEGIN_T] = 0;
-		}
-		if (e2_option_bool_get ("auto-refresh"))
-		{
-			//don't cancel check-dirty timer, to reduce risk that FAM
-			//dirty-reports queue may over-fill
-			LISTS_LOCK
-			curr_view->listcontrols.norefresh = TRUE;
-			other_view->listcontrols.norefresh = TRUE;
-			LISTS_UNLOCK
-		}
-		if (e2_option_bool_get ("auto-refresh-config"))
-		{
-			//CHECKME just block this too?
-			//(can't reasonably be many changes to the config file)
-			if (app.timers[CONFIG_T] > 0)
-			{
-				g_source_remove (app.timers[CONFIG_T]);
-				app.timers[CONFIG_T] = 0;
-			}
-		}
-
-#ifndef E2_STATUS_DEMAND
-		e2_window_disable_status_update ();
-#endif
+		g_atomic_int_add (&curr_view->listcontrols.refresh_refcount, 1);
+		g_atomic_int_add (&other_view->listcontrols.refresh_refcount, 1);
 	}
+	//same as e2_option_disable_config_checks()
+	if (e2_option_bool_get ("auto-refresh-config"))
+		g_atomic_int_add (&cfg_refresh_refcount, 1);
 }
 /**
 @brief re-enable polling of pane filelists and config data
-
-Decrease ref count, if == 0, re-enable/restart the processes which detect
-changes of filelist content and/or config data
-Config timer is bundled here because the same func refreshes the panes
-Expects BGL open
-
+Decreases all ref-counts, which will enable refresh for any that are now 0
 @return
 */
 void e2_filelist_enable_refresh (void)
 {
-	refresh_refcount--;
-#ifdef E2_REFRESH_DEBUG
-	printd (DEBUG, "enable refresh, refresh ref now = %d", refresh_refcount);
-#endif
-	if (refresh_refcount < 0)
+	if (e2_option_bool_get ("auto-refresh"))
 	{
-		printd (WARN, "The auto-refresh refresh count is < 0");
-		gdk_threads_enter ();
-		e2_window_output_show (NULL, NULL);
-		e2_output_print_error (
-		_("Something is wrong with the auto-refresh mechanism"), FALSE);
-		gdk_threads_leave ();
+		if (g_atomic_int_exchange_and_add (&curr_view->listcontrols.refresh_refcount, -1) < 1)
+			g_atomic_int_set (&curr_view->listcontrols.refresh_refcount, 0);
+		if (g_atomic_int_exchange_and_add (&other_view->listcontrols.refresh_refcount, -1) < 1)
+			g_atomic_int_set (&other_view->listcontrols.refresh_refcount, 0);
 	}
-
-	if (refresh_refcount == 0)
+	//same as e2_option_enable_config_checks()
+	if (e2_option_bool_get ("auto-refresh-config"))
 	{
-		if (e2_option_bool_get ("auto-refresh"))
-		{
-			LISTS_LOCK
-			curr_view->listcontrols.norefresh = FALSE;
-			other_view->listcontrols.norefresh = FALSE;
-			LISTS_UNLOCK
-		}
-		//this timer is used, even with fam backends
-		if (e2_option_bool_get ("auto-refresh-config"))
-			app.timers[CONFIG_T] =
-#ifdef USE_GLIB2_14
-			g_timeout_add_seconds (E2_CONFIGCHECK_INTERVAL_S,
-#else
-			g_timeout_add (E2_CONFIGCHECK_INTERVAL,
-#endif
-				(GSourceFunc) e2_option_check_config_files, NULL);
-
-#ifndef E2_STATUS_DEMAND
-		e2_window_enable_status_update (-1);
-#endif
+		if (g_atomic_int_exchange_and_add (&cfg_refresh_refcount, -1) < 1)
+			g_atomic_int_set (&cfg_refresh_refcount, 0);
 	}
 }
 /**
-@brief idle function to run refresh hooklist
+@brief idle function to run the refresh-hooklist of @s view
+This happens after the end of the refresh timer callback, so in principle,
+another refresh, or a cd, could start before the hook funcs are called.
+So they should check for those races, and respond accordingly.
 @param view pointer to view data struct
+
 @return FALSE to abort the source
 */
-static gboolean _e2_fileview_more_refresh (ViewInfo *view)
+static gboolean _e2_filelist_propogate_refresh (ViewInfo *view)
 {
 	e2_hook_list_run (&view->hook_refresh, view);
 	return FALSE;
 }
 /**
-@brief timer destroy function after cancelling filelists checking
-@param data UNUSED data specified when the timer was established
+@brief timer destroy function
+
+@param data pointerised index of the timer that ended
+
 @return
 */
-static void _e2_filelist_timer_shutdown (gpointer data)
+void e2_filelist_timer_shutdown (gpointer data)
 {
-	app.timers[REFRESHBEGIN_T] = 0;
+	guint index = GPOINTER_TO_UINT (data);
+	app.timers[index] = 0;
+#ifdef E2_REFRESH_DEBUG
+	if (index == REFRESHBEGIN_T)
+		printd (DEBUG, "timer REFRESHBEGIN_T id set to 0");
+#endif
 }
+
+#if 1
+/**
+@brief thread-function which performs filelist refresh if it can
+If refreshing is blocked, there is no deferral
+@param view pointer to data struct for the filelist to be refreshed
+@return NULL always
+*/
+static gpointer _e2_filelist_refresh_view (ViewInfo *view)
+{
+	e2_utils_block_thread_signals ();
+
+	if (g_atomic_int_get (&view->listcontrols.refresh_refcount) == 0)
+	{	//this refresh is not disabled
+		if (g_atomic_int_compare_and_exchange (&view->listcontrols.refresh_requested, 1, 0))
+		{
+			gboolean hook = FALSE;
+			gboolean ready;
+retest:
+			ready = !(
+			g_atomic_int_get (&view->listcontrols.refresh_working) ||	//or busy
+			g_atomic_int_get (&view->listcontrols.cd_working)
+#ifdef E2_STATUS_BLOCK
+			|| g_atomic_int_get (&app.status_working)
+#endif
+			);
+
+			if (ready)
+			{
+#ifdef E2_VFSTMP
+				//FIXME dir when not mounted local
+#else
+				printd (DEBUG, "accepting request to refresh %s", view->dir);
+#endif
+				//refresh function expects BGL open
+				if (e2_fileview_refresh_list (view) == GINT_TO_POINTER(1))
+				{
+					hook = TRUE;	//send it downstream
+					//maybe again/now/still want to do this pane
+					if (g_atomic_int_get (&view->listcontrols.refresh_refcount) == 0
+						&&
+						g_atomic_int_compare_and_exchange (&view->listcontrols.refresh_requested, 1, 0))
+					{
+						printd (DEBUG, "Sequential refresh requested for %s", view->dir);
+						usleep (10000);
+						goto retest;
+					}
+				}
+			}
+			//initiate refresh hooklist here, not downstream, so that refreshes are batched,
+			//and in idle to avoid any UI impact
+			//(i.e. small risk of cd / another refresh starting before hook is called)
+			if (hook)
+				g_idle_add ((GSourceFunc)_e2_filelist_propogate_refresh, view);
+		}
+	}
+
+	return NULL;
+}
+#else //original approach to refreshing
 /**
 @brief timer callback function which refreshes filelist(s) as appropriate, as soon as any block is gone
-@param data pointerised value of current timer delay
+
+@param unused_data UNUSED pointer set when timer was initiated (actually, it has the timer index, REFRESHBEGIN_T)
+
 @return FALSE when there's nothing left to refresh or more waiting is needed
 */
-static gboolean _e2_filelist_refresh_manage (gpointer data)
+static gboolean _e2_filelist_refresh_manage (gpointer unused_data)
 {
+	ViewInfo *cv, *ov;
 	pthread_t thisID;
 	gpointer result;
-	gboolean cvdone, ovdone, cvhook, ovhook;
+	gboolean cvreq, ovreq, cvdone, ovdone, cvhook, ovhook;
+	gint ovsave;
+
 	LISTS_LOCK
-	ViewInfo *cv = curr_view;	//log these, in case active pane is changed during this process
-	ViewInfo *ov = other_view;
+	//log these, in case active pane is changed during this process
+	cv = curr_view;
+	ov = other_view;
+	//ensure that refresh is started if either pane needs it now
+	cvreq = g_atomic_int_get (&cv->listcontrols.refresh_requested);
+	ovreq = g_atomic_int_get (&ov->listcontrols.refresh_requested);
+
 	cvhook = ovhook = FALSE;
+	ovsave = 0;
 
 	e2_utf8_set_name_conversion_if_requested ();
 
-retest:
-	if (cv->listcontrols.refresh_requested)
+	if (cvreq)
 	{
+retestc:
 		cvdone = !(
-		cv->listcontrols.norefresh || //norefresh flags set when refresh is disabled
-		cv->listcontrols.refresh_working ||
-		cv->listcontrols.cd_working
+		g_atomic_int_get (&cv->listcontrols.refresh_refcount) || //this refresh is disabled
+		g_atomic_int_get (&cv->listcontrols.refresh_working) ||	//or busy
+		g_atomic_int_get (&cv->listcontrols.cd_working)
 #ifdef E2_STATUS_BLOCK
-		|| app.status_working
+		|| g_atomic_int_get (&app.status_working)
 #endif
 		);
 		if (cvdone)
@@ -347,28 +452,31 @@ retest:
 			if (pthread_create (&thisID, NULL,
 				(gpointer(*)(gpointer))e2_fileview_refresh_list, cv) == 0)
 			{
+				g_atomic_int_set (&cv->listcontrols.refresh_requested, 0); //not needed downstream
 				pthread_join (thisID, &result); //only 1 refresh at a time
-				cvhook = TRUE;	//send it downstream
+				if (result == GINT_TO_POINTER (1))
+					cvhook = TRUE;	//send it downstream
+				else if (result == NULL) //try again
+					g_atomic_int_set (&cv->listcontrols.refresh_requested, 1); //revert
 			}
 			else
 				printd (WARN, "Failed to create list refresh thread");
+
 			LISTS_LOCK
-			//clear flag, too bad if a request happened late in the process
-			if (result != NULL)
-				cv->listcontrols.refresh_requested = FALSE;
 		}
 	}
 	else
-		cvdone = TRUE;
-retest2:
-	if (ov->listcontrols.refresh_requested)
+		cvdone = FALSE;
+
+	if (ovreq)
 	{
+retesto:
 		ovdone = !(
-		ov->listcontrols.norefresh ||
-		ov->listcontrols.refresh_working ||
-		ov->listcontrols.cd_working
+		g_atomic_int_get (&ov->listcontrols.refresh_refcount) ||
+		g_atomic_int_get (&ov->listcontrols.refresh_working) ||
+		g_atomic_int_get (&ov->listcontrols.cd_working)
 #ifdef E2_STATUS_BLOCK
-		|| app.status_working
+		|| g_atomic_int_get (&app.status_working)
 #endif
 		);
 		if (ovdone)
@@ -382,51 +490,64 @@ retest2:
 			if (pthread_create (&thisID, NULL,
 				(gpointer(*)(gpointer))e2_fileview_refresh_list, ov) == 0)
 			{
+				g_atomic_int_set (&ov->listcontrols.refresh_requested, 0); //not needed downstream
 				pthread_join (thisID, &result); //only 1 refresh at a time
-				ovhook = TRUE;
+				if (result == GINT_TO_POINTER (1))
+					ovhook = TRUE;
+				else if (result == NULL) //try again
+					g_atomic_int_set (&ov->listcontrols.refresh_requested, 1);
 			}
 			else
 				printd (WARN, "Failed to create list refresh thread");
+
 			LISTS_LOCK
-			if (result != NULL)
-				ov->listcontrols.refresh_requested = FALSE;
 		}
 	}
 	else
-		ovdone = TRUE;
+		ovdone = FALSE;
 
-	if (cv->listcontrols.refresh_requested || ov->listcontrols.refresh_requested)
+	if (cvreq || ovreq)
 	{
-		//again/now/still want to do one or both panes
-		//but only go back now not blocked before
-		if (cv->listcontrols.refresh_requested && cvdone)
+		if (ovdone)
+			ovsave++;	//save satus
+		//maybe again/now/still want to do one or both panes
+		//but only go back now if not blocked before
+		if (cvdone && g_atomic_int_get (&cv->listcontrols.refresh_requested))
 		{
-			LISTS_UNLOCK
-			goto retest;
-		}
-		if (ov->listcontrols.refresh_requested && ovdone)
-		{
-			LISTS_UNLOCK
-			goto retest2;
+			usleep (10000);
+			ovreq = FALSE; //don't (yet) repeat ov (ok cuz cvreq is still TRUE)
+			goto retestc; //sets ovdone FALSE
 		}
 
-		printd (DEBUG, "more refresh request(s), deferred because busy");
+		if (ovsave > 0 && g_atomic_int_get (&ov->listcontrols.refresh_requested))
+		{
+			usleep (10000);
+			ovreq = TRUE; //reinstate after any cv repeat
+			ovsave = 0;
+			goto retesto;
+		}
+
+#ifdef E2_REFRESH_DEBUG
+		printd (DEBUG, "start timer REFRESHBEGIN_T");
+#endif
 		app.timers[REFRESHBEGIN_T] = g_timeout_add_full (G_PRIORITY_HIGH, 200,
-			(GSourceFunc) _e2_filelist_refresh_manage, GINT_TO_POINTER (200),
-				(GDestroyNotify) _e2_filelist_timer_shutdown);
+			(GSourceFunc) _e2_filelist_refresh_manage,
+			GUINT_TO_POINTER (REFRESHBEGIN_T),
+			(GDestroyNotify) e2_filelist_timer_shutdown);
 	}
 
 	LISTS_UNLOCK
-	//CHECKME do this separately e.g. in idle, to avoid any delay ?
+	//initiate refresh hooklists here, not downstream, so that refreshes are batched,
+	//and in idle to avoid any UI impact
+	//(i.e. small risk of cd / another refresh starting before hook is called)
 	if (cvhook)
-		g_idle_add ((GSourceFunc)_e2_fileview_more_refresh, curr_view);
-//		e2_hook_list_run (&curr_view->hook_refresh, curr_view);
+		g_idle_add ((GSourceFunc)_e2_filelist_propogate_refresh, curr_view);
 	if (ovhook)
-//		e2_hook_list_run (&other_view->hook_refresh, other_view);
-		g_idle_add ((GSourceFunc)_e2_fileview_more_refresh, other_view);
+		g_idle_add ((GSourceFunc)_e2_filelist_propogate_refresh, other_view);
 
 	return FALSE;
 }
+#endif
 /**
 @brief log request, and possibly initiate, filelist(s) refresh if it/they currently show @a dir
 If a refresh is initiated, either or both flagged lists will be refreshed
@@ -442,27 +563,31 @@ gboolean e2_filelist_request_refresh (gchar *dir, gboolean immediate)
 #ifdef E2_VFSTMP
 	//CHECKME v-dir ok
 #endif
-	if (!strcmp (curr_view->dir, dir))
+	if (strcmp (app.pane1.view.dir, dir) == 0)
 	{
 		matched = TRUE;
-		LISTS_LOCK
-		curr_view->listcontrols.refresh_requested = TRUE;
-		LISTS_UNLOCK
+		g_atomic_int_set (&app.pane1.view.listcontrols.refresh_requested, 1);
 	}
 #ifdef E2_VFSTMP
 	//CHECKME v-dir ok
 #endif
-	if (!strcmp (other_view->dir, dir))
+	if (strcmp (app.pane2.view.dir, dir) == 0)
 	{
 		matched = TRUE;
-		LISTS_LOCK
-		other_view->listcontrols.refresh_requested = TRUE;
-		LISTS_UNLOCK
+		g_atomic_int_set (&app.pane2.view.listcontrols.refresh_requested, 1);
 	}
 
 	if (matched && immediate)
-		//clear any fs report about dirty list, then do the refresh
-		e2_filelist_check_dirty (NULL);
+	{
+#ifdef E2_REFRESH_DEBUG
+		printd (DEBUG, "In e2_filelist_request_refresh, goto e2_filelist_check_dirty");
+#endif
+		e2_filelist_check_dirty (GINT_TO_POINTER (1));
+	}
+#ifdef E2_REFRESH_DEBUG
+	else
+		printd (DEBUG, "In e2_filelist_request_refresh, NO e2_filelist_check_dirty");
+#endif
 
 	return matched;
 }
@@ -472,7 +597,7 @@ This is called when a 'dirty' flag has been detected
 
 @param pane enumerator for pane 1 or pane 2, which was reported dirty
 
-@return TRUE when
+@return TRUE to process the current request now
 */
 static gboolean _e2_filelist_refresh_ok (E2_ListChoice pane)
 {
@@ -516,128 +641,274 @@ static gboolean _e2_filelist_refresh_ok (E2_ListChoice pane)
 	return retval;
 }
 /**
-@brief request refresh of filepane(s) contents (and with FAM, config too) if the corresponding changed is detected
+@brief initiate refresh of filepane(s) contents if requested (elsewhere)
+ or if changed content has been, or is now, detected
 
 Among other uses, this is the timer callback function for filelists auto refresh
-When FAM/gamin in use, it also checks for and flags (not processes)
-config file updates, as that data comes from the same data stream.
 In certain circumstances any dirty flag might be ignored, or deferred to lengthen
-the effective interval between refreshes
+the effective interval between refreshes.
+A directory that is 'gone' will have been flagged as 'dirty', and later the
+list-refesh function handles choosing a replacement dir.
+Non-NULL @a userdata forces refresh of both panes ASAP.
+If this is a timer callback, the cfgdirty flag may be refreshed, but not otherwise
+acted upon.
+If there has been no refresh during the last QUIET_SECONDS, then
+app.timers[DIRTYCHECK_T] is shut down. Otherwise, the time of the last refresh is
+updated.
 This expects gtk's BGL to be off/open
 
-@param userdata data specified when the timer was initialised (NULL), or when called directly, non-NULL
+@param userdata data specified when the timer was initialised (NULL), or when called directly, usually non-NULL
 
 @return TRUE so the timer keeps working
 */
 gboolean e2_filelist_check_dirty (gpointer userdata)
 {
 	static gboolean busy = FALSE;
+	gboolean dop1, dop2;
+
 	if (busy)
-		return TRUE;	//no reentrant usage
-	busy = TRUE;
-#ifdef E2_FAM
-	static gboolean cfgdeferred = FALSE;
-#endif
-
-	gboolean p1dirty, p2dirty;
-#ifdef E2_FAM
-	gboolean localcfgdirty;	//use a local copy so that the extern value doesn't get cleared before it's noticed
-	extern gboolean cfgdirty;
-
-	if (userdata != NULL
-		 && app.monitor_type != E2_MONITOR_DEFAULT)
-			g_usleep (E2_FAMWAIT);	//wait for things to be noticed
-#endif
-
-#ifndef E2_FAM
-	if (userdata != NULL)
 	{
-		p1dirty = p2dirty = TRUE;	//force a refresh
+		if (userdata == NULL)
+		{	//timer callback, not a specific request
+#ifdef E2_REFRESH_DEBUG
+			printd (DEBUG, "e2_filelist_check_dirty is busy, exit immediately");
+#endif
+			return TRUE;	//no re-entrant usage, run the timer again
+		}
+		else
+		{
+			while (busy)
+				usleep (50000);
+		}
+	}
+	busy = TRUE;
+
+#ifdef E2_REFRESH_DEBUG
+	printd (DEBUG, "In e2_filelist_check_dirty()");
+#endif
+
+	if (userdata == NULL)
+	{	//a timer callback
+	/* if E2_FAM_KERNEL backend is working, that will have already set *dirty
+		flag(s) as appropriate*/
+#ifdef E2_FAM_KERNEL
+		if (app.monitor_type == E2_MONITOR_DEFAULT)
+		{
+#endif
+			e2_fs_FAM_check_dirty (&p1dirty, &p2dirty
+#ifdef E2_FAM
+			//cfgdirty flag is logged, but not otherwise processed here
+			, &cfgdirty
+#endif
+			);
+#ifdef E2_FAM_KERNEL
+		}
+#endif
 	}
 	else
-#endif
-	//this reports 'gone' as 'dirty', and then the list-refesh
-	//function handles choosing a replacement dir
-	e2_fs_FAM_check_dirty (&p1dirty, &p2dirty
-#ifdef E2_FAM
-	, &localcfgdirty
-#endif
-	);
+	{	//specific request, start fresh
+		pane1repeats = 0;
+		pane2repeats = 0;
+	}
 
-	p1dirty = p1dirty || pane1repeats;
-	p2dirty = p2dirty || pane2repeats;
+	//ASAP check and clear flags used by backend
+	dop1 = g_atomic_int_compare_and_exchange (&p1dirty, 1, 0);
+	dop2 = g_atomic_int_compare_and_exchange (&p2dirty, 1, 0);
 
-	if (p1dirty || p2dirty)
+	if (dop1 && (userdata != NULL || _e2_filelist_refresh_ok (PANE1)))
 	{
-		LISTS_LOCK
-		if (p1dirty && (userdata != NULL || _e2_filelist_refresh_ok (PANE1)))
-			app.pane1.view.listcontrols.refresh_requested = TRUE;
+		g_atomic_int_set (&app.pane1.view.listcontrols.refresh_requested, 1);
+#ifdef E2_REFRESH_DEBUG
+		printd (DEBUG,"refresh-requested flag set for pane 1");
+#endif
+	}
+	else
+	{
+#ifdef E2_REFRESH_DEBUG
+		if (dop1)
+			printd (DEBUG,"dirty set FALSE for pane 1");
+# ifdef E2_FAM_KERNEL
 		else
-			p1dirty = FALSE;
-		if (p2dirty && (userdata != NULL || _e2_filelist_refresh_ok (PANE2)))
-			app.pane2.view.listcontrols.refresh_requested = TRUE;
+			printd (DEBUG,"pane 1 is clean");
+# endif
+#endif
+		dop1 = g_atomic_int_get (&app.pane1.view.listcontrols.refresh_requested);
+	}
+
+	if (dop2 && (userdata != NULL || _e2_filelist_refresh_ok (PANE2)))
+	{
+		g_atomic_int_set (&app.pane2.view.listcontrols.refresh_requested, 1);
+#ifdef E2_REFRESH_DEBUG
+		printd (DEBUG,"refresh-requested flag set for pane 2");
+#endif
+	}
+	else
+	{
+#ifdef E2_REFRESH_DEBUG
+		if (dop2)
+			printd (DEBUG,"dirty set FALSE for pane 2");
+# ifdef E2_FAM_KERNEL
 		else
-			p2dirty = FALSE;
-		LISTS_UNLOCK
-		if (p1dirty || p2dirty)
+			printd (DEBUG,"pane 2 is clean");
+# endif
+#endif
+		dop2 = g_atomic_int_get (&app.pane2.view.listcontrols.refresh_requested);
+	}
+
+	if (dop1 || dop2)
+	{
+		last_work_time = time (NULL);
+#if 1
+		pthread_t thisID;
+		pthread_attr_t attr;
+
+		e2_utf8_set_name_conversion_if_requested ();
+
+		pthread_attr_init (&attr);
+		pthread_attr_setdetachstate (&attr, PTHREAD_CREATE_DETACHED);
+		//priority for active pane
+		if (g_atomic_int_get (&curr_view->listcontrols.refresh_requested))
 		{
-			if (app.timers[REFRESHBEGIN_T] > 0)
+			if (pthread_create (&thisID, &attr,
+				(gpointer(*)(gpointer))_e2_filelist_refresh_view, curr_view) != 0)
 			{
-				//clear any incomplete refresh request
-				g_source_remove (app.timers[REFRESHBEGIN_T]);
+				//TODO handle error
+				printd (WARN,"refresh-dir-thread-create error!");
 			}
+		}
+		if (g_atomic_int_get (&other_view->listcontrols.refresh_requested))
+		{
+			if (pthread_create (&thisID, &attr,
+				(gpointer(*)(gpointer))_e2_filelist_refresh_view, other_view) != 0)
+			{
+				//TODO handle error
+				printd (WARN,"refresh-dir-thread-create error!");
+			}
+		}
+
+		pthread_attr_destroy (&attr);
+#else
+		if (app.timers[REFRESHBEGIN_T] > 0)
+		{
+			//clear any incomplete refresh request
+#ifdef E2_REFRESH_DEBUG
+			printd (DEBUG, "stop timer REFRESHBEGIN_T");
+#endif
+			g_source_remove (app.timers[REFRESHBEGIN_T]);
+		}
 		//the delay between attempts will be lengthened later, if appropriate
-//CHECKME we may be in a timer callback here - are nested timers still acceptable to glib/gtk ?
-			app.timers[REFRESHBEGIN_T] = g_timeout_add_full (G_PRIORITY_HIGH, 5,
-				(GSourceFunc) _e2_filelist_refresh_manage, GINT_TO_POINTER (5),
-				(GDestroyNotify) _e2_filelist_timer_shutdown);
-		}
+//CHECKME we may already be in a timer callback here - are nested timers acceptable to glib/gtk ?
+#ifdef E2_REFRESH_DEBUG
+		printd (DEBUG, "Start timer REFRESHBEGIN_T to initiate refresh");
+#endif
+		app.timers[REFRESHBEGIN_T] = g_timeout_add_full (G_PRIORITY_HIGH, 5,
+			(GSourceFunc) _e2_filelist_refresh_manage,
+			GUINT_TO_POINTER (REFRESHBEGIN_T),
+			(GDestroyNotify) e2_filelist_timer_shutdown);
+#endif
 	}
-
-#ifdef E2_FAM
-	if (localcfgdirty || cfgdeferred)
+	else
 	{
-		if (!app.rebuild_enabled)
+		if (time (NULL) >= last_work_time + QUIET_SECONDS)
 		{
-			cfgdeferred = TRUE;
-			printd (DEBUG, "config refresh deferred");
-		}
-		else
-		{
-			cfgdeferred = FALSE;
-			if (localcfgdirty)
-			{
-				cfgdirty = TRUE;	//set, but don't clear, the main flag
-				printd (DEBUG, "config refresh flag set");
-			}
+			printd (DEBUG, "Suspend operation of timer DIRTYCHECK_T");
+			g_source_remove (app.timers[DIRTYCHECK_T]);
+			app.timers[DIRTYCHECK_T] = 0;
+			//arrange to restart upon activity
+			g_signal_connect (G_OBJECT (app.main_window), "event",
+				G_CALLBACK (e2_filelist_repoll), GUINT_TO_POINTER (DIRTYCHECK_T));
 		}
 	}
-#endif	//def E2_FAM
 
 	busy = FALSE;
 	return TRUE;
 }
 /**
-@brief initialise timer which polls for various 'dirty' indicator(s)
-
-The timer is never suspended or paused, to avoid the risk of overflow from
-kernel-fam processes. It may be stopped permanently if change-monitoring is
-abandoned via a config dialog
+@brief stop change-monitoring
+Either stop a timer which polls for various 'dirty' indicator(s), or stop
+the 'kernel-backend' process(es)  in which case it expects app.monitor_type
+to be set appropriately
+No check here for "auto-refresh" option status
 
 @return
 */
-void e2_filelist_start_refresh_polling (void)
+void e2_filelist_stop_refresh_checks (void)
 {
-	if (e2_option_bool_get ("auto-refresh"))
-	{
-		app.timers[DIRTYCHECK_T] =
-#ifdef USE_GLIB2_14
-			g_timeout_add_seconds (E2_FILESCHECK_INTERVAL_S,
-#else
-			g_timeout_add (E2_FILESCHECK_INTERVAL,
+#ifdef E2_FAM_KERNEL
+	//stop backend working
+	e2_fs_FAM_cancel_monitor_dir (&app.pane1);
+	e2_fs_FAM_cancel_monitor_dir (&app.pane2);
 #endif
+	//this timer is used whatever app.monitor_type is, with E2_FAM_KERNEL
+	//TODO stopping it also shuts down cfg monitoring, which shares the mechanism
+	if (app.timers[DIRTYCHECK_T] != 0)
+	{
+#ifdef E2_REFRESH_DEBUG
+		printd (DEBUG, "stop timer DIRTYCHECK_T");
+#endif
+		g_source_remove (app.timers[DIRTYCHECK_T]);
+		app.timers[DIRTYCHECK_T] = 0;
+	}
+	g_atomic_int_set (&app.pane1.view.listcontrols.refresh_refcount, 0);
+	g_atomic_int_set (&app.pane2.view.listcontrols.refresh_refcount, 0);
+}
+/**
+@brief start change-monitoring
+Either start a timer which polls for various 'dirty' indicator(s), or start
+the 'kernel-backend' process(es)- in which case it expects app.monitor_type
+to be set appropriately
+No check here for "auto-refresh" option status
+The timer (if any) may later be stopped permanently if change-monitoring is
+abandoned, probably via a config dialog. But it won't be stopped merely if
+change-monitoring is disabled i.e. suspended
+
+@return
+*/
+void e2_filelist_start_refresh_checks (void)
+{
+	g_atomic_int_set (&app.pane1.view.listcontrols.refresh_refcount, 0);
+	g_atomic_int_set (&app.pane2.view.listcontrols.refresh_refcount, 0);
+#ifdef E2_FAM_KERNEL
+/* app.monitor_type setup needs a prior call to setup kernel-monitoring if
+	that's in the build */
+	if (app.monitor_type == E2_MONITOR_DEFAULT)
+	{
+		if (app.timers[DIRTYCHECK_T] == 0)
+		{
+# ifdef E2_REFRESH_DEBUG
+			printd (DEBUG, "start timer DIRTYCHECK_T");
+# endif
+			app.timers[DIRTYCHECK_T] =
+# ifdef USE_GLIB2_14
+				g_timeout_add_seconds (E2_FILESCHECK_INTERVAL_S,
+# else
+				g_timeout_add (E2_FILESCHECK_INTERVAL,
+# endif
+					(GSourceFunc) e2_filelist_check_dirty, NULL);
+		}
+	}
+	else
+	{
+		//start backend working
+		e2_fs_FAM_change (app.pane1.path, &app.pane1);
+		e2_fs_FAM_change (app.pane2.path, &app.pane2);
+		e2_fs_FAM_resume ();
+	}
+#else //ndef E2_FAM_KERNEL
+	if (app.timers[DIRTYCHECK_T] == 0)
+	{
+# ifdef E2_REFRESH_DEBUG
+		printd (DEBUG, "start timer DIRTYCHECK_T");
+# endif
+		app.timers[DIRTYCHECK_T] =
+# ifdef USE_GLIB2_14
+			g_timeout_add_seconds (E2_FILESCHECK_INTERVAL_S,
+# else
+			g_timeout_add (E2_FILESCHECK_INTERVAL,
+# endif
 				(GSourceFunc) e2_filelist_check_dirty, NULL);
 	}
+#endif
 }
 /**
 @brief idle function to clear old liststores
@@ -718,7 +989,7 @@ GtkListStore *e2_filelist_fill_store (GList *entries, ViewInfo *view)
 
 	//format for dates display
 	gchar *strf_string;
-	switch	(e2_option_int_get ("date-string"))
+	switch (e2_option_int_get ("date-string"))
 	{
 		case 1:
 			strf_string = "%d/%m/%y %H:%M";	//standard
@@ -730,13 +1001,13 @@ GtkListStore *e2_filelist_fill_store (GList *entries, ViewInfo *view)
 			strf_string = "%Y-%m-%dT%H:%M";	//ISO8601
 			break;
 		case 4:
-/*			{
-				strf_string = nl_langinfo (D_T_FMT);
-				if (strf_string != NULL && *strf_string != '\0')
-					break;
-			} */
 			strf_string = "%x %X";	//localised
 			break;
+		case 5:
+			strf_string = e2_option_str_get ("custom-date-format");
+			if (strf_string != NULL && *strf_string != '\0')
+				//no reliable check for format validity ?
+				break;
 		default:
 			strf_string = "%b %d %H:%M";
 			break;
@@ -759,9 +1030,27 @@ GtkListStore *e2_filelist_fill_store (GList *entries, ViewInfo *view)
 
 	gboolean anyconvert = FALSE;
 	gboolean caseignore = ! e2_option_bool_get ("namesort-case-sensitive");
+#ifdef E2_RAINBOW
+	gboolean anycase = e2_option_bool_get ("anycase-filetypes");
+#endif
+/*
+#ifdef USE_GTK3_0
+	GdkRBGA *normal;
+	GtkStyleContext *context;
+	context = gtk_widget_get_style_context (curr_view->treeview);
+	gtk_style_context_get (context, GTK_STATE_NORMAL, GTK_STYLE_PROPERTY_COLOR,
+		&normal, NULL);
+	GdkColor normal2 = {0,  (guint16)(65535 * normal->red),
+							(guint16)(65535 * normal->green),
+							(guint16)(65535 * normal->blue) } ;
+	GdkColor *default_color = &normal2;
+	gdk_rgba_free (normal);
+#else
 	GtkStyle *style = gtk_rc_get_style (curr_view->treeview);
 	GdkColor *default_color = &style->text[GTK_STATE_NORMAL];
-
+#endif
+*/
+	GdkColor *default_color = NULL;
 #ifdef E2_ASSISTED
 	GdkColor *back_color = (e2_option_bool_get ("color-background-set")) ?
 		e2_option_color_get ("color-background") : NULL;
@@ -965,7 +1254,16 @@ GtkListStore *e2_filelist_fill_store (GList *entries, ViewInfo *view)
 				{
 					ext++;	//pass the '.'
 					//use its color, if any
-					foreground = g_hash_table_lookup (app.colors, ext);
+					if (anycase)
+					{ //infoptr->filename is localised text
+						gchar *utf = F_FILENAME_FROM_LOCALE (ext);
+						gchar *ext2 = g_utf8_casefold (utf, -1);
+						foreground = g_hash_table_lookup (app.colors, ext2);
+						g_free (ext2);
+						F_FREE (utf, ext);
+					}
+					else
+						foreground = g_hash_table_lookup (app.colors, ext);
 					if (foreground != NULL)
 						break;
 				}
