@@ -1,6 +1,6 @@
-/* $Id: e2_filetype.c 1349 2008-11-14 06:14:46Z tpgww $
+/* $Id: e2_filetype.c 2412 2011-12-20 20:00:46Z tpgww $
 
-Copyright (C) 2003-2008 tooar <tooar@gmx.net>
+Copyright (C) 2003-2010 tooar <tooar@emelfm2.net>
 Portions copyright (C) 1999 Michael Clark.
 
 This file is part of emelFM2.
@@ -15,8 +15,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 #include "emelfm2.h"
@@ -60,7 +59,7 @@ Not much use - in the config data, the categories are user-editable !!
 					if (gtk_tree_model_iter_children (set->ex.tree.model, &iter3, &iter2))
 					{
 						gtk_tree_model_get (set->ex.tree.model, &iter2, 1, &extension, -1);
-						if (g_str_equal (extension, _C(13)))  //extensions node found
+						if (!strcmp (extension, _C(13)))  //extensions node found
 						{
 							g_free (extension);
 							do
@@ -116,9 +115,9 @@ const gchar **e2_filetype_get_actions (gchar *ext)
 {
 	const gchar **actions;
 
-	if ( g_str_equal (ext, _("<directory>"))
-	  || g_str_equal (ext, _("<executable>"))
-	  || g_str_equal (ext, _("<none>"))
+	if ( !strcmp (ext, _("<directory>"))
+	  || !strcmp (ext, _("<executable>"))
+	  || !strcmp (ext, _("<none>"))
 	  || (!e2_option_bool_get ("anycase-filetypes") && e2_fs_mount_is_cased (curr_view)))
 		actions = (const gchar **) g_hash_table_lookup (app.filetypes, ext);
 	else
@@ -198,7 +197,7 @@ void e2_filetype_add_all (void)
 					gtk_tree_model_get (set->ex.tree.model, &iter2, 1, &extension, -1);
 					if (gtk_tree_model_iter_children (set->ex.tree.model, &iter3, &iter2))
 					{
-						if (g_str_equal (extension, _C(13)))  //extensions node found
+						if (!strcmp (extension, _C(13)))  //extensions node found
 						{
 							g_free (extension);
 							do
@@ -208,7 +207,7 @@ void e2_filetype_add_all (void)
 								g_ptr_array_add (e_ray, extension);
 							} while (gtk_tree_model_iter_next (set->ex.tree.model, &iter3));
 						}
-						else if (g_str_equal (extension, _C(6)))  //commands node found
+						else if (!strcmp (extension, _C(6)))  //commands node found
 						{
 							g_free (extension);
 							do
@@ -311,7 +310,7 @@ static gboolean _e2_filetype_config (gpointer from, E2_ActionRuntime *art)
 	//open the config dialog at the appropriate place
 	gchar *utf = F_FILENAME_FROM_LOCALE (ext);	//DISPLAY ??
 	e2_filetype_config_show (utf);
-	F_FREE (utf);
+	F_FREE (utf, ext);
 	return TRUE;
 }
 /**
@@ -474,9 +473,9 @@ void e2_filetype_apply_allnew (void)
 */
 void e2_filetype_actions_register (void)
 {
-	gchar *action_name = g_strconcat(_A(3),".",_A(45),NULL);
-	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
-		_e2_filetype_config, NULL, FALSE);
+	E2_Action action =
+	{g_strconcat(_A(3),".",_A(45),NULL),_e2_filetype_config, FALSE, E2_ACTION_TYPE_ITEM, 0, NULL, NULL};
+	e2_action_register (&action);
 }
 /**
 @brief install default tree options for filetypes
@@ -540,7 +539,7 @@ static void _e2_filetype_tree_defaults (E2_OptionSet *set)
 			g_strdup ("\t\t|txt|"),
 		g_strconcat ("\t|",_C(6),"|",NULL),
 //not loaded	g_strconcat ("\t\t|",PLUGIN,":",_("View"),"|",_A(6),".",_("view"),NULL),  //conformed name
-			g_strconcat ("\t\t|",_("View"),"|",_A(6),".",_A(105),NULL), //_("file.view") //<
+			g_strconcat ("\t\t|",_("View"),"|",_A(6),".",_A(106),NULL), //_("file.view") //<
 			g_strconcat ("\t\t|",_("_Edit"),"|",_A(6),".",_A(43),NULL), //_("file.edit") //<
 //this removed in favour of the default context menu item open-with
 //			g_strconcat ("\t\t|",_("Edit _with.."),"|%{(editors)@",_("Editor command:"),"} %p",NULL),
@@ -576,7 +575,9 @@ static void _e2_filetype_tree_defaults (E2_OptionSet *set)
 //			g_strdup ("\t\t|sxp|"),
 		g_strconcat ("\t|",_C(6),"|",NULL),
 			g_strdup ("\t\t|_gview|gview"),
-			g_strdup ("\t\t|G_Qview|gqview"),
+			g_strdup ("\t\t|gee_qie|geeqie"),
+			g_strdup ("\t\t|_viewnior|viewnior"),
+//			g_strdup ("\t\t|G_Qview|gqview"),
 //			g_strdup ("\t\t|GImage_View|gimageview"),
 //			g_strconcat ("\t\t|",_("_Oo impress"),"|ooimpress",NULL),
 //			g_strconcat ("\t\t|",_("Oo _draw"),"|oodraw",NULL),
@@ -600,10 +601,10 @@ static void _e2_filetype_tree_defaults (E2_OptionSet *set)
 			g_strconcat ("\t\t|",_("Unpack"),"|tar -xf %f",NULL),
 			g_strconcat ("\t\t|",_("Unpack in other pane"),"|",_A(14),".",_A(75),";x tar -C %D -xf %f;",_A(14),".",_A(74),NULL), //<*2
 #ifdef E2_VFS
-			g_strconcat ("\t\t|",_("_Open"),"|",_A(6),".",_A(103),NULL),
-			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(6),".",_A(122),NULL),
+			g_strconcat ("\t\t|",_("_Open"),"|",_A(6),".",_A(104),NULL),
+			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(6),".",_A(123),NULL),
 #else
-			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(6),".",_A(103),NULL),  //conformed name
+			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(6),".",_A(104),NULL),  //conformed name
 #endif
 			g_strconcat ("\t\t|",_("_List contents"),"|>tar -tf %f \\| less",NULL),
 	g_strconcat (_("gzip tarballs"),"||",NULL),
@@ -614,10 +615,10 @@ static void _e2_filetype_tree_defaults (E2_OptionSet *set)
 			g_strconcat ("\t\t|",_("_Unpack"),"|tar -zxf  %f",NULL),
 			g_strconcat ("\t\t|",_("Unpack in _other pane"),"|",_A(14),".",_A(75),";tar -C %D -zxf %f;",_A(14),".",_A(74),NULL), //<*2
 #ifdef E2_VFS
-			g_strconcat ("\t\t|",_("_Open"),"|",_A(6),".",_A(103),NULL),
-			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(6),".",_A(122),NULL),
+			g_strconcat ("\t\t|",_("_Open"),"|",_A(6),".",_A(104),NULL),
+			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(6),".",_A(123),NULL),
 #else
-			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(6),".",_A(103),NULL),   //conformed name
+			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(6),".",_A(104),NULL),   //conformed name
 #endif
 			g_strconcat ("\t\t|",_("_List contents"),"|>tar -ztf %f \\| less",NULL),
 	g_strconcat (_("bzip2 tarballs"),"||",NULL),
@@ -629,10 +630,10 @@ static void _e2_filetype_tree_defaults (E2_OptionSet *set)
 			g_strconcat ("\t\t|",_("_Unpack"),"|>bzip2 -d -c %f \\| tar -xf -",NULL),
 			g_strconcat ("\t\t|",_("Unpack in _other pane"),"|",_A(14),".",_A(75),";>bzip2 -d -c %f \\| tar -C %D -xf -;",_A(14),".",_A(74),NULL), //<*2
 #ifdef E2_VFS
-			g_strconcat ("\t\t|",_("_Open"),"|",_A(6),".",_A(103),NULL),
-			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(6),".",_A(122),NULL),
+			g_strconcat ("\t\t|",_("_Open"),"|",_A(6),".",_A(104),NULL),
+			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(6),".",_A(123),NULL),
 #else
-			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(6),".",_A(103),NULL),    //conformed name
+			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(6),".",_A(104),NULL),    //conformed name
 #endif
 			g_strconcat ("\t\t|",_("_List contents"),"|>bzip2 -d -c %f \\| tar -tf - \\| less",NULL),
 	g_strconcat (_("lzma tarballs"),"||",NULL),
@@ -643,24 +644,37 @@ static void _e2_filetype_tree_defaults (E2_OptionSet *set)
 			g_strconcat ("\t\t|",_("_Unpack"),"|>lzma -d -c %f \\| tar -xf -",NULL),
 			g_strconcat ("\t\t|",_("Unpack in _other pane"),"|",_A(14),".",_A(75),";>lzma -d -c %f \\| tar -C %D -xf -;",_A(14),".",_A(74),NULL), //<*2
 #ifdef E2_VFS
-			g_strconcat ("\t\t|",_("_Open"),"|",_A(6),".",_A(103),NULL),
-			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(6),".",_A(122),NULL),
+			g_strconcat ("\t\t|",_("_Open"),"|",_A(6),".",_A(104),NULL),
+			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(6),".",_A(123),NULL),
 #else
-			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(6),".",_A(103),NULL),    //conformed name
+			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(6),".",_A(104),NULL),    //conformed name
 #endif
 			g_strconcat ("\t\t|",_("_List contents"),"|>lzma -d -c %f \\| tar -tf - \\| less",NULL),
+	g_strconcat (_("xz tarballs"),"||",NULL),
+		g_strconcat ("\t|",_C(13),"|",NULL),
+			g_strdup ("\t\t|tar.xz|"),
+			g_strdup ("\t\t|txz|"),
+		g_strconcat ("\t|",_C(6),"|",NULL),
+			g_strconcat ("\t\t|",_("_Unpack"),"|>xz -d -c %f \\| tar -xf -",NULL),
+			g_strconcat ("\t\t|",_("Unpack in _other pane"),"|",_A(14),".",_A(75),";>xz -d -c %f \\| tar -C %D -xf -;",_A(14),".",_A(74),NULL), //<*2
+#ifdef E2_VFS
+			g_strconcat ("\t\t|",_("_Open"),"|",_A(6),".",_A(104),NULL),
+			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(6),".",_A(123),NULL),
+#else
+			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(6),".",_A(104),NULL),    //conformed name
+#endif
+			g_strconcat ("\t\t|",_("_List contents"),"|>xz -d -c %f \\| tar -tf - \\| less",NULL),
 	g_strconcat (_("zip archives"),"||",NULL),
 		g_strconcat ("\t|",_C(13),"|",NULL),
 			g_strdup ("\t\t|zip|"),
 		g_strconcat ("\t|",_C(6),"|",NULL),
 			g_strconcat ("\t\t|",_("_Unzip"),"|unzip %f",NULL),
-			//space after %F in next line is needed ?
-			g_strconcat ("\t\t|",_("Unzip in _other pane"),"|",_A(14),".",_A(94),";unzip %F;",_A(14),".",_A(94),NULL), //<*2
+			g_strconcat ("\t\t|",_("Unzip in _other pane"),"|",_A(14),".",_A(95),";unzip %P;",_A(14),".",_A(95),NULL), //<*2
 #ifdef E2_VFS
-			g_strconcat ("\t\t|",_("_Open"),"|",_A(6),".",_A(103),NULL),
-			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(6),".",_A(122),NULL),
+			g_strconcat ("\t\t|",_("_Open"),"|",_A(6),".",_A(104),NULL),
+			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(6),".",_A(123),NULL),
 #else
-			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(6),".",_A(103),NULL),    //conformed name
+			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(6),".",_A(104),NULL),    //conformed name
 #endif
 			g_strconcat ("\t\t|",_("_List contents"),"|>unzip -l %f \\| less",NULL),
 /* plugin functionality not tested yet, for these
@@ -669,20 +683,20 @@ static void _e2_filetype_tree_defaults (E2_OptionSet *set)
 			g_strdup ("\t\t|arj|"),
 		g_strconcat ("\t|",_C(6),"|",NULL),
 #ifdef E2_VFS
-			g_strconcat ("\t\t|",_("_Open")"|",_A(6),".",_A(103),NULL),
-			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(6),".",_A(122),NULL),
+			g_strconcat ("\t\t|",_("_Open")"|",_A(6),".",_A(104),NULL),
+			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(6),".",_A(123),NULL),
 #else
-			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(6),".",_A(103),NULL),
+			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(6),".",_A(104),NULL),
 #endif
 	g_strconcat (_("rar archives"),"||",NULL),
 		g_strconcat ("\t|",_C(13),"|",NULL),
 			g_strdup ("\t\t|rar|"),
 		g_strconcat ("\t|",_C(6),"|",NULL),
 #ifdef E2_VFS
-			g_strconcat ("\t\t|",_("_Open")"|",_A(6),".",_A(103),NULL),
-			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(6),".",_A(122),NULL),
+			g_strconcat ("\t\t|",_("_Open")"|",_A(6),".",_A(104),NULL),
+			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(6),".",_A(123),NULL),
 #else
-			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(6),".",_A(103),NULL),
+			g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|",_A(6),".",_A(104),NULL),
 #endif
 */
 	g_strconcat (_("RPM packages"),"||",NULL),
@@ -692,8 +706,8 @@ static void _e2_filetype_tree_defaults (E2_OptionSet *set)
 			g_strconcat ("\t\t|",_("In_formation"),"|rpm -qlip %f",NULL),
 			g_strconcat ("\t\t|",_("_Install"),"|su rpm -Uvh %f",NULL),
 #ifdef E2_VFS
-			g_strconcat ("\t\t|",_("_Open"),"|",_A(6),".",_A(103),NULL),
-			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(6),".",_A(122),NULL),
+			g_strconcat ("\t\t|",_("_Open"),"|",_A(6),".",_A(104),NULL),
+			g_strconcat ("\t\t|",_("O_pen in other"),"|",_A(6),".",_A(123),NULL),
 #endif
 	g_strconcat (_("source code files"),"||",NULL),
 		g_strconcat ("\t|",_C(13),"|",NULL),
