@@ -1,6 +1,6 @@
-/* $Id: e2_keybinding.c 2033 2009-12-21 02:36:51Z tpgww $
+/* $Id: e2_keybinding.c 2469 2012-03-16 00:19:24Z tpgww $
 
-Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
+Copyright (C) 2004-2012 tooar <tooar@emelfm2.net>
 
 This file is part of emelfm2.
 emelfm2 is free software; you can redistribute it and/or modify it
@@ -14,8 +14,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 /**
@@ -698,21 +697,24 @@ static gboolean _e2_keybinding_key_press_cb (GtkWidget *widget,
 	if (e2_utils_key_is_modifier (event))
 #endif
 		return FALSE;
-//	printd (DEBUG,"bindings key-press cb begins, key %u", event->keyval);
-	gboolean retval = FALSE;
+	guint keyval = event->keyval;
 	guint modifiers = gtk_accelerator_get_default_mod_mask ();
-	modifiers = event->state & modifiers;
-//	printd (DEBUG, "key 1: %d - modifiers: %x", event->keyval, modifiers);
-	/*workaround for matching our storage of <Shift>-letter
-	with gtk's approach to returning keycodes
-	See comments in _e2_keybinding_sync_one () */
-//	if (event->state & GDK_LOCK_MASK)
-//		modifiers |= GDK_SHIFT_MASK;
-	//get rid of shift flag if that's the only one
-	if ((event->keyval < 0xF000 || event->keyval > 0xFFFF)
-			&& (modifiers & E2_MODIFIER_MASK) == GDK_SHIFT_MASK)
-		modifiers &= ~(GDK_SHIFT_MASK);
+	modifiers &= event->state;
+//	printd (DEBUG, "key 1: %d - modifiers: %x", keyval, modifiers);
+	/*workaround for matching our storage of <Shift>-letter with gtk's approach
+	  to returning keycodes
+	  See comments in _e2_keybinding_sync_one() */
+	if (keyval < 0xF000 || keyval > 0xFFFF)
+	{
+		//get rid of shift flag if that's the only one
+		if ((modifiers & E2_MODIFIER_MASK) == GDK_SHIFT_MASK)
+			modifiers &= ~(GDK_SHIFT_MASK);
+		//lcase keycode if shift + other flag(s)
+		else if (modifiers & GDK_SHIFT_MASK)
+			keyval = gdk_keyval_to_lower (keyval);
+	}
 
+	gboolean retval = FALSE;
 	//if there was a chain from the last key, check if this one is a valid
 	//part of the chain. If so we need to update a pointer and adjust the timer
 	gpointer timeout_id = g_object_get_data (G_OBJECT (widget), CHAINTIMER_DATA_KEY);
@@ -727,7 +729,7 @@ static gboolean _e2_keybinding_key_press_cb (GtkWidget *widget,
 			for (member = (GSList *)chainkey; member != NULL; member = member->next)
 			{
 				E2_KeyRuntime *k = (E2_KeyRuntime *)member->data;
-				if ((k->keyval == event->keyval) && (k->mask == modifiers))
+				if ((k->keyval == keyval) && (k->mask == modifiers))
 				{
 					if (k->chained != NULL)
 					{
@@ -775,7 +777,7 @@ static gboolean _e2_keybinding_key_press_cb (GtkWidget *widget,
 				 member = member->next)
 			{
 				E2_KeyRuntime *k = member->data;
-				if ((k->keyval == event->keyval) && (k->mask == modifiers))
+				if ((k->keyval == keyval) && (k->mask == modifiers))
 				{
 					g_object_set_data (G_OBJECT (widget), CHAINKEY_DATA_KEY, k->chained);
 					if (k->chained != NULL)
