@@ -1,6 +1,6 @@
-/* $Id: e2_main.c 1962 2009-10-07 03:06:10Z tpgww $
+/* $Id: e2_main.c 2469 2012-03-16 00:19:24Z tpgww $
 
-Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
+Copyright (C) 2003-2012 tooar <tooar@emelfm2.net>
 Portions copyright (C) 1999 Michael Clark.
 
 This file is part of emelFM2.
@@ -15,8 +15,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 /**
@@ -112,6 +111,7 @@ The following items are covered:
 #include "emelfm2.h"
 #include <string.h>
 #include <unistd.h>
+#include <locale.h>
 #include <signal.h>
 #include <pthread.h>
 #include "e2_dialog.h"
@@ -126,11 +126,12 @@ The following items are covered:
 
 #define UPGRADE_PNAME "e2p_upgrade.so"
 
-extern gint refresh_refcount;
+extern volatile gint cfg_refresh_refcount;
 extern GList *open_history;
 #ifdef USE_GLIB2_10
 extern GHashTable *actions_hash;
 #endif
+extern time_t last_work_time;
 
 #ifdef WAIT_POLL
 pthread_mutex_t wait_mutex; // = PTHREAD_MUTEX_INITIALIZER;	if no recursion
@@ -140,7 +141,9 @@ pthread_mutex_t wait_mutex; // = PTHREAD_MUTEX_INITIALIZER;	if no recursion
 
 static void _e2_main_system_shutdown (gint num); //__attribute__ ((noreturn));
 
-#ifndef NATIVE_BGL
+#ifdef NATIVE_BGL
+# warning "using native gdk mutex"
+#else
 /**
 @brief local management of gdk threads mutex
 These allow a conservative approach to locking without risk of deadlock
@@ -219,8 +222,8 @@ static gboolean _e2_main_defer_init (gpointer data)
 {
 	if (app.pane1.view.dir == NULL || *app.pane1.view.dir == '\0'
 	 || app.pane2.view.dir == NULL || *app.pane2.view.dir == '\0'
-	 || app.pane1.view.listcontrols.cd_working
-	 || app.pane2.view.listcontrols.cd_working)
+	 || g_atomic_int_get (&app.pane1.view.listcontrols.cd_working)
+	 || g_atomic_int_get (&app.pane2.view.listcontrols.cd_working))
 	{
 		if (data != NULL)	//not first first callback
 			return TRUE;
@@ -230,20 +233,40 @@ static gboolean _e2_main_defer_init (gpointer data)
 			GINT_TO_POINTER (100));
 			return FALSE;
 	}
-	//this func may be called at some racy time when these need to be reset
-	app.pane1.view.listcontrols.norefresh = FALSE;
-	app.pane2.view.listcontrols.norefresh = FALSE;
-	//set the 'real' initial conversion funcs
-	e2_utf8_set_name_conversion (app.pane1.view.convert || app.pane2.view.convert);
+	//in case async setup of initial dirs results in pane2 completion after pane 1
+	if (curr_pane == &app.pane2)
+	{
+		gdk_threads_enter ();
+		e2_pane_activate_other ();
+		gdk_threads_leave ();
+	}
+#ifdef E2_FAM
+	//hookup to a file monitor if possible
+	e2_fs_FAM_connect ();
+#endif
+	last_work_time = time (NULL);	//actually, only needed when auto-refreshing
+	if (e2_option_bool_get ("auto-refresh"))
+		//initiate filelist dirty-checks
+		e2_filelist_start_refresh_checks ();
+	else
+	{
+		//must cleanup the flags in case of later use
+		app.pane1.view.listcontrols.refresh_refcount = 0;
+		app.pane2.view.listcontrols.refresh_refcount = 0;
+	}
+	if (e2_option_bool_get ("auto-refresh-config"))
+		//downstream uses change-monitor-type, so do this after FAM_connect()
+		e2_option_start_config_checks ();
+	else
+		cfg_refresh_refcount = 0;
 
 #ifdef E2_REFRESH_DEBUG
-	printd (DEBUG, "enable refresh, session start after filelists filled");
-#endif
-	e2_filelist_enable_refresh ();
-#ifndef E2_REFRESH_DEBUG
 	printd (DEBUG, "all change-polling initialised after filelists filled");
 #endif
 
+	//set the 'real' initial conversion funcs
+	e2_utf8_set_name_conversion (app.pane1.view.convert || app.pane2.view.convert);
+
 #ifdef E2_HAL
 	e2_hal_init ();
 #endif
@@ -251,16 +274,20 @@ static gboolean _e2_main_defer_init (gpointer data)
 	e2_devkit_init ();
 #endif
 
-#ifndef USE_GTK2_99
-	//add a discard if gtk is old or does not provide a stock discard icon as it should
-//	GtkStyle *style = gtk_widget_get_style (app.main_window);
+#ifndef USE_GTK3_99
+	//add a discard if gtk does not provide a stock discard icon as it should (still none, in 3.0)
+# ifdef USE_GTK3_0
+	GtkStyleContext *sc = gtk_widget_get_style_context (app.main_window);
+	GtkIconSet *iset = gtk_style_context_lookup_icon_set (sc, GTK_STOCK_DISCARD);
+# else //gtk 2
 	GtkIconSet *iset = gtk_style_lookup_icon_set (
-#ifdef USE_GTK2_14
+#  ifdef USE_GTK2_14
 		gtk_widget_get_style (app.main_window),
-#else
+#  else
 		app.main_window->style,
-#endif
+#  endif
 		GTK_STOCK_DISCARD);
+# endif
 	if (iset == NULL)
 	{
 		GtkIconSource *isrc = gtk_icon_source_new ();
@@ -340,9 +367,14 @@ static void _e2_main_check_locale (void)
 //	printd (DEBUG, "check_locale ()");
 	if (g_getenv ("G_BROKEN_FILENAMES") != NULL)
 	{
-		const gchar *locale = gtk_set_locale (); //repeat: already done by gtk_init()
-		printd (NOTICE, "current locale is '%s'", locale);
-		_e2_main_check_locale_broken (locale);
+		const gchar *locale = setlocale(LC_ALL, (gchar *) NULL);
+		if (locale != NULL)
+		{
+			printd (NOTICE, "current locale is '%s'", locale);
+			_e2_main_check_locale_broken (locale);
+		}
+		else
+			printf ("Your locale setup is broken!\n");//no point in translating this
 	}
 }
 
@@ -454,7 +486,8 @@ static void _e2_main_loop_clean_BGL (E2_MainLoop *loopdata)
 #endif
 /**
 @brief block until wait is ended (as indicated by flag in @a loopdata) then cleanup
-Function is abort-safe. BGL is expected to be on/closed
+Function is abort-safe. Refreshing may be disabled - if so, it's reverted while here
+BGL is expected to be on/closed
 @param loopdata pointer to loop data struct
 
 @return
@@ -471,7 +504,8 @@ void e2_main_loop_run (E2_MainLoop *loopdata)
 	gdk_flush ();
 	pthread_cleanup_push ((gpointer)e2_filelist_reset_refresh, NULL);
 		//enable refresh
-		saverefcount = refresh_refcount;
+		//BAD if cfg_refresh_refcount adjusted individually ?
+		saverefcount = g_atomic_int_get (&cfg_refresh_refcount); //proxy for all refcounters
 		if (saverefcount > 0)
 		{
 #ifdef E2_REFRESH_DEBUG
@@ -656,7 +690,9 @@ gint main (gint argc, gchar *argv[])
 	e2_main_replace_gdklock (FALSE);
 #endif
 
+#ifndef USE_GLIB2_32
 	g_thread_init (NULL);
+#endif
 #ifndef NATIVE_BGL
 	gdk_threads_set_lock_functions (e2_main_close_gdklock, e2_main_open_gdklock);
 #endif
@@ -707,12 +743,14 @@ gint main (gint argc, gchar *argv[])
 
 	if (confile_ok)
 	{
-		//ASAP after config settled, CHECKME custom dialog ok ?
+		e2_option_date_style (); //customise date-format option
+		//ASAP after config settled
 		if (strcmp (app.cfgfile_version, VERSION RELEASE) < 0)
 		{
 			Plugin *p = e2_plugins_open1 (PLUGINS_DIR G_DIR_SEPARATOR_S UPGRADE_PNAME);	//localised path
 			if (p != NULL)
 			{
+				//CHECKME downstream custom dialog ok without explicit BGL close ?
 				if (!p->plugin_init (p))	//do any upgrades
 					printd (ERROR, "Can't initialize upgrade plugin: "UPGRADE_PNAME);
 				gdk_threads_enter ();
@@ -747,45 +785,7 @@ gint main (gint argc, gchar *argv[])
 	//if this is a fresh start,
 	//do this before bars/menus created
 	e2_plugins_load_all (); // load plugins specified in the config data
-
-	//create and show the main window before the filelist creation (in case there's a big delay)
-	printd (DEBUG, "create main window");
-	e2_window_create (&app.window);
-
-#ifdef E2_FAM
-	//before change dir, signal unknown state
-	app.pane1.FAMreq = -1;
-	app.pane2.FAMreq = -1;
-	//hookup to a file monitor if possible
-	e2_fs_FAM_connect ();
-	//CHECKME ensure FAM reports don't pile up due to lack of polling
-#endif
-	//initate dirty-checks
-	e2_filelist_start_refresh_polling ();
-	//config-change checks are disabled during list creation
-	//so the mechanism needs to be initialised beforehand
-//	app.config_timer_id = 0;
-	e2_option_enable_config_checks ();
-
-#ifndef E2_STATUS_DEMAND
-	e2_window_enable_status_update (-1);
-#endif
-	//set refresh counter to its real initial value
-	refresh_refcount = 0;
-#ifdef E2_REFRESH_DEBUG
-	printd (DEBUG, "disable refresh, session start before filelists built");
-#endif
-	//prevent any actual refreshing until after the initial filelists are created
-	e2_filelist_disable_refresh ();
-
-	e2_button_setup_labels ();	//maybe needed by dialog when opening dir
-	e2_utils_translate_keys (TRUE, FALSE);	//setup for specific keycode management
-	//all widgets are in place, register bindings in case needed in dialog
-	e2_keybinding_register_all ();
-#ifdef E2_MOUSECUSTOM
-	e2_mousebinding_register_all (); //includes gestures if relevant
-#endif
-
+	//some of these are used during pane creation
 #ifdef E2_VFS
 	app.vdir_cache = g_hash_table_new_full (e2_fileview_vdir_hash,
 		e2_fileview_vdir_equal, e2_fieview_vdir_free, NULL); //uses same data for key and value
@@ -798,15 +798,39 @@ gint main (gint argc, gchar *argv[])
 #endif
 		e2_fileview_clean1_history
 	);
-	//session-starts with BGL closed
-//	gdk_threads_leave ();
+
+	//create and show the main window before the filelist creation (in case there's a big delay)
+	e2_window_create (&app.window);
+#ifndef E2_STATUS_DEMAND
+	e2_window_enable_status_update (-1);
+#endif
+	e2_button_setup_labels ();	//maybe needed by dialog when opening dir
+	e2_utils_translate_keys (TRUE, FALSE);	//setup for specific keycode management
+	//all widgets are in place, register bindings in case needed in dialog
+	e2_keybinding_register_all ();
+#ifdef E2_MOUSECUSTOM
+	e2_mousebinding_register_all (); //includes gestures if relevant
+#endif
+
+	//set big refresh-counters, to block refresh-things before real start of monitoring
+	app.pane1.view.listcontrols.refresh_refcount = 10;
+	app.pane2.view.listcontrols.refresh_refcount = 10;
+	cfg_refresh_refcount = 10;
 	//threaded cd's may start at any time, including after main loop is running
-	//and cd completion depends on speed of dir loading
-	//this func will probably not set app.pane2.view.dir immediately
-	e2_utils_goto_accessible_path (&app.pane2);
-	//show pane 1 last, making it the default for session start
-	e2_utils_goto_accessible_path (&app.pane1);
-//	gdk_threads_enter ();
+	//and cd completion depends on speed of dir loading, so these funcs won't
+	//set respective view.dir's synchronously
+#ifdef E2_FAM
+	//NOTE FAM mechanism not set up until after initial filelists created, but
+	//because of values below and/or app.monitor_type starts as E2_MONITOR_DEFAULT,
+	//nothing is done by backend before setup
+	//signal unknown state
+	app.pane1.FAMreq = -1;
+	app.pane2.FAMreq = -1;
+#endif
+	e2_pane_goto_accessible_path (&app.pane2);
+	//do pane 1 last, possibly making it the default at session start
+	//(active pane is changed if needed in _e2_main_defer_init())
+	e2_pane_goto_accessible_path (&app.pane1);
 
 	gint pos = gtk_paned_get_position (GTK_PANED (app.window.output_paned));
 	if (pos == 0)
@@ -890,7 +914,7 @@ gint main (gint argc, gchar *argv[])
 	gdk_threads_enter ();
 	gtk_main ();
 	printd (DEBUG, "leave top-level main loop");
-	gdk_threads_leave ();	//should never get to here
+	gdk_threads_leave ();
 	exit (0);
 }
 
@@ -971,6 +995,11 @@ gboolean e2_main_closedown (gboolean compulsory, gboolean saveconfig, gboolean d
 		//FIXME handle maximised window
 		gtk_widget_get_allocation (app.main_window, &app.main_alloc);
 #endif
+		if (e2_option_bool_get ("cache-history"))
+		{	//limit the extent of cached history
+			e2_pane_trim_history (&app.pane1);
+			e2_pane_trim_history (&app.pane2);
+		}
 		e2_cache_file_write ();
 		printd (DEBUG, "touch config dir");
 		e2_fs_touch_config_dir (); //signal to the world that changes saved
@@ -1046,7 +1075,7 @@ gboolean e2_main_closedown (gboolean compulsory, gboolean saveconfig, gboolean d
 #endif
 //	printd (DEBUG, "clear dirs hash");
 	g_hash_table_destroy (app.dir_history);
-	g_list_free (app.pane1.opendirs);	//this for non-cached list without heaped data
+	g_list_free (app.pane1.opendirs);	//list data are owned by app.dir_history
 	g_list_free (app.pane2.opendirs);	//ditto
 /*	printd (DEBUG, "clean bookmarks");
 	e2_bookmark_clean ();
