@@ -1,6 +1,6 @@
-/* $Id: e2_mousebinding.c 1316 2008-10-07 10:49:25Z tpgww $
+/* $Id: e2_mousebinding.c 2064 2010-03-12 13:15:36Z tpgww $
 
-Copyright (C) 2008 tooar <tooar@gmx.net>
+Copyright (C) 2008-2010 tooar <tooar@emelfm2.net>
 
 This file is part of emelfm2.
 emelfm2 is free software; you can redistribute it and/or modify it
@@ -14,8 +14,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 /**
@@ -258,14 +257,17 @@ static void _e2_mousebinding_sync_one (GtkTreeModel *model, GtkTreeIter *iter,
 	if (valid)
 	{
 		E2_Action *act = e2_action_check (action);
-		valid = (act == NULL || !(act->exclude & E2_ACTION_EXCLUDE_ACCEL));
+		valid = (act == NULL || !(act->exclude & E2_ACTION_EXCLUDE_POINTER));
 	}
 	if (valid)
 	{
 		b = ALLOCATE (E2_BtnRuntime);
+#if (CHECKALLOCATEDWARN)
 		CHECKALLOCATEDWARN (b, valid = FALSE;)
+#else
 		if (b == NULL)
 			valid = FALSE;
+#endif
 	}
 	else
 		b = NULL; //warning prevention
@@ -957,7 +959,7 @@ static gboolean _e2_mousebinding_button_event_cb (GtkWidget *widget,
 						guint32 releases = 0;
 #endif
 						pausedata = _e2_mousebinding_match_chain (widget, event, binding, EVENT_BEFORE);
-						if (pausedata != NULL)	//there's already a 'preceeding' watch in progress for this event
+						if (pausedata != NULL)	//there's already a 'preceding' watch in progress for this event
 						{
 #ifdef MCOL_REL
 							if (event->type == GDK_BUTTON_RELEASE)
@@ -991,9 +993,7 @@ static gboolean _e2_mousebinding_button_event_cb (GtkWidget *widget,
 						else	//just run it CHECKME in another thread or idle cb?
 						{
 							//commands and at least some actions need BGL closed
-//							gdk_threads_leave ();
 							e2_action_run_simple_from (binding->action, binding->action_data, widget);
-//							gdk_threads_enter ();
 							//remember how far up this branch we've gone, to prevent repeats for other categories
 //CHECKME					if (topnode == NULL || g_node_depth (node) < g_node_depth (topnode))
 								topnode = node;
@@ -1029,20 +1029,20 @@ static gboolean _e2_mousebinding_button_event_cb (GtkWidget *widget,
 */
 void e2_mousebinding_register_all (void)
 {
-	e2_mousebinding_enrol (NULL, _C(17), NULL);
+	e2_mousebinding_enrol (NULL, _C(17), (void(*)(E2_OptionSet*))NULL);
 #ifdef E2_PTRGESTURES
-	e2_mousegesture_enrol (NULL, _C(17), NULL);
+	e2_mousegesture_enrol (NULL, _C(17), (void(*)(E2_OptionSet*))NULL);
 #endif
 
 	gchar *category = g_strconcat(_C(17),".",_C(23),NULL);  //_("general.main
-	e2_mousebinding_enrol (NULL, category, NULL);
+	e2_mousebinding_enrol (NULL, category, (void(*)(E2_OptionSet*))NULL);
 #ifdef E2_PTRGESTURES
-	e2_mousegesture_enrol (NULL, category, NULL);
+	e2_mousegesture_enrol (NULL, category, (void(*)(E2_OptionSet*))NULL);
 #endif
 	g_free (category);
 
-	e2_fileview_register_pointerbindings (app.pane1_view.treeview, &app.pane1_view);
-	e2_fileview_register_pointerbindings (app.pane2_view.treeview, &app.pane2_view);
+	e2_fileview_register_pointerbindings (app.pane1.view.treeview, &app.pane1.view);
+	e2_fileview_register_pointerbindings (app.pane2.view.treeview, &app.pane2.view);
 
 	GList *member;
 	for (member = app.command_lines; member != NULL ; member = g_list_next (member))
@@ -1050,11 +1050,11 @@ void e2_mousebinding_register_all (void)
 		E2_CommandLineRuntime *line = (E2_CommandLineRuntime *) member->data;
 		e2_command_line_register_pointerbindings (line);
 	}
-
+	//tab bindings are also set when the tab is created - so that new ones are covered
 	for (member = app.tabslist; member != NULL ; member = g_list_next (member))
 	{
 		E2_OutputTabRuntime *tab = (E2_OutputTabRuntime *)member->data;
-		e2_output_register_pointerbindings (tab);
+		e2_output_register_pointerbindings (GTK_WIDGET(tab->text));
 	}
 }
 /**
@@ -1074,7 +1074,7 @@ transient item that uses the category)
 @return
 */
 void e2_mousebinding_enrol (GtkWidget *widget, const gchar *category,
-	gpointer defaults_func)
+	void (*defaults_func)(E2_OptionSet*))
 {
 	GNode *member = _e2_mousebinding_get_category (category);
 	if (member == NULL)
@@ -1095,15 +1095,14 @@ void e2_mousebinding_enrol (GtkWidget *widget, const gchar *category,
 		rt->ref = gtk_tree_row_reference_new (set->ex.tree.model, path);
 		gtk_tree_path_free (path);
 #ifdef E2_TRANSIENTBINDINGS
-		if (defaults_func != NULL && !old)
+		if (defaults_func != (void(*)(E2_OptionSet*))NULL && !old)
 		{
 			//run the defaults-install function, which in turn calls
 			//e2_option_tree_setup_defaults() which sets up a pointer array at
-			//dummy_set.ex.tree.def
-			void (*install_func) (E2_OptionSet *) = defaults_func;
+			//dummy_set.ex.tree.def.tree_strings
 			E2_OptionSet dummyset;
-			(*install_func) (&dummyset);
-			gchar **split = (gchar **) dummyset.ex.tree.def;
+			(*defaults_func) (&dummyset);
+			gchar **split = dummyset.ex.tree.def.tree_strings;
 
 			//adjust start-index of parsed array
 			const gchar *p = strrchr (category, '.');
@@ -1129,7 +1128,7 @@ void e2_mousebinding_enrol (GtkWidget *widget, const gchar *category,
 
 	if (G_LIKELY (widget != NULL))
 	{
-		printd (DEBUG, "enrol widget %x in btnbinding category '%s'", widget, category);
+		printd (DEBUG, "register widget %x in btnbinding category '%s'", widget, category);
 		GPtrArray *boundto = (GPtrArray *)
 				g_object_get_data (G_OBJECT (widget), BUTTON_BINDINGS_KEY);
 		if (boundto == NULL)	//this is the first time widget is enrolled
@@ -1198,16 +1197,21 @@ static gboolean _e2_mousebinding_walk_unregister (GNode *node, E2_NodeData *data
 		{
 			if (data->widget == widgets->data || data->widget == NULL)
 			{
-	printd (DEBUG, "UNregister widget");
-				//in case widget isn't to be destroyed
-				g_signal_handlers_disconnect_matched (widgets->data,
-					G_SIGNAL_MATCH_FUNC, 0, 0, NULL, _e2_mousebinding_button_event_cb, NULL);
-				gpointer boundto = g_object_get_data (G_OBJECT (widgets->data),
-						BUTTON_BINDINGS_KEY);
-				if (boundto != NULL)
+				if (GTK_IS_WIDGET (widgets->data))
 				{
-					while (g_ptr_array_remove_fast ((GPtrArray *)boundto, rt)); //probably only one
+					//in case widget isn't to be destroyed
+					g_signal_handlers_disconnect_matched (widgets->data,
+						G_SIGNAL_MATCH_FUNC, 0, 0, NULL, _e2_mousebinding_button_event_cb, NULL);
+					printd (DEBUG, "UNregister buttonbinding for widget");
+					gpointer boundto = g_object_get_data (G_OBJECT (widgets->data),
+							BUTTON_BINDINGS_KEY);
+					if (boundto != NULL)
+					{
+						while (g_ptr_array_remove_fast ((GPtrArray *)boundto, rt)); //probably only one
+					}
 				}
+				else
+					printd (DEBUG, "No UNregister button-binding for destroyed widget");
 				widgets->data = NULL;	//don't remove, list pointer cannot change yet
 			}
 		}
@@ -1224,7 +1228,7 @@ static gboolean _e2_mousebinding_walk_unregister (GNode *node, E2_NodeData *data
 */
 void e2_mousebinding_disrol (GtkWidget *widget, const gchar *category)
 {
-	printd (DEBUG, "e2_mousebinding_unregister, category: %s, widget: _",
+	printd (DEBUG, "e2_mousebinding_disrol, category: %s, widget: _",
 		(category) ? category : "ALL");
 	GQuark qname;
 	if (category != NULL)
@@ -1460,7 +1464,6 @@ void e2_mousebinding_output_help (gchar *parameters)
 		{
 			do
 			{
-				GtkTreeIter iter3;
 				gchar *cat, *btn, *act, *arg, *numptr, *temp;
 				const gchar *fmt;
 				gboolean dbl, tripl;
@@ -1476,66 +1479,137 @@ void e2_mousebinding_output_help (gchar *parameters)
 					MCOL_ACT, &act, MCOL_ARG, &arg, -1);
 				if (*cat != '\0')
 				{
+					GtkTreeIter iter3;
 					g_string_append_printf (str, "\t%s\n", cat);
 					g_free (cat);
 					if (gtk_tree_model_iter_children (mdl, &iter3, &iter2))
 					{
 						do
 						{
-							gtk_tree_model_get (mdl, &iter3, MCOL_BTN, &btn,
+							gtk_tree_model_get (mdl, &iter3, MCOL_CAT, &cat, MCOL_BTN, &btn,
 								MCOL_DBL, &dbl, MCOL_TRP, &tripl,
 #ifdef MCOL_REL
 								MCOL_REL, &rel,
 #endif
 								MCOL_ACT, &act, MCOL_ARG, &arg, -1);
-#ifdef MCOL_REL
-							if (rel)
+							if (*cat != '\0')
 							{
-								if (dbl)
-									fmt = fmts[1];
-								else if (tripl)
-									fmt = fmts[2];
-								else
-									fmt = fmts[0];
-								snprintf (detail, sizeof (detail), fmt, extras[1]);
-							}
-							else //press
-							{
-#endif
-								if (dbl)
+								GtkTreeIter iter4;
+								g_string_append_printf (str, "\t\t%s\n", cat);
+								g_free (cat);
+								if (gtk_tree_model_iter_children (mdl, &iter4, &iter3))
 								{
-									fmt = fmts[1];
-									snprintf (detail, sizeof (detail), fmt, extras[0]);
-								}
-								else if (tripl)
-								{
-									fmt = fmts[2];
-									snprintf (detail, sizeof (detail), fmt, extras[0]);
-								}
-								else
-									*detail = '\0';
+									do
+									{
+										gtk_tree_model_get (mdl, &iter4, MCOL_BTN, &btn,
+											MCOL_DBL, &dbl, MCOL_TRP, &tripl,
 #ifdef MCOL_REL
-							}
+											MCOL_REL, &rel,
 #endif
-							numptr = strrchr (btn, '>');
-							if (numptr != NULL)
-							{
-								//split into parts
-								temp = g_strdup (++numptr);
-								*numptr = '\0';
-								numptr = temp;
+											MCOL_ACT, &act, MCOL_ARG, &arg, -1);
+#ifdef MCOL_REL
+										if (rel)
+										{
+											if (dbl)
+												fmt = fmts[1];
+											else if (tripl)
+												fmt = fmts[2];
+											else
+												fmt = fmts[0];
+											snprintf (detail, sizeof (detail), fmt, extras[1]);
+										}
+										else //press
+										{
+#endif
+											if (dbl)
+											{
+												fmt = fmts[1];
+												snprintf (detail, sizeof (detail), fmt, extras[0]);
+											}
+											else if (tripl)
+											{
+												fmt = fmts[2];
+												snprintf (detail, sizeof (detail), fmt, extras[0]);
+											}
+											else
+												*detail = '\0';
+#ifdef MCOL_REL
+										}
+#endif
+										numptr = strrchr (btn, '>');
+										if (numptr != NULL)
+										{
+											//split into parts
+											temp = g_strdup (++numptr);
+											*numptr = '\0';
+											numptr = temp;
+										}
+										else
+										{
+											numptr = "";
+										}
+
+										g_string_append_printf (str, "\t\t\t%s%s%s\t%s\t%s\n", btn, detail, numptr, act, arg);
+										g_free (btn);
+										g_free (act);
+										g_free (arg);
+										if (*numptr != '\0')
+											g_free (numptr);
+									} while (gtk_tree_model_iter_next (mdl, &iter4));
+								}
 							}
 							else
 							{
-								numptr = "";
+								//FIXME do this more efficiently
+#ifdef MCOL_REL
+								if (rel)
+								{
+									if (dbl)
+										fmt = fmts[1];
+									else if (tripl)
+										fmt = fmts[2];
+									else
+										fmt = fmts[0];
+									snprintf (detail, sizeof (detail), fmt, extras[1]);
+								}
+								else //press
+								{
+#endif
+									if (dbl)
+									{
+										fmt = fmts[1];
+										snprintf (detail, sizeof (detail), fmt, extras[0]);
+									}
+									else if (tripl)
+									{
+										fmt = fmts[2];
+										snprintf (detail, sizeof (detail), fmt, extras[0]);
+									}
+									else
+										*detail = '\0';
+#ifdef MCOL_REL
+								}
+#endif
+								numptr = strrchr (btn, '>');
+								if (numptr != NULL)
+								{
+									//split into parts
+									temp = g_strdup (++numptr);
+									*numptr = '\0';
+									numptr = temp;
+								}
+								else
+								{
+									numptr = "";
+								}
+								g_string_append_printf (str, "\t\t%s%s%s\t%s\t%s\n", btn, detail, numptr, act, arg);
+								g_free (cat);
+								g_free (btn);
+								g_free (act);
+								g_free (arg);
+								if (*numptr != '\0')
+									g_free (numptr);
 							}
-
-							g_string_append_printf (str, "\t\t%s%s%s\t%s\t%s\n", btn, detail, numptr, act, arg);
-							g_free (btn);
-							g_free (act);
-							g_free (arg);
-							if (*numptr != '\0')
-								g_free (numptr);
 						} while (gtk_tree_model_iter_next (mdl, &iter3));
 					}
 				}
@@ -1697,10 +1771,9 @@ static gboolean _e2_mousebinding_issue (gpointer from, E2_ActionRuntime *art)
 void e2_mousebinding_actions_register (void)
 {
 #ifdef WITH_BUTTONFAKE
-//	gchar *action_name = g_strconcat(_A(125),".",_A(124),NULL);	//_("button.fake"
-	gchar *action_name = g_strdup ("button.fake");	//for debugging
-	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
-		_e2_mousebinding_issue, NULL, TRUE);
+	E2_Action action =
+	{g_strconcat(_A(126),".",_A(125),NULL),_e2_mousebinding_issue,TRUE,E2_ACTION_TYPE_ITEM,0,NULL,NULL};
+	e2_action_register (&action);
 #endif
 }
 /**
@@ -1728,10 +1801,12 @@ static void _e2_mousebinding_tree_defaults (E2_OptionSet *set)
 	g_strconcat("\t\t\t|4|||",_A(13),".",_A(49),"|",NULL), //"go back
 	g_strconcat("\t\t\t|5|||",_A(13),".",_A(50),"|",NULL), //"go forward
 	g_strconcat("\t\t",_C(12),"|||||",NULL),  //_"directory line"
-	g_strconcat("\t\t\t|<Control>2|||",_A(13),".",_A(51),"|",NULL), //"go up
-	g_strconcat("\t\t\t|<Alt>2|||",_A(13),".",_A(64),"|",NULL), //"open
+	g_strconcat("\t\t\t|1|true||",_A(5),".",_A(29),"|",NULL), //"dirline.activate
+	g_strconcat("\t\t\t|<Control>2|||",_A(13),".",_A(51),"|",NULL), //"pane.go up
+	g_strconcat("\t\t\t|<Alt>2|||",_A(13),".",_A(64),"|",NULL), //"pane.open
 	g_strconcat("\t\t",_C(5),"|||||",NULL),  //_("command line"
-	g_strconcat("\t\t\t|<Alt>2|||",_A(1),".",_A(80),"|",NULL), //"command.send"
+	g_strconcat("\t\t\t|1|true||",_A(1),".",_A(29),"|",NULL), //"command.activate
+	g_strconcat("\t\t\t|<Alt>2|||",_A(1),".",_A(81),"|",NULL), //"command.send"
 	g_strconcat("\t\t",_C(28),"|||||",NULL),  //_("output"
 	g_strconcat("\t\t\t|1|true||",_A(10),".",_A(29),"|",NULL), //"activate
 	g_strconcat("\t\t\t|<Control>2|||",_A(10),".",_A(31),"|1",NULL), //"adjust ratio
@@ -1767,10 +1842,10 @@ void e2_mousebinding_options_register (void)
 #endif
 	e2_option_tree_add_column (set, _("Action"), E2_OPTION_TREE_TYPE_SEL, 0, "",
 		0, e2_mousebinding_visible_check_cb,
-		GINT_TO_POINTER (E2_ACTION_EXCLUDE_ACCEL | E2_ACTION_EXCLUDE_LAYOUT));
+		GINT_TO_POINTER (E2_ACTION_EXCLUDE_LAYOUT));
 	e2_option_tree_add_column (set, _("Argument"), E2_OPTION_TREE_TYPE_SEL , 0, "",
 		0, e2_mousebinding_visible_check_cb,
-		GINT_TO_POINTER (E2_ACTION_EXCLUDE_GENERAL | E2_ACTION_EXCLUDE_ACCEL
+		GINT_TO_POINTER (E2_ACTION_EXCLUDE_GENERAL | E2_ACTION_EXCLUDE_ACCEL //CHECKME EXCLUDE_POINTER ?
 		| E2_ACTION_EXCLUDE_LAYOUT | E2_ACTION_EXCLUDE_TOGGLE));
 	e2_option_tree_create_store (set);
 
