@@ -1,6 +1,6 @@
-/* $Id: e2_mousegesture.c 1316 2008-10-07 10:49:25Z tpgww $
+/* $Id: e2_mousegesture.c 2319 2011-01-27 03:52:56Z tpgww $
 
-Copyright (C) 2008 tooar <tooar@gmx.net>
+Copyright (C) 2008-2011 tooar <tooar@emelfm2.net>
 
 This file is part of emelfm2.
 emelfm2 is free software; you can redistribute it and/or modify it
@@ -14,8 +14,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 /**
@@ -278,11 +277,11 @@ static gboolean _stroke_watcher_pointermove_cb (GtkWidget *widget,
 		// (? and for race-insurance, check btns & mods too ?)
 			if (gw->handle == NULL)	//WHERE IS BEST FOR THIS ?
 			{
-				GtkAllocation alloc =
-#ifdef USE_GTK2_99
-					gtk_widget_get_allocation (widget);
+				GtkAllocation alloc;
+#ifdef USE_GTK2_18
+				gtk_widget_get_allocation (widget, &alloc);
 #else
-					widget->allocation;
+				alloc = widget->allocation;
 #endif
 				printd (DEBUG, "create new %d X %d stroke handle for widget: %x",
 					alloc.width, alloc.height, widget);
@@ -330,10 +329,13 @@ static gboolean _stroke_watcher_key_event_cb (GtkWidget *widget,
 	if (e2_utils_key_is_modifier (event))
 #endif
 	{
-		//FIXME multi-pointers
 		gint x, y;
-		GdkWindow *win = gdk_window_get_pointer (event->window, &x, &y, NULL);
-		if (win == event->window)
+#ifdef USE_GTK3_0 //CHECKME pointer position
+		if (gdk_window_get_device_position (event->window, gw->device, &x, &y, NULL)
+			== event->window)
+#else
+		if (gdk_window_get_pointer (event->window, &x, &y, NULL) == event->window)
+#endif
 		{
 /* CHECKME
 			GPtrArray *watches = (GPtrArray *) g_object_get_data (G_OBJECT (widget), WATCHES_DATA_KEY); //must exist here
@@ -525,7 +527,12 @@ static gboolean _stroke_watcher_button_event_cb (GtkWidget *widget,
 			printd (DEBUG, "btns all gone for widget: %x", widget);
 			//clear redundant things before any complications arising from a match and therefore callback task
 			//can't be any more release cb's to clear more, so zap it all now
-/*			if (!GTK_WIDGET_REALIZED (widget)) //FIXME
+/*
+#ifdef USE_GTK2_20
+			if (!gtk_widget_get_realized(widget)) //FIXME
+#else
+			if (!GTK_WIDGET_REALIZED (widget)) //FIXME
+#endif
 			{
 				gint mask = gtk_widget_get_events (widget);
 				mask &= ~GDK_POINTER_MOTION_MASK;
@@ -682,14 +689,17 @@ static void _e2_mousegesture_sync_one (GtkTreeModel *model,
 	if (valid)
 	{
 		E2_Action *act = e2_action_check (action);
-		valid = (act == NULL || !(act->exclude & E2_ACTION_EXCLUDE_ACCEL));
+		valid = (act == NULL || !(act->exclude & E2_ACTION_EXCLUDE_POINTER));
 	}
 	if (valid)
 	{
 		g = ALLOCATE0 (gesture_runtime);
+#if (CHECKALLOCATEDWARN)
 		CHECKALLOCATEDWARN (g, valid = FALSE;)
+#else
 		if (g == NULL)
 			valid = FALSE;
+#endif
 	}
 	else
 		g = NULL; //warning prevention
@@ -907,7 +917,7 @@ static void _e2_mousegesture_run_task (GtkWidget *from, gpointer action_string)
 			if (rt == NULL || workdir == NULL)
 				workdir = curr_view->dir;
 			else
-				workdir = rt->view->dir;
+				workdir = rt->view.dir;
 #ifdef E2_VFSTMP
 	CHECKME
 #endif
@@ -935,7 +945,7 @@ transient item that uses the category)
 @return
 */
 void e2_mousegesture_enrol (GtkWidget *widget, const gchar *category,
-	gpointer defaults_func)
+	void (*defaults_func)(E2_OptionSet*))
 {
 	GNode *member = _e2_mousegesture_get_category (category);
 	if (member == NULL)
@@ -955,15 +965,14 @@ void e2_mousegesture_enrol (GtkWidget *widget, const gchar *category,
 		rt->ref = gtk_tree_row_reference_new (set->ex.tree.model, path);
 		gtk_tree_path_free (path);
 #ifdef E2_TRANSIENTBINDINGS
-		if (defaults_func != NULL && !old)
+		if (defaults_func != (void(*)(E2_OptionSet*))NULL && !old)
 		{
 			//run the defaults-install function, which in turn calls
 			//e2_option_tree_setup_defaults() which sets up a pointer array at
-			//dummy_set.ex.tree.def
-			void (*install_func) (E2_OptionSet *) = defaults_func;
+			//dummy_set.ex.tree.def.tree_strings
 			E2_OptionSet dummyset;
-			(*install_func) (&dummyset);
-			gchar **split = (gchar **) dummyset.ex.tree.def;
+			(*defaults_func) (&dummyset);
+			gchar **split = dummyset.ex.tree.def.tree_strings;
 
 			//adjust start-index of parsed array
 			const gchar *p = strrchr (category, '.');
@@ -1057,16 +1066,22 @@ static gboolean _e2_mousegesture_walk_unregister (GNode *node, E2_NodeData *data
 		{
 			if (data->widget == widgets->data || data->widget == NULL)
 			{
-	printd (DEBUG, "UNregister widget");
-				//in case widget isn't to be destroyed
-				g_signal_handlers_disconnect_matched (widgets->data,
-					G_SIGNAL_MATCH_FUNC, 0, 0, NULL, _stroke_watcher_button_event_cb, NULL);
-				gpointer boundto = g_object_get_data (G_OBJECT (widgets->data),
-						GESTURE_BINDINGS_KEY);
-				if (boundto != NULL)
+				if (GTK_IS_WIDGET (widgets->data))
 				{
-					while (g_ptr_array_remove_fast ((GPtrArray *)boundto, rt)); //probably only one
+					//in case widget isn't to be destroyed
+					g_signal_handlers_disconnect_matched (widgets->data,
+						G_SIGNAL_MATCH_FUNC, 0, 0, NULL, _stroke_watcher_button_event_cb, NULL);
+					printd (DEBUG, "UNregister gesture-binding for widget");
+					gpointer boundto = g_object_get_data (G_OBJECT (widgets->data),
+							GESTURE_BINDINGS_KEY);
+					if (boundto != NULL)
+					{
+						while (g_ptr_array_remove_fast ((GPtrArray *)boundto, rt)); //probably only one
+					}
 				}
+				else
+					printd (DEBUG, "No UNregister gesture-binding for destroyed widget");
+
 				widgets->data = NULL;	//don't remove, list pointer cannot change yet
 			}
 		}
@@ -1083,7 +1098,7 @@ static gboolean _e2_mousegesture_walk_unregister (GNode *node, E2_NodeData *data
 */
 void e2_mousegesture_disrol (GtkWidget *widget, const gchar *category)
 {
-	printd (DEBUG, "e2_mousebinding_unregister, category: %s, widget: _",
+	printd (DEBUG, "e2_mousegesture_disrol, category: %s, widget: _",
 		(category) ? category : "ALL");
 	GQuark qname;
 	if (category != NULL)
@@ -1308,10 +1323,10 @@ void e2_mousegesture_options_register (void)
 		0, e2_mousebinding_visible_check_cb, NULL);
 	e2_option_tree_add_column (set, _("Action"), E2_OPTION_TREE_TYPE_SEL, 0, "",
 		0, e2_mousebinding_visible_check_cb,
-		GINT_TO_POINTER (E2_ACTION_EXCLUDE_ACCEL | E2_ACTION_EXCLUDE_LAYOUT));
+		GINT_TO_POINTER (E2_ACTION_EXCLUDE_LAYOUT));
 	e2_option_tree_add_column (set, _("Argument"), E2_OPTION_TREE_TYPE_SEL , 0, "",
 		0, e2_mousebinding_visible_check_cb,
-		GINT_TO_POINTER (E2_ACTION_EXCLUDE_GENERAL | E2_ACTION_EXCLUDE_ACCEL
+		GINT_TO_POINTER (E2_ACTION_EXCLUDE_GENERAL | E2_ACTION_EXCLUDE_ACCEL //CHECKME EXCLUDE POINTER ?
 		| E2_ACTION_EXCLUDE_LAYOUT | E2_ACTION_EXCLUDE_TOGGLE));
 	e2_option_tree_create_store (set);
 
