@@ -1,6 +1,6 @@
-/* $Id: e2_plugins.c 2005 2009-11-11 22:11:02Z tpgww $
+/* $Id: e2_plugins.c 2427 2012-01-22 02:08:21Z tpgww $
 
-Copyright (C) 2003-2009 tooar <tooar@emelfm2.net>
+Copyright (C) 2003-2012 tooar <tooar@emelfm2.net>
 Portions copyright (C) 1999 Michael Clark.
 
 This file is part of emelFM2.
@@ -15,8 +15,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 /**
@@ -125,6 +124,22 @@ Plugins' configuration data are loaded into the corresponding config treestore a
 #include "e2_dialog.h"
 #include "e2_filelist.h"
 
+//show plugin file path and name in plugins GUI
+//#define SHOW_PLUGPATHS
+
+//plugins config treestore columns
+enum
+{
+	LOAD_COL,
+	MENU_COL,
+	LABEL_COL,
+	ICON_COL,
+	TIP_COL,
+	FILE_COL,
+	PATH_COL,
+	SIG_COL
+};
+
 typedef struct _E2P_Ref
 {
 	gchar *name;
@@ -221,6 +236,18 @@ static gint _e2_plugins_unref (GList **list, gchar *name)
 		return (gint)newrefcount;
 	}
 }
+/**
+@brief list foreach func to check if plugin data @a p has signature @a signature
+See also: e2_plugins_check_installed()
+@param p pointer to plugin data struct
+@param signature string to be found
+
+@return 0 if @a p has signature @a signature
+*/
+static gint _e2_plugins_match_sig (Plugin *p, gchar *signature)
+{
+	return (strcmp (p->signature, signature));
+}
 /* *
 @brief get function address
 
@@ -260,7 +287,7 @@ Any child plugins are cleared, also from app.plugins
 Adjusts app.plugins, so is not very suited to use during a walk of that list
 Any related config treestore data are not affected - any update of those
 must be done before coming here
-Expects BGL on/closed
+Expects BGL on/closed (for any error message display)
 
 @param p plugin data struct
 @param force TRUE to force removal even if not cleaned up by the plugin
@@ -283,10 +310,10 @@ gboolean e2_plugins_unload1 (Plugin *p, gboolean force)
 			for (member = p->child_list; member != NULL; member = member->next)
 			{
 				pc = member->data;
-				if (pc->cleanflags & E2P_CLEANICON)
-					g_free (pc->icon);
 				if (pc->cleanflags & E2P_CLEANLABEL)
 					g_free (pc->menu_name);
+				if (pc->cleanflags & E2P_CLEANICON)
+					g_free (pc->icon);
 				if (pc->cleanflags & E2P_CLEANTIP)
 					g_free (pc->description);
 				DEALLOCATE (Plugin, pc);
@@ -392,7 +419,7 @@ void e2_plugins_clean (void)
 
 @param filepath path+name string for the plugin, probably localised (API doc is silent)
 
-@return plugin data struct for the loaded plugin, or NULL if plugin not found or has no initialize fn
+@return allocated plugin data struct for the loaded plugin, or NULL if plugin not found or has no initialize fn
 */
 Plugin *e2_plugins_open1 (gchar *filepath)
 {
@@ -421,7 +448,7 @@ Plugin *e2_plugins_open1 (gchar *filepath)
 }
 /**
 @brief process child UI data into config store and cleanup
-This does not affect app.plugins
+This does not affect app.plugins. @ap ->child_list is cleared
 @param model model for plugins config treestore
 @param iter pointer to parent-iter in @a model
 @param p pointer to data struct for parent plugin
@@ -438,10 +465,18 @@ static void _e2_plugins_store_child_data (GtkTreeModel *model, GtkTreeIter *iter
 		{
 			E2_Sextet *uidata = (E2_Sextet *)member->data;
 			gtk_tree_store_insert_before (GTK_TREE_STORE (model), &iter2, iter, NULL);
-			//loaded flag is irrelevant for child plugins, set to match on_menu
 			gtk_tree_store_set (GTK_TREE_STORE (model), &iter2,
-				0, p->show_in_menu, 1, p->show_in_menu, 2, uidata->a,
-				3, uidata->b, 4, uidata->c, 5, "", 6, uidata->d, -1);
+				LOAD_COL, p->show_in_menu,//loaded flag is irrelevant for child plugins, set to match parent
+				MENU_COL, p->show_in_menu,
+				LABEL_COL, uidata->a,
+				ICON_COL, uidata->b,
+				TIP_COL, uidata->c,
+//#ifdef SHOW_PLUGPATHS // ? or always, to cleanup old-format config data
+				FILE_COL, "",			//clear display
+				PATH_COL, "",
+//#endif
+				SIG_COL, uidata->d,
+				-1);
 			e2_utils_sextet_destroy (uidata);
 		}
 		g_list_free (p->child_list);	//finished interim usage
@@ -469,23 +504,38 @@ void e2_plugins_store_data (GtkTreeModel *model, GtkTreeIter *iter,
 	const gchar *description = (p->description != NULL) ? p->description : NULL;
 	gchar *local = strrchr (localpath, G_DIR_SEPARATOR);	//should never fail
 	*local = '\0';	//truncate plocal at trailing /
-	gchar *utf = F_FILENAME_FROM_LOCALE (local + 1);
-	gchar *utfp;
-	if (!strcmp (localpath, PLUGINS_DIR))	//from Makefile this is localised, no trailer
+	gchar *utfn, *utfp;
+#ifdef SHOW_PLUGPATHS
+	utfn = F_FILENAME_FROM_LOCALE (local + 1); //NOT DISPLAYNAME
+	if (!strcmp (localpath, PLUGINS_DIR))	//from Makefile PLUGINS_DIR is localised, no trailer
 		utfp = "";
 	else
 		utfp = F_FILENAME_FROM_LOCALE (localpath);
+#else
+	utfn = local + 1;
+	utfp = (!strcmp (localpath, PLUGINS_DIR)) ? "" : localpath;	//from Makefile PLUGINS_DIR is localised, no trailer
+#endif
 
 	gtk_tree_store_set (GTK_TREE_STORE (model),	iter,
-		0, inmenu, 1, inmenu, 2, menu_name, 3, icon, 4, description, 5, utf, 6, utfp, -1);
+		LOAD_COL, inmenu,
+		MENU_COL, inmenu,
+		LABEL_COL, menu_name,
+		ICON_COL, icon,
+		TIP_COL, description,
+		FILE_COL, utfn,
+		PATH_COL, utfp,
+		SIG_COL, p->signature,
+		-1);
 	p->show_in_menu = inmenu;	//any child-plugin uses this
 	//process any child-plugin data too
 	_e2_plugins_store_child_data (model, iter, p);
 
 	*local = G_DIR_SEPARATOR;
+#ifdef SHOW_PLUGPATHS
 	if (*utfp != '\0')
 		F_FREE (utfp, localpath);
-	F_FREE (utf, local + 1);
+	F_FREE (utfn, local + 1);
+#endif
 }
 /* *
 @brief reconcile on-disk plugin files with plugins-config-data
@@ -505,10 +555,12 @@ void e2_plugins_store_data (GtkTreeModel *model, GtkTreeIter *iter,
 		gchar *utfpath, *utfname;
 		do
 		{
-			gtk_tree_model_get (model, &iter, 5, &utfname, 6, &utfpath, -1);
+			gtk_tree_model_get (model, &iter, FILE_COL, &utfname, PATH_COL, &utfpath, -1);
 			if (*utfpath == '\0')
 			{
+#ifdef SHOW_PLUGPATHS
 				construct localpath
+#endif
 				if stat() fails
 				{
 					remove iter from store
@@ -542,18 +594,6 @@ void e2_plugins_store_data (GtkTreeModel *model, GtkTreeIter *iter,
 }
 */
 /**
-@brief check if plugin data @a child has signature @a signature
-
-@param child pointer to child plugin data struct
-@param signature string to be found in @a child
-
-@return 0 if @a child has signature @a signature
-*/
-static gint _e2_plugins_match_child (Plugin *child, gchar *signature)
-{
-	return (strcmp (child->signature, signature));
-}
-/**
 @brief process "child plugin" data when a plugin has > 1 action
 As needed, this updates the config treestore data, and appends "child" plugins
 to app.plugins
@@ -589,27 +629,29 @@ static void _e2_plugins_record_children (GtkTreeModel *model, GtkTreeIter *iter,
 			do
 			{
 				gchar *childsig;
-				gtk_tree_model_get (model, &iter2, 6, &childsig, -1); //from original p->signature
+				gtk_tree_model_get (model, &iter2, SIG_COL, &childsig, -1); //from original p->signature
 				//plugin init creates p->child_list in same order as child-signature-index
 				//that index is a string of the form "n-ANAME" where n= 0,1, ...
 				//but in the config store, the lines can be in any order
 				member = g_list_find_custom (p->child_list, childsig,
-					(GCompareFunc)_e2_plugins_match_child);
+					(GCompareFunc)_e2_plugins_match_sig);
 				if (member != NULL) //found a child plugin matching the store iter (member->data = NULL handled in search func))
 				{
 					gboolean childon;
-					gtk_tree_model_get (model, iter, 1, &childon, -1);
+					gtk_tree_model_get (model, iter, MENU_COL, &childon, -1);
 					if (childon)	//parent is on-menu
-						gtk_tree_model_get (model, &iter2, 1, &childon, -1); //keep original value
+						gtk_tree_model_get (model, &iter2, MENU_COL, &childon, -1); //keep original value
 					else
 						p->show_in_menu = FALSE;
 					pc = (Plugin *)member->data;
 					gtk_tree_store_set (GTK_TREE_STORE (model), &iter2,
-						0, TRUE,	//for cosmetic reasons only, set this flag same as for parent
-						1, childon,
-						5, "",	//clear any default filename string
-						//this repetition not really needed, but just in case the user edited the sig ...
-						6, pc->signature,	//formatted as n-ANAME, n=0,1,....
+						LOAD_COL, TRUE,	//for cosmetic reasons only, set this flag same as for parent
+						MENU_COL, childon,
+#ifdef SHOW_PLUGPATHS
+						FILE_COL, "",	//clear any default filename string
+						PATH_COL, "",  //and path
+#endif
+						SIG_COL, pc->signature,	//formatted as n-ANAME, n=0,1,....
 						-1);
 					pc->show_in_menu = childon;
 					if (pc->cleanflags & E2P_CLEANICON)
@@ -619,10 +661,10 @@ static void _e2_plugins_record_children (GtkTreeModel *model, GtkTreeIter *iter,
 					if (pc->cleanflags & E2P_CLEANTIP)
 						g_free (pc->description);
 					gtk_tree_model_get (model, &iter2,
-//						1, &pc->show_in_menu,
-						2, &pc->menu_name,
-						3, &pc->icon,
-						4, &pc->description,
+//						MENU_COL, &pc->show_in_menu,
+						LABEL_COL, &pc->menu_name,
+						ICON_COL, &pc->icon,
+						TIP_COL, &pc->description,
 						-1);
 					pc->cleanflags = E2P_CLEANALL;
 					//progressively build replacement list in same order as config data
@@ -650,14 +692,16 @@ static void _e2_plugins_record_children (GtkTreeModel *model, GtkTreeIter *iter,
 					gtk_tree_store_append (GTK_TREE_STORE (model), &iter2, iter);
 					gtk_tree_store_set (GTK_TREE_STORE (model), &iter2,
 #endif
-						0, TRUE,	//just for config dialog cosmetics
-						1, FALSE,
-						2, (pc->menu_name != NULL) ? pc->menu_name : "",
-						3, (pc->icon != NULL) ? pc->icon : "",
-						4, (pc->description != NULL) ? pc->description : "",
-						5, "",	//clear the default filename string
-						//instead of path (the parent will cover that) log the matching-signature
-						6, pc->signature,	//formatted as n-ANAME, n=0,1,....
+						LOAD_COL, TRUE,	//just for config dialog cosmetics
+						MENU_COL, FALSE,
+						LABEL_COL, (pc->menu_name != NULL) ? pc->menu_name : "",
+						ICON_COL, (pc->icon != NULL) ? pc->icon : "",
+						TIP_COL, (pc->description != NULL) ? pc->description : "",
+#ifdef SHOW_PLUGPATHS
+						FILE_COL, "",	//clear the default filename string
+						PATH_COL, "",	//and path
+#endif
+						SIG_COL, pc->signature,	//formatted as n-ANAME, n=0,1,....
 						-1);
 				}
 			}
@@ -683,16 +727,22 @@ static void _e2_plugins_record_children (GtkTreeModel *model, GtkTreeIter *iter,
 		{
 			pc = (Plugin *)member->data;
 			pc->show_in_menu = p->show_in_menu;	//match the parent's setting
+#ifdef USE_GTK2_10
+			gtk_tree_store_insert_with_values (GTK_TREE_STORE (model), &iter2, iter, -1,
+#else
 			gtk_tree_store_append (GTK_TREE_STORE (model), &iter2, iter);
 			gtk_tree_store_set (GTK_TREE_STORE (model), &iter2,
-				0, TRUE,
-				1, pc->show_in_menu,
-				2, (pc->menu_name != NULL) ? pc->menu_name : "",
-				3, (pc->icon != NULL) ? pc->icon : "",
-				4, (pc->description != NULL) ? pc->description : "",
-				5, "",	//clear the default filename string
-				//instead of path (the parent will cover that) provide a matching-string
-				6, pc->signature,	//formatted as n-ANAME, n=0,1,....
+#endif
+				LOAD_COL, TRUE,
+				MENU_COL, pc->show_in_menu,
+				LABEL_COL, (pc->menu_name != NULL) ? pc->menu_name : "",
+				ICON_COL, (pc->icon != NULL) ? pc->icon : "",
+				TIP_COL, (pc->description != NULL) ? pc->description : "",
+#ifdef SHOW_PLUGPATHS
+				FILE_COL, "",	//clear the default filename string
+				PATH_COL, "",	//and path
+#endif
+				SIG_COL, pc->signature,	//formatted as n-ANAME, n=0,1,....
 				-1);
 		}
 		//append children to the main list, in the order defined by the init func
@@ -718,13 +768,24 @@ static void _e2_plugins_load1 (GtkTreeModel *model, GtkTreeIter *iter)
 	void (*unload)(Plugin *);
 	gboolean load_this, in_menu;
 	gchar *label, *icon, *tip;	//these are for checking if currently in config
-	gchar *utf_name, *utf_path, *localdir, *localpath;
+	gchar *pname, *pdir, *localdir, *localpath;
 
-	gtk_tree_model_get (model, iter, 0, &load_this, 1, &in_menu,
-		2, &label, 3, &icon, 4, &tip, 5, &utf_name, 6, &utf_path, -1);
+	gtk_tree_model_get (model, iter,
+		LOAD_COL, &load_this,
+		MENU_COL, &in_menu,
+		LABEL_COL, &label,
+		ICON_COL, &icon,
+		TIP_COL, &tip,
+		FILE_COL, &pname,
+		PATH_COL, &pdir,
+		-1);
 
-	if (*utf_path != '\0')
-		localdir = F_FILENAME_TO_LOCALE (utf_path);
+	if (pdir != NULL && *pdir != '\0')
+#ifdef SHOW_PLUGPATHS
+		localdir = F_FILENAME_TO_LOCALE (pdir);
+#else
+		localdir = pdir;
+#endif
 	else
 		//no path specified, so use the default plugins path
 		localdir = PLUGINS_DIR;	//localised
@@ -736,10 +797,13 @@ static void _e2_plugins_load1 (GtkTreeModel *model, GtkTreeIter *iter)
 	if (e2_fs_is_dir3 (localdir E2_ERR_NONE()))	//only local places for plugins
 #endif
 	{
-		gchar *file_name = F_FILENAME_TO_LOCALE (utf_name);
-		localpath = g_build_filename (localdir, file_name, NULL);
-		F_FREE (file_name, utf_name);
-
+#ifdef SHOW_PLUGPATHS
+		gchar *local = F_FILENAME_TO_LOCALE (pname);
+		localpath = g_build_filename (localdir, local, NULL);
+		F_FREE (local, pname);
+#else
+		localpath = g_build_filename (localdir, pname, NULL);
+#endif
 		if (load_this || in_menu //this is one we want to keep
 			|| *label == '\0')	//no data logged CHECKME is this test ok ?
 		{
@@ -748,7 +812,7 @@ static void _e2_plugins_load1 (GtkTreeModel *model, GtkTreeIter *iter)
 			{
 				if (load_this || in_menu)
 				{
-	//				p->api_lookup = e2_plugins_api_lookup;
+//					p->api_lookup = e2_plugins_api_lookup;
 					p->action = NULL; //ensure "real" init
 					if (p->plugin_init (p))
 					{
@@ -756,7 +820,7 @@ static void _e2_plugins_load1 (GtkTreeModel *model, GtkTreeIter *iter)
 						p->show_in_menu = in_menu;
 						if (*label == '\0' && p->menu_name != NULL)
 							gtk_tree_store_set (GTK_TREE_STORE (model), iter,
-								2, p->menu_name, -1);
+								LABEL_COL, p->menu_name, -1);
 						else if (p->menu_name == NULL || strcmp (p->menu_name, label))
 						{
 							if (p->menu_name != NULL && (p->cleanflags & E2P_CLEANLABEL))
@@ -766,7 +830,7 @@ static void _e2_plugins_load1 (GtkTreeModel *model, GtkTreeIter *iter)
 						}
 						if (*icon == '\0' && p->icon != NULL)
 							gtk_tree_store_set (GTK_TREE_STORE (model), iter,
-								3, p->icon, -1);
+								ICON_COL, p->icon, -1);
 						else if (p->icon == NULL || strcmp (p->icon, icon))
 						{
 							if (p->icon != NULL && (p->cleanflags & E2P_CLEANICON))
@@ -776,7 +840,7 @@ static void _e2_plugins_load1 (GtkTreeModel *model, GtkTreeIter *iter)
 						}
 						if (*tip == '\0' && p->description != NULL)
 							gtk_tree_store_set (GTK_TREE_STORE (model), iter,
-								4, p->description, -1);
+								TIP_COL, p->description, -1);
 						else if (p->description == NULL || strcmp (p->description, tip))
 						{
 							if (p->description != NULL && (p->cleanflags & E2P_CLEANTIP))
@@ -784,6 +848,8 @@ static void _e2_plugins_load1 (GtkTreeModel *model, GtkTreeIter *iter)
 							p->description = g_strdup (tip);
 							p->cleanflags |= E2P_CLEANTIP;
 						}
+						gtk_tree_store_set (GTK_TREE_STORE (model), iter,
+								SIG_COL, p->signature, -1);
 
 						if (e2_plugins_check_installed (p->signature) == NULL)
 						{
@@ -793,7 +859,7 @@ static void _e2_plugins_load1 (GtkTreeModel *model, GtkTreeIter *iter)
 							{	//no child plugin(s)
 								if (*tip == '\0' && p->description != NULL)
 									gtk_tree_store_set (GTK_TREE_STORE (model), iter,
-										4, p->description, -1);
+										TIP_COL, p->description, -1);
 								else if (p->description == NULL || strcmp (p->description, tip))
 									p->description = g_strdup (tip);
 								//FIXME if there were children before, make sure gone
@@ -803,7 +869,7 @@ static void _e2_plugins_load1 (GtkTreeModel *model, GtkTreeIter *iter)
 							{
 								//when child plugin(s) exist, parent tip won't show in menu
 								 //so make submenu item's tip blank
-								gtk_tree_store_set (GTK_TREE_STORE (model), iter, 4, "", -1);
+								gtk_tree_store_set (GTK_TREE_STORE (model), iter, TIP_COL, "", -1);
 								//setup children data
 								_e2_plugins_record_children (model, iter, p);
 							}
@@ -812,16 +878,16 @@ static void _e2_plugins_load1 (GtkTreeModel *model, GtkTreeIter *iter)
 						{
 							printd (WARN, "aborted attempt to load 2nd copy of plugin: %s", p->signature);
 						    //CHECKME does this also remove any children from store
-						    gtk_tree_store_remove (GTK_TREE_STORE (model), iter); //CHECKME effect on store iteration
+							gtk_tree_store_remove (GTK_TREE_STORE (model), iter); //CHECKME effect on store iteration
 							goto killit;
 						}
 					}
 					else	//init function failed
 					{
-						printd (WARN, "can't initialize plugin: %s", utf_name);
+						printd (WARN, "can't initialize plugin: %s", pname);
 						//mark it as un-available (i.e. don't completely remove from store)
 						gtk_tree_store_set (GTK_TREE_STORE (model), iter,
-							0, FALSE, 1, FALSE, -1);
+							LOAD_COL, FALSE, MENU_COL, FALSE, SIG_COL, "", -1);
 						GtkTreeIter iter2;
 						if (gtk_tree_model_iter_children (model, &iter2, iter))
 						{
@@ -829,7 +895,7 @@ static void _e2_plugins_load1 (GtkTreeModel *model, GtkTreeIter *iter)
 							do
 							{
 								gtk_tree_store_set (GTK_TREE_STORE (model), &iter2,
-									0, FALSE, 1, FALSE, -1);
+									LOAD_COL, FALSE, MENU_COL, FALSE, SIG_COL, "", -1);
 							} while (gtk_tree_model_iter_next (model, &iter2));
 						}
 killit:
@@ -848,16 +914,16 @@ killit:
 					//{
 						if (p->menu_name != NULL)
 							gtk_tree_store_set (GTK_TREE_STORE (model), iter,
-								2, p->menu_name, -1);
+								LABEL_COL, p->menu_name, -1);
 						if (*icon == '\0' && p->icon != NULL)
 							gtk_tree_store_set (GTK_TREE_STORE (model), iter,
-								3, p->icon, -1);
+								ICON_COL, p->icon, -1);
 						if (*tip == '\0')
 						{
 							const gchar *faketip = (p->description != NULL) ?
 								p->description : _("Plugin not loaded");	//some extra user info
 							gtk_tree_store_set (GTK_TREE_STORE (model), iter,
-								4, faketip, -1);
+								TIP_COL, faketip, -1);
 						}
 						//get store and list data for children, if any
 						p->show_in_menu = FALSE;	//set flags in store
@@ -876,7 +942,7 @@ killit:
 			}
 			else	//plugin file not loaded
 			{
-				printd (WARN, "plugin %s doesn't exist or won't load", utf_name);
+				printd (WARN, "plugin %s doesn't exist or won't load", pname);
 				gtk_tree_store_remove (GTK_TREE_STORE (model), iter); //CHECKME effect on store iteration
 			}
 		}
@@ -884,16 +950,18 @@ killit:
 	}
 	else //no valid dir for the plugin
 	{
-		printd (WARN, "plugin dir doesn't exist: %s", utf_path);
+		printd (WARN, "plugin dir doesn't exist: %s", pdir);
 		gtk_tree_store_remove (GTK_TREE_STORE (model), iter); //CHECKME effect on store iteration
 	}
-	if (*utf_path != '\0')
-		F_FREE (localdir, utf_path);
+#ifdef SHOW_PLUGPATHS
+	if (pdir != NULL && *pdir != '\0')
+		F_FREE (localdir, pdir);
+#endif
 	g_free (label);
 	g_free (icon);
 	g_free (tip);
-	g_free (utf_name);
-	g_free (utf_path);
+	g_free (pname);
+	g_free (pdir);
 }
 /**
 @brief load all plugins that are recorded in plugins config data
@@ -927,14 +995,14 @@ void e2_plugins_load_all (void)
 This is the 'apply' fn for the plugins dialog, also used in general config dialog
 It performs a differential load/unload relative to the status quo
 Status quo is determined by checking menu name, which is in both Plugin and model
-( so if a user edits the menu name, the plugin will be improperly, but not fatally,
-reloaded )
+(so if a user edits the menu name, the plugin will be improperly, but not fatally,
+reloaded)
 
 @return
 */
 void e2_plugins_update (void)
 {
-	gchar *icon, *name, *tip;
+	gchar *icon, *label, *tip;
 	gboolean load, menu, freestrings;
 	GList *oldlist, *member;
 	Plugin *p;
@@ -946,104 +1014,107 @@ void e2_plugins_update (void)
 		app.plugins = NULL;
 		do
 		{
+			gchar *sig;
 			freestrings = TRUE;
 			gtk_tree_model_get (set->ex.tree.model, &iter,
-				0, &load, 1, &menu, 2, &name, 3, &icon, 4, &tip, -1);
+				LOAD_COL, &load,
+				MENU_COL, &menu,
+				LABEL_COL, &label,
+				ICON_COL, &icon,
+				TIP_COL, &tip,
+				SIG_COL, &sig,
+				-1);
 			if (menu && !load) //fix any mistake by user
 			{
 				gtk_tree_store_set (GTK_TREE_STORE (set->ex.tree.model), &iter,
-					0, TRUE, -1);
+					LOAD_COL, TRUE, -1);
 				load = TRUE;
 			}
 			//check if this one is already in play
-			//FIXME menu_name is not a reliable indicator. Enable the signature to
-			//be stored in treestore, but not displayed so user can't change it
-			//(it's already stored, in column 6, for child plugins, but that's editable)
-			if (name == NULL)
-				member = NULL;
-			else
+			for (member = oldlist; member != NULL; member = member->next)
 			{
-				for (member = oldlist; member != NULL; member = member->next)
-				{
-					if (!strcmp (name, ((Plugin *)member->data)->menu_name))
-						break;
-				}
+				if (!strcmp (sig, ((Plugin *)member->data)->signature))
+					break;
 			}
 
-			if (load && member != NULL)
-			{	//this one is to keep
-				oldlist = g_list_remove_link (oldlist, member);
-				app.plugins = g_list_concat (app.plugins, member);
-				p = (Plugin *)member->data;
-				//update on-menu status in case was edited
-				p->show_in_menu = menu;
-				//update strings in case they were edited
-				if (p->icon != NULL && (p->cleanflags & E2P_CLEANICON))
-					g_free (p->icon);
-				p->icon = icon;
-				p->cleanflags |= E2P_CLEANICON;
-				if (p->menu_name != NULL && (p->cleanflags & E2P_CLEANLABEL))
-					g_free (p->menu_name);
-				p->menu_name = name;
-				p->cleanflags |= E2P_CLEANLABEL;
-				if (p->description != NULL && (p->cleanflags & E2P_CLEANTIP))
-					g_free (p->description);
-				p->description = tip;
-				p->cleanflags |= E2P_CLEANTIP;
-				freestrings = FALSE;
-				//process any child plugins
-				if (p->child_list != NULL)
-				{
-					/*_e2_plugins_record_children() expects the child plugins
-					not to be in app.plugins, and the parent plugin to be at
-					the end of app.plugins. Should not be any child back in
-					app.plugins yet, but in case the user has moved a child
-					badly out of order ... */
-					GList *childmember;
-					for (childmember = p->child_list; childmember != NULL;
-						childmember = childmember->next)
-						app.plugins = g_list_remove_all (app.plugins, childmember->data);
-
-					_e2_plugins_record_children (set->ex.tree.model, &iter, p);
-				}
-			}
-			else	// !load and/or member == NULL
-				if (member != NULL) //we need to unload this one if we can
+			if (member != NULL)
 			{
-				//process any child plugins' flags before dumping
-				p = (Plugin *)member->data;
-				if (p->child_list != NULL)
-				{
-					GList *node;
-					for (node = p->child_list; node != NULL; node = node->next)
+				if (load)
+				{ //this one is to keep
+					oldlist = g_list_remove_link (oldlist, member);
+					app.plugins = g_list_concat (app.plugins, member);
+					p = (Plugin *)member->data;
+					//update on-menu status in case was edited
+					p->show_in_menu = menu;
+					//update strings in case they were edited
+					if (p->icon != NULL && (p->cleanflags & E2P_CLEANICON))
+						g_free (p->icon);
+					p->icon = icon;
+					p->cleanflags |= E2P_CLEANICON;
+					if (p->menu_name != NULL && (p->cleanflags & E2P_CLEANLABEL))
+						g_free (p->menu_name);
+					p->menu_name = label;
+					p->cleanflags |= E2P_CLEANLABEL;
+					if (p->description != NULL && (p->cleanflags & E2P_CLEANTIP))
+						g_free (p->description);
+					p->description = tip;
+					p->cleanflags |= E2P_CLEANTIP;
+					freestrings = FALSE;
+					//process any child plugins
+					if (p->child_list != NULL)
 					{
-						p = (Plugin *)node->data;
-						iter2 = iter;
-						if (e2_tree_find_iter_from_str_simple (set->ex.tree.model,
-							6, p->signature, &iter2, FALSE))
-							gtk_tree_store_set (GTK_TREE_STORE (set->ex.tree.model),
-								&iter2, 0, FALSE, 1, FALSE, -1);
+						/*_e2_plugins_record_children() expects the child plugins
+						not to be in app.plugins, and the parent plugin to be at
+						the end of app.plugins. Should not be any child back in
+						app.plugins yet, but in case the user has moved a child
+						badly out of order ... */
+						GList *childmember;
+						for (childmember = p->child_list; childmember != NULL;
+							childmember = childmember->next)
+							app.plugins = g_list_remove_all (app.plugins, childmember->data);
+
+						_e2_plugins_record_children (set->ex.tree.model, &iter, p);
 					}
 				}
-				if (e2_plugins_unload1 (member->data, FALSE))
-					oldlist = g_list_delete_link (oldlist, member);
-				else
-					//some strange unload problem, revert the status
-					gtk_tree_store_set (GTK_TREE_STORE (set->ex.tree.model),
-						&iter, 0, TRUE,-1);
-					//TOO BAD ABOUT ANY CHILD PLUGINS' FLAGS, HERE
+				else // !load - we need to unload this one if we can
+				{
+					//process any child plugins' flags before dumping
+					p = (Plugin *)member->data;
+					if (p->child_list != NULL)
+					{
+						GList *node;
+						for (node = p->child_list; node != NULL; node = node->next)
+						{
+							p = (Plugin *)node->data;
+							iter2 = iter;
+							if (e2_tree_find_iter_from_str_simple (set->ex.tree.model,
+									SIG_COL, p->signature, &iter2, FALSE))
+								gtk_tree_store_set (GTK_TREE_STORE (set->ex.tree.model),
+									&iter2, LOAD_COL, FALSE, MENU_COL, FALSE, SIG_COL, "", -1);
+						}
+					}
+					if (e2_plugins_unload1 (member->data, FALSE))
+						oldlist = g_list_delete_link (oldlist, member);
+					else
+						//some strange unload problem, revert the status
+						gtk_tree_store_set (GTK_TREE_STORE (set->ex.tree.model),
+							&iter, LOAD_COL, TRUE,-1);
+						//TOO BAD ABOUT ANY CHILD PLUGINS' FLAGS, HERE
+				}
 			}
 			else	//member == NULL, load = ?
 				if (load) //we need to load this one
 			{
 				_e2_plugins_load1 (set->ex.tree.model, &iter);
 			}
+
 			if (freestrings)
 			{
-				g_free (name);
+				g_free (label);
 				g_free (icon);
 				g_free (tip);
 			}
+			g_free (sig);
 		} while (gtk_tree_model_iter_next (set->ex.tree.model, &iter));
 
 		if (oldlist != NULL)
@@ -1081,18 +1152,17 @@ This is intended to be used by plugins, if they want to interact
 */
 Plugin *e2_plugins_check_installed (const gchar *signature)
 {
-	GList *tmp;
-	Plugin *p;
-	for (tmp = app.plugins; tmp != NULL; tmp = tmp->next)
+	GList *node;
+	for (node = app.plugins; node != NULL; node = node->next)
 	{
-		p = tmp->data;
+		Plugin *p = node->data;
 		if (!strcmp (p->signature, signature))
 			return p;
 	}
 	return NULL;
 }
 /**
-@brief load plugin with specified @a signature, if it's not loaded already
+@brief load (non-child) plugin with specified @a signature, if it's not loaded already
 
 @param signature unique ID string for the desired plugin, may be localised
 
@@ -1103,7 +1173,7 @@ Plugin *e2_plugins_load_plugin (const gchar *signature)
 	Plugin *p = e2_plugins_check_installed (signature);
 	if (p == NULL)
 	{
-		gchar *ppath, *pname, *local, *s;
+		gchar *ppath, *pname, *s;
 
 		ppath = NULL;
 		s = strstr (signature, VERSION);
@@ -1116,17 +1186,21 @@ Plugin *e2_plugins_load_plugin (const gchar *signature)
 		gboolean inconfig = FALSE;
 		if (gtk_tree_model_get_iter_first (mdl, &iter))
 		{
-			if (e2_tree_find_iter_from_str_same (mdl, 5, pname, &iter))
+			if (e2_tree_find_iter_from_str_same (mdl, FILE_COL, pname, &iter))
 			{
 				inconfig = TRUE;
-				gtk_tree_model_get (mdl, &iter, 6, &ppath, -1);
+				gtk_tree_model_get (mdl, &iter, PATH_COL, &ppath, -1);
 				if (ppath != NULL && *ppath != '\0')
 				{	//plugin is recorded in config data
 					s = ppath;
-					local = F_FILENAME_TO_LOCALE (s);
+#ifdef SHOW_PLUGPATHS
+					gchar *local = F_FILENAME_TO_LOCALE (s);
 					ppath = g_build_filename (local, pname, NULL); //create path from config data
-					g_free (s);
 					F_FREE (local, s);
+#else
+					ppath = g_build_filename (s, pname, NULL); //create path from config data
+#endif
+					g_free (s);
 				}
 				else if (ppath != NULL)
 				{
@@ -1150,14 +1224,17 @@ Plugin *e2_plugins_load_plugin (const gchar *signature)
 				p = NULL;
 			}
 			if (inconfig)
-				gtk_tree_store_set (GTK_TREE_STORE (set->ex.tree.model), &iter, 0, TRUE, -1);
+				gtk_tree_store_set (GTK_TREE_STORE (set->ex.tree.model), &iter,
+					LOAD_COL, TRUE, SIG_COL, p->signature, -1);
 			else
 			{
 				//add to config data
 				const gchar *label = (p->menu_name != NULL) ? p->menu_name : "";
 				const gchar *icon = (p->icon != NULL) ? p->icon : "";
 				const gchar *tip = (p->description != NULL) ? p->description : "";
+#ifdef SHOW_PLUGPATHS
 				gchar *name = F_FILENAME_FROM_LOCALE (pname);
+#endif
 #ifdef USE_GTK2_10
 				gtk_tree_store_insert_with_values (GTK_TREE_STORE (set->ex.tree.model),
 					&iter, NULL, -1,
@@ -1166,9 +1243,22 @@ Plugin *e2_plugins_load_plugin (const gchar *signature)
 				gtk_tree_store_set (GTK_TREE_STORE (set->ex.tree.model), &iter,
 #endif
 					//if path is not already in config, then it must be default
-					0, TRUE, 1, FALSE, 2, label, 3, icon, 4, tip, 5, name, 6, "", -1);
-
+					LOAD_COL, TRUE,
+					MENU_COL, FALSE,
+					LABEL_COL, label,
+					ICON_COL, icon,
+					TIP_COL, tip,
+#ifdef SHOW_PLUGPATHS
+					FILE_COL, name,
+#else
+					FILE_COL, pname,
+#endif
+					PATH_COL, "",
+					SIG_COL, p->signature,
+					-1);
+#ifdef SHOW_PLUGPATHS
 				F_FREE (name, pname);
+#endif
 			}
 		}
 		else
@@ -1309,17 +1399,25 @@ E2_OptionSet *e2_plugins_option_register (E2_OptionType type, gchar *name,
 				break;
 			case E2_OPTION_TYPE_SEL:
 				set = e2_option_sel_register (name, group, desc, tip, depends,
-				ex->exsel.def, ex->exsel.values, flags);
+					ex->exsel.def, ex->exsel.values, flags);
+				break;
+			case E2_OPTION_TYPE_STR:
+				set = e2_option_str_register (name, group, desc, tip, depends,
+					ex->exstr, flags);
 				break;
-//			case E2_OPTION_TYPE_STR:
-//				break;
 //			case E2_OPTION_TYPE_FONT:
+//				set = e2_option_font_register (name, group, desc, tip, depends,
+//					ex->exstr, flags);
 //				break;
 //			case E2_OPTION_TYPE_COLOR:
+//				set = e2_option_color_register (name, group, desc, tip, depends,
+//					ex->exstr, flags);
 //				break;
 //			case E2_OPTION_TYPE_ICON:
+//				set =
 //				break;
 //			case E2_OPTION_TYPE_TREE:
+//				set =
 //				break;
 			default:
 				printd (ERROR, "BAD attempt to register unsupported plugin option");
@@ -1348,11 +1446,42 @@ gboolean e2_plugins_option_unregister (gchar *name)
 		return FALSE;
 }
 /**
+@brief (de)sensitize option tree buttons for selected option tree row
+Config dialog page buttons are de-sensitized if the row is not a 'child' plugin
+@param selection pointer to selection
+@param model UNUSED
+@param path
+@param path_currently_selected
+@param set data struct for the plugins option UNUSED
+@return TRUE always (the row is always selectable)
+*/
+static gboolean _e2_plugin_tree_selection_check_cb
+	(GtkTreeSelection *selection, GtkTreeModel *model, GtkTreePath *path,
+	gboolean path_currently_selected, E2_OptionSet *set)
+{
+	GtkTreeView *view = gtk_tree_selection_get_tree_view (selection);
+	gboolean result = gtk_tree_path_get_depth (path) == 1;
+	e2_option_tree_adjust_buttons (view, result);
+	return TRUE;
+}
+/**
+@brief decide whether tree row is draggable
+Checks whether row depth is 1 i.e. not a child plugin. If so, the row is draggable
+@param drag_source GtkTreeDragSource data struct
+@param path tree path to a row on which user is initiating a drag
+@return TRUE if the row can be dragged
+*/
+static gboolean _e2_plugin_tree_draggable_check_cb
+	(GtkTreeDragSource *drag_source, GtkTreePath *path)
+{
+	return (gtk_tree_path_get_depth (path) == 1);
+}
+/**
 @brief install default tree options for plugins
 This function is called only if the default is missing from the config file
 This is essentially a list of all the expected plugin filenames
-the menu name and description at collected from the plugin file
-the 'path' field at the end is empty if the plugins default dir is to be used
+The menu name, icon and tip are retrieved from the plugin file
+The 'path' field near the end is empty if the plugins default dir is to be used
 @param set pointer to set data
 
 @return
@@ -1361,44 +1490,42 @@ static void _e2_plugins_tree_defaults (E2_OptionSet *set)
 {
 	e2_option_tree_setup_defaults (set,
 	g_strdup("plugins=<"), //internal name
-	g_strdup ("true|true||||e2p_glob.so|"),
-	g_strdup ("false|false||||e2p_selmatch.so|"),
-	g_strdup ("false|false||||e2p_tag.so|"),
-	g_strdup ("true|true||||e2p_for_each.so|"),
-	g_strdup ("true|true||||e2p_rename.so|"),
-	g_strdup ("true|true||||e2p_find.so|"),
+	g_strdup ("true|true||||e2p_glob.so||"),
+	g_strdup ("false|false||||e2p_selmatch.so||"),
+	g_strdup ("false|false||||e2p_tag.so||"),
+	g_strdup ("true|true||||e2p_for_each.so||"),
+	g_strdup ("true|true||||e2p_rename.so||"),
+	g_strdup ("true|true||||e2p_find.so||"),
 #ifdef E2_TRACKER
-	g_strdup ("false|false||||e2p_track.so|"),
+	g_strdup ("false|false||||e2p_track.so||"),
 #endif
 #ifdef E2_THUMBNAILS
-	g_strdup ("true|true||||e2p_thumbs.so|"),
+	g_strdup ("true|true||||e2p_thumbs.so||"),
 #endif
-	g_strdup ("true|true||||e2p_pack.so|"),
-	g_strdup ("true|false||||e2p_unpack.so|"),
-	g_strdup ("true|true||||e2p_names_clip.so|"),
-	g_strdup ("false|false||||e2p_cpbar.so|"),
-	g_strdup ("false|false||||e2p_mvbar.so|"),
-	g_strdup ("false|false||||e2p_times.so|"),
-	g_strdup ("false|false||||e2p_dircmp.so|"),
+	g_strdup ("true|true||||e2p_pack.so||"),
+	g_strdup ("true|false||||e2p_unpack.so||"),
+	g_strdup ("true|true||||e2p_names_clip.so||"),
+	g_strdup ("false|false||||e2p_cpbar.so||"),
+	g_strdup ("false|false||||e2p_mvbar.so||"),
+	g_strdup ("false|false||||e2p_times.so||"),
+	g_strdup ("false|false||||e2p_dircmp.so||"),
 #ifdef E2_ACL
-	g_strdup ("false|false||||e2p_acl.so|"),
+	g_strdup ("false|false||||e2p_acl.so||"),
 #endif
-	g_strdup ("false|false||||e2p_config.so|"),
+	g_strdup ("false|false||||e2p_config.so||"),
 //#ifdef E2_POLKIT
-//	g_strdup ("false|false||||e2p_privilege.so|"),
+//	g_strdup ("false|false||||e2p_privilege.so||"),
 //#endif
-	g_strdup ("false|false||||e2p_clone.so|"),
-	g_strdup ("false|false||||e2p_view.so|"),
-	g_strdup ("false|false||||e2p_sort_by_ext.so|"),
-	g_strdup ("true|true||||e2p_du.so|"),
+	g_strdup ("false|false||||e2p_clone.so||"),
+	g_strdup ("false|false||||e2p_view.so||"),
+	g_strdup ("false|false||||e2p_sort_by_ext.so||"),
+	g_strdup ("true|true||||e2p_du.so||"),
 #ifdef E2_VFS
-	g_strdup ("false|false||||e2p_vfs.so|"),
-	g_strdup ("false|false||||e2p_gvfs.so|"),
+	g_strdup ("false|false||||e2p_vfs.so||"),
+	g_strdup ("false|false||||e2p_gvfs.so||"),
 #endif
 	g_strdup(">"),
 	NULL);
-	//FIXME setup some mapping between filename and signature name to enable
-	//lookup by signature
 }
 /**
 @brief register plugins actions
@@ -1410,9 +1537,10 @@ void e2_plugins_options_register (void)
 	//no screen rebuilds needed after any change to this option
 	gchar *group_name = _C(34); //_("plugins")
 	E2_OptionSet *set = e2_option_tree_register ("plugins", group_name,
-		group_name, NULL, NULL, NULL,
+		group_name, NULL, _e2_plugin_tree_selection_check_cb, _e2_plugin_tree_draggable_check_cb,
 		E2_OPTION_TREE_LIST | E2_OPTION_TREE_UP_DOWN | E2_OPTION_TREE_ADD_DEL,
 		E2_OPTION_FLAG_ADVANCED | E2_OPTION_FLAG_BUILDPLUGS);
+	//must conform to *.COL enum
 	e2_option_tree_add_column (set, _("Loaded"), E2_OPTION_TREE_TYPE_BOOL, TRUE, "true",
 		0, NULL, NULL);
 	e2_option_tree_add_column (set, _("Menu"), E2_OPTION_TREE_TYPE_BOOL, TRUE, "true",
@@ -1423,10 +1551,19 @@ void e2_plugins_options_register (void)
 		0, NULL, NULL);
 	e2_option_tree_add_column (set, _("Tooltip"), E2_OPTION_TREE_TYPE_STR, 0, "",
 		0, NULL, NULL);
+#ifdef SHOW_PLUGPATHS
 	e2_option_tree_add_column (set, _("Filename"), E2_OPTION_TREE_TYPE_STR, 0, "?.so",
-		0, NULL, NULL);
-	e2_option_tree_add_column (set, _("Path"), E2_OPTION_TREE_TYPE_STR, 0, "",
-		0, NULL, NULL);
+		0, NULL, NULL); //E2_OPTION_TREE_COL_NOT_EDITABLE flag ?
+	e2_option_tree_add_column (set, _("Location"), E2_OPTION_TREE_TYPE_STR, 0, "",
+		0, NULL, NULL); //file-path (for no-child- and parent-plugins)
+#else
+	e2_option_tree_add_column (set, "", E2_OPTION_TREE_TYPE_HIDDENSTR, 0, "",
+		0, NULL, NULL); //E2_OPTION_TREE_COL_NOT_EDITABLE flag ?
+	e2_option_tree_add_column (set, "", E2_OPTION_TREE_TYPE_HIDDENSTR, 0, "",
+		0, NULL, NULL); //file-path (no-child- and parent-plugins)
+#endif
+	e2_option_tree_add_column (set, "", E2_OPTION_TREE_TYPE_HIDDENSTR, 0, "",
+		0, NULL, NULL); //signature
 	e2_option_tree_create_store (set);
 
 	e2_option_tree_prepare_defaults (set, _e2_plugins_tree_defaults);
