@@ -1,6 +1,6 @@
-/* $Id: e2_window.c 2023 2009-12-04 23:29:09Z tpgww $
+/* $Id: e2_window.c 2469 2012-03-16 00:19:24Z tpgww $
 
-Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
+Copyright (C) 2004-2012 tooar <tooar@emelfm2.net>
 Portions copyright (C) 1999 Michael Clark.
 
 This file is part of emelFM2.
@@ -15,8 +15,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 /**
@@ -47,6 +46,8 @@ ToDo - description of how this works
 # include "e2_mousebinding.h"
 #endif
 
+static gboolean _e2_window_unadvise (gpointer data);
+
 extern GList *cols_data;
 extern gint col_width_store[2][MAX_COLUMNS];
 extern gint stored_col_order[2][MAX_COLUMNS];
@@ -54,8 +55,6 @@ extern gint stored_col_order[2][MAX_COLUMNS];
 gint real_width;
 gint real_height;
 
-static gdouble _e2_window_get_pos (GtkWidget *paned);
-
   /*****************/
  /***** utils *****/
 /*****************/
@@ -65,9 +64,6 @@ static gdouble _e2_window_get_pos (GtkWidget *paned);
 
 Depending on whether the file panes are horiz or vert, a vert or horiz paned
 widget is packed inside two boxes
-The 'separator' between the panes is set to approximately its correct position
-(this is before pane content is added, so that un-necessary content mapping is
-minimized)
 
 @param rt ptr to window data struct
 
@@ -75,20 +71,20 @@ minimized)
 */
 static void _e2_window_create_pane_boxes (E2_WindowRuntime *rt)
 {
-	gint base;
+#ifdef USE_GTK3_0
+	rt->panes_horizontal = e2_option_bool_get ("panes-horizontal");
+	GtkOrientation or = (rt->panes_horizontal) ? GTK_ORIENTATION_HORIZONTAL : GTK_ORIENTATION_VERTICAL;
+	rt->panes_outer_box = gtk_box_new (or, 0);
+	or = (rt->panes_horizontal) ? GTK_ORIENTATION_VERTICAL : GTK_ORIENTATION_HORIZONTAL;
+	rt->panes_inner_box = gtk_box_new (or, 0);
+	rt->panes_paned = gtk_paned_new (or);
+#else
 	if (e2_option_bool_get ("panes-horizontal"))
 	{
 		rt->panes_horizontal = TRUE;
 		rt->panes_outer_box = gtk_hbox_new (FALSE, 0);
 		rt->panes_inner_box = gtk_vbox_new (FALSE, 0);
 		rt->panes_paned = gtk_vpaned_new ();
-		base =
-#ifdef USE_GTK2_18
-			app.main_alloc.height
-#else
-			app.main_window->allocation.height
-#endif
-			- 28;  //28 = notional height of commandbar
 	}
 	else
 	{
@@ -96,23 +92,10 @@ static void _e2_window_create_pane_boxes (E2_WindowRuntime *rt)
 		rt->panes_outer_box = gtk_vbox_new (FALSE, 0);
 		rt->panes_inner_box = gtk_hbox_new (FALSE, 0);
 		rt->panes_paned = gtk_hpaned_new ();
-		base =
-#ifdef USE_GTK2_18
-			app.main_alloc.width
-#else
-			app.main_window->allocation.width
-#endif
-			- 5;  //5 = trial'n'error value to minimise separator jitter
 	}
-	if (base < 2)
-		base = 2;
-	//set interim panes sizes, to minimize un-necessary content mapping
-	//real size set again, later
-	gtk_paned_set_position (GTK_PANED (rt->panes_paned),
-		base * rt->panes_paned_ratio);
-
-	gtk_box_pack_start_defaults (GTK_BOX (rt->panes_outer_box), rt->panes_inner_box);
-	gtk_box_pack_start_defaults (GTK_BOX (rt->panes_inner_box), rt->panes_paned);
+#endif
+	gtk_box_pack_start (GTK_BOX (rt->panes_inner_box), rt->panes_paned, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (rt->panes_outer_box), rt->panes_inner_box, TRUE, TRUE, 0);
 }
 /**
 @brief get ratio value for position of paned widget divider
@@ -123,12 +106,19 @@ static void _e2_window_create_pane_boxes (E2_WindowRuntime *rt)
 */
 static gdouble _e2_window_get_pos (GtkWidget *paned)
 {
+#ifdef USE_GTK3_0
+	gint pos, maxpos;
+	g_object_get (G_OBJECT (paned), "position", &pos, "max-position", &maxpos, NULL);
+	if (G_LIKELY (maxpos > 0))
+		return ((gdouble) pos / maxpos);
+#else
 	GtkPaned *_paned = GTK_PANED (paned);
 	gint pos = gtk_paned_get_position (_paned);
-#ifdef USE_GTK2_99
-   ->max_position
+	if (G_LIKELY (_paned->max_position > 0))
+		return ((gdouble) pos / _paned->max_position);
 #endif
-	return ((gdouble) pos / _paned->max_position);
+	else
+		return 1.0;
 }
 /**
 @brief change position of the divider for @a paned between filelists or lists and output
@@ -230,40 +220,51 @@ static gboolean _e2_window_adjust_panes_ratio (GtkPaned *paned,
 	printd (DEBUG, "_e2_window_adjust_panes_ratio (paned:,ratio_last:%f3.2,ratio_new:%f3.2,ratio_log:%f3.2)",
 		*ratio_last, ratio_new, ratio_log);
 
+#ifdef USE_GTK3_0
+	gint pos, maxpos;
+	g_object_get (G_OBJECT (paned), "position", &pos, "max-position", &maxpos, NULL);
+#endif
 	if (p1 != p2 && (ratio_new < -0.0001 || ratio_log < -0.0001))
 	{	// '*' in either or both parts of 2-part arg
 		if (ratio_new >= -0.0001)
 		{
-#ifdef USE_GTK2_99
-   ->max_position
-#endif
+#ifdef USE_GTK3_0
+			res = (gfloat)pos/maxpos;
+#else
 			res = (gfloat)gtk_paned_get_position(paned)/paned->max_position;
+#endif
 			ratio_log = (gfloat)*ratio_last;
 			if (ratio_log < (res - 0.0001) || ratio_log > (res + 0.0001)) //floats !=
 				ratio_new = ratio_log;
 		}
 		else if (ratio_log >= -0.0001)
 		{
-#ifdef USE_GTK2_99
-   ->max_position
-#endif
+#ifdef USE_GTK3_0
+			res = (gfloat)pos/maxpos;
+#else
 			res = (gfloat)gtk_paned_get_position(paned)/paned->max_position;
+#endif
 			if (ratio_log < (res - 0.0001) || ratio_log > (res + 0.0001)) //floats !=
 				ratio_new = ratio_log;
 			ratio_log = (gfloat)*ratio_last;
 		}
 		else // '*' in both parts
 		{
-#ifdef USE_GTK2_99
-   ->max_position
-#endif
+#ifdef USE_GTK3_0
+			ratio_new = ratio_log = maxpos * (gfloat)*ratio_last;
+#else
 			ratio_new = ratio_log = paned->max_position * (gfloat)*ratio_last;
+#endif
 		}
 	}
 
 	if (ratio_new < -0.0001)
 		ratio_new = *ratio_last;
+#ifdef USE_GTK3_0
+	gtk_paned_set_position (paned, (gint)(maxpos * ratio_new));
+#else
 	gtk_paned_set_position (paned, paned->max_position * ratio_new);
+#endif
 	//typically arg has 1 part, and ratio_log == ratio_new
 	if (ratio_log > 0.01 && ratio_log < 0.99)
 		*ratio_last = (gdouble)ratio_log;
@@ -408,17 +409,17 @@ This happens when file panes divider moves (among other instances)
 
 @return
 */
-static void _e2_window_filepanes_adjusted_cb (GtkWidget *widget,
+static void _e2_window_pane1box_allocated_cb (GtkWidget *widget,
 	GtkAllocation *alloc, E2_WindowRuntime *rt)
 {
 	static gint prev_size = -1;
 	if ((rt->panes_horizontal && alloc->height != prev_size)
 	 || (!rt->panes_horizontal && alloc->width != prev_size))
 	{	//this is not just some gtk housekeeping
-		printd (DEBUG, "_e2_window_filepanes_adjusted_cb (widget:,alloc:%d-%d,rt:_)", alloc->width, alloc->height);
+//		printd (DEBUG, "_e2_window_pane1box_allocated_cb (widget:,alloc:x=%d,y=%d,w=%d,h=%d,rt:_)", alloc->x, alloc->y, alloc->width, alloc->height);
 		prev_size = (rt->panes_horizontal) ? alloc->height : alloc->width;
 		rt->panes_paned_ratio = _e2_window_get_pos (rt->panes_paned);
-//		printd (DEBUG, "1ratio: %f", rt->panes_paned_ratio);
+//		printd (DEBUG, "new panes ratio: %f", rt->panes_paned_ratio);
 		if (rt->panes_paned_ratio > 0.001 && rt->panes_paned_ratio < 0.999)
 			rt->panes_paned_ratio_last = rt->panes_paned_ratio;
 	}
@@ -435,6 +436,7 @@ This fixes the toggle button when the panes divider is dragged
 */
 static void _e2_window_map_pane_cb (GtkWidget *widget, gpointer state)
 {
+//	printd (DEBUG, "_e2_window_map_pane_cb (widget:,state:%s)", state ? "true" : "false");
 	E2_ToggleType num = (widget == app.pane1.pane_sw) ?
 		E2_TOGGLE_PANE2FULL : E2_TOGGLE_PANE1FULL;
 	if (e2_toolbar_toggle_button_get_state (toggles_array [num])
@@ -442,25 +444,52 @@ static void _e2_window_map_pane_cb (GtkWidget *widget, gpointer state)
 		e2_toolbar_button_toggle (toggles_array [num]);
 }
 /**
-@brief callback for signal emitted when output pane size is allocated
-This happens when output content change varies the size of @a widget,
-and also when the output-paned divider is moved
+@brief callback for signal emitted when output-pane size is allocated
+This is called when output content change varies the size of @a widget, and also
+when the output-paned divider is moved, either by the user (drag or action) or
+as a result of window resizing
 @param widget UNUSED the resized widget, app.outbook
 @param alloc the allocation for @a widget
 @param rt pointer to window data struct
 
-@return TRUE, always
+@return
 */
-static void _e2_window_outputpane_adjusted_cb (GtkWidget *widget,
+static void _e2_window_panesbox_allocated_cb (GtkWidget *widget,
 	GtkAllocation *alloc, E2_WindowRuntime *rt)
 {
 	static gint prev_height = -1;
+//	printd (DEBUG, "_e2_window_panesbox_allocated_cb (widget:,alloc:x=%d,y=%d,w=%d,h=%d,rt:_)", alloc->x, alloc->y, alloc->width, alloc->height);
 	if (alloc->height != prev_height)
-	{	//this is not just more output being displayed
-		printd (DEBUG, "_e2_window_outputpane_adjusted_cb (widget:,alloc:%d, %d, %d-%d,rt:_)", alloc->x, alloc->y, alloc->width, alloc->height);
+	{
+		if (prev_height > 1 && alloc->height > 1)
+		{	//not [un]hiding the pane
+			//adjust scroll-positions of all tabs, to make it seem that textview
+			//content remains stationary within the displayed window
+			GtkAdjustment *vadj;
+			vadj = gtk_scrolled_window_get_vadjustment
+				(GTK_SCROLLED_WINDOW (app.tab.scroll));
+			gdouble val = gtk_adjustment_get_value (vadj);
+			gtk_adjustment_set_value (vadj, val + prev_height - alloc->height);
+			//TODO other attached tabs at idle time
+			GList *node;
+			for (node = app.tabslist; node != NULL; node = node->next)
+			{
+				E2_OutputTabRuntime *tab = (E2_OutputTabRuntime *)node->data;
+				if (tab != curr_tab
+#ifdef E2_TABS_DETACH
+					&& !tab->detached
+#endif
+				)
+				{
+					vadj = gtk_scrolled_window_get_vadjustment (
+						GTK_SCROLLED_WINDOW (tab->scroll));
+					gtk_adjustment_set_value (vadj, val + prev_height - alloc->height);
+				}
+			}
+		}
 		prev_height = alloc->height;
-		rt->output_paned_ratio = _e2_window_get_pos (app.window.output_paned);
-//		printd (DEBUG, "2ratio: %f", rt->output_paned_ratio);
+		rt->output_paned_ratio = _e2_window_get_pos (rt->output_paned);
+//		printd (DEBUG, "new output ratio: %f", rt->output_paned_ratio);
 		//remember visibility state
 		app.output.visible = (rt->output_paned_ratio != 1.0);
 		if (rt->output_paned_ratio > 0.001 && rt->output_paned_ratio < 0.999)
@@ -469,7 +498,7 @@ static void _e2_window_outputpane_adjusted_cb (GtkWidget *widget,
 		gboolean truenow =
 		e2_toolbar_toggle_button_get_state (toggles_array [E2_TOGGLE_OUTPUTFULL]);
 		if ((!truenow && rt->output_paned_ratio < 0.001)
-		  ||(truenow && rt->output_paned_ratio >= 0.001))
+		 ||(truenow && rt->output_paned_ratio >= 0.001))
 			e2_toolbar_button_toggle (toggles_array [E2_TOGGLE_OUTPUTFULL]);
 	}
 }
@@ -595,17 +624,24 @@ static void _e2_window_show_cb (GtkWidget *window, E2_WindowRuntime *rt)
 	//notice future window-state changes
 	g_signal_connect (G_OBJECT(app.main_window), "window-state-event",
 		G_CALLBACK(_e2_window_change_cb), NULL);
-#ifdef USE_GTK2_99
-   ->max_position
-#endif
+#ifdef USE_GTK3_0
+	gint maxpos;
+	g_object_get (G_OBJECT (rt->panes_paned), "max-position", &maxpos, NULL);
+	gtk_paned_set_position (GTK_PANED (rt->panes_paned),
+		(gint)(maxpos * rt->panes_paned_ratio));
+	g_object_get (G_OBJECT (rt->output_paned), "max-position", &maxpos, NULL);
+	gtk_paned_set_position (GTK_PANED (rt->output_paned),
+		(gint)(maxpos * rt->output_paned_ratio));
+#else
 	gtk_paned_set_position (GTK_PANED (rt->panes_paned),
 		GTK_PANED (rt->panes_paned)->max_position * rt->panes_paned_ratio);
-	gtk_paned_set_position (GTK_PANED (app.window.output_paned),
-		GTK_PANED (app.window.output_paned)->max_position * rt->output_paned_ratio);
+	gtk_paned_set_position (GTK_PANED (rt->output_paned),
+		GTK_PANED (rt->output_paned)->max_position * rt->output_paned_ratio);
+#endif
 	//ensure inter-list divider-position is logged when divider is moved
 	g_signal_connect (G_OBJECT (app.pane1.outer_box), "size-allocate",
-			G_CALLBACK (_e2_window_filepanes_adjusted_cb), rt);
-	//ensure correct toggle button is displayed when a pane divider is dragged
+			G_CALLBACK (_e2_window_pane1box_allocated_cb), rt);
+	//ensure correct toggle button is displayed when pane divider is dragged
 	g_signal_connect (G_OBJECT (app.pane1.pane_sw), "map",
 			G_CALLBACK (_e2_window_map_pane_cb), GINT_TO_POINTER (TRUE));
 	g_signal_connect (G_OBJECT (app.pane1.pane_sw), "unmap",
@@ -615,8 +651,11 @@ static void _e2_window_show_cb (GtkWidget *window, E2_WindowRuntime *rt)
 	g_signal_connect (G_OBJECT (app.pane2.pane_sw), "unmap",
 			G_CALLBACK (_e2_window_map_pane_cb), GINT_TO_POINTER (FALSE));
 	//ensure inter-pane divider-position is logged when divider is moved
-	g_signal_connect (G_OBJECT (app.outbook), "size-allocate",
-			G_CALLBACK (_e2_window_outputpane_adjusted_cb), rt);
+	//("notify::position" signal doesn't work !)
+//	g_signal_connect (G_OBJECT (app.outbook), "size-allocate",
+	g_signal_connect (G_OBJECT (rt->panes_outer_box), "size-allocate",
+			G_CALLBACK (_e2_window_panesbox_allocated_cb), rt);
+	//ensure correct toggle button is displayed when pane divider is dragged
 	g_signal_connect (G_OBJECT (app.outbook), "map",
 			G_CALLBACK (_e2_window_map_output_cb), GINT_TO_POINTER (TRUE));
 	g_signal_connect (G_OBJECT (app.outbook), "unmap",
@@ -643,6 +682,26 @@ static gboolean _e2_window_gone_cb (GtkWidget *widget, GdkEvent *event, gpointer
 	e2_main_closedown (GPOINTER_TO_INT (forced), TRUE, TRUE);
 	return TRUE;
 }
+#ifdef USE_GTK3_0
+/**
+@brief callback for several windows' "key-press-event" & "key-release-event" signals
+
+@param widget the window widget
+@param event pointer to event data
+@param user_data pointer specified when callback was connected
+
+@return FALSE always
+*/
+gboolean e2_window_key_cb (GtkWidget *widget, GdkEventKey *event,
+	gpointer user_data)
+{
+	if (event->is_modifier)
+		e2_utils_save_state (widget);
+	else
+		e2_utils_key_translate_cb (widget, event, user_data);
+	return FALSE;
+}
+#endif
 
   /*******************/
  /***** actions *****/
@@ -650,22 +709,23 @@ static gboolean _e2_window_gone_cb (GtkWidget *widget, GdkEvent *event, gpointer
 /**
 @brief toggle between horizontal and vertical file panes
 
-@param from the button, menu item etc which was activated
-@param art action runtime data
+@param from UNUSED the button, menu item etc which was activated
+@param art UNUSED action runtime data
 
 @return TRUE
 */
 static gboolean _e2_window_toggle_panes_direction
 	(gpointer from, E2_ActionRuntime *art)
 {
+	//FIXME Q this
 	e2_option_bool_toggle ("panes-horizontal");
-	e2_window_recreate (&app.window);	//FIXME Q this
+	e2_window_recreate (&app.window);
 	return TRUE;
 }
 /**
 @brief change the position of the filepanes separator
 
-This is a wrapper for _e2_window_adjust_panes_ratio ()
+This is a wrapper for _e2_window_adjust_panes_ratio()
 Downstream expects BGL to be on/closed
 
 @param from the button, menu item etc which was activated
@@ -737,7 +797,7 @@ static gboolean _e2_window_toggle_full_output (gpointer from, E2_ActionRuntime *
 	!e2_toolbar_button_toggle (toggles_array [E2_TOGGLE_OUTPUTFULL]);
 
 	if (next_state)
-		e2_output_get_bottom_iter (&app.tab);	//set mark for current bottom position
+		e2_output_mark_end (&app.tab);	//set mark for current bottom position
 
 	gboolean retval =
 	_e2_window_adjust_panes_ratio (GTK_PANED (app.window.output_paned),
@@ -747,10 +807,10 @@ static gboolean _e2_window_toggle_full_output (gpointer from, E2_ActionRuntime *
 	{
 		//cancelling full-window reverts to "last" split ratio, never to hidden
 		//if appropriate, adjust scroll position to show the stuff at the bottom of the former view
-		e2_output_scroll_to_bottom (&app.tab);	//get back to the bottom portion
+		e2_output_scroll_to_end (&app.tab);	//get back to the bottom portion
 /*		E2_OutputTabRuntime *trt = &app.tab;
-		GtkAdjustment *vadj = gtk_scrolled_window_get_vadjustment
-			(GTK_SCROLLED_WINDOW (trt->scroll));
+		GtkAdjustment *vadj;
+		vadj = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (trt->scroll));
 		gdouble value = gtk_adjustment_get_value (vadj);
 		if (value >= (vadj->upper - vadj->page_size))
 		{	//we're showing the end of the text
@@ -807,14 +867,14 @@ static gboolean _e2_window_adjust_output_ratio_action
 					((t = strchr(arg,',')) == NULL || t > s));
 	gboolean shade = (star && app.window.output_paned_ratio < 0.01);
 	if (shade)
-		e2_output_get_bottom_iter (&app.tab);	//set mark for bottom position
+		e2_output_mark_end (&app.tab);	//set mark at end-position
 
 	gboolean retval =
 	_e2_window_adjust_panes_ratio (GTK_PANED (app.window.output_paned),
 		&app.window.output_paned_ratio_last, arg);
 
 	if (retval && shade)
-		e2_output_scroll_to_bottom (&app.tab);	//get back to the bottom portion of the former view
+		e2_output_scroll_to_end (&app.tab);	//get back to the bottom portion of the former view
 
 	return retval;
 }
@@ -976,7 +1036,11 @@ void e2_window_set_cursor (GdkCursorType type)
 		app.main_window->window,
 #endif
 		cursor);
+#ifdef USE_GTK3_0
+	g_object_unref (G_OBJECT (cursor));
+#else
 	gdk_cursor_unref (cursor);
+#endif
 	gdk_flush ();
 }
 /* *
@@ -1016,7 +1080,7 @@ gboolean e2_window_update_status_bar (gpointer userdata)
 	//CHECKME make this happen after specific events (refresh, refilter, cd, swap panes)
 	//not on a timer
 #ifdef E2_STATUS_BLOCK
-	if (app.status_working)
+	if (g_atomic_int_get (&app.status_working))
 	{
 		printd (DEBUG, "update status bar BLOCKED");
 		return TRUE;
@@ -1024,7 +1088,7 @@ gboolean e2_window_update_status_bar (gpointer userdata)
 #endif
 //	printd (DEBUG, "e2_window_update_status_bar");
 #ifdef E2_STATUS_BLOCK
-	app.status_working = TRUE;
+	g_atomic_int_set (&app.status_working, 1);
 #endif
 	gint selected_rows, total_rows;
 	GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW (curr_view->treeview));
@@ -1077,7 +1141,7 @@ gboolean e2_window_update_status_bar (gpointer userdata)
 		g_free (status_text2);
 	}
 #ifdef E2_STATUS_BLOCK
-	app.status_working = FALSE;
+	g_atomic_int_set (&app.status_working, 0);
 #endif
 //	printd (DEBUG, "e2_window_update_status_bar ends");
 	return TRUE;  //never turn this off
@@ -1106,7 +1170,7 @@ void e2_window_enable_status_update (gint interval)
 	{
 #endif
 #ifdef E2_STATUS_BLOCK
-		app.status_working = FALSE;
+		g_atomic_int_set (&app.status_working, 0);
 #endif
 		app.timers[STATUS_T] = g_timeout_add (interval, e2_window_update_status_bar, NULL);
 #ifdef E2_STATUS_REF
@@ -1139,28 +1203,6 @@ void e2_window_disable_status_update (void)
 }
 #endif	//ndef E2_STATUS_DEMAND
 /* *
-@brief display status bar custom message
-
-The specified string is added to the end of the hbox, but only displayed
-if it is the first message in the box
-This allows multiple messages to be queued
-FIXME the messages may be cancelled in any order if threads are in force ...
-
-@param message string to be displayed, may have pango markup
-
-@return
-*/
-/* UNUSED
-void e2_window_show_status_message (gchar *message)
-{
-	GtkWidget *label = gtk_label_new (NULL);
-	gtk_label_set_markup (GTK_LABEL (label), message);
-	gtk_box_pack_start (GTK_BOX (app.status_bar_box3), label, FALSE, FALSE, 0);
-	GList *children = GTK_BOX (app.status_bar_box3)->children;
-	if (g_list_length (children) == 1)
-		gtk_widget_show_all (app.status_bar_box3);
-} */
-/* *
 @brief cancel display of status bar custom message
 
 @return
@@ -1184,33 +1226,63 @@ void e2_window_remove_status_message (void)
 		gtk_container_remove (GTK_CONTAINER (app.status_bar_box3), label);
 	}
 } */
+#ifdef USE_GTK2_20
+static GtkWidget *mover = NULL;
+#endif
 /**
 @brief show statusline message @a labeltext
-Any existing message is simply replaced
+Any existing message is simply replaced.
 Gtk's BGL is expected to be open/off
-@param message utf-8 string, which may include pango markup
-
+@param message UTF-8 string, which may include pango markup
+#ifdef USE_GTK2_20
+@param with_spinner TRUE to append a GtkSpinner to the displayed message
+#endif
 @return
 */
-void e2_window_show_status_message (gchar *message)
+void e2_window_show_status_message (const gchar *message
+#ifdef USE_GTK2_20
+	, gboolean with_spinner
+#endif
+)
 {
+	printd (DEBUG, "In e2_window_show_status_message()");
+	//abort idle(s) which may remove the message being added
+	while (g_idle_remove_by_data ((gpointer) _e2_window_unadvise))
+		printd (DEBUG, "Killed status idle");
+	gdk_threads_enter ();
+	//kill the current label FIXME make this a more-friendly Q
 	GList *children = gtk_container_get_children (GTK_CONTAINER (app.status_bar_box3));
 	if (children != NULL)
 	{
-		gdk_threads_enter ();	//maybe the message is already displayed
-		if (GTK_IS_WIDGET (children->data))
-			gtk_widget_destroy (children->data);	//FIXME make this a friendly Q
-		gdk_threads_leave ();
+		if (GTK_IS_LABEL (children->data))
+			gtk_widget_destroy (children->data);
 		g_list_free (children);
 	}
-//	GtkWidget *label = gtk_label_new (message);
+//	gdk_threads_leave ();
 	GtkWidget *label = gtk_label_new (NULL);
 	gchar *public = g_markup_escape_text (message, -1);
 	gtk_label_set_markup (GTK_LABEL (label), public);
 	g_free (public);
-	gdk_threads_enter ();
-//	gtk_box_pack_start (GTK_BOX (app.status_bar_box3), label, FALSE, FALSE, 0);
+//	gdk_threads_enter ();
 	gtk_container_add (GTK_CONTAINER (app.status_bar_box3), label);
+#ifdef USE_GTK2_20
+	if (with_spinner)
+	{
+		if (mover == NULL)
+		{
+			mover = gtk_spinner_new ();
+			gtk_box_pack_start (GTK_BOX(app.status_bar_box3), mover, TRUE, TRUE, E2_PADDING);
+			gtk_widget_show (mover);
+			gtk_spinner_start ((GtkSpinner*) mover);
+		}
+	}
+	else	//no spinner from now
+		if (mover != NULL)
+	{
+		gtk_widget_destroy (mover);
+		mover = NULL;
+	}
+#endif
 	gtk_widget_show_all (app.status_bar_box3);
 	gdk_threads_leave ();
 }
@@ -1221,19 +1293,25 @@ void e2_window_show_status_message (gchar *message)
 */
 static gboolean _e2_window_unadvise (gpointer data)
 {
+	printd (DEBUG, "In _e2_window_unadvise()");
 	gdk_threads_enter ();
 	gtk_widget_hide (app.status_bar_box3);
-	gdk_threads_leave ();
-//	GList *children = GTK_BOX (app.status_bar_box3)->children;
-//	GtkWidget *label = ((GtkBoxChild *) children->data)->widget;
-//	gtk_container_remove (GTK_CONTAINER (app.status_bar_box3), label);
+
 	GList *children = gtk_container_get_children (GTK_CONTAINER (app.status_bar_box3));
 	if (children != NULL)
 	{
-		if (GTK_IS_WIDGET (children->data))
-			gtk_widget_destroy (children->data);	//FIXME make this a more-friendly Q
+		GList *member;
+		for (member = children; member != NULL; member = member->next)
+		{
+			if (GTK_IS_WIDGET (member->data))
+				gtk_widget_destroy (member->data);	//FIXME make this a more-friendly Q
+		}
 		g_list_free (children);
 	}
+#ifdef USE_GTK2_20
+	mover = NULL;
+#endif
+	gdk_threads_leave ();
 	return FALSE;
 }
 /**
@@ -1243,7 +1321,8 @@ In case this change is done at a busy time for X/gdk, it's done asynchronously
 */
 void e2_window_clear_status_message (void)
 {
-	g_idle_add ((GSourceFunc) _e2_window_unadvise, NULL);
+	//data pointer may be used for killing the idle
+	g_idle_add ((GSourceFunc) _e2_window_unadvise, (gpointer) _e2_window_unadvise);
 }
 /**
 @brief create and show main window
@@ -1255,13 +1334,14 @@ This is called only at session-start, from main ()
 */
 void e2_window_create (E2_WindowRuntime *rt)
 {
+	printd (DEBUG, "create main window");
 	_e2_window_set_icon ();
 
 	//setup main window
 	app.main_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-	gtk_window_set_role (GTK_WINDOW (app.main_window), "main");
 	gtk_window_set_title (GTK_WINDOW (app.main_window), PROGNAME);
-	gtk_window_set_wmclass (GTK_WINDOW (app.main_window), "main", BINNAME);
+//	gtk_window_set_role (GTK_WINDOW (app.main_window), "main");
+//	gtk_window_set_wmclass (GTK_WINDOW (app.main_window), "main", BINNAME);
 	gtk_widget_set_size_request (app.main_window, 0, 0);	//window as small as possible
 	//default size 640 x 480
 	//FIXME these are wrong when window is maximised
@@ -1282,9 +1362,18 @@ void e2_window_create (E2_WindowRuntime *rt)
 #ifdef E2_COMPOSIT
 	e2_window_set_opacity (app.main_window, -1);
 #endif
+#ifdef USE_GTK3_0
+	g_signal_connect (G_OBJECT (app.main_window), "key-press-event",
+		G_CALLBACK (e2_window_key_cb), GUINT_TO_POINTER(1));	//includes translation, non-NULL data to avoid disconnection during bindings setup
+	g_signal_connect (G_OBJECT (app.main_window), "key-release-event",
+		G_CALLBACK (e2_window_key_cb), NULL); //includes translation
+#else
 	//arrange for translation of mod-alphabetic-keycodes from locale to ascii
 	g_signal_connect (G_OBJECT (app.main_window), "key-press-event",
 		G_CALLBACK (e2_utils_key_translate_cb), GUINT_TO_POINTER(1));	//non-NULL data to avoid disconnection during bindings setup
+	g_signal_connect (G_OBJECT (app.main_window), "key-release-event",
+		G_CALLBACK (e2_utils_key_translate_cb), NULL);
+#endif
 /*	//create top-level bindings
 	e2_keybinding_enrol (app.main_window, _C(17), (void(*)(E2_OptionSet*))NULL);
 	//(must be done before file- or output-pane contents are created)
@@ -1311,7 +1400,7 @@ void e2_window_create (E2_WindowRuntime *rt)
 		G_CALLBACK (system_shutdown), NULL); */
 	//trap signals that might (but probably don't) get issued when a
 	//session-manager initiates a shutdown
-//	g_signal_connect (GTK_OBJECT (app.main_window), "destroy",
+//	g_signal_connect (GTK_OBJECT (app.main_window), "destroy", gtk2 only
 //		G_CALLBACK (_e2_window_destroy_cb), NULL);
 	g_signal_connect (G_OBJECT (app.main_window), "destroy-event",
 		G_CALLBACK (_e2_window_gone_cb), (gpointer)TRUE);
@@ -1319,7 +1408,7 @@ void e2_window_create (E2_WindowRuntime *rt)
 	app.hbox_main = gtk_hbox_new (FALSE, 0);
 	app.vbox_main = gtk_vbox_new (FALSE, 0);
 	gtk_container_add (GTK_CONTAINER (app.main_window), app.hbox_main);
-	gtk_box_pack_start_defaults (GTK_BOX (app.hbox_main), app.vbox_main);
+	gtk_box_pack_start (GTK_BOX (app.hbox_main), app.vbox_main, TRUE, TRUE, 0);
 
 #ifdef E2_RAINBOW
 	e2_option_color_filetypes_sync ();
@@ -1328,8 +1417,7 @@ void e2_window_create (E2_WindowRuntime *rt)
 	//file panes infrastructure
 	e2_cache_double_register ("file-pane-ratio-last", &rt->panes_paned_ratio_last, 0.55);
 	e2_cache_double_register ("file-pane-ratio", &rt->panes_paned_ratio, 0.55);
-	_e2_window_create_pane_boxes (rt);	//setup paned widget, with approximately 'correct' ratio
-
+	_e2_window_create_pane_boxes (rt);	//setup outer boxes and paned widget for filelists
 	//output pane (which can be completed relatively quickly)
 	e2_cache_double_register ("output-pane-ratio-last", &rt->output_paned_ratio_last, 0.85);
 	e2_cache_double_register ("output-pane-ratio", &rt->output_paned_ratio, 0.85);
@@ -1338,14 +1426,27 @@ void e2_window_create (E2_WindowRuntime *rt)
 	//set visibility state
 	app.output.visible = (rt->output_paned_ratio != 1.0);
 	rt->output_paned = gtk_vpaned_new ();
-#ifdef USE_GTK2_18
+	//CHECKME why block focus here?
+/*#ifdef USE_GTK2_18
 	gtk_widget_set_can_focus (rt->output_paned, FALSE);
 #else
 	GTK_WIDGET_UNSET_FLAGS (rt->output_paned, GTK_CAN_FOCUS);
 #endif
+*/
 	gtk_paned_pack1 (GTK_PANED (rt->output_paned), rt->panes_outer_box, TRUE, TRUE);
 	gtk_paned_pack2 (GTK_PANED (rt->output_paned), wid, TRUE, TRUE);
+	gtk_box_pack_start (GTK_BOX (app.vbox_main), rt->output_paned, TRUE, TRUE, 0);
 
+	//before pane content is added, set separators between panes to approximately
+	//their correct position (so that un-necessary content mapping is minimized).
+ 	//Final positions set later, in _e2_window_show_cb()
+	gtk_paned_set_position (GTK_PANED (rt->panes_paned),
+#ifdef USE_GTK2_18
+		app.main_alloc.width
+#else
+		app.main_window->allocation.width
+#endif
+		* rt->panes_paned_ratio);
 	gtk_paned_set_position (GTK_PANED (rt->output_paned),
 #ifdef USE_GTK2_18
 		app.main_alloc.height
@@ -1354,9 +1455,7 @@ void e2_window_create (E2_WindowRuntime *rt)
 #endif
 		* rt->output_paned_ratio);
 
-	gtk_box_pack_start (GTK_BOX (app.vbox_main), rt->output_paned, TRUE, TRUE, 0);
-
-	gtk_widget_show_all (app.hbox_main); //before panes (with toggle-buttons)
+	gtk_widget_show_all (app.hbox_main); //all, before panes (with toggle-buttons)
 
 	//do this registration here, rather that in the pane context,
 	//as it covers both panes, and to avoid attempted re-registers
@@ -1503,9 +1602,12 @@ void e2_window_create (E2_WindowRuntime *rt)
 	e2_pane_flag_active ();
 	//setup correct focus
 	gtk_widget_grab_focus (curr_view->treeview);
+
+//	gdk_threads_enter (); //CHECKME needeed for tooltips etc?
 	//other toolbars
 	e2_toolbar_create (&app.toolbar);
 	e2_toolbar_create (&app.commandbar);
+//	gdk_threads_leave ();
 
 	if (e2_option_bool_get_direct (app.commandbar.show)) //FIXME button could be on any bar
 	{
@@ -1593,12 +1695,12 @@ void e2_window_recreate (E2_WindowRuntime *rt)
 	while (TRUE)
 	{
 		gboolean busy;
-		LISTS_LOCK
-		busy = curr_view->listcontrols.cd_working
-			|| curr_view->listcontrols.refresh_working
-			|| other_view->listcontrols.cd_working
-			|| other_view->listcontrols.refresh_working;
-		LISTS_UNLOCK
+//		LISTS_LOCK
+		busy = g_atomic_int_get (&curr_view->listcontrols.cd_working)
+			|| g_atomic_int_get (&curr_view->listcontrols.refresh_working)
+			|| g_atomic_int_get (&other_view->listcontrols.cd_working)
+			|| g_atomic_int_get (&other_view->listcontrols.refresh_working);
+//		LISTS_UNLOCK
 		if (busy)
 			usleep (50000);
 		else
@@ -1695,7 +1797,7 @@ void e2_window_recreate (E2_WindowRuntime *rt)
 #ifdef USE_GTK2_18
 	gtk_widget_get_allocation (app.main_window, &app.main_alloc);
 #endif
-	_e2_window_create_pane_boxes (rt);
+	_e2_window_create_pane_boxes (rt);//setup outer boxes and paned widget for filelists
 	gtk_widget_show_all (rt->panes_outer_box); //before bars with toggles
 
 	e2_pane_create_part (&app.pane1);
@@ -1785,16 +1887,23 @@ void e2_window_recreate (E2_WindowRuntime *rt)
 #endif
 	//set output visibility flag
 	app.output.visible = (rt->output_paned_ratio != 1.0);
-	GtkCornerType where = (GtkCornerType) e2_option_int_get ("scrollbar-position");
+	GtkCornerType where = (GtkCornerType) e2_option_sel_get ("scrollbar-position");
 	//update each bar position
 	GList *member;
 	for (member = app.tabslist; member != NULL; member = member->next)
 		gtk_scrolled_window_set_placement (
 		GTK_SCROLLED_WINDOW (((E2_OutputTabRuntime *)member->data)->scroll), where);
+#ifdef ADJACENT_TABS
+	if (where == GTK_CORNER_TOP_LEFT || where == GTK_CORNER_BOTTOM_LEFT)
+		gtk_notebook_set_tab_pos (GTK_NOTEBOOK (app.outbook), GTK_POS_RIGHT);
+	else
+		gtk_notebook_set_tab_pos (GTK_NOTEBOOK (app.outbook), GTK_POS_LEFT);
+#else
 	if (where == GTK_CORNER_TOP_LEFT || where == GTK_CORNER_BOTTOM_LEFT)
 		gtk_notebook_set_tab_pos (GTK_NOTEBOOK (app.outbook), GTK_POS_LEFT);
 	else
 		gtk_notebook_set_tab_pos (GTK_NOTEBOOK (app.outbook), GTK_POS_RIGHT);
+#endif
 	e2_output_update_style ();
 	e2_filelist_enable_refresh ();
 	gtk_widget_grab_focus (curr_view->treeview);
