@@ -1,6 +1,6 @@
-/* $Id: emelfm2.h 1315 2008-10-07 03:19:03Z tpgww $
+/* $Id: emelfm2.h 2484 2012-04-06 10:07:40Z tpgww $
 
-Copyright (C) 2003-2008 tooar <tooar@gmx.net>
+Copyright (C) 2003-2012 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -14,8 +14,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 #ifndef __EMELFM2_H__
@@ -35,35 +34,19 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 //some general things ...
 #include <stdio.h>
 #include <stdlib.h>
+#include <pthread.h>
 
 //#define GTK_DISABLE_DEPRECATED see Makefile
 #include <gtk/gtk.h>
-#include <gdk/gdkkeysyms.h>
-#include <gmodule.h>
-
-#define GETTEXT_PACKAGE BINNAME
-#include <glib/gi18n-lib.h>
 
 //#define E2_ASSISTED see build.h
 #ifdef E2_ASSISTED
 # include <atk/atk.h>
 #endif
 
-#ifndef PRIu64
-//for systems that don't define PRI??? macros
-# if defined(__WORDSIZE) && __WORDSIZE == 64
-#  define PRIu64 "lu"
-# else
-//guess it's a 32-bit platform
-#  define PRIu64 "llu"
-# endif
-#endif
-
-//include code valid only for gtk versions > 2.6, according to the
-//version in use when compiling
-//#define E2_CURRENTLIBS see Makefile and build.h
-//mechanism for using glib/gtk facilities > min standard (2.6.0)
 #ifdef E2_CURRENTLIBS
+//include extra/different code valid only for glib/gtk versions > minimum standard
+//(2.6 or 3.0), according to the version in use when compiling
 # if GLIB_CHECK_VERSION (2,8,0)
 #  define USE_GLIB2_8
 # endif
@@ -79,6 +62,31 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 # if GLIB_CHECK_VERSION (2,16,0)
 #  define USE_GLIB2_16
 # endif
+# if GLIB_CHECK_VERSION (2,18,0)
+#  define USE_GLIB2_18
+# endif
+# if GLIB_CHECK_VERSION (2,20,0)
+#  define USE_GLIB2_20
+# endif
+# if GLIB_CHECK_VERSION (2,22,0)
+#  define USE_GLIB2_22
+# endif
+# if GLIB_CHECK_VERSION (2,24,0)
+#  define USE_GLIB2_24
+# endif
+# if GLIB_CHECK_VERSION (2,26,0)
+#  define USE_GLIB2_26
+# endif
+# if GLIB_CHECK_VERSION (2,28,0)
+#  define USE_GLIB2_28
+# endif
+# if GLIB_CHECK_VERSION (2,30,0)
+#  define USE_GLIB2_30
+# endif
+# if GLIB_CHECK_VERSION (2,32,0)
+#  define USE_GLIB2_32
+# endif
+
 # if GTK_CHECK_VERSION (2,8,0)
 #  define USE_GTK2_8
 # endif
@@ -91,14 +99,81 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 # if GTK_CHECK_VERSION (2,14,0)
 #  define USE_GTK2_14
 # endif
+# if GTK_CHECK_VERSION (2,16,0)
+#  define USE_GTK2_16
+# endif
+# if GTK_CHECK_VERSION (2,18,0)
+#  define USE_GTK2_18
+# endif
+# if GTK_CHECK_VERSION (2,20,0)
+#  define USE_GTK2_20
+# endif
+# if GTK_CHECK_VERSION (2,22,0)
+#  define USE_GTK2_22
+# endif
+# if GTK_CHECK_VERSION (2,24,0)
+#  define USE_GTK2_24
+# endif
+# if GTK_CHECK_VERSION (3,0,0)
+#  define USE_GTK3_0
+# endif
+# if GTK_CHECK_VERSION (3,2,0)
+#  define USE_GTK3_2
+# endif
+# if GTK_CHECK_VERSION (3,4,0)
+#  define USE_GTK3_4
+# endif
+# if GTK_CHECK_VERSION (3,6,0)
+#  define USE_GTK3_6
+# endif
+
+#elif defined(E2_MIN_GTK3)
+//GTK3 is available and either: explicitly requested or GTK2 is not available
+//include extra/different code valid only for gtk 3.0 and corresponding glib 2.28
+# define USE_GLIB2_8
+# define USE_GLIB2_10
+# define USE_GLIB2_12
+# define USE_GLIB2_14
+# define USE_GLIB2_16
+# define USE_GLIB2_18
+# define USE_GLIB2_20
+# define USE_GLIB2_22
+# define USE_GLIB2_24
+# define USE_GLIB2_26
+# define USE_GLIB2_28
+# define USE_GTK2_8
+# define USE_GTK2_10
+# define USE_GTK2_12
+# define USE_GTK2_14
+# define USE_GTK2_16
+# define USE_GTK2_18
+# define USE_GTK2_20
+# define USE_GTK2_22
+# define USE_GTK2_24
+# define USE_GTK3_0
+#endif
+
+#define GETTEXT_PACKAGE BINNAME //before gi18n-lib.h
+
+#ifdef USE_GLIB2_32
+# include <glib/gi18n.h>
+#else
+# include <gmodule.h>
+# include <glib/gi18n-lib.h>
 #endif
-//some things which didn't make it into 2.14 have been re-badged as USE_GTK2_99 CHECK LATER
 
 #ifdef USE_GTK2_12
 //this tag governs use of gtk 2.12 tooltips
-#define USE_GTK2_12TIPS
+# define USE_GTK2_12TIPS
 //this tag governs use of gtk 2.12 notebook-tab DnD
-#define USE_GTK2_12DND
+# define USE_GTK2_12DND
+#endif
+
+#ifdef USE_GTK3_0
+# include <gdk/gdkkeysyms-compat.h>
+# define GTK3_COMBO_FIX //include workarounds for gtk3.0.x bad behaviour
+#else
+# include <gdk/gdkkeysyms.h>
 #endif
 
 //longest item name (bytes, NOT chars) is a *NIX define
@@ -149,30 +224,47 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 //tag for refcounting enable/disable status-bar updates
 //#define E2_STATUS_REF
 //allow checking for status-in progess when updating filelists
-//#define E2_STATUS_BLOCK
+#define E2_STATUS_BLOCK
 #endif
 
 //include error-message code regardless of debug state
 //#define DEBUG_MESSAGES_ALWAYS
 
-//setup for kernel-based file monitoring, as opposed to fam/gamin
-//see Makefile & build.h for definition of these variables
-#if defined(E2_FAM_INOTIFY) || defined(E2_FAM_DNOTIFY) || defined(E2_FAM_KQUEUE) || defined(E2_FAM_PORTEVENT)
-# define E2_FAM_KERNEL
-//this is needed too
-# define E2_FAM
+#if defined(DEBUG_MESSAGES) || defined(DEBUG_MESSAGES_ALWAYS)
+# define printall(str, args...)
 #else
-//include code for file-change detection using FAM or gamin
-//#define E2_FAM see Makefile & build.h
-# ifdef E2_FAM
-#  include <fam.h>
+# define printall(str, args...) printf("e2: "str"\n", ## args)
+#endif
+
+#ifndef PRIu64
+//for systems that don't define PRI??? macros
+# if defined(__WORDSIZE) && __WORDSIZE == 64
+#  define PRIu64 "lu"
+# else
+//guess it's a 32-bit platform
+#  define PRIu64 "llu"
 # endif
 #endif
-#ifdef E2_FAM
+
+//output-pane tabs on the same side as scrollbars
+//#define ADJACENT_TABS
+
+//setup for kernel-based file monitoring, as opposed to fam/gamin
+//see Makefile & build.h for definition of these variables
+#ifdef E2_GAMIN
+//include code for file-change detection using gamin
+# include <fam.h>
+#endif
+#if defined(E2_FAM_INOTIFY) || defined(E2_FAM_KQUEUE) || defined(E2_FAM_PORTEVENT)
+# define E2_FAM_KERNEL
+#endif
+#if defined(E2_GAMIN) || defined(E2_FAM_KERNEL)
+//this is needed too, ATM
+# define E2_FAM
 //usec interval after file events before FAM is polled
 //needs to be long enough for most/all changes to be noticed
 //by the monitor, but not so long that the user perceives a delay
-#define E2_FAMWAIT 100000
+# define E2_FAMWAIT 100000
 #endif
 
 //include code for making incremental changes to
@@ -184,7 +276,7 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #define E2_UNBLOCK
 
 //include debug messages about refreshing
-//#define E2_REFRESH_DEBUG
+#define E2_REFRESH_DEBUG
 //include debug messages about mkdir dialog shutdown
 //#define RACE_CHECK
 
@@ -239,6 +331,10 @@ typedef struct _E2_ColorData
 //#define VOLATILE volatile
 #define VOLATILE
 
+//interval between last-detected 'dirty' and suspension of refresh-poll timers
+//20 mins = 1200 seconds
+#define QUIET_SECONDS 1200
+
 //include code for handling [%]%e [%]%E macros
 #define E2_BADQUOTES
 
@@ -299,6 +395,13 @@ typedef struct _E2_ColorData
 # endif
 #endif
 
+//use libmagic instead of file(1)
+#define E2_MAGIC
+#ifdef E2_MAGIC
+//for use with dlopen()
+# define MAGIC_LIB_NAME "libmagic.so.1"
+#endif
+
 //enable filelist liststore copying instead of getting fresh data from source
 #ifdef E2_VFS
 #define STORECOPY
@@ -320,11 +423,12 @@ typedef struct _E2_ColorData
 # define E2_ERR_MSGL(a) a, __sys_err->message
 # define E2_ERR_IS(num) (__sys_err != NULL && __sys_err->code == num)
 # define E2_ERR_ISNOT(num) (__sys_err == NULL || __sys_err->code != num)
+# define E2_ERR_CLEAR if (__sys_err != NULL) { g_error_free (__sys_err); __sys_err = NULL; }
 //these are for use in a func with GError ** argument
 # define E2_ERR_PIS(num) (__sys_err != NULL && (*__sys_err)->code == num)
 # define E2_ERR_PISNOT(num) (__sys_err == NULL || (*__sys_err)->code != num)
 # define E2_ERR_BACKUP(a) GError *a = NULL;if (__sys_err == NULL) __sys_err = &a;
-# define E2_ERR_CLEAR if (__sys_err != NULL) { g_error_free (__sys_err); __sys_err = NULL; }
+# define E2_ERR_PCLEAR if (__sys_err != NULL && *__sys_err != NULL) { g_error_free (*__sys_err); *__sys_err = NULL; }
 #else
 # define E2_ERR_NAME
 # define E2_ERR_INIT
@@ -338,9 +442,10 @@ typedef struct _E2_ColorData
 # define E2_ERR_MSGL(a)
 # define E2_ERR_IS(num) (errno==num)
 # define E2_ERR_ISNOT(num) (errno!=num)
+# define E2_ERR_CLEAR
 # define E2_ERR_PIS(num) (errno==num)
 # define E2_ERR_PISNOT(num) (errno!=num)
-# define E2_ERR_CLEAR
+# define E2_ERR_PCLEAR
 # define E2_ERR_BACKUP(a)
 #endif
 
@@ -355,6 +460,20 @@ typedef struct _E2_ColorData
 # define VPSTR(p) (gchar*)p
 #endif
 
+#ifdef E2_VFS
+# define CURRDIR curr_view->dir.path
+# define OTHRDIR other_view->dir.path
+# define VIEWDIR view->dir.path
+#else
+# define CURRDIR curr_view->dir
+# define OTHRDIR other_view->dir
+# define VIEWDIR view->dir
+#endif
+
+#define ActivateFunc void(*)(GtkEntry*,gpointer)
+#define ResponseFunc void(*)(GtkDialog*,gint,gpointer)
+#define HookFunc gboolean(*)(gpointer,gpointer)
+
 //type of "dir" in a pane
 typedef enum
 {
@@ -407,10 +526,9 @@ typedef struct _E2_PlaceInfo
 //data for vfs operations uning path-argument(s)
 typedef struct _E2_Vpath
 {
-	const gchar *localpath; /*first, so that vpath* == gchar **
+	const gchar *path; /*first in the struct, so that vpath* == gchar **
 						typically an absolute path string
-						encoding may actually be utf-8 if a handler lib
-						requires that */
+						encoding may be localised or UTF-8 */
 	PlaceInfo *spacedata;
 //	GError *err; //replacement for error args in fs functions
 } vpath;
@@ -478,41 +596,45 @@ typedef struct _E2_Vpath
        _res; })
 #endif
 
-//do not implement slices everywhere until session-end cleanups are coded
-//#ifdef USE_GLIB2_10
-#if 0
-# define ALLOCATE0(type) (type *) g_slice_alloc0 (sizeof (type));
-# define ALLOCATE(type) (type *) g_slice_alloc (sizeof (type));
-# define DEALLOCATE(type,ptr) g_slice_free1 (sizeof (type), ptr);
+//do not implement slices everywhere until comprehensive session-end cleanups are done
+#if 0 //def USE_GLIB2_10
+# define ALLOCATE(type) (type*)g_slice_alloc(sizeof(type));
+# define ALLOCATE0(type) (type*) g_slice_alloc0(sizeof(type));
+# define DEALLOCATE(type,ptr) g_slice_free1(sizeof(type),ptr);
+#elif defined (USE_GLIB2_8)
+# define ALLOCATE(type) (type*)g_try_malloc(sizeof(type));
+# define ALLOCATE0(type) (type*)g_try_malloc0(sizeof(type));
+# define DEALLOCATE(type,ptr) g_free(ptr);
 #else
-# ifdef USE_GLIB2_8
-#  define ALLOCATE0(type) (type *) g_try_malloc0 (sizeof (type));
-# else
-//this really should be free'd not g_free'd
-#  define ALLOCATE0(type) (type *) calloc (1, sizeof (type));
-# endif
-# define ALLOCATE(type) (type *) g_try_malloc (sizeof (type));
-# define DEALLOCATE(type,ptr) g_free (ptr);
+# define ALLOCATE(type) (type*)malloc(sizeof(type));
+# define ALLOCATE0(type) (type*)calloc(1,sizeof(type));
+# define DEALLOCATE(type,ptr) free(ptr);
 #endif
 //for ALLOCATE's that are too small for a slice
 #ifdef USE_GLIB2_8
-# define MALLOCATE0(type) (type *) g_try_malloc0 (sizeof (type));
+# define MALLOCATE(type) (type*)g_try_malloc(sizeof(type));
+# define MALLOCATE0(type) (type*)g_try_malloc0(sizeof(type));
+# define DEMALLOCATE(type,ptr) g_free(ptr);
 #else
-# define MALLOCATE0(type) (type *) calloc (1, sizeof (type));
+# define MALLOCATE(type) (type*)malloc(sizeof(type));
+# define MALLOCATE0(type) (type*)calloc(1,sizeof(type));
+# define DEMALLOCATE(type,ptr) free(ptr);
 #endif
-#define MALLOCATE(type) (type *) g_try_malloc (sizeof (type));
-#define DEMALLOCATE(type,ptr) g_free (ptr);
 
 #ifdef USE_GLIB2_8
-# define NEW(type,n) g_try_new (type, n)
-# define NEW0(type,n) g_try_new0 (type, n)
+# define NEW(type,n) g_try_new(type,n)
+# define NEW0(type,n) g_try_new0(type,n)
 # define FREENEW g_free
 #else
-# define NEW(type,n) (type*)malloc (sizeof(type)*n)
-# define NEW0(type,n) (type*)calloc (n, sizeof(type))
+# define NEW(type,n) (type*)malloc(sizeof(type)*n)
+# define NEW0(type,n) (type*)calloc(n,sizeof(type))
 # define FREENEW free
 #endif
 
+#ifdef USE_GLIB2_30
+# define g_atomic_int_exchange_and_add g_atomic_int_add
+#endif
+
 //survive or exit gracefully when allocation fails
 //can implement this now, if so desired
 //#define CHECKALLOCATEDFATAL(p) if (p==NULL) e2_utils_memory_error ();
@@ -526,15 +648,15 @@ typedef struct _E2_Vpath
 
 //index enumerator for cancellable timers
 //CHECKME some other repeating, non-trivial-delay, timers are non-static
+//NOTE also need to kill any [un]mount timers for devices(s) in hal/devkit
 enum
 {
-	FAMPOLL_T,	//polling for reports from file alteration monitor
 	DIRTYCHECK_T,	//for polling whether filelists are in need of refresh
-	REFRESHBEGIN_T,	//for waiting to start refreshing filelist(s)
-	STATUS_T,	//for status-line updates
-	CONFIG_T,	//for config-file change checking
+	CONFIG_T,	//for config-file change polling
+	REFRESHBEGIN_T,	//for ensuring open BGL when calling _e2_filelist_refresh_manage()
 	REFRESHWAIT_T,	//for pausing until a prior refresh is completed
 	CDWAIT_T,	//for pausing until a prior cd is completed
+	STATUS_T,	//for status-line updates
 	ASCROLL_T,	//DnD auto-scroll timer
 	MAX_TIMERS
 };
@@ -548,10 +670,18 @@ typedef enum
 } E2_FAMonitor;
 #endif
 
+//define to use local polling instead of full-blown main loop for blocking dialogs
+#define WAIT_POLL
 typedef struct _E2_MainLoop
 {
-	pthread_t threadID;	//which thread started this loop
+	pthread_t threadID;	//thread which started this loop
+#ifdef WAIT_POLL
+	gboolean maincontext; //TRUE when wait was initiated from main thread, in
+						// which case the watching is done by separate, dedicated, thread
+	gboolean finished;	//TRUE when the wait is ready to be ended
+#else
 	GMainLoop *loop;
+#endif
 } E2_MainLoop;
 
 typedef struct _E2_MainData
@@ -561,23 +691,24 @@ typedef struct _E2_MainData
 	GtkWidget *hbox_main;
 	GtkWidget *vbox_main;
 	GtkWidget *outbook; //notebook container for tabbed output-panes
+//#ifdef E2_POLKIT
+//	GtkWidget *status_bar_label1;
+//#endif
 	GtkWidget *status_bar_label2;
 	//this is a hbox, at the right of the status bar, for general usage
 	// needs to be shown when used
 	GtkWidget *status_bar_box3;
 	E2_WindowRuntime window;
-	E2_PaneRuntime pane1;
-	E2_PaneRuntime pane2;
-	ViewInfo pane1_view;
-	ViewInfo pane2_view;
+	E2_PaneRuntime pane1;	//includes ViewInfo
+	E2_PaneRuntime pane2;	//ditto
 	E2_ToolbarRuntime toolbar;
 	E2_ToolbarRuntime commandbar;
 	E2_AliasRuntime aliases;
-	E2_OutputRuntime output;	//non-tab-specific data for output pane
-	E2_OutputTabRuntime tab;	//tab-specific data for output pane
-	GList *tabslist;	//list of E2_OutputTabRuntime's for each tab
-	gint tabcount;	//no. of tabs (cached)
-	E2_ToolbarData **bars;	//null-terminated array of pointers to toolbar data
+	E2_OutputRuntime output;//non-tab-specific data for output pane
+	E2_OutputTabRuntime tab;//tab-specific data for output pane
+	GList *tabslist;		//list of E2_OutputTabRuntime's for each tab
+	gint tabcount;			//no. of tabs (cached)
+	E2_ToolbarData **bars;	//NULL-terminated array of pointers to toolbar data
 	GList *command_lines;	//list of rt data structs of command-lines
 
 #if defined(E2_FAM) && !defined(E2_FAM_KERNEL)
@@ -586,41 +717,56 @@ typedef struct _E2_MainData
 #endif
 #ifdef E2_FAM
 	E2_FAMonitor monitor_type;
-	gint FAMreq;	//request no used for monitoring config file changes
+	gint FAMreq;			//request no used for monitoring config file changes
 #endif
-	guint timers[MAX_TIMERS];	//glib timer id's
+	guint timers[MAX_TIMERS];//glib timer id's
 	time_t config_mtime;	//last-logged (seconds) timestamp of config file
 #ifndef USE_GTK2_12TIPS
 	GtkTooltips *tooltips;
+	gint gtkversion;		//runtime version, = major * 10000 + minor * 100 + micro
 #endif
+#ifdef E2_VFS
+	GHashTable *vdir_cache; //table of vath's for places used in the session, each with 0x1 data
+#endif
+	GHashTable *dir_history;//table of E2_DirHistoryEntry's for dirs opened in session
+	GHashTable *keysnative;	//table of keycodes for asciifying localised alphabetic
+							//bindings, each index is a pointerised locale-specific
+							//keycode, each corresponding value is a pointerized
+							//lowercase code GDK_a .. GDK_z
+	GHashTable *keyslocal;	//table of strings for localising alphabetic key-bindings,
+							//each index is a lowercase letter string "a" .. "z",
+							//each corresponding value is a UTF-8 string
+							//representing the corresponding localised letter
+							//This table is not often used
 	GList *plugins;	//list of data structs for all loaded plugins, "non-children"
 					//in menu-order (children-submenu-order depends on a list in
 					//corresponding parent)
 	GHashTable *filetypes;
-	GSList *typelist;	//list of string arrays, needed tor cleanup
+	GSList *typelist;		//list of string arrays, needed tor cleanup
 #ifdef E2_IMAGECACHE
-	GHashTable *icons;	//table of E2_Images, for cached icons
+	GHashTable *icons;		//table of E2_Images, for cached icons
 #endif
 #ifdef E2_RAINBOW
 	GHashTable *colors;
 	GList *colorchunks;
 #endif
-	GSList *mainloops;	//local mainloop information in E2_MainLoop's
-	GSList *used_stores;	//for deferred liststore clearing
+	GSList *mainloops;			//local mainloop information in E2_MainLoop's
+	GSList *used_stores;		//for deferred liststore clearing
 	GList *taskhistory;
-#ifndef E2_FILES_UTF8ONLY
-	gboolean utf8_filenames;	//TRUE when filesystem coding is utf-8 or ascii only
-#endif
+//#ifndef E2_FILES_UTF8ONLY
+//	gboolean utf8_filenames;	//TRUE when filesystem coding is utf-8 or ascii only
+//#endif
 	GdkWindowState mainwindow_state;	//flags indicating changes to main_window
 	GtkWidget *context_menu;	//main context-menu when it is popped up
-#ifdef USE_GTK2_99
+#ifdef USE_GTK2_18
 	GtkAllocation main_alloc;	//main window dimensions for cache
 #endif
-	GtkAllocation cfg_alloc; //config dialog dimensions for cache
-	gchar cfgfile_version[20];  //store for config version string eg "0.1.9", used only for upgrades
-	gboolean rebuild_enabled;	//set FALSE to block window recreation
+	GtkAllocation cfg_alloc;	//config dialog dimensions for cache
+	gchar cfgfile_version[20];	//store for config version string eg "0.1.9", used only for upgrades
+//	gboolean keytrans;			//TRUE for automatic translation next time keybindings are sync'd
+	gboolean reconvert_requested; //TRUE when a change of encoding-status has been logged during a filelist refill
 #ifdef E2_STATUS_BLOCK
-	gboolean status_working;	//set TRUE to block reentrant status-checking etc
+	volatile gint status_working;	//1 to block reentrant status-checking etc
 #endif
 	GHookList hook_pane_focus_changed;
 } E2_MainData;
@@ -642,20 +788,29 @@ pthread_mutex_t history_mutex;
 
 //for testing - define to use gtk's own mutex for BGL
 //#define NATIVE_BGL
-#ifndef NATIVE_BGL
+#ifdef NATIVE_BGL
+	//WAIT_FOR_EVENTS expects BGL closed
+# define CLOSEBGL_IF_OPEN \
+	WAIT_FOR_EVENTS_UNLOCKED
+# define OPENBGL_IF_CLOSED
+#else //ndef NATIVE_BGL
 //local management of gdk mutex, to enable tolerant re-locking by the same thread
-#include <pthread.h>
 //direct mutex-manipulation
-#define gdk_threads_enter e2_main_close_gdklock
-#define gdk_threads_leave e2_main_open_gdklock
 pthread_mutex_t gdklock;	//BGL replacement
 void e2_main_close_gdklock (void);
 void e2_main_open_gdklock (void);
+# ifdef DEBUG_MESSAGES
+#  define gdk_threads_enter e2_main_close_gdklock
+#  define gdk_threads_leave e2_main_open_gdklock
+#else
+#  define gdk_threads_enter() pthread_mutex_lock (&gdklock)
+#  define gdk_threads_leave() pthread_mutex_unlock (&gdklock)
+#endif
 void e2_main_replace_gdklock (gboolean free);
 
 	//not locked now and not already locked - this should never happen
 	//prevent unlock
-#define CLOSEBGL_IF_OPEN \
+# define CLOSEBGL_IF_OPEN \
 	gint _lockres; \
 	_lockres = pthread_mutex_trylock (&gdklock); \
 	printd (DEBUG, "%s BGL trylock result %d", __PRETTY_FUNCTION__, _lockres); \
@@ -666,22 +821,15 @@ void e2_main_replace_gdklock (gboolean free);
 		_lockres = 1; \
 	}
 
-#define	OPENBGL_IF_CLOSED \
+# define	OPENBGL_IF_CLOSED \
 	if (_lockres == 0) \
 		pthread_mutex_unlock (&gdklock);
-
-#else //def NATIVE_BGL
-	//WAIT_FOR_EVENTS expects BGL closed
-#define CLOSEBGL_IF_OPEN \
-	WAIT_FOR_EVENTS_UNLOCKED
-#define	OPENBGL_IF_CLOSED
 #endif
 //void e2_main_cleanup_gdklock (gpointer data);
 
 E2_MainLoop *e2_main_loop_new (gboolean mainctx);
-//void e2_main_loop_run (E2_MainLoop *loopdata);
-#define e2_main_loop_run(d) g_main_loop_run(d->loop)
-gboolean e2_main_loop_quit (E2_MainLoop *loopdata);
+void e2_main_loop_run (E2_MainLoop *loopdata);
+void e2_main_loop_quit (E2_MainLoop *loopdata);
 gboolean e2_main_loop_abort (pthread_t ID);
 gboolean e2_main_closedown (gboolean compulsory, gboolean saveconfig, gboolean doexit);
 gboolean e2_main_user_shutdown (gpointer from, E2_ActionRuntime *art);
