@@ -1,6 +1,6 @@
-/* $Id: e2_fs.c 1424 2009-02-10 22:11:01Z tpgww $
+/* $Id: e2_fs.c 2454 2012-03-09 11:53:45Z tpgww $
 
-Copyright (C) 2005-2009 tooar <tooar@gmx.net>
+Copyright (C) 2005-2012 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -14,8 +14,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 /**
@@ -49,6 +48,10 @@ Not implemented yet.
 #ifndef MNT_LOCAL
 # include <sys/statvfs.h>
 #endif
+#ifdef E2_MAGIC
+# include <magic.h>
+# include <dlfcn.h>
+#endif
 #include "e2_fs.h"
 #include "e2_dialog.h"
 #ifdef E2_VFS
@@ -82,10 +85,7 @@ static gboolean _e2_fs_S_ISEXE (mode_t mode)
 }
 #endif //0
 
-#ifdef E2_VFSTMP
-	//FIXME may need to check coding for each displayed dir, if that's possible
-#endif
-#ifndef E2_FILES_UTF8ONLY
+/* #ifndef E2_FILES_UTF8ONLY
 //check whether a character encoding is compatible with utf
 static gboolean _e2_fs_charset_is_utf (gchar *filecoding)
 {
@@ -104,12 +104,13 @@ static gboolean _e2_fs_language_is_english (void)
 		lang = g_getenv ("LANG");
 	return (lang != NULL && strstr (lang, "en_") != NULL);
 }
-/**
+*/
+/* *
 @brief check whether the native filesystem encoding is utf-8 (or ascii), and setup for conversion accordingly
 
 @return
 */
-void e2_fs_check_coding (void)
+/*void e2_fs_check_coding (void)
 {
 	gchar *filecoding;
 	app.utf8_filenames = FALSE;
@@ -133,7 +134,7 @@ void e2_fs_check_coding (void)
 			else
 				app.utf8_filenames = _e2_fs_language_is_english ();
 		}
-		else if (g_str_equal (filecoding, "@locale"))
+		else if (!strcmp (filecoding, "@locale"))
 		{
 			const gchar *charset;
 			app.utf8_filenames = g_get_charset (&charset)
@@ -163,13 +164,12 @@ void e2_fs_check_coding (void)
 		e2_fname_from_locale = e2_fname_dupfrom_locale = e2_utf8_filename_from_locale;
 		e2_fname_free = g_free;
 	}
-}
-#endif //ndef E2_FILES_UTF8ONLY
+} */
+//#endif //ndef E2_FILES_UTF8ONLY
 
 /**
 @brief check whether @a localpath exists
-Can't use the function access(), which (for glibc at least) looks
-through links
+Can't use the function access(), which (for glibc at least) looks through links
 Errors other than non-readable are ignored
 NOTE probably this does not work properly on FAT*, which are case-insensitive
 @param localpath item to test, absolute localised string
@@ -315,7 +315,7 @@ gboolean e2_fs_ingroup (gid_t gid)
 			gint i = 0;
 	        while (grp_buf->gr_mem [i] != NULL)
 			{
-				if (g_str_equal (grp_buf->gr_mem [i], myname))
+				if (!strcmp (grp_buf->gr_mem [i], myname))
 				{
 					ingroup = TRUE;
 					break;
@@ -330,7 +330,7 @@ gboolean e2_fs_ingroup (gid_t gid)
 /**
 @brief execute the file(1) command on @a localpath, and search each line of its output for @a string
 Used in e2_fs_is_text, e2_fs_is_exec2, and e2_fs_is_executable
-If @a locapath is a symlink, it will be traversed.
+If @a localpath is a symlink, it will be traversed.
 
 @param localpath localised name of file to test
 @param string ascii string for which to search, in the output from file command
@@ -339,55 +339,101 @@ If @a locapath is a symlink, it will be traversed.
 */
 static gboolean _e2_fs_grep_file_output (VPATH *localpath, gchar *string E2_ERR_ARG())
 {
+	E2_ERR_PCLEAR
+
 #ifdef E2_VFS
 	if (e2_fs_item_is_mounted (localpath))
 	{
 #endif
+#ifdef E2_MAGIC
+		MagicIface ifc;
+
+		if (e2_utils_fill_magic_iface (&ifc))
+		{
+			magic_t handle = ifc.open (
+				MAGIC_PRESERVE_ATIME | MAGIC_RAW | MAGIC_ERROR | MAGIC_SYMLINK);
+			if (handle != NULL)
+			{
+				ifc.load (handle, NULL); //load failure will result in NULL msg
+				const gchar *msg = ifc.file (handle, VPCSTR (localpath));
+				if (msg != NULL)
+				{
+					gboolean retval = (strstr (msg, string) != NULL);
+					ifc.close (handle);
+					dlclose (ifc.libhandle);
+					return retval;
+				}
+				else
+				{
+					//FIXME warning
+# ifdef E2_VFSTMP
+					if (E2_ERR_NAME != NULL)
+					{
+						populate *E2_ERR_NAME
+						X = ifc.error (handle);
+					}
+# endif
+					ifc.close (handle);
+				}
+			}
+			else
+			{
+				//FIXME warning
+# ifdef E2_VFSTMP
+				if (E2_ERR_NAME != NULL)
+				{
+					populate *E2_ERR_NAME
+				}
+# endif
+			}
+			dlclose (ifc.libhandle);
+		}
+#else
 		//CHECKME weirdness when > 1 opening of the same file,
 		//avoided by setting the cwd before opening the pipe
-		gchar *local = F_FILENAME_TO_LOCALE (curr_view->dir);
-#ifdef E2_VFS
+		gchar *local = D_FILENAME_TO_LOCALE (curr_view->dir); //always dup, to avoid dirchange race
+# ifdef E2_VFS
 		VPATH data = { local, NULL };	//always local
-		if (e2_fs_chdir_local (&data E2_ERR_NONE()))
-#else
+		if (e2_fs_chdir_local (&data E2_ERR_SAMEARG()))
+# else
 		if (e2_fs_chdir_local (local E2_ERR_NONE()))
-#endif
+# endif
 		{
 		// FIXME warn user
-			F_FREE (local);
+			g_free (local);
 			return FALSE;
 		}
-		F_FREE (local);
+		g_free (local);
 
 		size_t bsize = E2_MAX_LEN;
-#ifdef USE_GLIB2_10
+# ifdef USE_GLIB2_10
 		gchar *buf = (gchar *) g_slice_alloc ((gulong) bsize);
-#else
+# else
 		gchar *buf = (gchar *) g_try_malloc ((gulong) bsize);
-#endif
+# endif
 		CHECKALLOCATEDWARN (buf, return FALSE;);
 
 //tag E2_BADQUOTES
 		gchar *qp = e2_utils_quote_string (VPCSTR (localpath));
-		gchar *command = e2_utils_strcat ("file -L ", qp);	//want localised
+		gchar *command = e2_utils_strcat ("file -Lpr ", qp);	//want localised
 		g_free (qp);
 		E2_FILE *pipe = e2_fs_open_pipe (command);
 		g_free (command);
 		if (pipe == NULL)
 		{
-#ifdef USE_GLIB2_10
+# ifdef USE_GLIB2_10
 			g_slice_free1 (bsize, buf);
-#else
+# else
 			g_free (buf);
-#endif
+# endif
 			return FALSE;
 		}
 		gboolean retval = FALSE;
-#ifdef __USE_GNU
+# ifdef __USE_GNU
 		while (getdelim (&buf, &bsize, '\n', pipe) > 1)	//not an empty line
-#else
+# else
 		while (fgets (buf, bsize, pipe) != NULL && *buf != '\n')
-#endif
+# endif
 		{
 			if (strstr (buf, string))
 			{
@@ -396,27 +442,23 @@ static gboolean _e2_fs_grep_file_output (VPATH *localpath, gchar *string E2_ERR_
 			}
 		}
 		e2_fs_pipe_close (pipe);
-#ifdef USE_GLIB2_10
+# ifdef USE_GLIB2_10
 		g_slice_free1 (bsize, buf);
-#else
+# else
 		g_free (buf);
-#endif
+# endif
 		return retval;
+#endif
 #ifdef E2_VFS
 	}
 	else	//item is virtual
 	{
-/*		if (E2_ERR_NAME != NULL && *E2_ERR_NAME != NULL)
-		{
-			g_error_free (*E2_ERR_NAME);	//just in case?
-			*E2_ERR_NAME = NULL;
-		} */
 # ifdef E2_VFSTMP
 		//get information about virtual item, how ??
 # endif
-		return FALSE;
 	}
 #endif
+	return FALSE;
 }
 /**
 @brief test whether @a localpath is a text file
@@ -504,11 +546,6 @@ gboolean e2_fs_is_executable (FileInfo *info, ViewInfo *view)
 			}
 			else	//item is virtual
 			{
-/*				if (E2_ERR_NAME != NULL && *E2_ERR_NAME != NULL)
-				{
-					g_error_free (*E2_ERR_NAME);	//just in case?
-					*E2_ERR_NAME = NULL;
-				} */
 # ifdef E2_VFSTMP
 			//FIXME full path for vfs item
 			//FIXME do this some other way for virtual items e.g. test mimetype
@@ -635,13 +672,7 @@ gboolean e2_fs_recurse_mkdir (VPATH *localpath, gint mode E2_ERR_ARG())
 			if (E2_ERR_PISNOT (ENOENT) || e2_fs_mkdir (localpath, mode E2_ERR_NONE()))	//FIXME vfs
 			{
 				*s = c;
-#ifdef E2_VFS
-				if (E2_ERR_NAME != NULL && *E2_ERR_NAME != NULL)
-				{
-					g_error_free (*E2_ERR_NAME);
-					*E2_ERR_NAME = NULL;
-				}
-#endif
+				E2_ERR_PCLEAR
 				return TRUE;
 			}
 		}
@@ -657,13 +688,7 @@ gboolean e2_fs_recurse_mkdir (VPATH *localpath, gint mode E2_ERR_ARG())
 	{
 		if (E2_ERR_PISNOT (ENOENT) || e2_fs_mkdir (localpath, mode E2_ERR_NONE()))	//FIXME vfs
 		{
-#ifdef E2_VFS
-			if (E2_ERR_NAME != NULL && *E2_ERR_NAME != NULL)
-			{
-				g_error_free (*E2_ERR_NAME);
-				*E2_ERR_NAME = NULL;
-			}
-#endif
+			E2_ERR_PCLEAR
 			return TRUE;
 		}
 	}
@@ -704,7 +729,7 @@ gboolean e2_fs_chdir (gchar *utfpath E2_ERR_ARG())
 		retval = TRUE;
 
 	E2_ERR_CLEARBACKUP (localerr);
-	F_FREE (local);
+	F_FREE (local, utfpath);
 	return retval;
 }
 /**
@@ -751,25 +776,25 @@ gboolean e2_fs_cd_isok (gchar *utfpath E2_ERR_ARG())
 			success = FALSE;
 		}
 	}
-	else if (e2_fs_dir_is_native (utfpath, TRUE))
-		message = g_strdup_printf (_("Directory '%s' does not exist"), utfpath);
 	else
 	{
-#ifdef E2_VFS
-		if (E2_ERR_NAME != NULL && *E2_ERR_NAME != NULL)
+		//dir doesn't exist
+		if (e2_fs_dir_is_native (utfpath, TRUE))
 		{
-			g_error_free (*E2_ERR_NAME);
-			*E2_ERR_NAME = NULL;
+			message = g_strdup_printf (_("Directory '%s' does not exist"), utfpath);
+		}
+		else
+		{
+			E2_ERR_PCLEAR
+			success = TRUE;	//missing path ok when it's non-mounted
 		}
-#endif
-		success = TRUE;	//missing path ok when it's non-mounted
 	}
 
 	if (!success)
 	{
 		e2_output_print_error (message, TRUE);
 	}
-	F_FREE (local);
+	F_FREE (local, utfpath);
 	return success;
 }
 /**
@@ -832,7 +857,7 @@ gboolean e2_fs_get_valid_path (gchar **utfpath, gboolean accessible E2_ERR_ARG()
 		}
 
 #ifdef E2_VFS
-		data.localpath = local;
+		data.path = local;
 		data.spacedata = NULL;
 		if (e2_fs_is_dir3 (&data E2_ERR_NONE())
 			&& (!accessible || !e2_fs_access (&data, R_OK | X_OK E2_ERR_NONE())))
@@ -852,7 +877,7 @@ gboolean e2_fs_get_valid_path (gchar **utfpath, gboolean accessible E2_ERR_ARG()
 	//FIXME new path may be virtual item eg archive ?
 #endif
 #ifdef E2_VFS
-//		data.localpath = local; e2_utils_get_parent_path() doesn't change path
+//		data.path = local; e2_utils_get_parent_path() doesn't change path
 		if (e2_fs_is_dir3 (&data E2_ERR_NONE())
 			&& (!accessible || !e2_fs_access (&data, R_OK | X_OK E2_ERR_NONE())))
 #else
@@ -883,7 +908,7 @@ gboolean e2_fs_get_valid_path (gchar **utfpath, gboolean accessible E2_ERR_ARG()
 @brief complete a directory string in @a entry after a keypress @a pressed_char
 This works only for native directories
 @param entry the entry widget to be updated
-@param pressed_char utf8-string form of pressed key which triggered the completion
+@param pressed_char UTF-8 string-form of pressed key which triggered the completion
 @param pane enumerator of pane to use for default dir, 1,2,0=current
 
 @return TRUE if completion was performed
@@ -896,6 +921,20 @@ static gboolean _e2_fs_complete_dir (GtkWidget *entry, gchar *pressed_char, guin
 	gint pos = start + 1;	//+1 to include the key just pressed
 	gchar *entrytext = gtk_editable_get_chars (GTK_EDITABLE (entry), 0, start);
 	gchar *text = g_strconcat (entrytext, pressed_char, NULL);
+
+	//do macros/variables replacement CHECKME worth it ?
+	printd (DEBUG, "_e2_fs_complete_dir, macro/var interpretation");
+	gchar *tmp = text;
+	gchar *freeme = e2_utils_expand_macros (text, NULL); //note %% changes to %
+	if (freeme == NULL || freeme == GINT_TO_POINTER (0x1))
+		freeme = tmp;
+	text = e2_utils_replace_vars (freeme, TRUE);
+	if (freeme != tmp)
+		g_free (freeme);
+	if (strcmp (tmp, text))
+		pos = g_utf8_strlen (text, -1);
+	g_free (tmp);
+
 	gchar *utfdir;
 	if (g_path_is_absolute (text))
 		utfdir = g_path_get_dirname (text);
@@ -904,17 +943,17 @@ static gboolean _e2_fs_complete_dir (GtkWidget *entry, gchar *pressed_char, guin
 		switch (pane)
 		{
 			case E2PANE1:
-				utfdir = app.pane1_view.dir;
+				utfdir = app.pane1.view.dir;
 				break;
 			case E2PANE2:
-				utfdir = app.pane2_view.dir;
+				utfdir = app.pane2.view.dir;
 				break;
 //			case E2PANECUR:
 			default:
 				utfdir = curr_view->dir;
 				break;
 		}
-		gchar *freeme = g_strconcat (utfdir, text, NULL);
+		freeme = g_strconcat (utfdir, text, NULL);
 		utfdir = g_path_get_dirname (freeme);
 		g_free (freeme);
 	}
@@ -932,6 +971,7 @@ static gboolean _e2_fs_complete_dir (GtkWidget *entry, gchar *pressed_char, guin
 			gchar *newtext = g_strconcat (text, entrytext, NULL);
 			gtk_entry_set_text (GTK_ENTRY (entry), newtext);
 			gtk_editable_set_position (GTK_EDITABLE (entry), pos);
+			e2_command_line_update_highlight (entry, newtext);
 			g_free (newtext);
 			e2_list_free_with_data (&found);
 		}
@@ -1044,12 +1084,13 @@ retry:
 	}
 	if (!(fstatus.f_flags & MNT_LOCAL))
 	{
-//		if (errno == EINTR)
+//		if (errno == EINTR) CHECKME relevance ?
 //			goto retry;
-		else if (errno == ENOENT && descend && e2_utils_get_parent_path (localpath, TRUE))
+//		else
+			if (errno == ENOENT && descend && e2_utils_get_parent_path (localpath, TRUE))
 			goto retry;
 		g_free (localpath);
-		return FALSE;
+		return (errno == EACCES);	//no permission implies a local dir
 	}
 #else
 	struct statvfs fstatus;
@@ -1065,7 +1106,7 @@ retry:
 		else if (errno == ENOENT && descend && e2_utils_get_parent_path (localpath, TRUE))
 			goto retry;
 		g_free (localpath);
-		return FALSE;
+		return (errno == EACCES);	//no permission implies a local dir
 	}
 #endif
 	//FIXME MORE TESTS HERE e.g.
@@ -1228,7 +1269,7 @@ static gpointer _e2_fs_read_mounted_dir (E2_DRFuncArgs *data)
 			if (processor != NULL)
 			{
 #ifdef E2_VFS
-				ddata.localpath = data->localpath;
+				ddata.path = data->localpath;
 				if (!processor (&ddata, u.entry.d_name, &entries, data->cb_data))
 #else
 				if (!processor (data->localpath, u.entry.d_name, &entries, data->cb_data))
@@ -1343,12 +1384,27 @@ static gpointer _e2_fs_progress_monitor (E2_DRead *data)
 		pthread_mutex_unlock (&condition_mutex);
 		printd (DEBUG,"dir-monitor-thread (ID=%lu) %d-sec sleep ended", (gulong) data->mid, secs);
 
-		gboolean shown = (data->dialog != NULL && GTK_WIDGET_VISIBLE (data->dialog));
+		gboolean shown = (data->dialog != NULL &&
+#ifdef USE_GTK2_18
+			gtk_widget_get_visible (data->dialog));
+#else
+			GTK_WIDGET_VISIBLE (data->dialog));
+#endif
 		if (data->dialog == NULL) //once-only, create the dialog
+		{
+			gdk_threads_enter ();
 			data->dialog = e2_dialog_slow (_("Reading directory data"),
-				_("directory read"), _e2_fs_slowread_response_cb, data);
+				_("directory read"),
+			(ResponseFunc)_e2_fs_slowread_response_cb, data);
+			gdk_threads_leave ();
+		}
 
-		if (data->dialog != NULL && !GTK_WIDGET_VISIBLE (data->dialog))
+		if (data->dialog != NULL &&
+#ifdef USE_GTK2_18
+			!gtk_widget_get_visible (data->dialog));
+#else
+			!GTK_WIDGET_VISIBLE (data->dialog))
+#endif
 		{
 			gdk_threads_enter ();
 			gtk_widget_show (data->dialog);
@@ -1414,8 +1470,9 @@ gpointer e2_fs_dir_foreach (VPATH *localpath, E2_FsReadWatch monitor,
 		//create joinable read thread, not with glib thread funcs, as we need to
 		//kill thread(s)
 		pthread_t ID;
-		if (!pthread_create (&ID, NULL, (gpointer)_e2_fs_read_mounted_dir, &args))
-			printd (DEBUG,"read-dir-thread (ID=%lu) started", ID);
+		if (pthread_create (&ID, NULL,
+			(gpointer(*)(gpointer))_e2_fs_read_mounted_dir, &args) == 0)
+				printd (DEBUG,"read-dir-thread (ID=%lu) started", ID);
 		else
 		{
 			//FIXME message to user
@@ -1430,7 +1487,11 @@ gpointer e2_fs_dir_foreach (VPATH *localpath, E2_FsReadWatch monitor,
 //		CHECKALLOCATEDWARN (data, return (GINT_TO_POINTER (E2DREAD_ENOMEM));)
 //		data->aid = ID;
 
-		e2_window_show_status_message (_("Reading directory data"));
+		e2_window_show_status_message (_("Reading directory data")
+#ifdef USE_GTK2_20
+			, FALSE
+#endif
+		);
 		//make status message show, at session start at least
 		WAIT_FOR_EVENTS_UNLOCKED	//_SLOWLY
 
@@ -1444,7 +1505,7 @@ gpointer e2_fs_dir_foreach (VPATH *localpath, E2_FsReadWatch monitor,
 		pthread_attr_setdetachstate (&attr, PTHREAD_CREATE_DETACHED);
 
 		if (pthread_create (&data.mid, &attr,
-			(gpointer) _e2_fs_progress_monitor, &data) == 0)
+			(gpointer(*)(gpointer))_e2_fs_progress_monitor, &data) == 0)
 				printd (DEBUG,"dir-monitor-thread (ID=%lu) started", data.mid);
 		else
 		{
@@ -1499,6 +1560,70 @@ gpointer e2_fs_dir_foreach (VPATH *localpath, E2_FsReadWatch monitor,
 #endif
 	return entries;
 }
+
+/**
+@brief get the target of symbolic link represented by @a localpath
+The string in targetbuf will be 0-terminated and stripped of irrelevant leading
+relativities
+@param localpath descriptor for the symlink, with localised path
+@param targetbuf buffer into which the result can be copied
+@param targetsize the length of @a targetbuf
+
+@return the length of the string in @a targetbuf, or < 0 upon error
+*/
+gint e2_fs_readlink (VPATH *localpath, gchar *targetbuf, size_t targetsize E2_ERR_ARG())
+{
+	gint len;
+	const gchar *t;
+#ifdef E2_VFS
+	VPATH ddata;
+#endif
+
+#ifdef E2_VFSTMP
+	if (e2_fs_item_is_mounted (localpath))
+	{
+#endif
+		len = readlink (VPCSTR(localpath), targetbuf, targetsize);
+		if (len < 0)
+		{
+			*targetbuf = '\0';
+			return len;
+		}
+//		else if (len == targetsize)
+//			FIXME can't tell if target is longer
+#ifdef E2_VFSTMP
+	}
+	else	//need to do virtual
+	{
+		FIXME vfs version
+		if (len < 0)
+		{
+			e2_fs_set_error_from_errno (E2_ERR_NAME);
+			*targetbuf = '\0';
+			return len;
+		}
+	}
+#endif
+	if (len == targetsize)
+		len--;
+	targetbuf[len] = '\0';
+
+#ifdef E2_VFS
+	ddata.path = targetbuf;
+	ddata.spacedata = localpath->spacedata;
+	t = e2_utils_skip_relative_path (&ddata);
+#else
+	t = e2_utils_skip_relative_path (targetbuf);
+#endif
+	if (t > targetbuf)
+	{
+		memmove (targetbuf, t, len + targetbuf - t + 1);
+		len -= (t-targetbuf);
+	}
+
+	return len;
+}
+
 /**
 @brief get real path for @a local_path if it's a link
 
@@ -1508,6 +1633,7 @@ gpointer e2_fs_dir_foreach (VPATH *localpath, E2_FsReadWatch monitor,
 */
 gboolean e2_fs_walk_link (gchar **local_path E2_ERR_ARG())
 {
+//CHECKME use e2_utils_skip_relative_path (VPATH *path); here ?
 	struct stat sb;
 #ifdef E2_VFSTMP
 	FIXME get proper spacedata
@@ -1546,14 +1672,10 @@ gboolean e2_fs_walk_link (gchar **local_path E2_ERR_ARG())
 		}
 		else	//item is virtual
 		{
-/*			if (E2_ERR_NAME != NULL && *E2_ERR_NAME != NULL)
-			{
-				g_error_free (*E2_ERR_NAME);	//just in case?
-				*E2_ERR_NAME = NULL;
-			} */
-	# ifdef E2_VFSTMP
+			E2_ERR_PCLEAR
+# ifdef E2_VFSTMP
 			//FIXME vfs if relevant, get information about virtual item, how ??
-	# endif
+# endif
 			return FALSE;
 		}
 #endif
@@ -1731,26 +1853,6 @@ gint e2_fs_symlink (VPATH *target, VPATH *name, GError **E2_ERR_NAME)
 	return result;
 }
 
-gint e2_fs_readlink (VPATH *localpath, gchar *targetbuf, size_t bufsize,
-	GError **E2_ERR_NAME)
-{
-	gint result;
-	if (e2_fs_item_is_mounted (localpath))
-	{
-		result = readlink (VPCSTR(localpath), targetbuf, bufsize);
-		if (result != 0)
-			e2_fs_set_error_from_errno (E2_ERR_NAME);
-		return result;
-	}
-	else	//need to do virtual
-	{
-#ifdef E2_VFSTMP
-		//FIXME do virtual change
-#endif
-		return -1;
-	}
-}
-
 gint e2_fs_unlink (VPATH *localpath, GError **E2_ERR_NAME)
 {
 	gint result;
@@ -2039,9 +2141,8 @@ gboolean e2_fs_touchnow (VPATH *localpath E2_ERR_ARG())
 /**
 @brief update m/a/c times of config dir
 
-This should be called after writing the config/cache files
-to let any other instance know that it should re-read those
-files (if that option is in force)
+This should be called after writing config/cache files, to let any other
+instance know that it should re-read those files (if that option is in force).
 Error message expects BGL to be closed
 
 @return
@@ -2090,7 +2191,7 @@ void e2_fs_touch_config_dir (void)
 #endif
 		E2_ERR_CLEAR
 	}
-	F_FREE (local);
+	F_FREE (local, e2_cl_options.config_dir);
 }
 /**
 @brief tolerantly open file @a localpath and return its descriptor
@@ -2141,7 +2242,7 @@ E2_FILE *e2_fs_open_writestream (const gchar *localpath E2_ERR_ARG())
 			gchar *msg = g_strdup_printf (_("Cannot open '%s' for writing - %s"),
 				utf, g_strerror (errno));
 			e2_output_print_error (msg, TRUE);
-			F_FREE (utf);
+			F_FREE (utf, localpath);
 		}
 #ifdef E2_VFS
 	}
@@ -2192,7 +2293,7 @@ gint e2_fs_put_stream (E2_FILE *stream, const gchar *string, gchar *localpath
 #endif
 				);
 			e2_output_print_error (msg, TRUE);
-			F_FREE (utf);
+			F_FREE (utf, localpath);
 			E2_ERR_CLEARBACKUP (localerr);
 		}
 #ifdef E2_VFSTMP
@@ -2208,7 +2309,7 @@ gint e2_fs_put_stream (E2_FILE *stream, const gchar *string, gchar *localpath
 /**
 @brief read from file into buffer
 
-@param descriptor file descriptor
+@param descriptor file descriptor for the file to read
 @param buffer start of memory to hold the read data
 @param bufsize size of @a buffer
 
@@ -2275,7 +2376,7 @@ gboolean e2_fs_get_file_contents (VPATH *localpath, gpointer *contents,
 			struct stat sb;
 			if (!e2_fs_fstat (fdesc, &sb))
 			{
-				size_t len = sb.st_size;
+				size_t nread, len = sb.st_size;
 				if (terminate)
 					len += sizeof (gchar);	//in case we need to append a 0
 
@@ -2294,37 +2395,45 @@ gboolean e2_fs_get_file_contents (VPATH *localpath, gpointer *contents,
 
 				retval = TRUE;
 				fmt = NULL;
-				ssize_t nread = e2_fs_read (fdesc, *contents, sb.st_size E2_ERR_SAMEARG());
-				if (nread < 0 //|| nread < (gulong)sb.st_size
-				   )
+				if (sb.st_size > 0)
 				{
-					g_free (*contents);	//or free() if needed
-					*contents = NULL;
-					nread = 0;
-					retval = FALSE;
-					fmt = _("Error reading file %s");
-				}
-				else if (terminate)
-				{
-					gchar *s;
-					if (nread >= sizeof (gchar))
+					nread = e2_fs_read (fdesc, *contents, sb.st_size E2_ERR_SAMEARG());
+					if (nread < 0 //|| nread < (gulong)sb.st_size
+					   )
 					{
-						s = *contents + nread - sizeof (gchar);
-						if (*s == '\0')
-							nread -= sizeof (gchar);	//OK ?
-						else
+						g_free (*contents);	//or free() if needed
+						*contents = NULL;
+						nread = 0;
+						retval = FALSE;
+						fmt = _("Error reading file %s");
+					}
+					else if (terminate)
+					{
+						gchar *s;
+						if (nread >= sizeof (gchar))
 						{
-							s += sizeof (gchar);
+							s = *contents + nread - sizeof (gchar);
+							if (*s == '\0')
+								nread -= sizeof (gchar);	//OK ?
+							else
+							{
+								s += sizeof (gchar);
+								*s = '\0';
+							}
+						}
+						else	//nothing read
+						{
+							s = *contents;
 							*s = '\0';
 						}
 					}
-					else	//nothing read
-					{
-						s = *contents;
-						*s = '\0';
-					}
 				}
-
+				else
+				{
+					nread = 0;
+					if (terminate)
+						*contents = '\0';
+				}
 				if (contlength != NULL)
 					*contlength = nread;
 			}
@@ -2365,10 +2474,10 @@ gboolean e2_fs_get_file_contents (VPATH *localpath, gpointer *contents,
 	return retval;
 }
 /**
-@brief read from file into buffer
+@brief write from buffer to file
 
-@param descriptor file descriptor
-@param buffer start of memory to hold the write data
+@param descriptor file descriptor of file to write
+@param buffer start of memory holding the write data
 @param bufsize size of @a buffer
 
 @return no. of bytes written, < 0 on error
@@ -2387,7 +2496,7 @@ ssize_t e2_fs_write (gint descriptor, gpointer buffer, /*size_t*/ gulong bufsize
 }
 /**
 @brief write whole file @a localpath, from memory starting at @a contents
-No overwrite checking is done
+No overwrite checking is done, no data syncing is done
 @param localpath path of file to be write, localised string
 @param contents ptr to the start of the data to save
 @param contlength byte-length of data to save
@@ -2426,7 +2535,10 @@ gboolean e2_fs_set_file_contents (VPATH *localpath, gpointer contents,
 				}
 			}
 			if (retval)
-				ftruncate (fdesc, written);
+			{
+				//CHECKME e2_fs_writeflush (fdesc);
+				TEMP_FAILURE_RETRY (ftruncate (fdesc, written));
+			}
 			e2_fs_safeclose (fdesc);
 		}
 
@@ -2469,6 +2581,27 @@ gint e2_fs_safeclose (gint file_desc)
 	return res;
 }
 /**
+@brief flush file contents to storage
+File descriptors are only relevant for local files, so no vfs treatment
+is needed
+@param file_desc file descriptor
+
+@return 0 on success, -1 on error
+*/
+gint e2_fs_writeflush (gint file_desc)
+{
+	gint res;
+	errno = 0;
+#ifdef __E2BSD__
+	res = fsync (file_desc);
+#else
+	res = fdatasync (file_desc);
+#endif
+	if (res == -1 && errno == EINVAL)
+		res = 0;
+	return res;
+}
+/**
 @brief perform a native, blockwise, file copy
 Error messages assume BGL is open
 @param vsrc localised string, absolute path of item to copy
@@ -3019,7 +3152,7 @@ gboolean e2_fs_error_local (const gchar *format, VPATH *local
 		, reason
 #endif
 		);
-	F_FREE (utf);
+	F_FREE (utf, VPSTR(local));
 	g_free (msg);
 	return FALSE;
 }
@@ -3039,7 +3172,7 @@ gboolean e2_fs_error_simple (const gchar *format, VPATH *local)
 	gdk_threads_enter ();
 	e2_output_print_error (msg, TRUE);
 	gdk_threads_leave ();
-	F_FREE (utf);
+	F_FREE (utf, VPSTR(local));
 	return FALSE;
 }
 
