@@ -1,5 +1,6 @@
-/* $Id: e2_fs_mount.c 1277 2008-09-27 01:10:48Z tpgww $
+/* $Id: e2_fs_mount.c 2309 2011-01-17 00:12:22Z tpgww $
 
+Copyright (C) 2007-2011 tooar <tooar@emelfm2.net>
 Copyright (C) 2004-2007 Florian Zaehringer <flo.zaehringer@web.de>
 Copyright (C) 1999-2004 Bill Wilson <bill@gkrellm.net>
 
@@ -15,8 +16,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 /**
@@ -190,67 +190,22 @@ CHECKME sometimes, test may need to include
 //guint uid;
 //time_t fstab_mtime;
 
-/**
-@brief check whether @a localpath is a filesystem mountpoint
-
-@param localpath localised absolute path string
-
-@return TRUE if @a localpath is a mountpoint
-*/
-gboolean e2_fs_mount_is_mountpoint (VPATH *localpath)
-{
-	struct stat sb;
-#ifdef E2_VFS
-	if (localpath->spacedata != NULL)
-		return FALSE;	//mountpoints are always local
-#endif
-	//root path is not detected by mountpoints lister
-	if (g_str_equal (VPSTR (localpath), G_DIR_SEPARATOR_S))
-		return TRUE;
-
-	if (e2_fs_lstat (localpath, &sb E2_ERR_NONE()) || !S_ISDIR (sb.st_mode))
-		return FALSE;
-
-	gboolean matched = FALSE;
-#ifdef E2_VFS
-	gchar *utf = F_FILENAME_FROM_LOCALE (VPSTR(localpath));
-#else
-	gchar *utf = F_FILENAME_FROM_LOCALE (localpath);
-#endif
-	GList *points = e2_fs_mount_get_mounts_list ();
-	if (points != NULL)
-	{
-		matched = (g_list_find_custom (points, utf, (GCompareFunc) e2_list_strcmp) != NULL);
-		e2_list_free_with_data (&points);
-	}
-#if defined(__linux__) || defined(__FreeBSD__)
-	if (!matched)
-	{
-		points = e2_fs_mount_get_fusemounts_list ();
-		if (points != NULL)
-		{
-			matched = (g_list_find_custom (points, utf, (GCompareFunc) e2_list_strcmp) != NULL);
-			e2_list_free_with_data (&points);
-		}
-	}
-#endif
-	F_FREE (utf);
-	return matched;
-}
 #ifdef E2_HAL
 /**
 @brief find mountpoint dir whose device is @a device
-@param device localised string with device of mountpoint to be found
+@param device localised string with device (i.e. not volume with trailing number) of mountpoint to be found
 
 @return newly allocated string with mountpoint, or NULL upon error or no match
 */
 gchar *e2_fs_mount_get_mountpoint_for_device (const gchar *device)
 {
+	//FIXME this is useless if cached data (in fstab etc) omits device path
 	//FIXME convert this func to check & get any related string(s):
 	//	device, mountpoint, mount options
 	gchar *mountpath = NULL;
 	//CHECKME lock data file while accessing it ?? re-entrant forms of functions ?
 #if defined(__E2BSD__) || defined(__linux__)
+	//FIXME better to use the _last-mounted_ device when >1 share the mountpoint
     struct fstab *fs;
 
 	if (!setfsent ())
@@ -337,192 +292,18 @@ gchar *e2_fs_mount_get_mountpoint_for_device (const gchar *device)
 #endif	//which OS
 	return mountpath;
 }
-#endif
-/**
-@brief add @a dir to list @a mounts
+#endif //def E2_HAL
 
-The string is copied, with conversion to utf-8 if need be
-
-@param dir localised string with path of mountpoint to be added
-@param points store of pointer to the list to be updated
-
-@return
-*/
-static void _e2_fs_mount_add_to_list (gchar *dir, GList **points)
-{
-	gchar *utf = D_FILENAME_FROM_LOCALE (dir);
-	*points = g_list_append (*points, utf);
-}
-/**
-@brief check whether @a localpath is a fuse mountpoint
-
-@param localpath localised absolute path string
-
-@return TRUE if @a localpath is a fuse mountpoint
-*/
-gboolean e2_fs_mount_is_fusepoint (VPATH *localpath)
-{
-#if defined(__linux__) || defined(__FreeBSD__)
-# ifdef E2_VFS
-	if (localpath->spacedata != NULL)
-		return FALSE;	//mountpoints are always local
-# endif
-	gboolean retval;
-#ifdef __linux__
-	struct mntent *mounted;
-	E2_FILE *f;
-/* When the linux proc filesystem is mounted, the files /etc/mtab and /proc/mounts
-	have very similar contents. The former has somewhat more information (CHECKME),
-	such as the mount options used, but is not necessarily up-to-date */
-	if ((f = setmntent ("/proc/mounts", "r")) == NULL
-		&& (f = setmntent ("/etc/mtab", "r")) == NULL)
-			return FALSE;
-
-	retval = FALSE;
-	while ((mounted = getmntent (f)) != NULL)
-	{
-		if (!strcmp (mounted->mnt_type, "fuse")
-			&& g_str_has_prefix (VPCSTR (localpath), mounted->mnt_dir))
-		{
-			retval = TRUE;
-			break;
-		}
-	}
-	endmntent (f);
-#elif defined(__FreeBSD__)
-	retval = FALSE;
-	struct statfs *mounted;
-	//NOTE: data provided by this is not process- or thread-safe
-	gint i, count = getmntinfo (&mounted, MNT_NOWAIT);
-
-	for (i=0; i<count; i++)
-	{
-		if (!strcmp (mounted[i].f_fstypename, "fuse")
-			&& g_str_has_prefix (VPCSTR (localpath), mounted[i].f_mntonname))
-		{
-			retval = TRUE;
-			break;
-		}
-	}
-#endif
-	return retval;
-#else
-	return FALSE;
-#endif
-}
-#if defined(__linux__) || defined(__FreeBSD__)
-/**
-@brief create list of fuse mountpoints
-The list is created by the appropriate os-specifc protocol.
-Mountpoints are stored as utf8 strings
-
-@return list of utf-8 mountpoints, or NULL
-*/
-GList *e2_fs_mount_get_fusemounts_list (void)
-{
-	GList *mounts_list = NULL;
-	//FIXME lock the data source while accessing it
-#if defined(__linux__)
-	struct mntent *mounted;
-	E2_FILE *f;
-/* When the linux proc filesystem is mounted, the files /etc/mtab and /proc/mounts
-	have very similar contents. The former has somewhat more information (CHECKME),
-	such as the mount options used, but is not necessarily up-to-date */
-	if ((f = setmntent ("/proc/mounts", "r")) == NULL)
-		if ((f = setmntent ("/etc/mtab", "r")) == NULL)
-			return NULL;	//FIXME report an error
-
-	while ((mounted = getmntent (f)) != NULL)
-	{
-		if (!strcmp (mounted->mnt_type, "fuse"))
-			_e2_fs_mount_add_to_list (mounted->mnt_dir, &mounts_list);
-	}
-	endmntent (f);
-#elif defined(__FreeBSD__)
-	struct statfs *mounted;
-
-	//NOTE: data provided by this is not process- or thread-safe
-	gint i, count = getmntinfo (&mounted, MNT_NOWAIT);
-
-	for (i=0; i<count; i++)
-	{
-		if (!strcmp (mounted[i].f_fstypename, "fuse"))
-			_e2_fs_mount_add_to_list (mounted[i].f_mntonname, &mounts_list);
-	}
-#endif
-	return mounts_list;
-}
-#endif //which OS
-/* *
-@brief replace escaped character codes in @a buf
-@param buf buffer with string from fs/mtab, presumably locale-encoded
-@return
-*/
-/* UNUSED
-static void _e2_fs_mount_fix_fstab_name (gchar *buf)
-{
-	gchar *rp, *wp;
-
-	if (buf[0] == '\0')
-		return;
-	rp = buf;
-	wp = buf;
-	do	// This loop same as in libc6 getmntent()
-		if (rp[0] == '\\' && rp[1] == '0' && rp[2] == '4' && rp[3] == '0')
-		{
-			*wp++ = ' ';		// \040 is a SPACE
-			rp += 3;
-		}
-		else if (rp[0] == '\\' && rp[1] == '0' && rp[2] == '1' && rp[3] == '2')
-		{
-			*wp++ = '\t';		// \012 is a TAB
-			rp += 3;
-		}
-		else if (rp[0] == '\\' && rp[1] == '\\')
-		{
-			*wp++ = '\\';		// \\ is a \
-			rp += 1;
-		}
-		else
-			*wp++ = *rp;
-	while (*rp++ != '\0');
-} */
-/* *
-@brief check whether current user is permitted to [u]mount mountpoint @a device
-@param device mountpoint path-string, presumably locale-encoded
-@param options is a string with the (4th) parameters string from a line in fs/mtab
-@return TRUE if permitted
-*/
-/* UNUSED FIXME, if this is used, needs to account for sudo & group membership etc
- and work properly for all os's
-static gboolean _e2_fs_mount_permission (gchar* device, gchar *options)
-{
-	if (uid == 0 	//root can always do it
-		|| strstr (options, "dev,") != NULL //CHECKME _I(
-		|| (strstr (options, "user") != NULL && strstr (options, "nouser") == NULL)) //CHECKME _I(
-		return TRUE;
-#if defined (__linux__)
-	struct stat my_stat;
-	//no need to localise *device before stat
-	return (e2_fs_stat (device, &my_stat E2_ERR_NONE()) == 0
-		&& my_stat.st_uid == uid
-		&& strstr (options, "owner") != NULL //CHECKME _I(
-	   );
-#else
-	return FALSE;
-#endif
-} */
-
-#ifdef E2_HAL
+#if defined (E2_HAL) || defined (E2_DEVKIT)
 /**
 @brief get device corresponding to mounted directory @a utfpath
-@param absolute path, utf8, may have trailing separator
+@param absolute path, UTF-8, may have trailing separator
 This gets the device for the mounted point with the longest match between
 the point's path and @a utfpath. Only works properly when @a utfpath is mounted.
 Simple string comparisons are performed. @a utfpath may be quoted.
 @return newly-allocated string, or NULL
 */
-gchar *_e2_fs_get_mounted_device (const gchar *utfpath)
+static gchar *_e2_fs_mount_get_device_for_mountpoint (const gchar *utfpath)
 {
 	gint local_length, current_length, longest_sofar;
 	gchar *clean, *local, *thisdev, *bestdev;
@@ -792,10 +573,228 @@ gchar *_e2_fs_get_mounted_device (const gchar *utfpath)
 #endif	//which OS
 //=========================
 	g_free (clean);
-	F_FREE (local);
+	F_FREE (local, clean);
 	return bestdev;
 }
-#endif	//def E2_HAL
+#endif	//def (E2_HAL) || def (E2_DEVKIT)
+/**
+@brief check whether @a localpath is a filesystem mountpoint
+
+@param localpath localised absolute path string
+
+@return TRUE if @a localpath is a mountpoint
+*/
+gboolean e2_fs_mount_is_mountpoint (VPATH *localpath)
+{
+	struct stat sb;
+#ifdef E2_VFS
+	if (localpath->spacedata != NULL)
+		return FALSE;	//mountpoints are always local
+#endif
+	//root path is not detected by mountpoints lister
+	if (!strcmp (VPSTR (localpath), G_DIR_SEPARATOR_S))
+		return TRUE;
+
+	if (e2_fs_lstat (localpath, &sb E2_ERR_NONE()) || !S_ISDIR (sb.st_mode))
+		return FALSE;
+
+	gboolean matched = FALSE;
+	gchar *utf = F_FILENAME_FROM_LOCALE (VPSTR(localpath));
+	GList *points = e2_fs_mount_get_mounts_list ();
+	if (points != NULL)
+	{
+		matched = (g_list_find_custom (points, utf, (GCompareFunc) e2_list_strcmp) != NULL);
+		e2_list_free_with_data (&points);
+	}
+#if defined(__linux__) || defined(__FreeBSD__)
+	if (!matched)
+	{
+		points = e2_fs_mount_get_fusemounts_list ();
+		if (points != NULL)
+		{
+			matched = (g_list_find_custom (points, utf, (GCompareFunc) e2_list_strcmp) != NULL);
+			e2_list_free_with_data (&points);
+		}
+	}
+#endif
+	F_FREE (utf, VPSTR(localpath));
+	return matched;
+}
+/**
+@brief add @a dir to list @a mounts
+
+The string is copied, with conversion to utf-8 if need be
+
+@param dir localised string with path of mountpoint to be added
+@param points store of pointer to the list to be updated
+
+@return
+*/
+static void _e2_fs_mount_add_to_list (gchar *dir, GList **points)
+{
+	gchar *utf = D_FILENAME_FROM_LOCALE (dir);
+	*points = g_list_append (*points, utf);
+}
+/**
+@brief check whether @a localpath is a fuse mountpoint
+
+@param localpath localised absolute path string
+
+@return TRUE if @a localpath is a fuse mountpoint
+*/
+gboolean e2_fs_mount_is_fusepoint (VPATH *localpath)
+{
+#if defined(__linux__) || defined(__FreeBSD__)
+# ifdef E2_VFS
+	if (localpath->spacedata != NULL)
+		return FALSE;	//mountpoints are always local
+# endif
+	gboolean retval;
+#ifdef __linux__
+	struct mntent *mounted;
+	E2_FILE *f;
+/* When the linux proc filesystem is mounted, the files /etc/mtab and /proc/mounts
+	have very similar contents. The former has somewhat more information (CHECKME),
+	such as the mount options used, but is not necessarily up-to-date */
+	if ((f = setmntent ("/proc/mounts", "r")) == NULL
+		&& (f = setmntent ("/etc/mtab", "r")) == NULL)
+			return FALSE;
+
+	retval = FALSE;
+	while ((mounted = getmntent (f)) != NULL)
+	{
+		if (!strcmp (mounted->mnt_type, "fuse")
+			&& g_str_has_prefix (VPCSTR (localpath), mounted->mnt_dir))
+		{
+			retval = TRUE;
+			break;
+		}
+	}
+	endmntent (f);
+#elif defined(__FreeBSD__)
+	retval = FALSE;
+	struct statfs *mounted;
+	//NOTE: data provided by this is not process- or thread-safe
+	gint i, count = getmntinfo (&mounted, MNT_NOWAIT);
+
+	for (i=0; i<count; i++)
+	{
+		if (!strcmp (mounted[i].f_fstypename, "fuse")
+			&& g_str_has_prefix (VPCSTR (localpath), mounted[i].f_mntonname))
+		{
+			retval = TRUE;
+			break;
+		}
+	}
+#endif
+	return retval;
+#else
+	return FALSE;
+#endif
+}
+#if defined(__linux__) || defined(__FreeBSD__)
+/**
+@brief create list of fuse mountpoints
+The list is created by the appropriate os-specifc protocol.
+Mountpoints are stored as utf8 strings
+
+@return list of utf-8 mountpoints, or NULL
+*/
+GList *e2_fs_mount_get_fusemounts_list (void)
+{
+	GList *mounts_list = NULL;
+	//FIXME lock the data source while accessing it
+#if defined(__linux__)
+	struct mntent *mounted;
+	E2_FILE *f;
+/* When the linux proc filesystem is mounted, the files /etc/mtab and /proc/mounts
+	have very similar contents. The former has somewhat more information (CHECKME),
+	such as the mount options used, but is not necessarily up-to-date */
+	if ((f = setmntent ("/proc/mounts", "r")) == NULL)
+		if ((f = setmntent ("/etc/mtab", "r")) == NULL)
+			return NULL;	//FIXME report an error
+
+	while ((mounted = getmntent (f)) != NULL)
+	{
+		if (!strcmp (mounted->mnt_type, "fuse"))
+			_e2_fs_mount_add_to_list (mounted->mnt_dir, &mounts_list);
+	}
+	endmntent (f);
+#elif defined(__FreeBSD__)
+	struct statfs *mounted;
+
+	//NOTE: data provided by this is not process- or thread-safe
+	gint i, count = getmntinfo (&mounted, MNT_NOWAIT);
+
+	for (i=0; i<count; i++)
+	{
+		if (!strcmp (mounted[i].f_fstypename, "fuse"))
+			_e2_fs_mount_add_to_list (mounted[i].f_mntonname, &mounts_list);
+	}
+#endif
+	return mounts_list;
+}
+#endif //which OS
+/* *
+@brief replace escaped character codes in @a buf
+@param buf buffer with string from fs/mtab, presumably locale-encoded
+@return
+*/
+/* UNUSED
+static void _e2_fs_mount_fix_fstab_name (gchar *buf)
+{
+	gchar *rp, *wp;
+
+	if (buf[0] == '\0')
+		return;
+	rp = buf;
+	wp = buf;
+	do	// This loop same as in libc6 getmntent()
+		if (rp[0] == '\\' && rp[1] == '0' && rp[2] == '4' && rp[3] == '0')
+		{
+			*wp++ = ' ';		// \040 is a SPACE
+			rp += 3;
+		}
+		else if (rp[0] == '\\' && rp[1] == '0' && rp[2] == '1' && rp[3] == '2')
+		{
+			*wp++ = '\t';		// \012 is a TAB
+			rp += 3;
+		}
+		else if (rp[0] == '\\' && rp[1] == '\\')
+		{
+			*wp++ = '\\';		// \\ is a \
+			rp += 1;
+		}
+		else
+			*wp++ = *rp;
+	while (*rp++ != '\0');
+} */
+/* *
+@brief check whether current user is permitted to [u]mount mountpoint @a device
+@param device mountpoint path-string, presumably locale-encoded
+@param options is a string with the (4th) parameters string from a line in fs/mtab
+@return TRUE if permitted
+*/
+/* UNUSED FIXME, if this is used, needs to account for sudo & group membership etc
+ and work properly for all os's
+static gboolean _e2_fs_mount_permission (gchar* device, gchar *options)
+{
+	if (uid == 0 	//root can always do it
+		|| strstr (options, "dev,") != NULL //CHECKME _I(
+		|| (strstr (options, "user") != NULL && strstr (options, "nouser") == NULL)) //CHECKME _I(
+		return TRUE;
+#if defined (__linux__)
+	struct stat my_stat;
+	//no need to localise *device before stat
+	return (e2_fs_stat (device, &my_stat E2_ERR_NONE()) == 0
+		&& my_stat.st_uid == uid
+		&& strstr (options, "owner") != NULL //CHECKME _I(
+	   );
+#else
+	return FALSE;
+#endif
+} */
+
 /**
 @brief create list of mounted partitions (other than "/") in the native filesystem
 
@@ -944,7 +943,7 @@ GList *e2_fs_mount_get_mounts_list (void)	//gboolean permcheck)
 @brief create list of mountable partitions (other than "/") in the native filesystem
 
 The list is created by the appropriate os-specifc protocol.
-Root dir is exlcluded because we don't need to ever unmount that.
+Root dir is exlcluded because we don't ever need to mount that (? some other device chroot?)
 
 @return list of partitions (utf-8 strings), or NULL
 */
@@ -1090,7 +1089,7 @@ gchar *e2_fs_mount_get_enclosing_point (const gchar *localpath)
 				}
 			}
 		}
-		F_FREE (converted);
+		F_FREE (converted, localpath);
 		if (thispoint != NULL)
 			converted = D_FILENAME_TO_LOCALE (thispoint);
 		else
@@ -1100,9 +1099,9 @@ gchar *e2_fs_mount_get_enclosing_point (const gchar *localpath)
 	}
 	return g_strdup (G_DIR_SEPARATOR_S);	//NULL;
 }
-#ifdef E2_HAL
+#if defined (E2_HAL) || defined (E2_DEVKIT)
 /**
-@brief check whether device on which @a utfpath resides is removable
+@brief check whether device on which @a utfpath resides is ejectable
 This is intended only for native dirs, so no VPATH involved
 It analyses mounted-devices data, so can only be used when @a utfpath is mounted
 @param utfpath absolute path string, utf8, may be quoted, and/or have spaces
@@ -1112,17 +1111,22 @@ and/or trailing separator
 gboolean e2_fs_mount_is_ejectable (const gchar *utfpath)
 {
 	gboolean retval;
-	gchar *device = _e2_fs_get_mounted_device (utfpath);
+	gchar *device = _e2_fs_mount_get_device_for_mountpoint (utfpath);
 	if (device != NULL)
 	{
-		retval = e2_hal_device_is_ejectable (device);
+		retval =
+# ifdef E2_HAL
+		e2_hal_device_is_ejectable (device);
+# else
+		e2_devkit_device_is_ejectable (device);
+# endif
 		g_free (device);
 	}
 	else
 		retval = FALSE;
 	return retval;
 }
-#endif
+#endif //defined (E2_HAL) || defined (E2_DEVKIT)
 /**
 @brief check whether device on which @a utfpath resides is removable
 @param utfpath absolute path string, utf8, may be quoted, and/or have spaces
@@ -1134,11 +1138,16 @@ It analyses mounted-devices data, so can only be used when @a utfpath is mounted
 gboolean e2_fs_mount_is_removable (const gchar *utfpath)
 {
 	gboolean retval;
-#ifdef E2_HAL
-	gchar *device = _e2_fs_get_mounted_device (utfpath);
+#if defined (E2_HAL) || defined (E2_DEVKIT)
+	gchar *device = _e2_fs_mount_get_device_for_mountpoint (utfpath);
 	if (device != NULL)
 	{
-		retval = e2_hal_device_is_removable (device);
+		retval =
+# ifdef E2_HAL
+		e2_hal_device_is_removable (device);
+# else
+		e2_devkit_device_is_removable (device);
+# endif
 		g_free (device);
 	}
 	else
@@ -1187,7 +1196,7 @@ gboolean e2_fs_mount_is_cased (ViewInfo *view)
 		else
 		{
 #endif
-			gchar *local = F_FILENAME_TO_LOCALE (view->dir);
+			gchar *local = D_FILENAME_TO_LOCALE (view->dir); //always dup, to avoid dirchange race
 //=========== OS-specific stuff =============
 #if defined(__linux__)
 #include <sys/vfs.h>
@@ -1213,7 +1222,7 @@ retry:
 						break;
 				}
 			}
-#ifdef E2_HAL
+#if defined (E2_HAL) || defined (E2_DEVKIT)
 			if (e2_fs_mount_is_removable (view->dir)) //removable device
 #endif
 				view->case_sensitive_names |= E2FSCASE_NOCACHE;
@@ -1234,7 +1243,7 @@ retry:
 				else
 					view->case_sensitive_names = E2FSCASE_SENSITIVE;
 			}
-#ifdef E2_HAL
+#if defined (E2_HAL) || defined (E2_DEVKIT)
 			if (e2_fs_mount_is_removable (view->dir)) //on removable device
 #endif
 				view->case_sensitive_names |= E2FSCASE_NOCACHE;
@@ -1255,7 +1264,7 @@ retry:
 				else
 					view->case_sensitive_names = E2FSCASE_SENSITIVE;
 			}
-#ifdef E2_HAL
+#if defined (E2_HAL) || defined (E2_DEVKIT)
 			if (e2_fs_mount_is_removable (view->dir)) //on removable device
 #endif
 				view->case_sensitive_names |= E2FSCASE_NOCACHE;
@@ -1276,13 +1285,13 @@ retry:
 				else
 					view->case_sensitive_names = E2FSCASE_SENSITIVE;
 			}
-#ifdef E2_HAL
+#if defined (E2_HAL) || defined (E2_DEVKIT)
 			if (e2_fs_mount_is_removable (view->dir)) //on removable place
 #endif
 				view->case_sensitive_names |= E2FSCASE_NOCACHE;
 #endif	//which OS
 //=========================
-			F_FREE (local);
+			g_free (local);
 #ifdef E2_VFSTMP
 		}
 #endif
@@ -1297,10 +1306,8 @@ retry:
 */
 void e2_fs_mount_actions_register (void)
 {
-	gchar *action = g_strconcat (_A(61),".",_A(27), NULL);
-	e2_action_register (action, E2_ACTION_TYPE_ITEM,
-		e2_menu_create_mounts_menu, NULL, TRUE,
-		E2_ACTION_EXCLUDE_MENU,
-		NULL);
+	E2_Action action =
+	{g_strconcat(_A(61),".",_A(27),NULL),e2_menu_create_mounts_menu,TRUE,E2_ACTION_TYPE_ITEM,E2_ACTION_EXCLUDE_MENU,NULL,NULL};
+	e2_action_register (&action);
 }
 #endif //def E2_FS_MOUNTABLE
