@@ -1,6 +1,6 @@
-/* $Id: e2_hal.c 1427 2009-02-10 22:16:11Z tpgww $
+/* $Id: e2_hal.c 2309 2011-01-17 00:12:22Z tpgww $
 
-Copyright (C) 2008-2009 tooar <tooar@gmx.net>
+Copyright (C) 2008-2011 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -14,8 +14,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 /**
@@ -357,7 +356,7 @@ static gboolean _e2_hal_mount_volume (LibHalVolume *vol, const gchar *realdevice
 		g_free (str);
 		str = g_strdup_printf ("%s%d", mountpoint, count++);
 #ifdef E2_VFS
-		ddata.localpath = str;
+		ddata.path = str;
 #endif
 	}
 	g_free (mountpoint);
@@ -556,7 +555,7 @@ badexit:
 			do something
 		if (g_str_has_prefix (other_view->dir, utf))
 			do something
-		F_FREE (utf);
+		F_FREE (utf, mountpoint);
 	}
 */
 	//cleanup
@@ -824,6 +823,20 @@ static void _e2_hal_add_device_cb (LibHalContext *ctx, const gchar *udi)
 		dbus_error_free (&error);
 }
 /**
+@brief change displayed directory if necessary, after an unmount
+BGL may be open or closed
+@param rt pointer to pane data, curr_pane or other_pane
+@return
+*/
+static void _e2_hal_checkdir (E2_PaneRuntime *rt)
+{
+	gchar *elsewhere = g_strdup (rt->path);
+	g_strchug (elsewhere); //just in case ...
+	if (!e2_fs_get_valid_path (&elsewhere, TRUE E2_ERR_NONE()))
+		e2_pane_change_dir (rt, elsewhere);
+	g_free (elsewhere);
+}
+/**
 @brief timer callback after a device-remove event
 If the device is not unmounted yet (by the OS etc), unmount it now
 Also, always cleanup the mount data
@@ -833,6 +846,8 @@ Also, always cleanup the mount data
 */
 static gboolean _e2_hal_check_unmount (HalStorageData *devicedata)
 {
+	devicedata->timer_id = 0;	//signal to others
+
 	printd (DEBUG, "HAL: unmount timer callback");
 //	printd (DEBUG, "HAL: device %s, udi %s", devicedata->device_path, devicedata->udi);
 
@@ -878,7 +893,9 @@ static gboolean _e2_hal_check_unmount (HalStorageData *devicedata)
 		}
 		libhal_drive_free (drv);
 	}
-
+	//ensure neither filelist shows the unmounted device
+	_e2_hal_checkdir (curr_pane);
+	_e2_hal_checkdir (other_pane);
 	//CHECME mutex for processing this list ?
 	removable_devices = g_list_remove (removable_devices, devicedata);
 	_e2_hal_destroy_data (devicedata);
@@ -959,20 +976,21 @@ static void _e2_hal_remove_device_cb (LibHalContext *ctx, const gchar *udi)
 					dbus_error_init (&error);
 					if (!libhal_device_remove_property_watch (ctx, udi, &error))
 					{
-//						_e2_hal_advise (_("HAL: cannot set watch on device"), &error);
-						printd (WARN, "HAL: cannot set watch on device - %s", error.message);
+//						_e2_hal_advise (_("HAL: cannot remove watch from device"), &error);
+						printd (WARN, "HAL: cannot remove watch from device - %s", error.message);
 						dbus_error_free (&error);
 					}
 					else
-						printd (DEBUG, "HAL: Watch added to device udi %s", udi);
+						printd (DEBUG, "HAL: removed watch from device udi %s", udi);
 				}
 				//wait awhile (longer than the mount-timer interval) to check for unmount and then cleanup
+				store->timer_id =
 #ifdef USE_GLIB2_14
-				g_timeout_add_seconds (10,
+					g_timeout_add_seconds (10,
 #else
-				g_timeout_add (10000,
+					g_timeout_add (10000,
 #endif
-					(GSourceFunc) _e2_hal_check_unmount, store);
+						(GSourceFunc) _e2_hal_check_unmount, store);
 
 				break;
 			}
