@@ -1,6 +1,6 @@
-/* $Id: e2_button.c 1141 2008-08-28 07:54:14Z tpgww $
+/* $Id: e2_button.c 2064 2010-03-12 13:15:36Z tpgww $
 
-Copyright (C) 2004-2008 tooar <tooar@gmx.net>
+Copyright (C) 2004-2010 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -14,8 +14,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 #include "emelfm2.h"
@@ -145,12 +144,7 @@ void e2_button_set_indexed_text (GtkWidget *button, E2_ButtonText type)
 		e2_button_set_label (button, defined_button_labels[type]);
 		if (defined_button_tips[type] != NULL)
 		{
-#ifdef USE_GTK2_12TIPS
-			gtk_widget_set_tooltip_text (
-#else
-			e2_widget_set_tooltip (NULL,
-#endif
-			button, defined_button_tips[type]);
+			e2_widget_set_safetip (button, defined_button_tips[type]);
 		}
 		else
 		{
@@ -184,19 +178,32 @@ void e2_button_set_label (GtkWidget *button, const gchar *label)
 {
 	if (label != NULL && *label != '\0')
 	{
-		GtkWidget *bbox = GTK_BIN (GTK_BIN (button)->child)->child;
+		GtkWidget *bbox =
+#ifdef USE_GTK2_14
+			gtk_bin_get_child (GTK_BIN (button));
+		bbox = gtk_bin_get_child (GTK_BIN (bbox));
+		GList *children = gtk_container_get_children (GTK_CONTAINER (bbox));
+		GtkWidget *child = (GtkWidget*) children->data;
+		if (!GTK_IS_LABEL (child))
+			child = (GtkWidget*) children->next->data;
+		gtk_label_set_markup_with_mnemonic (GTK_LABEL (child), label);
+		g_list_free (children);
+#else
+			GTK_BIN (GTK_BIN (button)->child)->child;
 		GtkBoxChild *child = (GtkBoxChild *) GTK_BOX (bbox)->children->data;
-		if (GTK_WIDGET_TYPE (child->widget) != g_type_from_name ("GtkLabel"))
+		if (!GTK_IS_LABEL (child->widget))
 			child = (GtkBoxChild *) GTK_BOX (bbox)->children->next->data;
 		gtk_label_set_markup_with_mnemonic (GTK_LABEL (child->widget), label);
+#endif
 	}
 }
 /**
 @brief set new (or no) icon for existing button @a button
 
-e2 uses non-standard dialog buttons, containing alignment
-containing h/vbox, the latter with image (if any) packed 1st
-@a stock may be a gtk stock name or a custom icon filename
+e2 uses non-standard dialog buttons, containing alignment containing h/vbox,
+the latter with image (if any) packed first. @a stock may be a gtk stock name
+or a custom icon filename. The new image is shown immediately, so this func is
+not intended for buttons which are ultimately included in a "show all".
 
 @param button the button widget to be updated
 @param stock string describing icon or NULL to clear the icon
@@ -205,16 +212,38 @@ containing h/vbox, the latter with image (if any) packed 1st
 */
 void e2_button_set_image (GtkWidget *button, gchar *stock)
 {
-	if (e2_option_bool_get ("dialog-button-icons"))
+	gint choice = e2_option_sel_get ("dialog-button-icons");
+	if (choice == 0)
+	{
+#ifdef USE_GTK2_18
+		gboolean show;
+		GtkSettings* defs = gtk_settings_get_default ();
+		g_object_get (G_OBJECT (defs), "gtk-button-images", &show, NULL);
+		if (show)
+#endif
+			choice = 1;
+	}
+	if (choice == 1)
 	{
 		GtkWidget *image = (stock != NULL) ?
 			e2_widget_get_icon (stock, GTK_ICON_SIZE_BUTTON) : NULL;
 		if (stock == NULL || image != NULL)
 		{
-			GtkWidget *bbox = GTK_BIN (GTK_BIN (button)->child)->child;
+			GtkWidget *bbox =
+#ifdef USE_GTK2_14
+				gtk_bin_get_child (GTK_BIN (button));
+			bbox = gtk_bin_get_child (GTK_BIN (bbox));
+			GList *children = gtk_container_get_children (GTK_CONTAINER (bbox));
+			GtkWidget *child1 = (GtkWidget *) children->data;
+			if (GTK_IS_IMAGE (child1))
+				gtk_container_remove (GTK_CONTAINER (bbox), child1);
+			g_list_free (children);
+#else
+				GTK_BIN (GTK_BIN (button)->child)->child;
 			GtkBoxChild *child1 = (GtkBoxChild *) GTK_BOX (bbox)->children->data;
-			if (GTK_WIDGET_TYPE (child1->widget) == g_type_from_name ("GtkImage"))
+			if (GTK_IS_IMAGE (child1->widget))
 				gtk_container_remove (GTK_CONTAINER (bbox), child1->widget);
+#endif
 			if (image != NULL)
 			{
 				gtk_box_pack_start (GTK_BOX (bbox), image, FALSE, FALSE, 0);
@@ -225,25 +254,6 @@ void e2_button_set_image (GtkWidget *button, gchar *stock)
 	}
 }
 /**
-@brief get button-widget which can be default and focused
-
-@param label button label which may contain markup and/or mnemonic
-@param stock string describing icon, or NULL
-@param tip tooltip for the button, or NULL
-@param callback callback function for button's "clicked" signal, or NULL
-@param data data specified for the callback
-
-@return the button widget
-*/
-GtkWidget *e2_button_get (const gchar *label, const gchar *stock, const gchar *tip,
-	gpointer callback, gpointer data)
-{
-	GtkWidget *button = e2_button_get_full (label, stock, GTK_ICON_SIZE_BUTTON,
-		tip, callback, data, E2_BUTTON_CAN_DEFAULT | E2_BUTTON_CAN_FOCUS);
-	gtk_widget_show_all (button);
-	return button;
-}
-/**
 @brief create button
 This is used for toolbar buttons, and (maybe indirectly) for dialog buttons
 @param label button label which may contain markup and/or mnemonic, or NULL
@@ -254,12 +264,17 @@ This is used for toolbar buttons, and (maybe indirectly) for dialog buttons
 @param data data specified for the callback
 @param flags flags indicating button properties
 
-@return the button widget
+@return the (shown) button widget
 */
 GtkWidget *e2_button_get_full (const gchar *label, const gchar *stock, GtkIconSize size,
-	const gchar *tip, gpointer callback, gpointer data, E2_ButtonFlags flags)
+	const gchar *tip, void (*callback)(/*GtkButton*,gpointer*/), gpointer data,
+	E2_ButtonFlags flags)
 {
 	GtkWidget *button = gtk_button_new ();
+#ifdef USE_GTK2_18
+	gtk_widget_set_can_default (button, (flags & E2_BUTTON_CAN_DEFAULT) > 0);
+	gtk_widget_set_can_focus (button, (flags & E2_BUTTON_CAN_FOCUS) > 0);
+#else
 	if (flags & E2_BUTTON_CAN_DEFAULT)
 		GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
 	else
@@ -268,15 +283,10 @@ GtkWidget *e2_button_get_full (const gchar *label, const gchar *stock, GtkIconSi
 		GTK_WIDGET_SET_FLAGS (button, GTK_CAN_FOCUS);
 	else
 		GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
+#endif
 
 	if (tip != NULL)
-#ifdef USE_GTK2_12TIPS
-	gtk_widget_set_tooltip_text (
-#else
-	e2_widget_set_tooltip (NULL,
-#endif
-		button, tip);
-
+		e2_widget_set_safetip (button, tip);
 	if (callback != NULL)
 		g_signal_connect_data (G_OBJECT (button), "clicked",
 			G_CALLBACK (callback), data,
@@ -284,7 +294,7 @@ GtkWidget *e2_button_get_full (const gchar *label, const gchar *stock, GtkIconSi
 
 	GtkWidget *bbox;
 	if (flags & E2_BUTTON_ICON_ABOVE_LABEL)
-		bbox = gtk_vbox_new (FALSE, 2);	//fixed gap, not E2_PADDING_XSMALL
+		bbox = gtk_vbox_new (FALSE, 2);	//constant gap, not E2_PADDING_XSMALL
 	else
 		bbox = gtk_hbox_new (FALSE, 2);
 
@@ -293,7 +303,7 @@ GtkWidget *e2_button_get_full (const gchar *label, const gchar *stock, GtkIconSi
 	if (stock != NULL)
 	{
 		GtkWidget *img = e2_widget_get_icon (stock, size);
-		if ((flags & E2_BUTTON_SHOW_MISSING_ICON) && (img == NULL))
+		if (img == NULL && (flags & E2_BUTTON_SHOW_MISSING_ICON))
 			img = e2_widget_get_icon (GTK_STOCK_MISSING_IMAGE, size);
 		if (img != NULL)
 		{
@@ -308,6 +318,8 @@ GtkWidget *e2_button_get_full (const gchar *label, const gchar *stock, GtkIconSi
 		GtkWidget *lab = gtk_label_new ("");
 		gtk_label_set_markup_with_mnemonic (GTK_LABEL (lab), label);
 		gtk_box_pack_start (GTK_BOX (bbox), lab, TRUE, TRUE, 0);
+		if (stock != NULL)
+			gtk_widget_add_mnemonic_label (button, lab); //needed to make it work dunno why
 		need_align = TRUE;
 	}
 
@@ -319,10 +331,26 @@ GtkWidget *e2_button_get_full (const gchar *label, const gchar *stock, GtkIconSi
 	}
 	else	//should never happen
 		gtk_container_add (GTK_CONTAINER (button), bbox);
-
 	return button;
 }
 /**
+@brief get button-widget which can be default and focused
+Downstream code requires BGL closed
+@param label button label which may contain markup and/or mnemonic
+@param stock string describing icon, or NULL
+@param tip tooltip for the button, or NULL
+@param callback callback function for button's "clicked" signal, or NULL
+@param data data to send to the callback
+
+@return the (shown) button widget
+*/
+GtkWidget *e2_button_get (const gchar *label, const gchar *stock, const gchar *tip,
+	void (*callback)(/*GtkButton*,gpointer*/), gpointer data)
+{
+	return (e2_button_get_full (label, stock, GTK_ICON_SIZE_BUTTON,
+		tip, callback, data, E2_BUTTON_CAN_DEFAULT | E2_BUTTON_CAN_FOCUS));
+}
+/**
 @brief add button-sized button to the start of @a box
 
 @param box widget to hold the button
@@ -337,11 +365,10 @@ GtkWidget *e2_button_get_full (const gchar *label, const gchar *stock, GtkIconSi
 @return the button widget
 */
 GtkWidget *e2_button_add (GtkWidget *box, gboolean exp, guint pad, gchar *label,
-	gchar *stock, gchar *tip, gpointer callback, gpointer data)
+	gchar *stock, gchar *tip, void (*callback)(/*GtkButton*,gpointer*/), gpointer data)
 {
 	GtkWidget *button = e2_button_get (label, stock, tip, callback, data);
 	gtk_box_pack_start (GTK_BOX (box), button, exp, TRUE, pad);
-	gtk_widget_show_all (button);
 	return button;
 }
 /**
@@ -358,12 +385,11 @@ GtkWidget *e2_button_add (GtkWidget *box, gboolean exp, guint pad, gchar *label,
 
 @return the button widget
 */
-GtkWidget *e2_button_add_end (GtkWidget *box, gboolean exp, guint pad,
-	gchar *label, gchar *stock, gchar *tip, gpointer callback, gpointer data)
+GtkWidget *e2_button_add_end (GtkWidget *box, gboolean exp, guint pad, gchar *label,
+	gchar *stock, gchar *tip, void (*callback)(/*GtkButton*,gpointer*/), gpointer data)
 {
 	GtkWidget *button = e2_button_get (label, stock, tip, callback, data);
 	gtk_box_pack_end (GTK_BOX (box), button, exp, TRUE, pad);
-	gtk_widget_show_all (button);
 	return button;
 }
 /**
@@ -374,33 +400,31 @@ GtkWidget *e2_button_add_end (GtkWidget *box, gboolean exp, guint pad,
 @param label button label which may contain markup and/or mnemonic
 @param tip tooltip for the button, or NULL
 @param callback callback function for "toggled" signal, or NULL
-@param data data specified for the callback
+@param data data specified for @a callback
 
 @return the button widget
 */
-GtkWidget *e2_button_get_toggle (gboolean check, gboolean state,
-	gchar *label, gchar *tip, gpointer callback, gpointer data)
+GtkWidget *e2_button_get_toggle (gboolean check, gboolean state, gchar *label,
+	gchar *tip, void (*callback)(GtkToggleButton*,gpointer), gpointer data)
 {
 	GtkWidget *button;
 	if (check)
 		button = gtk_check_button_new_with_mnemonic (label);
 	else
 		button = gtk_toggle_button_new_with_mnemonic (label);
+#ifdef USE_GTK2_18
+	gtk_widget_set_can_default (button, TRUE);
+	gtk_widget_set_can_focus (button, TRUE);
+#else
 	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
 	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_FOCUS);
+#endif
 	if (state)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
 	if (tip != NULL)
-#ifdef USE_GTK2_12TIPS
-	gtk_widget_set_tooltip_text (
-#else
-	e2_widget_set_tooltip (NULL,
-#endif
-		button, tip);
+		e2_widget_set_safetip (button, tip);
 	if (callback != NULL)
 		g_signal_connect (G_OBJECT (button), "toggled", G_CALLBACK (callback), data);
-	gtk_widget_show (button);
-
 	return button;
 }
 /**
@@ -419,7 +443,8 @@ GtkWidget *e2_button_get_toggle (gboolean check, gboolean state,
 @return the button widget
 */
 GtkWidget *e2_button_add_toggle (GtkWidget *box, gboolean check, gboolean state,
-	gchar *label, gchar *tip, gboolean exp, guint pad, gpointer callback, gpointer data)
+	gchar *label, gchar *tip, gboolean exp, guint pad,
+	void (*callback)(GtkToggleButton*,gpointer), gpointer data)
 {
 	GtkWidget *button = e2_button_get_toggle (check, state, label, tip, callback, data);
 	gtk_box_pack_start (GTK_BOX (box), button, exp, TRUE, pad);
@@ -440,7 +465,8 @@ GtkWidget *e2_button_add_toggle (GtkWidget *box, gboolean check, gboolean state,
 @return the button widget
 */
 GtkWidget *e2_button_add_radio (GtkWidget *box, gchar *label, GSList *group,
-	gboolean state, gboolean exp, guint pad, gpointer callback, gpointer data)
+	gboolean state, gboolean exp, guint pad,
+	void (*callback)(GtkToggleButton*,gpointer), gpointer data)
 {
 	GtkWidget *radio_button = (label != NULL) ?
 		gtk_radio_button_new_with_mnemonic (group, label):
@@ -451,8 +477,6 @@ GtkWidget *e2_button_add_radio (GtkWidget *box, gchar *label, GSList *group,
 		g_signal_connect (G_OBJECT (radio_button), "toggled",
 			G_CALLBACK (callback), data);
 	gtk_box_pack_start (GTK_BOX (box), radio_button, exp, TRUE, pad);
-	gtk_widget_show (radio_button);
-
 	return radio_button;
 }
 /**
@@ -472,7 +496,7 @@ GtkWidget *e2_button_add_radio (GtkWidget *box, gchar *label, GSList *group,
 GtkWidget *e2_button_add_to_table (
 	GtkWidget *table,
 	gchar *label,
-	gpointer callback,
+	void (*callback)(GtkButton*,gpointer),
 	gpointer data,
 	gint left, gint right, gint top, gint bottom)
 {
@@ -482,8 +506,6 @@ GtkWidget *e2_button_add_to_table (
 		g_signal_connect (G_OBJECT (button), "clicked",
 			G_CALLBACK (callback), data);
 	gtk_table_attach_defaults (GTK_TABLE (table), button, left, right, top, bottom);
-	gtk_widget_show (button);
-
 	return button;
 }
 /**
@@ -505,7 +527,7 @@ GtkWidget *e2_button_add_toggle_to_table (
 	GtkWidget *table,
 	gchar *label,
 	gboolean state,
-	gpointer callback,
+	void (*callback)(/*GtkToggleButton*,gpointer*/),
 	gpointer data,
 	gint left, gint right, gint top, gint bottom)
 {
@@ -519,8 +541,6 @@ GtkWidget *e2_button_add_toggle_to_table (
 			G_CALLBACK (callback), data);
 	gtk_table_attach_defaults (GTK_TABLE (table), check_button,
 		                        left, right, top, bottom);
-	gtk_widget_show (check_button);
-
 	return check_button;
 }
 /**
@@ -530,7 +550,7 @@ GtkWidget *e2_button_add_toggle_to_table (
 @param label button label which may contain mnemonic
 @param group radio group, or NULL to start a group
 @param state the intitial state of the button
-@param callback callback function for "clicked" signal, or NULL
+@param callback callback function for "toggled" signal, or NULL
 @param data data specified for the callback
 @param left
 @param right
@@ -544,7 +564,7 @@ GtkWidget *e2_button_add_radio_to_table (
 	gchar *label,
 	GSList *group,
 	gboolean state,
-	gpointer callback,
+	void (*callback)(/*GtkToggleButton*,gpointer*/),
 	gpointer data,
 	gint left, gint right, gint top, gint bottom)
 {
@@ -556,7 +576,5 @@ GtkWidget *e2_button_add_radio_to_table (
 			G_CALLBACK (callback), data);
 	gtk_table_attach_defaults (GTK_TABLE (table), radio_button,
 		                        left, right, top, bottom);
-	gtk_widget_show (radio_button);
-
 	return radio_button;
 }
