@@ -1,12 +1,11 @@
-/* $Id: e2_combobox.c 2034 2009-12-21 02:37:51Z tpgww $
+/* $Id: e2_combobox.c 2464 2012-03-12 12:07:01Z tpgww $
 
-Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
+Copyright (C) 2004-2012 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
-emelFM2 is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3, or (at your option)
-any later version.
+emelFM2 is free software; you can redistribute it and/or modify it under the
+terms of the GNU General Public License as published by the Free Software
+Foundation; either version 3, or (at your option) any later version.
 
 emelFM2 is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,8 +13,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 /**
@@ -30,7 +28,14 @@ This file contains utility functions for the GtkComboBox widget
 #include "e2_pane.h"
 #include "e2_command_line.h"
 
-static void _e2_combobox_block_changed (GtkWidget *combo)
+/**
+@brief block the issue of "activate" signal during "changed" signal callback on @a combo
+
+@param combo the entry text combo box to be configured
+
+@return
+*/
+void e2_combobox_block_changed (GtkWidget *combo)
 {
 	g_object_set_data (G_OBJECT (combo), "e2_changed_blocked",
 		GINT_TO_POINTER (TRUE));
@@ -71,10 +76,7 @@ static gboolean _e2_combobox_key_press_cb (GtkWidget *combo, GdkEventKey *event,
 	gpointer data)
 {
 	if (!(event->keyval == GDK_Up || event->keyval == GDK_Down))
-	{
-		e2_utils_translate_key_event (event);
 		return FALSE;
-	}
 
 	if (event->keyval == GDK_Down &&
 		(event->state & gtk_accelerator_get_default_mod_mask ()) == GDK_CONTROL_MASK)
@@ -89,7 +91,7 @@ static gboolean _e2_combobox_key_press_cb (GtkWidget *combo, GdkEventKey *event,
 	gint pos = gtk_combo_box_get_active (GTK_COMBO_BOX (combo));
 	if (event->keyval == GDK_Down)
 	{
-		if (pos < _e2_combobox_last_index (GTK_COMBO_BOX (combo)))
+		if (pos < _e2_combobox_last_index ((GtkComboBox*)combo))
 			pos++;	//assumes -1 can go to 0 always
 		else
 			pos = 0;
@@ -99,17 +101,21 @@ static gboolean _e2_combobox_key_press_cb (GtkWidget *combo, GdkEventKey *event,
 		if (pos > 0)
 			pos--;
 		else if (pos == 0 || pos == -1)
-			pos = _e2_combobox_last_index (GTK_COMBO_BOX (combo));
+			pos = _e2_combobox_last_index ((GtkComboBox*)combo);
 	}
 	e2_combobox_set_active (combo, pos);
+#ifdef USE_GTK2_24
+	if (gtk_combo_box_get_has_entry ((GtkComboBox*)combo))
+#else
 	if (GTK_IS_COMBO_BOX_ENTRY (combo))
+#endif
 	{
 		GtkWidget *entry =
-#ifdef USE_GTK2_14
+# ifdef USE_GTK2_14
 			gtk_bin_get_child (GTK_BIN (combo));
-#else
+# else
 			GTK_BIN (combo)->child;
-#endif
+# endif
 		if (pos == -1)
 			gtk_entry_set_text (GTK_ENTRY (entry), "");
 		else
@@ -191,6 +197,50 @@ static gboolean _e2_combobox_key_press_cb2 (GtkWidget *entry, GdkEventKey *event
 	return FALSE;
 }
 /**
+@brief "scroll-event" signal callback for the entry belonging to a GtkComboBoxEntry
+
+@param entry the entry widget for the combo box
+@param event pointer to event data struct
+@param combo the parent widget
+
+@return
+*/
+static gboolean _e2_combobox_scroll_cb (GtkWidget *entry, GdkEventScroll *event,
+	GtkWidget *combo)
+{
+	gint pos;
+	guint modifiers;
+//	printd (DEBUG, "_e2_combobox_scroll_cb");
+	if (event->direction == GDK_SCROLL_UP)
+	{
+		pos = gtk_combo_box_get_active (GTK_COMBO_BOX (combo));
+		if (pos > 0)
+		{
+			modifiers = gtk_accelerator_get_default_mod_mask ();
+			if ((event->state & modifiers) == GDK_CONTROL_MASK)
+				e2_combobox_set_active (combo, 0);
+			else
+				e2_combobox_set_active (combo, pos-1);
+		}
+		return TRUE;
+	}
+	else if (event->direction == GDK_SCROLL_DOWN)
+	{
+		pos = gtk_combo_box_get_active (GTK_COMBO_BOX (combo));
+		guint lastpos = _e2_combobox_last_index (GTK_COMBO_BOX (combo));
+		if (pos == -1 || pos < lastpos)
+		{
+			modifiers = gtk_accelerator_get_default_mod_mask ();
+			if ((event->state & modifiers) == GDK_CONTROL_MASK)
+				e2_combobox_set_active (combo, lastpos);
+			else
+				e2_combobox_set_active (combo, pos+1);
+		}
+		return TRUE;
+	}
+	return FALSE;
+}
+/**
 @brief "changed" signal callback for a GtkComboBoxEntry
 When a history item is selected, if @a data is non-NULL (TRUE) the combobox's
 entry is focused. If @a data is NULL (FALSE) then an "activated" signal is
@@ -245,7 +295,7 @@ static void _e2_combobox_changed_cb (GtkWidget *combo, gpointer data)
 /**
 @brief "activate" signal callback for created GtkComboBoxEntry comboboxes
 
-This function is a callback for the "activate" signal of the GtkEntry that
+This function is a callback for the "activate" signal on the GtkEntry that
 belongs to a combobox with a text entry, unless that combobox is created with
 flag E2_COMBOBOX_NO_AUTO_HISTORY.
 The current contents of @a entry, if not empty, are prepended to the box's
@@ -264,7 +314,11 @@ void e2_combobox_activated_cb (GtkWidget *entry, gpointer data)
 	g_strchug (text);
 	if (*text != '\0')
 	{
+#ifdef USE_GTK2_24
+		GtkComboBoxText *combo = GTK_COMBO_BOX_TEXT (
+#else
 		GtkComboBox *combo = GTK_COMBO_BOX (
+#endif
 #ifdef USE_GTK2_14
 			gtk_widget_get_parent (entry)
 #else
@@ -275,7 +329,11 @@ void e2_combobox_activated_cb (GtkWidget *entry, gpointer data)
 		if (data == NULL)	// !GPOINTER_TO_INT (data))
 		{
 			GtkTreeIter iter;
+#ifdef USE_GTK2_24
+			GtkTreeModel *model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo));
+#else
 			GtkTreeModel *model = gtk_combo_box_get_model (combo);
+#endif
 			if (gtk_tree_model_get_iter_first (model, &iter))
 			{
 				//actually, there should only ever be 1 match at most
@@ -290,8 +348,11 @@ void e2_combobox_activated_cb (GtkWidget *entry, gpointer data)
 				}
 			}
 		}
-
+#ifdef USE_GTK2_24
+		gtk_combo_box_text_prepend_text (combo, text);
+#else
 		gtk_combo_box_prepend_text (combo, text);
+#endif
 		e2_combobox_set_active (GTK_WIDGET (combo), 0);
 	}
 	g_free (text);
@@ -328,7 +389,7 @@ entry, and moves the cursor to the end
 	gchar *text = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
 	gint pos = gtk_editable_get_position (GTK_EDITABLE (entry));
 
-//	_e2_combobox_block_changed (combo);	//CHECKME
+//	e2_combobox_block_changed (combo);	//CHECKME
 	e2_combobox_set_active (combo, count);
 //	_e2_combobox_unblock_changed (combo);	//CHECKME
 
@@ -380,7 +441,7 @@ gchar *e2_combobox_first_text (GtkComboBox *combo)
 void e2_combobox_set_active (GtkWidget *combo, gint num)
 {
 	printd (DEBUG, "e2_combobox_set_active: %d", num);
-	_e2_combobox_block_changed (combo);
+	e2_combobox_block_changed (combo);
 	gtk_combo_box_set_active (GTK_COMBO_BOX (combo), num);
 	_e2_combobox_unblock_changed (combo);
 }
@@ -393,8 +454,13 @@ At this time, this func is only used for GtkComboBoxEntry's
 */
 gchar *e2_combobox_get_active_text (GtkWidget *combo)
 {
+	//we don't want to return entry-content, if that's inconsistent with selection
 	if (gtk_combo_box_get_active (GTK_COMBO_BOX (combo)) != -1)
+#ifdef USE_GTK2_24 //TODO
+		return (gtk_combo_box_text_get_active_text (GTK_COMBO_BOX_TEXT (combo)));
+#else
 		return (gtk_combo_box_get_active_text (GTK_COMBO_BOX (combo)));
+#endif
 	return NULL;
 }
 /**
@@ -473,8 +539,7 @@ void e2_combobox_clear_value (GtkWidget *combo, const gchar *value, gboolean wit
 /**
 @brief append a GList to a GtkComboBox's history
 
-This function appends a GList of strings (@a list) to the history
-of a GtkComboBoxEntry
+This function appends a GList of strings (@a list) to the history of a GtkComboBoxEntry
 NOTE only for boxes created with gtk_combo_box_new_text ()
 @param combo GtkComboBoxEntry to work on
 @param list GList of utf8 strings
@@ -485,7 +550,11 @@ void e2_combobox_append_history (GtkWidget *combo, GList *list)
 {
 	GList *tmp;
 	for (tmp = list; tmp != NULL; tmp = g_list_next (tmp))
+#ifdef USE_GTK2_24
+		gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), tmp->data);
+#else
 		gtk_combo_box_append_text (GTK_COMBO_BOX (combo), tmp->data);
+#endif
 }
 /**
 @brief append string array to a GtkComboBox's history
@@ -501,13 +570,16 @@ void e2_combobox_append_history_counted (GtkWidget *combo, guint num, gchar **ar
 {
 	guint i;
 	for (i = 0; i < num; i++)
+#ifdef USE_GTK2_24
+		gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), array[i]);
+#else
 		gtk_combo_box_append_text (GTK_COMBO_BOX (combo), array[i]);
+#endif
 }
 /**
 @brief append NULL-terminated string array to a GtkComboBox's history
 
-This function appends array of strings @a strv to the history
-of a GtkComboBox.
+This function appends array of strings @a strv to the history of a GtkComboBox.
 NOTE only for boxes created with gtk_combo_box_new_text ()
 @param combo combobox
 @param strv a null-terminated array of utf8 strings
@@ -518,7 +590,11 @@ void e2_combobox_append_history_strv (GtkWidget *combo, gchar **strv)
 {
 	guint i;
 	for (i = 0; strv[i] != NULL; i++)
+#ifdef USE_GTK2_24
+		gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), strv[i]);
+#else
 		gtk_combo_box_append_text (GTK_COMBO_BOX (combo), strv[i]);
+#endif
 }
 /**
 @brief create a GtkComboBox
@@ -531,6 +607,8 @@ command line, or generally NULL for an ad-hoc combobox
 @a history size figures prominently in any (downstream) clearing of the combobox
 model, so it should not be static if there's a prospect of multiple widgets
 clearing the history
+If E2_COMBOBOX_MENU_STYLE flag is not set, on gtk2, calls gtk_widget_set_name()
+which MAY? need BGL closed
 @param activate_cb "activate" callback for the combobox entry, or NULL
 @param activate_data data to provide to the activate callback
 @param history history-list pointer, or NULL
@@ -542,10 +620,32 @@ GtkWidget *e2_combobox_get (void (*activate_cb)(GtkEntry*,gpointer),
 	gpointer activate_data, GList **history, E2_ComboBoxFlags flags)
 {
 	GtkWidget *combo = (flags & E2_COMBOBOX_HAS_ENTRY) ?
+#ifdef USE_GTK2_24
+		gtk_combo_box_text_new_with_entry () : gtk_combo_box_text_new ();
+#else
 		gtk_combo_box_entry_new_text () : gtk_combo_box_new_text ();
+#endif
 
 	if (!(flags & E2_COMBOBOX_MENU_STYLE))
 	{
+#ifdef USE_GTK3_0
+		//use distinct style-object for each combo, cuz all are destroyed during any config window-recreation
+		GtkCssProvider *liststyle = gtk_css_provider_new ();
+		const gchar *cssdata = "GtkComboBox { -GtkComboBox-appears-as-list:1; }";
+		GError *err = NULL;
+		if (gtk_css_provider_load_from_data (liststyle, cssdata, -1, &err))
+		{
+			GtkStyleContext *sc = gtk_widget_get_style_context (combo);
+			gtk_style_context_add_provider (sc, GTK_STYLE_PROVIDER(liststyle),
+				GTK_STYLE_PROVIDER_PRIORITY_USER);
+		}
+		else
+		{
+			printd (WARN, "Combobox styling failure: %s", err->message);
+			g_error_free (err);
+		}
+		g_object_unref (G_OBJECT (liststyle)); //back to refcount=1, or 0
+#else
 		static gboolean style_parsed = FALSE;
 		if (!style_parsed)
 		{
@@ -555,6 +655,7 @@ GtkWidget *e2_combobox_get (void (*activate_cb)(GtkEntry*,gpointer),
 			style_parsed = TRUE;
 		}
 		gtk_widget_set_name (combo, "list-style");
+#endif
 	}
 
 	if (history != NULL)
@@ -565,26 +666,37 @@ GtkWidget *e2_combobox_get (void (*activate_cb)(GtkEntry*,gpointer),
 #ifdef USE_GTK2_14
 		GtkWidget *child = gtk_bin_get_child (GTK_BIN (combo));
 #endif
-		//enable prompt UI updates and deletions
-		g_signal_connect (G_OBJECT
-#ifdef USE_GTK2_14
-			(child),
-#else
-			(GTK_BIN (combo)->child),
-#endif
-			"key-press-event",
-			G_CALLBACK (_e2_combobox_key_press_cb2), history); //for <Delete>, no key translation needed
-		g_object_set_data (G_OBJECT(combo), "e2-combo-history", history);
-		if (!(flags & E2_COMBOBOX_NO_AUTO_HISTORY))
+		if (history != NULL)
+		{
+			g_object_set_data (G_OBJECT(combo), "e2-combo-history", history);
+			//enable prompt UI updates and deletions
 			g_signal_connect (G_OBJECT
 #ifdef USE_GTK2_14
 				(child),
 #else
 				(GTK_BIN (combo)->child),
 #endif
-				"activate",
-				G_CALLBACK (e2_combobox_activated_cb),
-				GINT_TO_POINTER (flags & E2_COMBOBOX_ALLOW_DOUBLE));
+				"key-press-event",
+				G_CALLBACK (_e2_combobox_key_press_cb2), history); //for <Delete>, no key translation needed
+			g_signal_connect (G_OBJECT
+#ifdef USE_GTK2_14
+				(child),
+#else
+				(GTK_BIN (combo)->child),
+#endif
+				"scroll-event",
+				G_CALLBACK (_e2_combobox_scroll_cb), combo);
+			if (!(flags & E2_COMBOBOX_NO_AUTO_HISTORY))
+				g_signal_connect (G_OBJECT
+#ifdef USE_GTK2_14
+					(child),
+#else
+					(GTK_BIN (combo)->child),
+#endif
+					"activate",
+					G_CALLBACK (e2_combobox_activated_cb),
+					GINT_TO_POINTER (flags & E2_COMBOBOX_ALLOW_DOUBLE));
+		}
 		//connect caller-specified callback for "activate" signal to combobox's entry
 		if (activate_cb != NULL)
 			g_signal_connect (G_OBJECT
@@ -602,18 +714,18 @@ GtkWidget *e2_combobox_get (void (*activate_cb)(GtkEntry*,gpointer),
 	}
 
 	g_signal_connect (G_OBJECT ((combo)), "key-press-event",
-		G_CALLBACK (_e2_combobox_key_press_cb), //includes translation
+		G_CALLBACK (_e2_combobox_key_press_cb),
 		GUINT_TO_POINTER (flags & E2_COMBOBOX_CYCLE_HISTORY));
-	g_signal_connect (G_OBJECT (combo), "key-release-event",
-		G_CALLBACK (e2_utils_key_translate_cb), NULL);
 
 	return combo;
 }
 /**
 @brief pack a new GtkComboBox into @a box
 
-This function creates a GtkComboBox using e2_combobox_get(), and then packs
-it into @a box and connects @a func to the "activate" signal.
+Creates a GtkComboBox using e2_combobox_get(), and then packs it into @a box
+and connects @a func to the "activate" signal.
+If E2_COMBOBOX_MENU_STYLE flag is not set, on gtk2, downstream calls
+gtk_widget_set_name() which MAY? need BGL closed
 
 @param box parent box to add the GtkComboBox to
 @param expand packing property
