@@ -1,6 +1,6 @@
-/* $Id: e2_menu.c 2017 2009-11-19 20:36:36Z tpgww $
+/* $Id: e2_menu.c 2469 2012-03-16 00:19:24Z tpgww $
 
-Copyright (C) 2004-2009 tooar <tooar@emelfm2.net>
+Copyright (C) 2004-2012 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -14,8 +14,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 /**
@@ -25,9 +24,10 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 This file contains utility and helper functions for menus.
 */
 
-#include "emelfm2.h"
 #include <string.h>
 #include <pthread.h>
+
+#include "emelfm2.h"
 #include "e2_menu.h"
 #include "e2_dialog.h"
 #include "e2_plugins.h"
@@ -38,14 +38,30 @@ This file contains utility and helper functions for menus.
 
 extern pthread_mutex_t task_mutex;
 
+/**
+@brief clear data for each item in bookmarks menu
+@param data data to be cleared
+@param closure UNUSED
+@return
+*/
+void e2_menu_mark_clear (gpointer data, GClosure *closure)
+{
+	E2_MarkData *mdata = data;
+	printd (DEBUG, "e2_menu_mark_clear() for %s", mdata->mark_path);
+	g_free (mdata->mark_path);
+	if (mdata->tpath != NULL)
+		gtk_tree_path_free (mdata->tpath);
+	DEALLOCATE (E2_MarkData, data);
+}
+
   /*******************/
  /**** callbacks ****/
 /*******************/
 
 /**
-@brief execute action corresponding to item selected from a destroyable menu
-We do this outside the manu-item "activated" callback to avoid disrupting gtk's
-menu-activation process
+@brief Timer callback to execute action corresponding to item selected from a destroyable menu
+We do this outside the menu-item "activated" callback to avoid disrupting gtk's
+menu-activation process. @a art is cleared here.
 @param art pointer to action runtime data
 
 @return FALSE to remove the source
@@ -67,7 +83,7 @@ be destroyed after usage.
 To avoid disrupting gtk's menu-activation process, we do not run the action here.
 Note: depending on the menu-destruction process and whether the action is
 synchronous, menu_item may not persist until the action is finished running
-i.e. possible race
+i.e. possible race. Downstream clears @a data when finished with it
 @param menuitem the selected menu item widget
 @param data pointer to action runtime data
 
@@ -82,7 +98,6 @@ void e2_menu_action_activated_cb (GtkMenuItem *menu_item, gpointer data)
 	//an idle-callback is no good !?
 	g_timeout_add (100, (GSourceFunc)_e2_menu_action_deferred_cb, art);
 }
-
 /**
 @brief like _e2_menu_action_deferred_cb, but for menus which are not destroyed
 @param art pointer to action runtime data
@@ -99,7 +114,8 @@ static gboolean _e2_menu_action_deferred_cb2 (E2_ActionRuntime *art)
 	return FALSE;
 }
 /**
-@brief like e2_menu_action_activated_cb, but for menus which are not destroyed
+@brief like e2_menu_action_activated_cb, but for menus which are not destroyed.
+Thus, @a data is _not_ cleared downstream
 @param menuitem the selected menu item widget
 @param data pointer to action runtime data
 */
@@ -233,8 +249,8 @@ static GtkWidget *_e2_menu_add_tied_check (GtkWidget *menu, E2_OptionSet *set,
 Args of @a func should be (GtkMenuItem*,some pointer)
 @param menu menu widget
 @param label menu item text, optionally with mnemonic
-@param icon custom iconfile path, or stock-icon identifier
-@param tip tooltip string
+@param icon custom iconfile path, or stock-icon identifier, or NULL for no icon
+@param tip tooltip string, or NULL
 @param activate_cb callback function for handling menu item activation, or NULL
 @param data pointer to data to send to @a activate_cb
 
@@ -244,18 +260,27 @@ GtkWidget *e2_menu_add (GtkWidget *menu, gchar *labeltext, gchar *icon,
 	gchar *tip, void (*activate_cb)(), gpointer data)
 {
 	GtkWidget *menu_item;
-	gint choice = e2_option_sel_get ("menu-show-icons");
-	if (choice == 0 && icon != NULL)	//theme
+	gint choice;
+	if (icon == NULL)
+		choice = 2;	//no icon
+	else
+		choice = e2_option_sel_get ("menu-show-icons");
+	if (choice == 0)	//theme
 	{
-#ifdef USE_GTK2_18
-		gboolean show;
+		//ensure that the "gtk-menu-images" property is registered
+		static GtkWidget *dummy = NULL;
+		if (dummy == NULL)
+		{
+			dummy = gtk_image_menu_item_new ();
+			gtk_widget_destroy (dummy);
+		}
 		GtkSettings* defs = gtk_settings_get_default ();	//assume this always exists
+		gboolean show;
 		g_object_get (G_OBJECT (defs), "gtk-menu-images", &show, NULL);
 		if (show)
-#endif
 			choice = 1;
 	}
-	if (choice == 1 && icon != NULL)
+	if (choice == 1) //show icon
 	{
 		GtkWidget *image = e2_widget_get_icon (icon, GTK_ICON_SIZE_MENU);	//e2_option_int_get ("menu-isize") + 1);
 		if (image != NULL)
@@ -423,7 +448,7 @@ This expects to be called twice in succession for each toggle-item.
 None of the strings may be NULL
 @param menu the menu to which the item will be added
 @param label menu label string
-@param destroy TRUE if @a menu will be destroyed after use
+@param onetime TRUE if @a menu will be destroyed after use
 @param icon menu icon name
 @param tip menu tip string
 @param type string "toggle.on" or "toggle.off"
@@ -431,7 +456,7 @@ None of the strings may be NULL
 
 @return after the first of the pair, NULL: after the second, the menu item or NULL upon error
 */
-GtkWidget *e2_menu_add_toggle (GtkWidget *menu, gboolean destroy,
+GtkWidget *e2_menu_add_toggle (GtkWidget *menu, gboolean onetime,
 	gchar *label, gchar *icon, gchar *tip, gchar *type, gchar *cmd)
 {
 	static gboolean first = TRUE;
@@ -500,10 +525,13 @@ GtkWidget *e2_menu_add_toggle (GtkWidget *menu, gboolean destroy,
 	if (art != NULL)
 	{
 		item = e2_menu_add (menu, uselabel, useicon, usetip,
-		(destroy) ? e2_menu_action_activated_cb:e2_menu_action_activated_cb2,
-		art);
-		g_object_set_data_full (G_OBJECT(item), "e2-free-actruntime",
-			art, (GDestroyNotify)e2_action_free_runtime);
+			(onetime) ? e2_menu_action_activated_cb:e2_menu_action_activated_cb2,
+			art);
+		if (onetime)
+			g_object_set_data (G_OBJECT (item), "e2-free-actruntime", art); //art cleared in callback
+		else
+			g_object_set_data_full (G_OBJECT (item), "e2-free-actruntime", art,
+					(GDestroyNotify) e2_action_free_runtime);
 	}
 	else
 		item = NULL;
@@ -551,7 +579,7 @@ void _e2_menu_popup (GtkWidget *menu, gint x, gint y, gint button, guint32 time)
 */
 /**
 @brief add items for a series of config options to @a menu
-
+CHECKME possible that something downstream needs BGL closed
 @param controller "parent" widget for the created menu
 @param menu menu widget to populate, or NULL to create a new one
 @param set first of a NULL-terminated series of option data structs
@@ -589,65 +617,88 @@ GtkWidget *e2_menu_create_options_menu (GtkWidget *controller, GtkWidget *menu,
 /**
 @brief add items to bookmarks menu, recursively if children exist
 
-@param menu widget to which the items will be added
-@param onetime TRUE if @a menu is destroyed after use
+@param menu (sub)menu widget to which the items will be added
+@param top_menu top-level bookmarks-menu widget, maybe same as @a menu
 @param action pointer to data for the pseudo-action being processed
+@param markflags flags indicating how mark is to be processed, E2_MARKFLAG_ACTIONS etc
 @param panenum the pane whose path will be used when adding a mark (E2PANECUR, E2PANE1 or E2PANE2)
-@param mdl pointer to bookmarks config data tree model
+@param mdl pointer to config data treemodel for bookmarks
 @param iter pointer to iter to be used for interrogating @a mdl
 
 @return
 */
-void e2_menu_add_bookmark_items (GtkWidget *menu, gboolean onetime,
-	E2_Action *action, gint panenum, GtkTreeModel *mdl, GtkTreeIter *iter)
+void e2_menu_add_bookmark_items (GtkWidget *menu, GtkWidget *top_menu,
+	E2_Action *action, guint32 markflags, gint32 panenum,
+	GtkTreeModel *mdl, GtkTreeIter *iter)
 {
 	do
 	{
-		gchar *name, *icon, *_tip, *_path;
-		//don't support variables in label
-//		gchar *_name, *icon, *_tip, *_path;
-		gtk_tree_model_get (mdl, iter, 0, &name, 1, &icon, 2, &_tip, 3, &_path, -1);
-//		gchar *name = e2_utils_replace_vars (_name, FALSE);
-		gchar *tip = e2_utils_replace_vars (_tip, FALSE);
-		gchar *path = e2_utils_replace_vars (_path, TRUE);
-//		g_free (_name);
-		g_free (_tip);
-		g_free (_path);
+		GtkWidget *item;
+		gchar *label, *icon, *tip, *markpath;
+
+		E2_MarkData *mdata = ALLOCATE (E2_MarkData);
+		CHECKALLOCATEDWARN (mdata, return;)
+
+		mdata->pane_num = panenum;
+		mdata->mark_flags = markflags;
+		mdata->tpath = gtk_tree_model_get_path (mdl, iter);
+		mdata->top_menu = top_menu;
+
+		gtk_tree_model_get (mdl, iter, 0, &label, 1, &icon, 2, &tip, 3, &markpath, -1);
 
 		if (gtk_tree_model_iter_has_child (mdl, iter))
 		{
 			GtkTreeIter iter2;
+
+			mdata->mark_path = NULL; //no path clean
+
 			gtk_tree_model_iter_children (mdl, &iter2, iter);
-			//sub-menu items can't show tip, context menu etc
-			GtkWidget *submenu = e2_menu_add_submenu (menu, name, icon);
-			e2_menu_add_bookmark_items (submenu, onetime, action, panenum, mdl, &iter2);	//recurse
-			g_free (path);
+			//items with sub-menu can't show tip etc
+			item = e2_menu_add (menu, label, icon, NULL, NULL, NULL);
+			GtkWidget *submenu = gtk_menu_new ();
+			gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), submenu);
+			e2_menu_add_bookmark_items (submenu, top_menu, action, markflags,
+				panenum, mdl, &iter2);	//recurse
+			gtk_widget_show (submenu);
 		}
 		else
 		{
-			GtkWidget *item = e2_menu_add (menu, name, icon, tip, NULL, NULL);
-			g_signal_connect_data (G_OBJECT (item), "button-press-event",
-				G_CALLBACK (e2_bookmark_button_press_cb),
-				gtk_tree_model_get_path (mdl, iter),
-				(GClosureNotify) gtk_tree_path_free, 0);
-			void (*activatecb)() = (onetime) ?
-				e2_menu_action_activated_cb : e2_menu_action_activated_cb2;
+			void (*activatecb)();
+
+			mdata->mark_path = e2_utils_replace_vars (markpath, TRUE);
+
+			//don't support variables in label
+			gchar *_tip = e2_utils_replace_vars (tip, FALSE);
+			item = e2_menu_add (menu, label, icon, _tip, NULL, NULL);
+			g_free (_tip);
+
 			E2_ActionRuntime *art = e2_action_pack_runtime (action,
-				g_strdup_printf ("%d %s", panenum, path), g_free);
+				g_strdup_printf ("%d %s", panenum, mdata->mark_path), g_free); //data for add/open
+			if (markflags & E2_MARKFLAG_ONETIME)
+			{
+				activatecb = e2_menu_action_activated_cb;
+				g_object_set_data (G_OBJECT (item), "e2-free-actruntime", art); //art cleared after use
+			}
+			else
+			{
+				activatecb = e2_menu_action_activated_cb2;
+				g_object_set_data_full (G_OBJECT (item), "e2-free-actruntime", art,
+					(GDestroyNotify) e2_action_free_runtime);
+			}
 			//must connect_after to the activate signal, or else the
-			//button-press-event will never occur
+			//button-press-event callback will never occur
 			g_signal_connect_after (G_OBJECT (item), "activate",
 				G_CALLBACK (activatecb), art);
-			g_object_set_data_full (G_OBJECT (item), "e2-free-actruntime", art,
-				(GDestroyNotify) e2_action_free_runtime);
-			//path to be cleaned when item is destroyed
-			g_object_set_data_full (G_OBJECT (item), "bookmark-path", path, g_free);
-			//signal which pane (or default) to use when adding a mark or opening a dir
-			g_object_set_data (G_OBJECT (item), "bookmark-pane", GINT_TO_POINTER (panenum));
 		}
-		g_free (name);
+
+		g_signal_connect_data (G_OBJECT (item), "button-press-event",
+			G_CALLBACK (e2_bookmark_button_press_cb), mdata,
+			(GClosureNotify) e2_menu_mark_clear, 0);
+
+		g_free (label);
 		g_free (icon);
 		g_free (tip);
+		g_free (markpath);
 	} while (gtk_tree_model_iter_next (mdl, iter));
 }
 /**
@@ -678,9 +729,8 @@ void e2_menu_create_plugins_menu (GtkWidget *menu, gboolean onetime, gboolean is
 			void (*activatecb)() = (onetime) ? e2_menu_action_activated_cb:e2_menu_action_activated_cb2;
 			if (p->child_list != NULL)
 			{
-				//create item and child menu
-				GtkWidget *menu2 = e2_menu_add_submenu (menu, (gchar *)p->menu_name,
-					(gchar *)p->icon);
+				//create item and child menu, if relevant
+				GtkWidget *menu2 = gtk_menu_new ();
 				GList *member;
 				for (member = p->child_list; member != NULL; member = member->next)
 				{
@@ -692,11 +742,25 @@ void e2_menu_create_plugins_menu (GtkWidget *menu, gboolean onetime, gboolean is
 						{
 							item = e2_menu_add (menu2, (gchar *)pc->menu_name,
 								(gchar *)pc->icon, (gchar *)pc->description, activatecb, art);
-							g_object_set_data_full (G_OBJECT(item), "e2-free-actruntime",
-								art, (GDestroyNotify)e2_action_free_runtime);
+							if (onetime)
+								g_object_set_data (G_OBJECT (item), "e2-free-actruntime", art); //art cleared in callback
+							else
+								g_object_set_data_full (G_OBJECT (item), "e2-free-actruntime", art,
+									(GDestroyNotify) e2_action_free_runtime);
 						}
 					}
 				}
+				member = gtk_container_get_children (GTK_CONTAINER(menu2));
+				if (member)
+				{
+					g_list_free (member);
+					GtkWidget *menu_item = e2_menu_add (menu, (gchar *)p->menu_name,
+						(gchar *)p->icon, NULL, NULL, NULL);
+					gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), menu2);
+					gtk_widget_show (menu2);
+				}
+				else
+					gtk_widget_destroy (menu2);
 			}
 			else //just 1 action in the plugin, or it's a child
 				if (p->module != NULL	//it's not a child
@@ -707,8 +771,11 @@ void e2_menu_create_plugins_menu (GtkWidget *menu, gboolean onetime, gboolean is
 				{
 					item = e2_menu_add (menu, (gchar *)p->menu_name, (gchar *)p->icon,
 						(gchar *)p->description, activatecb, art);
-					g_object_set_data_full (G_OBJECT(item), "e2-free-actruntime",
-						art, (GDestroyNotify)e2_action_free_runtime);
+					if (onetime)
+						g_object_set_data (G_OBJECT (item), "e2-free-actruntime", art); //art cleared in callback
+					else
+						g_object_set_data_full (G_OBJECT (item), "e2-free-actruntime", art,
+							(GDestroyNotify) e2_action_free_runtime);
 				}
 			}
 		}
@@ -772,10 +839,18 @@ static void _e2_menu_add_custom_items (GtkTreeModel *model, GtkTreeIter *iter, G
 		}
 		else
 		{
-			GtkWidget *menu_item = e2_menu_add (menu, label, icon, tip,
+			if (strcmp (command, _A(21)) != 0)
+			{
+				GtkWidget *menu_item = e2_menu_add (menu, label, icon, tip,
 					_e2_menu_custom_cb, NULL);
-			g_object_set_data_full (G_OBJECT (menu_item), "custom-command",
-				command, (GDestroyNotify) g_free);
+				g_object_set_data_full (G_OBJECT (menu_item), "custom-command",
+					command, (GDestroyNotify) g_free);
+			}
+			else
+			{
+				e2_menu_add_separator (menu);
+				g_free (command);
+			}
 		}
 		//cleanup except command
 		g_free (icon);
@@ -786,7 +861,7 @@ static void _e2_menu_add_custom_items (GtkTreeModel *model, GtkTreeIter *iter, G
 /**
 @brief construct a menu of custom commands
 
-@param name utf8 string, name of custom menu to use
+@param name UTF8 string, name of custom menu to use
 
 @return the menu item widget, or NULL
 */
@@ -965,52 +1040,65 @@ static void _e2_menu_mount_cb (GtkCheckMenuItem *item, gpointer from)
 	gboolean newstate = gtk_check_menu_item_get_active (item);
 //	gint result;
 	gchar *elsewhere = NULL;
-	gchar *point;
+	gchar *mpoint;
 //#ifdef USE_GTK2_16
 //	point = gtk_menu_item_get_label (GTK_MENU_ITEM (item));
 //#else
-	point = g_object_get_data (G_OBJECT (item), "_mountpoint_");
+	mpoint = g_object_get_data (G_OBJECT (item), "_mountpoint_");
 //#endif
 #if defined (E2_HAL) || defined (E2_DEVKIT)
-	gboolean remove = e2_fs_mount_is_ejectable (point);	//check this before unmount
+	gboolean remove = e2_fs_mount_is_ejectable (mpoint);	//check this before unmount
 #endif
 	//CHECKME synchronous umount command to mimimize risk of CWD error
 //tag E2_BADQUOTES
-	gchar *qp = e2_utils_quote_string (point);
+	gchar *qp = e2_utils_quote_string (mpoint);
 	gchar *cmd = g_strconcat ((newstate) ? E2_MOUNTCOMMAND : "|"E2_UNMOUNTCOMMAND,
 		" ", qp, NULL);
 	if (!newstate //doing an unmount
 #ifdef E2_VFSTMP
 	//FIXME dir when not mounted local
 #else
-		&& g_str_has_prefix (curr_view->dir, point))
+		&& g_str_has_prefix (curr_view->dir, mpoint))
 #endif
 	{	//if possible, change CWD out of mountpoint so we don't ourself block the unmount
-		gchar *s = strrchr (point, G_DIR_SEPARATOR);	//assumes no trailer on mountpoint string
-		if (s > point+1) //not doing root dir
-			elsewhere = g_strndup (point, s - point);
+		gchar *s = strrchr (mpoint, G_DIR_SEPARATOR);	//assumes no trailer on mountpoint string
+		if (s > mpoint+1) //not doing root dir
+			elsewhere = g_strndup (mpoint, s - mpoint);
 		else //can't cd
 			elsewhere = g_strdup (G_DIR_SEPARATOR_S);
 
 		e2_fs_get_valid_path (&elsewhere, TRUE E2_ERR_NONE());
 		//if (
 			e2_fs_chdir (elsewhere E2_ERR_NONE()); //)
-		//result =
-		e2_command_run_at (cmd, elsewhere, E2_COMMAND_RANGE_DEFAULT, from	//has temporary change CWD
-#ifdef E2_COMMANDQ
-		, FALSE
+/*
+#ifdef USE_GLIB2_22
+		if (!e2_fs_mount_gio_unmount (mpoint)) //try for removable, first
 #endif
-		);
+*/
+//			result =
+			e2_command_run_at (cmd, elsewhere, E2_COMMAND_RANGE_DEFAULT, from	//has temporary change CWD
+#ifdef E2_COMMANDQ
+			, FALSE
+#endif
+			);
 		//else
 			//FIXME warn user about blockage
 	}
 	else
-//	result =
-		e2_command_run (cmd, E2_COMMAND_RANGE_DEFAULT, from
+	{
+/*
+#ifdef USE_GLIB2_22
+		if ((newstate && !e2_fs_mount_gio_mount (mpoint)) //mount && not removable
+		|| (!newstate && !e2_fs_mount_gio_unmount (mpoint))) //unmount && not removable
+//#endif
+*/
+//			result =
+			e2_command_run (cmd, E2_COMMAND_RANGE_DEFAULT, from
 #ifdef E2_COMMANDQ
-		, FALSE
+			, FALSE
 #endif
-		);
+			);
+	}
 	g_free (cmd);
 
 	if (newstate) //doing a mount
@@ -1083,7 +1171,13 @@ static GtkWidget *selectedmount = NULL;
 @param user_data UNUSED
 @return
 */
-void _e2_menu_mount_select_cb (GtkItem *item, gpointer user_data)
+void _e2_menu_mount_select_cb (
+#ifdef USE_GTK3_0
+	GtkMenuItem *item,
+#else
+	GtkItem *item,
+#endif
+	gpointer user_data)
 {
 	selectedmount = GTK_WIDGET(item);
 }
