@@ -1,6 +1,6 @@
-/* $Id: e2_tree.c 1253 2008-09-20 10:53:16Z tpgww $
+/* $Id: e2_tree.c 2064 2010-03-12 13:15:36Z tpgww $
 
-Copyright (C) 2004-2007 tooar <tooar@gmx.net>
+Copyright (C) 2004-2010 tooar <tooar@emelfm2.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -14,8 +14,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 #include "emelfm2.h"
@@ -82,7 +81,7 @@ gboolean e2_tree_find_iter_from_str_simple (GtkTreeModel *model, gint column,
 		gtk_tree_model_get (model, iter, column, &value, -1);
 		if (value == NULL)
 			continue;
-		if (g_str_equal (value, search))
+		if (!strcmp (value, search))
 		{
 			g_free (value);
 			if (with_children)
@@ -132,7 +131,7 @@ gboolean e2_tree_find_iter_from_str_same (GtkTreeModel *model, gint column,
 		gtk_tree_model_get (model, iter, column, &value, -1);
 		if (value != NULL)
 		{
-			if (g_str_equal (value, search))
+			if (!strcmp (value, search))
 			{
 				g_free (value);
 				return TRUE;
@@ -177,7 +176,7 @@ gboolean e2_tree_get_lowest_iter_for_str (GtkTreeModel *model, gint column,
 				{
 					gchar *value;
 					gtk_tree_model_get (model, iter, column, &value, -1);
-					found = (g_str_equal (value, split[i]));
+					found = (!strcmp (value, split[i]));
 					g_free (value);
 				} while (!found && gtk_tree_model_iter_next (model, iter));
 
@@ -264,7 +263,7 @@ matching iter in the model. Otherwise, iter is undefined at exit
 			{
 				gchar *value;
 				gtk_tree_model_get (model, iter, column, &value, -1);
-				found = (g_str_equal (value, split[i]));
+				found = (!strcmp (value, split[i]));
 				g_free (value);
 			} while (!found && gtk_tree_model_iter_next (model, iter));
 
@@ -387,12 +386,11 @@ gboolean e2_tree_ref_to_iter (GtkTreeStore *store, GtkTreeRowReference *ref,
 @param widget UNUSED the widget which activated the callback
 @param treeview the treeview to be collapsed
 
-@return TRUE (to prevent further handlers)
+@return
 */
-gboolean e2_tree_expand_all_cb (GtkWidget *widget, GtkTreeView *treeview)
+void e2_tree_expand_all_cb (GtkMenuItem *widget, GtkTreeView *treeview)
 {
 	gtk_tree_view_expand_all (treeview);
-	return TRUE;
 }
 /**
 @brief tree collapse-all callback
@@ -400,12 +398,11 @@ gboolean e2_tree_expand_all_cb (GtkWidget *widget, GtkTreeView *treeview)
 @param widget UNUSED the widget which activated the callback
 @param treeview the treeview to be collapsed
 
-@return TRUE (to prevent further handlers)
+@return
 */
-gboolean e2_tree_collapse_all_cb (GtkWidget *widget, GtkTreeView *treeview)
+void e2_tree_collapse_all_cb (GtkMenuItem *widget, GtkTreeView *treeview)
 {
 	gtk_tree_view_collapse_all (treeview);
-	return TRUE;
 }
 /**
 @brief find previous iter in a tree model
@@ -473,11 +470,9 @@ void e2_tree_delete (GtkTreeView *treeview)
 	GtkTreeSelection *sel = gtk_tree_view_get_selection (treeview);
 	if (gtk_tree_selection_count_selected_rows (sel) > 0)
 	{
-		GList *rowrefs = NULL;
-
 		GtkTreeModel *model;
+		GList *tmp, *rowrefs = NULL;
 		GList *selpaths = gtk_tree_selection_get_selected_rows (sel, &model);
-		GList *tmp;
 		//identify a place to goto after the deletion
 		//(current place, if we're at the start of the tree)
 		GtkTreePath *newpath = gtk_tree_path_copy (selpaths->data);
@@ -490,14 +485,17 @@ void e2_tree_delete (GtkTreeView *treeview)
 		//create treerowrefs for all selected rows
 		for (tmp = selpaths; tmp!=NULL; tmp=tmp->next)
 		{
-			if (gtk_tree_model_get_iter (model, &iter, (GtkTreePath *) tmp->data))
+			path = (GtkTreePath *) tmp->data;
+			if (gtk_tree_model_get_iter (model, &iter, path))
 				_e2_tree_reference_treerow (model, &iter, &rowrefs);
+			gtk_tree_path_free (path);
 		}
+		g_list_free (selpaths);
 
-		for (tmp = rowrefs; tmp!=NULL; tmp=tmp->next)
+		for (tmp = rowrefs; tmp != NULL; tmp = tmp->next)
 		{
-			if ((path = gtk_tree_row_reference_get_path
-				((GtkTreeRowReference *) tmp->data)) != NULL)
+			path = gtk_tree_row_reference_get_path ((GtkTreeRowReference *) tmp->data);
+			if (path != NULL)
 			{
 				if (gtk_tree_model_get_iter (model, &iter, path))
 				{
@@ -505,8 +503,11 @@ void e2_tree_delete (GtkTreeView *treeview)
 					//this deletes all children too
 					gtk_tree_store_remove (GTK_TREE_STORE (model), &iter);
 				}
+				gtk_tree_path_free (path);
 			}
+			gtk_tree_row_reference_free ((GtkTreeRowReference *) tmp->data);
 		}
+		g_list_free (rowrefs);
 
 		//go to the new place
 		if (!gtk_tree_model_get_iter (model, &iter, newpath)
@@ -518,11 +519,6 @@ void e2_tree_delete (GtkTreeView *treeview)
 		gtk_tree_view_set_cursor (treeview, newpath,
 			gtk_tree_view_get_column (treeview, 0), FALSE);
 
-		//cleanups
-		g_list_foreach (rowrefs, (GFunc) gtk_tree_row_reference_free, NULL);
-		g_list_free (rowrefs);
-		g_list_foreach (selpaths, (GFunc) gtk_tree_path_free, NULL);
-		g_list_free (selpaths);
 		gtk_tree_path_free (newpath);
 	}
 }
@@ -595,20 +591,20 @@ GList *e2_tree_copy (GtkTreeView *treeview)
 	if (gtk_tree_selection_count_selected_rows (sel) > 0)
 	{
 		GtkTreeModel *model;
-		GList *base = gtk_tree_selection_get_selected_rows (sel, &model);
-		GtkTreeIter iter;
+		GList *rowpath;
+		GList *selpaths = gtk_tree_selection_get_selected_rows (sel, &model);
 		gint columns = gtk_tree_model_get_n_columns (model);
-		GList *selpaths;
 
 		//copy the selected rows and their descendants
-		for (selpaths = base; selpaths != NULL; selpaths = selpaths->next)
+		for (rowpath = selpaths; rowpath != NULL; rowpath = rowpath->next)
 		{
-			if (gtk_tree_model_get_iter (model, &iter, (GtkTreePath *) selpaths->data))
+			GtkTreeIter iter;
+			if (gtk_tree_model_get_iter (model, &iter, (GtkTreePath *) rowpath->data))
 				_e2_tree_copy_branch (model, &iter, columns, &rowscopied);
+			gtk_tree_path_free ((GtkTreePath *) rowpath->data);
 		}
 
-		g_list_foreach (base, (GFunc) gtk_tree_path_free, NULL);
-		g_list_free (base);
+		g_list_free (selpaths);
 
 		printd (DEBUG, "copied %d rows", g_list_length (rowscopied));
 	}
@@ -799,11 +795,13 @@ void e2_tree_paste (GList *rowscopied, GtkTreeView *treeview)
 		g_list_free (selpaths);
 	}
 }
-#ifdef STORECOPY
+//#ifdef STORECOPY
 /**
 @brief helper function to recursively copy a treestore
-@param model treemodel for store being copied
-@param parent pointer to treeiter which has been confirmed to have child(ren)
+@param srcmodel treemodel for store being copied
+@param srcparent pointer to source treeiter which has been confirmed to have child(ren)
+@param deststore the store to which data is being copied
+@param destparent pointer to destination store treeiter
 @param ncols no. of columns in @a model
 @param colnums array of column indices
 @param types array of GTypes corresponding to columns
@@ -811,30 +809,39 @@ void e2_tree_paste (GList *rowscopied, GtkTreeView *treeview)
 
 @return
 */
-static void _e2_tree_copy_descendants (GtkTreeModel *model, GtkTreeIter *parent, gint ncols,
-	gint colnums[], GType types[], GValue values[])
+static void _e2_tree_copy_descendants (
+	GtkTreeModel *srcmodel, GtkTreeIter *srcparent,
+	GtkTreeStore *deststore, GtkTreeIter *destparent,
+	gint ncols, gint colnums[], GType types[], GValue values[])
 {
-	gint i;
-	GtkTreeStore *newtstore = GTK_TREE_STORE (model);
 	GtkTreeIter child;
-	gtk_tree_model_iter_children (model, &child, parent);
-	do
+	if (gtk_tree_model_iter_children (srcmodel, &child, srcparent))
 	{
-		if (gtk_tree_model_iter_has_child (model, &child))
-			_e2_tree_copy_descendants (model, &child, ncols, colnums, types, values);
-		//read iter into pointers array
-		for (i = 0; i < ncols; i++)
-			gtk_tree_model_get_value (model, &child, i, &values[i]);
+		gint i;
+		GtkTreeIter dest;
+		do
+		{
+			//read iter into pointers array
+			for (i = 0; i < ncols; i++)
+				gtk_tree_model_get_value (srcmodel, &child, i, &values[i]);
 #ifdef USE_GTK2_10
-		gtk_tree_store_insert_with_valuesv (newtstore, &child, parent, -1, colnums, values, ncols);
+			gtk_tree_store_insert_with_valuesv (deststore, &dest, destparent, -1,
+				colnums, values, ncols);
+			for (i = 0; i < ncols; i++)
+				g_value_unset (&values[i]);
 #else
-		gtk_tree_store_append (newtstore, &child, parent);
-		for (i = 0; i < ncols; i++)
-			gtk_tree_store_set_value (newtstore, &child, i, &values[i]);
+			gtk_tree_store_append (deststore, &dest, destparent);
+			for (i = 0; i < ncols; i++)
+			{
+				gtk_tree_store_set_value (deststore, &dest, i, &values[i]);
+				g_value_unset (&values[i]);
+			}
 #endif
-		for (i = 0; i < ncols; i++)
-			g_value_unset (&values[i]);
-	} while (gtk_tree_model_iter_next (model, &child));
+			if (gtk_tree_model_iter_has_child (srcmodel, &child))
+				_e2_tree_copy_descendants (srcmodel, &child, deststore, &dest,
+					ncols, colnums, types, values);
+		} while (gtk_tree_model_iter_next (srcmodel, &child));
+	}
 }
 /**
 @brief copy a liststore or treestore
@@ -852,9 +859,7 @@ same as the old store.
 */
 void e2_tree_store_copy (GtkTreeModel *model, gboolean treetype, gpointer *newstore)
 {
-	GtkTreeModel *newmodel;
 	GtkTreeIter src;
-	GtkTreeIter dest;
 	gint i, ncols = gtk_tree_model_get_n_columns (model);
 	gint colnums[ncols];
 	GType types[ncols];
@@ -869,39 +874,19 @@ void e2_tree_store_copy (GtkTreeModel *model, gboolean treetype, gpointer *newst
 
 	if (treetype)
 	{
-		GtkTreeStore *newtstore;
-		newtstore = gtk_tree_store_newv (ncols, types);
+		GtkTreeStore *newtstore = gtk_tree_store_newv (ncols, types);
 		if (gtk_tree_model_get_iter_first (model, &src))
-		{
-			newmodel = GTK_TREE_MODEL (newtstore);
-			do
-			{
-				//read iter into values array
-				for (i = 0; i < ncols; i++)
-					gtk_tree_model_get_value (model, &src, i, &values[i]);
-#ifdef USE_GTK2_10
-				gtk_tree_store_insert_with_valuesv (newtstore, &dest, NULL, -1, colnums, values, ncols);
-#else
-				gtk_tree_store_append (newtstore, &dest, NULL);
-				for (i = 0; i < ncols; i++)
-					gtk_tree_store_set_value (newtstore, &dest, i, &values[i]);
-#endif
-				for (i = 0; i < ncols; i++)
-					g_value_unset (&values[i]);
-				if (gtk_tree_model_iter_has_child (model, &src))
-					_e2_tree_copy_descendants (model, NULL, ncols, colnums, types, values);
+			_e2_tree_copy_descendants (model, NULL, newtstore, NULL,
+						ncols, colnums, types, values);
 
-			} while (gtk_tree_model_iter_next (model, &src));
-		}
 		*newstore = newtstore;
 	}
 	else
 	{
-		GtkListStore *newlstore;
-		newlstore = gtk_list_store_newv (ncols, types);
+		GtkListStore *newlstore = gtk_list_store_newv (ncols, types);
 		if (gtk_tree_model_get_iter_first (model, &src))
 		{
-			newmodel = GTK_TREE_MODEL (newlstore);
+			GtkTreeIter dest;
 			do
 			{
 				for (i = 0; i < ncols; i++)
@@ -914,4 +899,4 @@ void e2_tree_store_copy (GtkTreeModel *model, gboolean treetype, gpointer *newst
 		*newstore = newlstore;
 	}
 }
-#endif
+//#endif
