@@ -1,6 +1,6 @@
-/* $Id: e2_utf8.c 1412 2009-02-01 21:51:22Z tpgww $
+/* $Id: e2_utf8.c 2487 2012-04-06 10:19:53Z tpgww $
 
-Copyright (C) 2004-2009 tooar <tooar@gmx.net>
+Copyright (C) 2004-2012 tooar <tooar@emelfm2.net>
 Portion _maybe_ copyright (C) 2004-2005 Tarot Osuji
 
 This file is part of emelFM2.
@@ -15,8 +15,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with emelFM2; see the file GPL. If not, contact the Free Software
-Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+along with emelFM2; see the file GPL. If not, see http://www.gnu.org/licenses.
 */
 
 /**
@@ -48,17 +47,21 @@ version of str, but we need a pointer to str itself
 static const gchar *_e2_utf8_pointer_from_offset_skipping_decomp
 	(const gchar *str, gint offset)
 {
-	gsize decomp_len;
-	gunichar *decomp;
 	const gchar *p;
 
 	p = str;
 	while (offset > 0)
 	{
+		gsize decomp_len;
+#ifdef USE_GLIB2_30
+		decomp_len = g_unichar_fully_decompose(g_utf8_get_char (p), FALSE, NULL, 0);
+#else
+		gunichar *decomp;
 		decomp = g_unicode_canonical_decomposition (g_utf8_get_char (p), &decomp_len);
 		g_free (decomp);
-		p = g_utf8_next_char (p);
+#endif
 		offset -= decomp_len;
+		p = g_utf8_next_char (p);
 	}
 	return p;
 }
@@ -299,16 +302,67 @@ gchar *e2_utf8_unescape (const gchar *str, gunichar c)
 	printd (DEBUG, "ret: %s", ret);
 	return ret;
 }
+/**
+@brief if a request has been logged, adjust func pointers to enable or disable file path/name encoding conversion
+*/
+void e2_utf8_set_name_conversion_if_requested (void)
+{
+	if (app.reconvert_requested)
+	{
+		printd (DEBUG, "repoint encoding conversion funcs if necessary, before action starts");
+		e2_utf8_set_name_conversion (app.pane1.view.convert || app.pane2.view.convert);
+		app.reconvert_requested = FALSE;
+	}
+}
+/**
+@brief set func pointers to en/disable file path/name encoding conversion
+@param convert TRUE if conversion to/from UTF-8 is to be done henceforth
+
+@return
+*/
+void e2_utf8_set_name_conversion (gboolean convert)
+{
+	if (convert)
+	{
+		e2_display_from_locale = g_filename_display_name;
+		e2_fname_to_locale = e2_fname_dupto_locale = e2_utf8_filename_to_locale;
+		e2_fname_from_locale = e2_fname_dupfrom_locale = e2_utf8_filename_from_locale;
+//		e2_fname_free = g_free;
+	}
+	else
+	{
+		e2_display_from_locale = e2_fname_to_locale = e2_fname_from_locale
+			= e2_utf8_not_converted;
+ 		e2_fname_dupto_locale = e2_fname_dupfrom_locale = g_strdup;
+//		e2_fname_free = e2_utf8_not_freed;
+	}
+}
 
 //inline
 gchar *e2_utf8_not_converted (const gchar *d)
 {
 	return (gchar *) d;
 }
-void e2_utf8_not_freed (gpointer dummy)
-{}
 
-#ifndef E2_FILES_UTF8ONLY
+//void e2_utf8_not_freed (gpointer dummy)
+//{}
+
+/**
+@brief cleanup @a freeme if it's not the same as @a original
+This is robust to a change of encoding-functions (hence string allocation)
+between creation and cleanup of @a freeme.
+@param freeme the string to be free'd, can be NULL
+@param original the string from which @a freeme was derived
+
+@return
+*/
+void e2_utf8_fname_free (gchar *freeme, const gchar *original)
+{
+	if (freeme != (gchar *)original)
+		g_free (freeme);
+}
+
+//#ifndef E2_FILES_UTF8ONLY
 //with run-time checking, a pointer may be set to this func, so no inline
 //inline
 gchar *e2_utf8_filename_from_locale (const gchar *d)
@@ -370,7 +424,7 @@ gchar *e2_utf8_filename_to_locale (const gchar *d)
 	}
 	return ret;
 }
-#endif // ndef E2_FILES_UTF8ONLY
+//#endif //ndef E2_FILES_UTF8ONLY
 
 gchar *e2_utf8_from_locale (const gchar *d)
 {
@@ -394,7 +448,8 @@ gchar *e2_utf8_from_locale_fallback (const gchar *d)
 		printd (WARN, "fallback locale string to UTF8 conversion failed: %s",
 			error->message);
 		g_error_free (error);
-		ret = g_strdup(_("Unknown"));
+		ret = g_strconcat("\357\277\275 ",	//U+FFFD inverted '?' char
+			_("unknown encoding"), NULL);
 	}
 	return ret;
 }
@@ -481,10 +536,10 @@ const gchar *locale_table[ENCODE_COUNT][MAX_LOCALES] =
 const gchar *encoding_table [ENCODE_COUNT][ENCODING_TYPECOUNT] =
 {
 				/*  IANA          OpenI18N       Codepage */
-// /* LATIN1 */	{ "ISO-8859-1",  "ISO-8859-15", "CP1252" }, //superseded by latin9 (with euro symbol etc)
+// / * LATIN1 * /	{ "ISO-8859-1",  "ISO-8859-15", "CP1252" }, //superseded by latin9 (with euro symbol etc)
 /* LATIN2 */	{ "ISO-8859-2",  "ISO-8859-16", "CP1250" },
 /* LATIN3 */	{ "ISO-8859-3",  NULL,          NULL },
-// /* LATIN4 */	{ "ISO-8859-4",  "ISO-8859-13", "CP1257" },
+// / * LATIN4 * /	{ "ISO-8859-4",  "ISO-8859-13", "CP1257" },
 /* LATIN5 */	{ "ISO-8859-9",  NULL,          "CP1254" },
 /* LATIN6 */	{ "ISO-8859-10", NULL,          "CP1257" },
 /* LATIN7 */	{ "ISO-8859-13", NULL,          "CP1257" },
@@ -902,7 +957,7 @@ const gchar *e2_utf8_detect_charset (const guchar *text, gboolean *isutf8)
 	if (charset != NULL)	//FIXME test for short file length if UTF-32*
 	{
 		//CHECKME aliases for this encoding ?
-		*isutf8 = g_str_equal (charset, "UTF-8");
+		*isutf8 = !strcmp (charset, "UTF-8");
 		return charset;
 	}
 
